@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright © 2012 CohortFS, LLC.
  * Author: Adam C. Emerson <aemerson@linuxbox.com>
@@ -65,7 +66,7 @@ static inline void local_invalidate(struct ds *ds, struct fsal_export *export)
 		.len = sizeof(ds->wire.wire.vi)
 	};
 	up_async_invalidate(general_fridge, export->up_ops, export->fsal, &key,
-			    CACHE_INODE_INVALIDATE_ATTRS, NULL, NULL);
+			    MDCACHE_INVALIDATE_ATTRS, NULL, NULL);
 }
 
 /**
@@ -73,12 +74,11 @@ static inline void local_invalidate(struct ds *ds, struct fsal_export *export)
  *
  * @param[in] ds_pub The object to release
  */
-static void ds_release(struct fsal_ds_handle *const ds_pub)
+static void ds_handle_release(struct fsal_ds_handle *const ds_pub)
 {
 	/* The private 'full' DS handle */
 	struct ds *ds = container_of(ds_pub, struct ds, ds);
 
-	fsal_ds_handle_fini(&ds->ds);
 	gsh_free(ds);
 }
 
@@ -87,11 +87,10 @@ static void ds_release(struct fsal_ds_handle *const ds_pub)
  *
  * NFSv4.1 data server handles are disjount from normal
  * filehandles (in Ganesha, there is a ds_flag in the filehandle_v4_t
- * structure) and do not get loaded into cache_inode or processed the
+ * structure) and do not get loaded into mdcache or processed the
  * normal way.
  *
  * @param[in]  ds_pub           FSAL DS handle
- * @param[in]  req_ctx          Credentials
  * @param[in]  stateid          The stateid supplied with the READ operation,
  *                              for validation
  * @param[in]  offset           The offset at which to read
@@ -103,15 +102,14 @@ static void ds_release(struct fsal_ds_handle *const ds_pub)
  * @return An NFSv4.1 status code.
  */
 static nfsstat4 ds_read(struct fsal_ds_handle *const ds_pub,
-			struct req_op_context *const req_ctx,
 			const stateid4 *stateid, const offset4 offset,
 			const count4 requested_length, void *const buffer,
 			count4 * const supplied_length,
 			bool * const end_of_file)
 {
 	/* The private 'full' export */
-	struct export *export =
-	    container_of(req_ctx->fsal_export, struct export, export);
+	struct ceph_export *export =
+		container_of(op_ctx->fsal_export, struct ceph_export, export);
 	/* The private 'full' DS handle */
 	struct ds *ds = container_of(ds_pub, struct ds, ds);
 	/* The OSD number for this machine */
@@ -172,7 +170,6 @@ static nfsstat4 ds_read(struct fsal_ds_handle *const ds_pub,
  * and performs an MDS write.
  *
  * @param[in]  ds_pub           FSAL DS handle
- * @param[in]  req_ctx          Credentials
  * @param[in]  stateid          The stateid supplied with the READ operation,
  *                              for validation
  * @param[in]  offset           The offset at which to read
@@ -187,7 +184,6 @@ static nfsstat4 ds_read(struct fsal_ds_handle *const ds_pub,
  * @return An NFSv4.1 status code.
  */
 static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
-			 struct req_op_context *const req_ctx,
 			 const stateid4 *stateid, const offset4 offset,
 			 const count4 write_length, const void *buffer,
 			 const stable_how4 stability_wanted,
@@ -196,8 +192,8 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
 			 stable_how4 * const stability_got)
 {
 	/* The private 'full' export */
-	struct export *export =
-	    container_of(req_ctx->fsal_export, struct export, export);
+	struct ceph_export *export =
+		container_of(op_ctx->fsal_export, struct ceph_export, export);
 	/* The private 'full' DS handle */
 	struct ds *ds = container_of(ds_pub, struct ds, ds);
 	/* The OSD number for this host */
@@ -262,7 +258,7 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
 		}
 		ceph_status = fsal_ceph_ll_open(
 			export->cmount, ds->wire.wire.vi,
-			O_WRONLY, &descriptor, op_ctx->creds);
+			O_WRONLY, &descriptor, &op_ctx->creds);
 		if (ceph_status != 0) {
 			LogMajor(COMPONENT_FSAL,
 				 "Open failed with: %d", ceph_status);
@@ -326,11 +322,10 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
  *
  * NFSv4.1 data server filehandles are disjount from normal
  * filehandles (in Ganesha, there is a ds_flag in the filehandle_v4_t
- * structure) and do not get loaded into cache_inode or processed the
+ * structure) and do not get loaded into mdcache or processed the
  * normal way.
  *
  * @param[in]  ds_pub    FSAL DS handle
- * @param[in]  req_ctx   Credentials
  * @param[in]  offset    Start of commit window
  * @param[in]  count     Length of commit window
  * @param[out] writeverf Write verifier
@@ -338,14 +333,13 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
  * @return An NFSv4.1 status code.
  */
 static nfsstat4 ds_commit(struct fsal_ds_handle *const ds_pub,
-			  struct req_op_context *const req_ctx,
 			  const offset4 offset, const count4 count,
 			  verifier4 * const writeverf)
 {
 #ifdef COMMIT_FIX
 	/* The private 'full' export */
-	struct export *export =
-	    container_of(req_ctx->fsal_export, struct export, export);
+	struct ceph_export *export =
+		container_of(op_ctx->fsal_export, struct ceph_export, export);
 	/* The private 'full' DS handle */
 	struct ds *ds = container_of(ds_pub, struct ds, ds);
 	/* Error return from Ceph */
@@ -367,16 +361,6 @@ static nfsstat4 ds_commit(struct fsal_ds_handle *const ds_pub,
 	return NFS4_OK;
 }
 
-static void dsh_ops_init(struct fsal_dsh_ops *ops)
-{
-	memcpy(ops, &def_dsh_ops, sizeof(struct fsal_dsh_ops));
-
-	ops->release = ds_release;
-	ops->read = ds_read;
-	ops->write = ds_write;
-	ops->commit = ds_commit;
-}
-
 /**
  * @brief Try to create a FSAL data server handle from a wire handle
  *
@@ -411,7 +395,6 @@ static nfsstat4 make_ds_handle(struct fsal_pnfs_ds *const pds,
 	ds = gsh_calloc(1, sizeof(struct ds));
 
 	*handle = &ds->ds;
-	fsal_ds_handle_init(*handle, pds);
 
 	/* Connect lazily when a FILE_SYNC4 write forces us to, not
 	   here. */
@@ -426,7 +409,10 @@ void pnfs_ds_ops_init(struct fsal_pnfs_ds_ops *ops)
 {
 	memcpy(ops, &def_pnfs_ds_ops, sizeof(struct fsal_pnfs_ds_ops));
 	ops->make_ds_handle = make_ds_handle;
-	ops->fsal_dsh_ops = dsh_ops_init;
+	ops->dsh_release = ds_handle_release;
+	ops->dsh_read = ds_read;
+	ops->dsh_write = ds_write;
+	ops->dsh_commit = ds_commit;
 }
 
 #endif				/* CEPH_PNFS */
