@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright © 2012-2014, CohortFS, LLC.
  * Author: Adam C. Emerson <aemerson@linuxbox.com>
@@ -49,6 +50,12 @@
 #include "statx_compat.h"
 #include "sal_functions.h"
 #include "nfs_core.h"
+#ifdef CEPHFS_POSIX_ACL
+#include "nfs_exports.h"
+#endif				/* CEPHFS_POSIX_ACL */
+#ifdef USE_LTTNG
+#include "gsh_lttng/fsal_ceph.h"
+#endif
 
 /**
  * @brief Clean up an export
@@ -57,9 +64,6 @@
  * released.
  *
  * @param[in,out] export The export to be released
- *
- * @retval ERR_FSAL_NO_ERROR on success.
- * @retval ERR_FSAL_BUSY if the export is in use.
  */
 
 static void release(struct fsal_export *export_pub)
@@ -67,6 +71,8 @@ static void release(struct fsal_export *export_pub)
 	/* The private, expanded export */
 	struct ceph_export *export =
 			container_of(export_pub, struct ceph_export, export);
+	/* Ceph mount */
+	struct ceph_mount *cm = export->cm;
 
 	deconstruct_handle(export->root);
 	export->root = 0;
@@ -74,8 +80,41 @@ static void release(struct fsal_export *export_pub)
 	fsal_detach_export(export->export.fsal, &export->export.exports);
 	free_export_ops(&export->export);
 
-	ceph_shutdown(export->cmount);
+	PTHREAD_RWLOCK_wrlock(&cmount_lock);
+
+	/* Detach this export from the ceph_mount */
+	glist_del(&export->cm_list);
+
+	if (--cm->cm_refcnt == 0) {
+		/* This was the final reference */
+
+		ceph_shutdown(cm->cmount);
+
+		ceph_mount_remove(&cm->cm_avl_mount);
+
+		gsh_free(cm->cm_fs_name);
+		gsh_free(cm->cm_mount_path);
+		gsh_free(cm->cm_user_id);
+		gsh_free(cm->cm_secret_key);
+
+
+		gsh_free(cm);
+	} else if (cm->cm_export == export) {
+		/* Need to attach a different export for upcalls */
+		cm->cm_export = glist_first_entry(&cm->cm_exports,
+						  struct ceph_export,
+						  cm_list);
+	}
+
 	export->cmount = NULL;
+
+	PTHREAD_RWLOCK_unlock(&cmount_lock);
+
+	gsh_free(export->sec_label_xattr);
+	gsh_free(export->fs_name);
+	gsh_free(export->cmount_path);
+	gsh_free(export->user_id);
+	gsh_free(export->secret_key);
 	gsh_free(export);
 	export = NULL;
 }
@@ -99,7 +138,7 @@ static void release(struct fsal_export *export_pub)
 static fsal_status_t lookup_path(struct fsal_export *export_pub,
 				 const char *path,
 				 struct fsal_obj_handle **pub_handle,
-				 struct attrlist *attrs_out)
+				 struct fsal_attrlist *attrs_out)
 {
 	/* The 'private' full export handle */
 	struct ceph_export *export =
@@ -113,7 +152,7 @@ static fsal_status_t lookup_path(struct fsal_export *export_pub,
 	/* The buffer in which to store stat info */
 	struct ceph_statx stx;
 	/* Return code from Ceph */
-	int rc;
+	int rc, lmp;
 	/* Find the actual path in the supplied path */
 	const char *realpath;
 
@@ -138,23 +177,35 @@ static fsal_status_t lookup_path(struct fsal_export *export_pub,
 	 * must be a superset of the export fullpath, or the string
 	 * handling will be broken.
 	 */
-	if (strstr(realpath, op_ctx->ctx_export->fullpath) != realpath) {
+	if (strstr(realpath, CTX_FULLPATH(op_ctx)) != realpath) {
 		status.major = ERR_FSAL_SERVERFAULT;
 		return status;
 	}
 
+	PTHREAD_RWLOCK_rdlock(&cmount_lock);
+
+	lmp = strlen(export->cm->cm_mount_path);
+
+	if (lmp == 1) {
+		/* If cmount_path is "/" we need the leading '/'. */
+		lmp = 0;
+	}
+
 	/* Advance past the export's fullpath */
-	realpath += strlen(op_ctx->ctx_export->fullpath);
+	realpath += lmp;
+
+	PTHREAD_RWLOCK_unlock(&cmount_lock);
 
 	/* special case the root */
-	if (strcmp(realpath, "/") == 0) {
+	if (realpath[1] == '\0' || realpath[0] == '\0') {
 		assert(export->root);
 		*pub_handle = &export->root->handle;
 		return status;
 	}
 
 	rc = fsal_ceph_ll_walk(export->cmount, realpath, &i, &stx,
-				!!attrs_out, op_ctx->creds);
+				!!attrs_out, &op_ctx->creds);
+
 	if (rc < 0)
 		return ceph2fsal_error(rc);
 
@@ -164,6 +215,10 @@ static fsal_status_t lookup_path(struct fsal_export *export_pub,
 		ceph2fsal_attributes(&stx, attrs_out);
 
 	*pub_handle = &handle->handle;
+#ifdef USE_LTTNG
+	tracepoint(fsalceph, ceph_create_handle, __func__, __LINE__,
+		   &handle->handle);
+#endif
 	return status;
 }
 
@@ -247,6 +302,26 @@ fsal_status_t host_to_key(struct fsal_export *exp_hdl,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
+#ifndef USE_FSAL_CEPH_LOOKUP_VINO
+static int ceph_ll_lookup_vino(struct ceph_mount_info *cmount, vinodeno_t vino,
+			       Inode **inode)
+{
+	/* Check the cache first */
+	*inode = ceph_ll_get_inode(cmount, vino);
+	if (*inode)
+		return 0;
+
+	/*
+	 * We can't look up snap inodes w/o ceph_ll_lookup_vino. Just
+	 * return -ESTALE if we get one that's not already in cache.
+	 */
+	if (vino.snapid.val != CEPH_NOSNAP)
+		return -ESTALE;
+
+	return ceph_ll_lookup_inode(cmount, vino.ino, inode);
+}
+#endif /* USE_FSAL_CEPH_LOOKUP_VINO */
+
 /**
  * @brief Create a handle object from a wire handle
  *
@@ -262,7 +337,7 @@ fsal_status_t host_to_key(struct fsal_export *exp_hdl,
 static fsal_status_t create_handle(struct fsal_export *export_pub,
 				   struct gsh_buffdesc *desc,
 				   struct fsal_obj_handle **pub_handle,
-				   struct attrlist *attrs_out)
+				   struct fsal_attrlist *attrs_out)
 {
 	/* Full 'private' export structure */
 	struct ceph_export *export =
@@ -291,26 +366,13 @@ static fsal_status_t create_handle(struct fsal_export *export_pub,
 	vi.ino.val = hhdl->chk_ino;
 	vi.snapid.val = hhdl->chk_snap;
 
-	/* Check our local cache first */
-	i = ceph_ll_get_inode(export->cmount, vi);
-	if (!i) {
-		/*
-		 * Try the slow way, may not be in cache now.
-		 *
-		 * Currently, there is no interface for looking up a snapped
-		 * inode, so we just bail here in that case.
-		 */
-		if (hhdl->chk_snap != CEPH_NOSNAP)
-			return ceph2fsal_error(-ESTALE);
-
-		rc = ceph_ll_lookup_inode(export->cmount, vi.ino, &i);
-		if (rc)
-			return ceph2fsal_error(rc);
-	}
+	rc = ceph_ll_lookup_vino(export->cmount, vi, &i);
+	if (rc)
+		return ceph2fsal_error(rc);
 
 	rc = fsal_ceph_ll_getattr(export->cmount, i, &stx,
 		attrs_out ? CEPH_STATX_ATTR_MASK : CEPH_STATX_HANDLE_MASK,
-		op_ctx->creds);
+		&op_ctx->creds);
 	if (rc < 0)
 		return ceph2fsal_error(rc);
 
@@ -320,6 +382,10 @@ static fsal_status_t create_handle(struct fsal_export *export_pub,
 		ceph2fsal_attributes(&stx, attrs_out);
 
 	*pub_handle = &handle->handle;
+#ifdef USE_LTTNG
+	tracepoint(fsalceph, ceph_create_handle, __func__, __LINE__,
+		   &handle->handle);
+#endif
 	return status;
 }
 
@@ -359,12 +425,22 @@ static fsal_status_t get_fs_dynamic_info(struct fsal_export *export_pub,
 	info->total_files = vfs_st.f_files;
 	info->free_files = vfs_st.f_ffree;
 	info->avail_files = vfs_st.f_favail;
-	info->time_delta.tv_sec = 1;
-	info->time_delta.tv_nsec = 0;
+	info->time_delta.tv_sec = 0;
+	info->time_delta.tv_nsec = FSAL_DEFAULT_TIME_DELTA_NSEC;
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
+#ifdef CEPHFS_POSIX_ACL
+static fsal_aclsupp_t fs_acl_support(struct fsal_export *exp_hdl)
+{
+	if (!op_ctx_export_has_option(EXPORT_OPTION_DISABLE_ACL))
+		return fsal_acl_support(&exp_hdl->fsal->fs_info);
+	else
+		return 0;
+}
+#endif				/* CEPHFS_POSIX_ACL */
+
 void ceph_prepare_unexport(struct fsal_export *export_pub)
 {
 	struct ceph_export *export =
@@ -385,6 +461,30 @@ void ceph_prepare_unexport(struct fsal_export *export_pub)
 #endif
 }
 
+
+/**
+ * @brief Function to get the fasl_obj_handle that has fsal_fd as its global fd.
+ *
+ * @param[in]     exp_hdl   The export in which the handle exists
+ * @param[in]     fd        File descriptor in question
+ * @param[out]    handle    FSAL object handle
+ *
+ * @return the fsal_obj_handle.
+ */
+void get_fsal_obj_hdl(struct fsal_export *exp_hdl,
+				  struct fsal_fd *fd,
+				  struct fsal_obj_handle **handle)
+{
+	struct ceph_fd *my_fd = NULL;
+	struct ceph_handle *myself = NULL;
+
+	my_fd = container_of(fd, struct ceph_fd, fsal_fd);
+	myself = container_of(my_fd, struct ceph_handle, fd);
+
+	*handle = &myself->handle;
+}
+
+
 /**
  * @brief Set operations for exports
  *
@@ -396,7 +496,7 @@ void ceph_prepare_unexport(struct fsal_export *export_pub)
 
 void export_ops_init(struct export_ops *ops)
 {
-	ops->prepare_unexport = ceph_prepare_unexport,
+	ops->prepare_unexport = ceph_prepare_unexport;
 	ops->release = release;
 	ops->lookup_path = lookup_path;
 	ops->host_to_key = host_to_key;
@@ -404,7 +504,10 @@ void export_ops_init(struct export_ops *ops)
 	ops->create_handle = create_handle;
 	ops->get_fs_dynamic_info = get_fs_dynamic_info;
 	ops->alloc_state = ceph_alloc_state;
-	ops->free_state = ceph_free_state;
+	ops->get_fsal_obj_hdl = get_fsal_obj_hdl;
+#ifdef CEPHFS_POSIX_ACL
+	ops->fs_acl_support = fs_acl_support;
+#endif				/* CEPHFS_POSIX_ACL */
 #ifdef CEPH_PNFS
 	export_ops_pnfs(ops);
 #endif				/* CEPH_PNFS */
