@@ -374,6 +374,18 @@ static fsal_status_t get_dynamic_info(struct fsal_export *exp_hdl,
 	return status;
 }
 
+void gluster_free_state(struct state_t *state)
+{
+	struct glusterfs_fd *my_fd;
+
+	my_fd = &container_of(state, struct glusterfs_state_fd,
+			      state)->glusterfs_fd;
+
+	destroy_fsal_fd(&my_fd->fsal_fd);
+
+	gsh_free(state);
+}
+
 /**
  * @brief Allocate a state_t structure
  *
@@ -394,36 +406,17 @@ struct state_t *glusterfs_alloc_state(struct fsal_export *exp_hdl,
 	struct glusterfs_fd *my_fd;
 
 	state = init_state(gsh_calloc(1, sizeof(struct glusterfs_state_fd)),
-			   exp_hdl, state_type, related_state);
+			   gluster_free_state, state_type, related_state);
 
 	my_fd = &container_of(state, struct glusterfs_state_fd,
 			      state)->glusterfs_fd;
 
+	init_fsal_fd(&my_fd->fsal_fd, FSAL_FD_STATE, op_ctx->fsal_export);
 	my_fd->glfd = NULL;
-	my_fd->openflags = FSAL_O_CLOSED;
-	PTHREAD_RWLOCK_init(&my_fd->fdlock, NULL);
 
 	return state;
 }
 
-/**
- * @brief free a gluster_state_fd structure
- *
- * @param[in] exp_hdl  Export state_t will be associated with
- * @param[in] state    Related state if appropriate
- *
- */
-void glusterfs_free_state(struct fsal_export *exp_hdl, struct state_t *state)
-{
-	struct glusterfs_state_fd *state_fd =
-		container_of(state, struct glusterfs_state_fd, state);
-	struct glusterfs_fd *my_fd = &state_fd->glusterfs_fd;
-
-	PTHREAD_RWLOCK_destroy(&my_fd->fdlock);
-
-	gsh_free(state_fd);
-}
-
 /** @todo: We have gone POSIX way for the APIs below, can consider the CEPH way
  * in case all are constants across all volumes etc.
  */
@@ -442,6 +435,30 @@ static attrmask_t fs_supported_attrs(struct fsal_export *exp_hdl)
 	return supported_mask;
 }
 
+
+/**
+ * @brief Function to get the fasl_obj_handle that has fsal_fd as its global fd.
+ *
+ * @param[in]     exp_hdl   The export in which the handle exists
+ * @param[in]     fd        File descriptor in question
+ * @param[out]    handle    FSAL object handle
+ *
+ * @return the fsal_obj_handle.
+ */
+void get_fsal_obj_hdl(struct fsal_export *exp_hdl,
+				  struct fsal_fd *fd,
+				  struct fsal_obj_handle **handle)
+{
+	struct glusterfs_fd *my_fd = NULL;
+	struct glusterfs_handle *myself = NULL;
+
+	my_fd = container_of(fd, struct glusterfs_fd, fsal_fd);
+	myself = container_of(my_fd, struct glusterfs_handle, globalfd);
+
+	*handle = &myself->handle;
+}
+
+
 /**
  * @brief Registers GLUSTER FSAL exportoperation vector
  *
@@ -460,7 +477,7 @@ void export_ops_init(struct export_ops *ops)
 	ops->get_fs_dynamic_info = get_dynamic_info;
 	ops->fs_supported_attrs = fs_supported_attrs;
 	ops->alloc_state = glusterfs_alloc_state;
-	ops->free_state = glusterfs_free_state;
+	ops->get_fsal_obj_hdl = get_fsal_obj_hdl;
 }
 
 enum transport {
@@ -526,7 +543,7 @@ glusterfs_free_fs(struct glusterfs_fs *gl_fs)
 	int64_t refcnt;
 	int err     = 0;
 
-	PTHREAD_MUTEX_lock(&GlusterFS.lock);
+	PTHREAD_MUTEX_lock(&GlusterFS.glfs_lock);
 
 	refcnt = --(gl_fs->refcnt);
 
@@ -537,12 +554,12 @@ glusterfs_free_fs(struct glusterfs_fs *gl_fs)
 			 "There are still (%"PRIi64
 			 ")active shares for volume(%s)",
 			 gl_fs->refcnt, gl_fs->volname);
-		PTHREAD_MUTEX_unlock(&GlusterFS.lock);
+		PTHREAD_MUTEX_unlock(&GlusterFS.glfs_lock);
 		return;
 	}
 
 	glist_del(&gl_fs->fs_obj);
-	PTHREAD_MUTEX_unlock(&GlusterFS.lock);
+	PTHREAD_MUTEX_unlock(&GlusterFS.glfs_lock);
 
 	atomic_inc_int8_t(&gl_fs->destroy_mode);
 
@@ -598,7 +615,7 @@ glusterfs_get_fs(struct glexport_params params,
 	glfs_t  *fs = NULL;
 	struct glist_head *glist, *glistn;
 
-	PTHREAD_MUTEX_lock(&GlusterFS.lock);
+	PTHREAD_MUTEX_lock(&GlusterFS.glfs_lock);
 
 	glist_for_each_safe(glist, glistn, &GlusterFS.fs_obj) {
 		gl_fs = glist_entry(glist, struct glusterfs_fs,
@@ -667,13 +684,8 @@ glusterfs_get_fs(struct glexport_params params,
 		goto skip_upcall;
 
 #ifndef USE_GLUSTER_UPCALL_REGISTER
-	rc = initiate_up_thread(gl_fs);
-	if (rc != 0) {
-		LogCrit(COMPONENT_FSAL,
-			"Unable to create GLUSTERFSAL_UP_Thread. Volume: %s",
-			params.glvolname);
+	if (initiate_up_thread(gl_fs) != 0)
 		goto out;
-	}
 #else
 	/* We are mainly interested in INODE_INVALIDATE for now. Still
 	 * register for all the events
@@ -696,11 +708,11 @@ skip_upcall:
 
 found:
 	++(gl_fs->refcnt);
-	PTHREAD_MUTEX_unlock(&GlusterFS.lock);
+	PTHREAD_MUTEX_unlock(&GlusterFS.glfs_lock);
 	return gl_fs;
 
 out:
-	PTHREAD_MUTEX_unlock(&GlusterFS.lock);
+	PTHREAD_MUTEX_unlock(&GlusterFS.glfs_lock);
 	if (fs)
 		glfs_fini(fs);
 
