@@ -58,7 +58,8 @@ static void release(struct fsal_export *exp_hdl)
 	struct gpfs_fsal_export *myself =
 	    container_of(exp_hdl, struct gpfs_fsal_export, export);
 
-	gpfs_unexport_filesystems(myself);
+	unclaim_all_export_maps(exp_hdl);
+
 	fsal_detach_export(exp_hdl->fsal, &exp_hdl->exports);
 	free_export_ops(exp_hdl);
 	close(myself->export_fd);
@@ -323,6 +324,17 @@ gpfs_host_to_key(struct fsal_export *exp_hdl,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
+void gpfs_free_state(struct state_t *state)
+{
+	struct gpfs_fd *my_fd;
+
+	my_fd = &container_of(state, struct gpfs_state_fd, state)->gpfs_fd;
+
+	destroy_fsal_fd(&my_fd->fsal_fd);
+
+	gsh_free(state);
+}
+
 /**
  * @brief Allocate a state_t structure
  *
@@ -343,34 +355,37 @@ gpfs_alloc_state(struct fsal_export *exp_hdl, enum state_type state_type,
 	struct gpfs_fd *my_fd;
 
 	state = init_state(gsh_calloc(1, sizeof(struct gpfs_state_fd)),
-			   exp_hdl, state_type, related_state);
+			   gpfs_free_state, state_type, related_state);
 
 	my_fd = &container_of(state, struct gpfs_state_fd, state)->gpfs_fd;
 
+	init_fsal_fd(&my_fd->fsal_fd, FSAL_FD_STATE, op_ctx->fsal_export);
 	my_fd->fd = -1;
-	my_fd->openflags = FSAL_O_CLOSED;
-	PTHREAD_RWLOCK_init(&my_fd->fdlock, NULL);
 
 	return state;
 }
 
-/**
- * @brief free a gpfs_state_fd structure
- *
- * @param[in] exp_hdl  Export state_t will be associated with
- * @param[in] state    Related state if appropriate
- *
- */
-void
-gpfs_free_state(struct fsal_export *exp_hdl, struct state_t *state)
-{
-	struct gpfs_state_fd *state_fd = container_of(state,
-						      struct gpfs_state_fd,
-						      state);
-	struct gpfs_fd *my_fd = &state_fd->gpfs_fd;
 
-	PTHREAD_RWLOCK_destroy(&my_fd->fdlock);
-	gsh_free(state_fd);
+/**
+ * @brief Function to get the fasl_obj_handle that has fsal_fd as its global fd.
+ *
+ * @param[in]     exp_hdl   The export in which the handle exists
+ * @param[in]     fd        File descriptor in question
+ * @param[out]    handle    FSAL object handle
+ *
+ * @return the fsal_obj_handle.
+ */
+void get_fsal_obj_hdl(struct fsal_export *exp_hdl,
+				  struct fsal_fd *fd,
+				  struct fsal_obj_handle **handle)
+{
+	struct gpfs_fd *my_fd = NULL;
+	struct gpfs_fsal_obj_handle *myself = NULL;
+
+	my_fd = container_of(fd, struct gpfs_fd, fsal_fd);
+	myself = container_of(my_fd, struct gpfs_fsal_obj_handle, u.file.fd);
+
+	*handle = &myself->obj_handle;
 }
 
 /**
@@ -389,14 +404,14 @@ void gpfs_export_ops_init(struct export_ops *ops)
 	ops->get_quota = get_quota;
 	ops->set_quota = set_quota;
 	ops->alloc_state = gpfs_alloc_state;
-	ops->free_state = gpfs_free_state;
+	ops->get_fsal_obj_hdl = get_fsal_obj_hdl;
 }
 
 static void free_gpfs_filesystem(struct gpfs_filesystem *gpfs_fs)
 {
 	if (gpfs_fs->root_fd >= 0)
 		close(gpfs_fs->root_fd);
-	PTHREAD_MUTEX_destroy(&gpfs_fs->upvector_mutex);
+
 	gsh_free(gpfs_fs);
 }
 
@@ -477,10 +492,13 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs,
 			  struct fsal_export *exp,
 			  void **private_data)
 {
-	struct gpfs_filesystem *gpfs_fs;
+	struct gpfs_filesystem *gpfs_fs = NULL;
 	int retval;
-	struct gpfs_filesystem_export_map *map;
 	pthread_attr_t attr_thr;
+	int rc;
+	struct gpfs_fsal_export *myself;
+
+	LogFilesystem("GPFS CLAIM FS", "", fs);
 
 	if (strcmp(fs->type, "gpfs") != 0) {
 		LogEvent(COMPONENT_FSAL,
@@ -488,32 +506,11 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs,
 		return ENXIO;
 	}
 
-	gpfs_fs = *private_data;
+	myself = container_of(exp, struct gpfs_fsal_export, export);
 
-	if (gpfs_fs == NULL) { /* first export by GPFS */
-		gpfs_fs = gsh_calloc(1, sizeof(*gpfs_fs));
-		glist_init(&gpfs_fs->exports);
-		gpfs_fs->root_fd = -1;
-		gpfs_fs->fs = fs;
-		PTHREAD_MUTEX_init(&gpfs_fs->upvector_mutex, NULL);
-	}
+	myself->export_fd = open(CTX_FULLPATH(op_ctx), O_RDONLY | O_DIRECTORY);
 
-	/* Now map the file system and export */
-	map = gsh_calloc(1, sizeof(*map));
-	map->fs = gpfs_fs;
-	map->exp = container_of(exp, struct gpfs_fsal_export, export);
-
-	PTHREAD_MUTEX_lock(&gpfs_fs->upvector_mutex);
-
-	glist_add_tail(&gpfs_fs->exports, &map->on_exports);
-	glist_add_tail(&map->exp->filesystems, &map->on_filesystems);
-
-	PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
-
-	map->exp->export_fd = open(CTX_FULLPATH(op_ctx),
-				   O_RDONLY | O_DIRECTORY);
-
-	if (map->exp->export_fd < 0) {
+	if (myself->export_fd < 0) {
 		retval = errno;
 		LogMajor(COMPONENT_FSAL,
 			"Could not open GPFS export point %s: rc = %s (%d)",
@@ -521,19 +518,28 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs,
 		goto errout;
 	}
 
-	LogFullDebug(COMPONENT_FSAL, "export_fd %d path %s",
-			map->exp->export_fd, CTX_FULLPATH(op_ctx));
-
-	/* We have set up the export. If the file system is already claimed,
-	 * we are done.
-	 */
 	if (*private_data != NULL) {
-		/* file system was already claimed, nothing more to do. */
+		/* Already claimed, and private_data is already set, nothing to
+		 * do here.
+		 */
+		LogDebug(COMPONENT_FSAL,
+			 "file system %s is already claimed with fd %d private_data %p",
+			 fs->path, (int) (long) *private_data, *private_data);
 		return 0;
 	}
 
+	/* first export by GPFS */
+	gpfs_fs = gsh_calloc(1, sizeof(*gpfs_fs));
+	gpfs_fs->root_fd = -1;
+	gpfs_fs->fs = fs;
+
+	LogFullDebug(COMPONENT_FSAL,
+		     "export_fd %d path %s",
+		     myself->export_fd, CTX_FULLPATH(op_ctx));
+
 	/* Get an fd for the root and create an upcall thread */
 	retval = open_root_fd(gpfs_fs);
+
 	if (retval != 0) {
 		if (retval == ENOTTY) {
 			LogInfo(COMPONENT_FSAL,
@@ -546,21 +552,17 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs,
 
 	gpfs_fs->stop_thread = false;
 
-	if (pthread_attr_init(&attr_thr) != 0)
-		LogCrit(COMPONENT_THREAD, "can't init pthread's attributes");
+	PTHREAD_ATTR_init(&attr_thr);
+	PTHREAD_ATTR_setscope(&attr_thr, PTHREAD_SCOPE_SYSTEM);
+	PTHREAD_ATTR_setdetachstate(&attr_thr, PTHREAD_CREATE_JOINABLE);
+	PTHREAD_ATTR_setstacksize(&attr_thr, 2116488);
 
-	if (pthread_attr_setscope(&attr_thr, PTHREAD_SCOPE_SYSTEM) != 0)
-		LogCrit(COMPONENT_THREAD, "can't set pthread's scope");
+	rc = pthread_create(&gpfs_fs->up_thread, &attr_thr, GPFSFSAL_UP_Thread,
+			    gpfs_fs);
 
-	if (pthread_attr_setdetachstate(&attr_thr,
-					PTHREAD_CREATE_JOINABLE) != 0)
-		LogCrit(COMPONENT_THREAD, "can't set pthread's join state");
+	PTHREAD_ATTR_destroy(&attr_thr);
 
-	if (pthread_attr_setstacksize(&attr_thr, 2116488) != 0)
-		LogCrit(COMPONENT_THREAD, "can't set pthread's stack size");
-
-	if (pthread_create(&gpfs_fs->up_thread, &attr_thr, GPFSFSAL_UP_Thread,
-			   gpfs_fs)) {
+	if (rc != 0) {
 		retval = errno;
 		LogCrit(COMPONENT_THREAD,
 			"Could not create GPFSFSAL_UP_Thread, error = %d (%s)",
@@ -574,16 +576,12 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs,
 
 errout:
 
-	if (map->exp->export_fd >= 0) {
-		close(map->exp->export_fd);
-		map->exp->export_fd = -1;
+	if (myself->export_fd >= 0) {
+		close(myself->export_fd);
+		myself->export_fd = -1;
 	}
-	PTHREAD_MUTEX_lock(&gpfs_fs->upvector_mutex);
-	glist_del(&map->on_filesystems);
-	glist_del(&map->on_exports);
-	PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
-	gsh_free(map);
-	if (!fs->private_data)
+
+	if (gpfs_fs != NULL)
 		free_gpfs_filesystem(gpfs_fs);
 
 	return retval;
@@ -596,33 +594,12 @@ errout:
 void gpfs_unclaim_filesystem(struct fsal_filesystem *fs)
 {
 	struct gpfs_filesystem *gpfs_fs = fs->private_data;
-	struct glist_head *glist, *glistn;
-	struct gpfs_filesystem_export_map *map;
 	struct callback_arg callback = {0};
 	int reason = THREAD_STOP;
 
 	if (gpfs_fs == NULL)
 		goto out;
 
-	glist_for_each_safe(glist, glistn, &gpfs_fs->exports) {
-		map = glist_entry(glist, struct gpfs_filesystem_export_map,
-				  on_exports);
-
-		/* Remove this file system from mapping */
-		PTHREAD_MUTEX_lock(&map->fs->upvector_mutex);
-		glist_del(&map->on_filesystems);
-		glist_del(&map->on_exports);
-		PTHREAD_MUTEX_unlock(&map->fs->upvector_mutex);
-
-		if (map->exp->root_fs == fs)
-			LogInfo(COMPONENT_FSAL,
-				"Removing root_fs %s from GPFS export",
-				fs->path);
-
-		/* And free it */
-		gsh_free(map);
-	}
-
 	/* Terminate GPFS upcall thread */
 	callback.mountdirfd = gpfs_fs->root_fd;
 	callback.reason = &reason;
@@ -648,41 +625,6 @@ out:
 	LogInfo(COMPONENT_FSAL, "GPFS Unclaiming %s", fs->path);
 }
 
-/**
- *  @brief Unexport filesystem
- *  @param exp FSAL export
- */
-void gpfs_unexport_filesystems(struct gpfs_fsal_export *exp)
-{
-	struct glist_head *glist, *glistn;
-	struct gpfs_filesystem_export_map *map;
-
-	PTHREAD_RWLOCK_wrlock(&fs_lock);
-
-	glist_for_each_safe(glist, glistn, &exp->filesystems) {
-		map = glist_entry(glist, struct gpfs_filesystem_export_map,
-				  on_filesystems);
-
-		/* Remove this export from mapping */
-		PTHREAD_MUTEX_lock(&map->fs->upvector_mutex);
-		glist_del(&map->on_filesystems);
-		glist_del(&map->on_exports);
-		PTHREAD_MUTEX_unlock(&map->fs->upvector_mutex);
-
-		if (glist_empty(&map->fs->exports)) {
-			LogInfo(COMPONENT_FSAL,
-				"GPFS is no longer exporting filesystem %s",
-				map->fs->fs->path);
-			//unclaim_fs(map->fs->fs);
-		}
-
-		/* And free it */
-		gsh_free(map);
-	}
-
-	PTHREAD_RWLOCK_unlock(&fs_lock);
-}
-
 /* GPFS FSAL export config */
 static struct config_item export_params[] = {
 	CONF_ITEM_NOOP("name"),
@@ -836,7 +778,7 @@ gpfs_create_export(struct fsal_module *fsal_hdl, void *parse_node,
 	return status;
 
 unexport:
-	gpfs_unexport_filesystems(gpfs_exp);
+	unclaim_all_export_maps(exp);
 detach:
 	fsal_detach_export(fsal_hdl, &exp->exports);
 free:
