@@ -78,15 +78,14 @@ struct gpfs_ds {
 };
 
 struct gpfs_fd {
-	/** The open and share mode etc. */
-	fsal_openflags_t openflags;
-	/* rw lock to protect the file descriptor */
-	pthread_rwlock_t fdlock;
+	/** open and share mode plus fd management */
+	struct fsal_fd fsal_fd;
 	/** The gpfsfs file descriptor. */
 	int fd;
 };
 
 struct gpfs_state_fd {
+	/** state MUST be first to use default free_state */
 	struct state_t state;
 	struct gpfs_fd gpfs_fd;
 };
