@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /**
  * @file    fsal_rename.c
  * @date    $Date: 2006/01/24 13:45:37 $
@@ -40,16 +41,14 @@
  *  @param old_name Current name of the object to be moved/renamed.
  *  @param new_hdl Target parent directory for the object.
  *  @param new_name New name for the object.
- *  @param op_ctx Authentication context for the operation (user,...).
  *
  *  @return Major error codes :
  *        - ERR_FSAL_NO_ERROR     (no error)
- *        - Another error code if an error occured.
+ *        - Another error code if an error occurred.
  */
 fsal_status_t
 GPFSFSAL_rename(struct fsal_obj_handle *old_hdl, const char *old_name,
-		struct fsal_obj_handle *new_hdl, const char *new_name,
-		const struct req_op_context *op_ctx)
+		struct fsal_obj_handle *new_hdl, const char *new_name)
 {
 
 	fsal_status_t status;
