@@ -66,33 +66,12 @@ struct gpfs_fsal_export {
 struct gpfs_filesystem {
 	struct fsal_filesystem *fs;
 	int root_fd;
-	struct glist_head exports;
 	bool stop_thread;
 	pthread_t up_thread; /* upcall thread */
-
-	/* we have an upcall thread for each file system. We use upvector_mutex
-	 * to get an export from the list of exports in a file system. The
-	 * following up_vector points to up_ops in such an export.
-	 */
-	pthread_mutex_t upvector_mutex;
-	struct fsal_up_vector *up_vector;
-};
-
-/*
- * Link GPFS file systems and exports
- * Supports a many-to-many relationship
- */
-struct gpfs_filesystem_export_map {
-	struct gpfs_fsal_export *exp;
-	struct gpfs_filesystem *fs;
-	struct glist_head on_exports;
-	struct glist_head on_filesystems;
 };
 
 void gpfs_extract_fsid(struct gpfs_file_handle *fh, struct fsal_fsid__ *fsid);
 
-void gpfs_unexport_filesystems(struct gpfs_fsal_export *exp);
-
 fsal_status_t gpfs_merge(struct fsal_obj_handle *orig_hdl,
 			 struct fsal_obj_handle *dupe_hdl);
 
@@ -142,6 +121,8 @@ fsal_status_t gpfs_open2(struct fsal_obj_handle *obj_hdl,
 			 struct fsal_obj_handle **new_obj,
 			 struct fsal_attrlist *attrs_out,
 			 bool *caller_perm_check);
+fsal_openflags_t gpfs_status2(struct fsal_obj_handle *obj_hdl,
+			      struct state_t *state);
 fsal_status_t gpfs_reopen2(struct fsal_obj_handle *obj_hdl,
 			   struct state_t *state,
 			   fsal_openflags_t openflags);
@@ -166,6 +147,11 @@ fsal_status_t gpfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 			    fsal_lock_param_t *conflicting_lock);
 fsal_status_t gpfs_close2(struct fsal_obj_handle *obj_hdl,
 			  struct state_t *state);
+fsal_status_t gpfs_reopen_func(struct fsal_obj_handle *obj_hdl,
+			       fsal_openflags_t openflags,
+			       struct fsal_fd *fsal_fd);
+fsal_status_t gpfs_close_func(struct fsal_obj_handle *obj_hdl,
+			      struct fsal_fd *fd);
 fsal_status_t gpfs_setattr2(struct fsal_obj_handle *obj_hdl,
 			    bool bypass,
 			    struct state_t *state,
@@ -174,8 +160,9 @@ fsal_status_t gpfs_read_plus_fd(int my_fs,
 			uint64_t offset,
 			size_t buffer_size, void *buffer, size_t *read_amount,
 			bool *end_of_file, struct io_info *info, int expfd);
-fsal_status_t gpfs_seek(struct fsal_obj_handle *obj_hdl,
-			struct io_info *info);
+fsal_status_t gpfs_seek2(struct fsal_obj_handle *obj_hdl,
+			 struct state_t *state,
+			 struct io_info *info);
 fsal_status_t gpfs_io_advise(struct fsal_obj_handle *obj_hdl,
 			 struct io_hints *hints);
 fsal_status_t gpfs_share_op(struct fsal_obj_handle *obj_hdl, void *p_owner,
