@@ -69,10 +69,23 @@ struct gpfs_fsal_obj_handle *alloc_handle(struct gpfs_file_handle *fh,
 	hdl->handle = (struct gpfs_file_handle *)&hdl[1];
 	hdl->obj_handle.fs = fs;
 	memcpy(hdl->handle, fh, fh->handle_size);
+
+	/* See fsal_internal_get_handle_at() for details. Some versions
+	 * of GPFS may return 40 byte sized handles and 48 byte sized
+	 * handles for the same object. Having different sized file
+	 * handles for the same object causes multiple entries in the
+	 * cache leading to junction check failures. This code makes
+	 * sure that any client sending us the old 40 byte sized handle
+	 * will be modified here to match its 48 byte sized handle!
+	 */
+	if (hdl->handle->handle_size == 40) {
+		hdl->handle->handle_size = 48;
+	}
+
 	hdl->obj_handle.type = attributes->type;
 	if (hdl->obj_handle.type == REGULAR_FILE) {
 		hdl->u.file.fd.fd = -1;	/* no open on this yet */
-		hdl->u.file.fd.openflags = FSAL_O_CLOSED;
+		hdl->u.file.fd.fsal_fd.openflags = FSAL_O_CLOSED;
 	} else if (hdl->obj_handle.type == SYMBOLIC_LINK
 		   && link_content != NULL) {
 		size_t len = strlen(link_content) + 1;
@@ -86,6 +99,11 @@ struct gpfs_fsal_obj_handle *alloc_handle(struct gpfs_file_handle *fh,
 	hdl->obj_handle.fsid = attributes->fsid;
 	hdl->obj_handle.fileid = attributes->fileid;
 
+	if (hdl->obj_handle.type == REGULAR_FILE) {
+		init_fsal_fd(&hdl->u.file.fd.fsal_fd, FSAL_FD_GLOBAL,
+			     op_ctx->fsal_export);
+	}
+
 	if (myself->pnfs_mds_enabled)
 		hdl->obj_handle.obj_ops = &GPFS.handle_ops_with_pnfs;
 	else
@@ -934,17 +952,21 @@ static void release(struct fsal_obj_handle *obj_hdl)
 
 	LogFullDebug(COMPONENT_FSAL, "type %d", type);
 	if (type == REGULAR_FILE) {
-		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+		fsal_status_t st;
 
-		if (myself->u.file.fd.openflags != FSAL_O_CLOSED) {
-			fsal_internal_close(myself->u.file.fd.fd, NULL, 0);
-			myself->u.file.fd.fd = -1;
-			myself->u.file.fd.openflags = FSAL_O_CLOSED;
+		st = close_fsal_fd(obj_hdl, &myself->u.file.fd.fsal_fd, false);
+
+		if (FSAL_IS_ERROR(st)) {
+			LogCrit(COMPONENT_FSAL,
+				"Could not close hdl 0x%p, status %s error %s(%d)",
+				obj_hdl, fsal_err_txt(st),
+				strerror(st.minor), st.minor);
 		}
-
-		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 	}
 
+	if (myself->obj_handle.type == REGULAR_FILE)
+		destroy_fsal_fd(&myself->u.file.fd.fsal_fd);
+
 	fsal_obj_handle_fini(obj_hdl);
 
 	if (type == SYMBOLIC_LINK) {
@@ -972,7 +994,7 @@ void gpfs_handle_ops_init(struct fsal_obj_ops *ops)
 	ops->link = linkfile;
 	ops->rename = renamefile;
 	ops->unlink = file_unlink;
-	ops->seek = gpfs_seek;
+	ops->seek2 = gpfs_seek2;
 	ops->io_advise = gpfs_io_advise;
 	ops->close = gpfs_close;
 	ops->handle_to_wire = handle_to_wire;
@@ -983,12 +1005,15 @@ void gpfs_handle_ops_init(struct fsal_obj_ops *ops)
 	ops->removexattrs = removexattrs;
 	ops->listxattrs = listxattrs;
 	ops->open2 = gpfs_open2;
+	ops->status2 = gpfs_status2;
 	ops->reopen2 = gpfs_reopen2;
 	ops->read2 = gpfs_read2;
 	ops->write2 = gpfs_write2;
 	ops->commit2 = gpfs_commit2;
 	ops->setattr2 = gpfs_setattr2;
 	ops->close2 = gpfs_close2;
+	ops->close_func = gpfs_close_func;
+	ops->reopen_func = gpfs_reopen_func;
 	ops->lock_op2 = gpfs_lock_op2;
 	ops->merge = gpfs_merge;
 	ops->is_referral = fsal_common_is_referral;
