@@ -54,7 +54,7 @@ struct gpfs_fsal_module GPFS = {
 			.link_support = true,
 			.symlink_support = true,
 			.lock_support = true,
-			.lock_support_async_block = true,
+			.lock_support_async_block = false,
 			.named_attr = true,
 			.unique_handles = true,
 			.acl_support = FSAL_ACLSUPPORT_ALLOW |
@@ -118,6 +118,7 @@ static struct config_block gpfs_param = {
 	.dbus_interface_name = "org.ganesha.nfsd.config.fsal.gpfs",
 	.blk_desc.name = "GPFS",
 	.blk_desc.type = CONFIG_BLOCK,
+	.blk_desc.flags = CONFIG_UNIQUE,  /* too risky to have more */
 	.blk_desc.u.blk.init = noop_conf_init,
 	.blk_desc.u.blk.params = gpfs_params,
 	.blk_desc.u.blk.commit = noop_conf_commit
