@@ -394,6 +394,17 @@ static uint32_t lzfs_fsal_fs_umask(struct fsal_export *exp_hdl)
 	return fsal_umask(info);
 }
 
+void lzfs_free_state(struct state_t *state)
+{
+	struct lzfs_fsal_fd *my_fd;
+
+	my_fd = &container_of(state, struct lzfs_fsal_state_fd, state)->lzfs_fd;
+
+	destroy_fsal_fd(&my_fd->fsal_fd);
+
+	gsh_free(state);
+}
+
 /*! \brief Allocate a state_t structure
  *
  * \see fsal_api.h for more information
@@ -402,19 +413,40 @@ struct state_t *lzfs_fsal_alloc_state(struct fsal_export *exp_hdl,
 				      enum state_type state_type,
 				      struct state_t *related_state)
 {
-	return init_state(gsh_calloc(1, sizeof(struct lzfs_fsal_state_fd)),
-			  exp_hdl, state_type, related_state);
+	struct state_t *state;
+	struct lzfs_fsal_fd *my_fd;
+
+	state = init_state(gsh_calloc(1, sizeof(struct lzfs_fsal_state_fd)),
+			   lzfs_free_state, state_type, related_state);
+
+	my_fd = &container_of(state, struct lzfs_fsal_state_fd, state)->lzfs_fd;
+
+	init_fsal_fd(&my_fd->fsal_fd, FSAL_FD_STATE, op_ctx->fsal_export);
+
+	return state;
 }
 
-/*! \brief Free a state_t structure
+
+/**
+ * @brief Function to get the fasl_obj_handle that has fsal_fd as its global fd.
  *
- * \see fsal_api.h for more information
+ * @param[in]     exp_hdl   The export in which the handle exists
+ * @param[in]     fd        File descriptor in question
+ * @param[out]    handle    FSAL object handle
+ *
+ * @return the fsal_obj_handle.
  */
-void lzfs_fsal_free_state(struct fsal_export *exp_hdl, struct state_t *state)
+void get_fsal_obj_hdl(struct fsal_export *exp_hdl,
+				  struct fsal_fd *fd,
+				  struct fsal_obj_handle **handle)
 {
-	struct lzfs_fsal_state_fd *state_fd = container_of(
-				state, struct lzfs_fsal_state_fd, state);
-	gsh_free(state_fd);
+	struct lzfs_fsal_fd *my_fd = NULL;
+	struct lzfs_fsal_handle *myself = NULL;
+
+	my_fd = container_of(fd, struct lzfs_fsal_fd, fsal_fd);
+	myself = container_of(my_fd, struct lzfs_fsal_handle, fd);
+
+	*handle = &myself->handle;
 }
 
 void lzfs_fsal_export_ops_init(struct export_ops *ops)
@@ -435,6 +467,6 @@ void lzfs_fsal_export_ops_init(struct export_ops *ops)
 	ops->fs_supported_attrs = lzfs_fsal_fs_supported_attrs;
 	ops->fs_umask = lzfs_fsal_fs_umask;
 	ops->alloc_state = lzfs_fsal_alloc_state;
-	ops->free_state = lzfs_fsal_free_state;
+	ops->get_fsal_obj_hdl = get_fsal_obj_hdl;
 	lzfs_fsal_export_ops_pnfs(ops);
 }
