@@ -62,7 +62,7 @@ struct mem_fsal_module MEM = {
 			.case_preserving = true,
 			.link_support = true,
 			.symlink_support = true,
-			.lock_support = true,
+			.lock_support = false,
 			.lock_support_async_block = false,
 			.named_attr = false,
 			.unique_handles = true,
@@ -97,6 +97,7 @@ static struct config_block mem_block = {
 	.dbus_interface_name = "org.ganesha.nfsd.config.fsal.mem",
 	.blk_desc.name = "MEM",
 	.blk_desc.type = CONFIG_BLOCK,
+	.blk_desc.flags = CONFIG_UNIQUE,  /* too risky to have more */
 	.blk_desc.u.blk.init = noop_conf_init,
 	.blk_desc.u.blk.params = mem_items,
 	.blk_desc.u.blk.commit = noop_conf_commit
