@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:shiftwidth=8:tabstop=8:
  *
@@ -144,13 +145,13 @@ static void package_pseudo_handle(char *buff,
  *
  * @return void
  */
-static int fullpath(struct display_buffer *pathbuf,
-		    struct pseudo_fsal_obj_handle *this_node)
+static int create_fullpath(struct display_buffer *pathbuf,
+			   struct pseudo_fsal_obj_handle *this_node)
 {
 	int b_left;
 
 	if (this_node->parent != NULL)
-		b_left = fullpath(pathbuf, this_node->parent);
+		b_left = create_fullpath(pathbuf, this_node->parent);
 	else
 		b_left = display_start(pathbuf);
 
@@ -175,7 +176,7 @@ static struct pseudo_fsal_obj_handle
 *alloc_directory_handle(struct pseudo_fsal_obj_handle *parent,
 			const char *name,
 			struct fsal_export *exp_hdl,
-			struct attrlist *attrs)
+			struct fsal_attrlist *attrs)
 {
 	struct pseudo_fsal_obj_handle *hdl;
 	char path[MAXPATHLEN] = "\0";
@@ -199,7 +200,7 @@ static struct pseudo_fsal_obj_handle
 	hdl->handle = (char *) &hdl[1];
 
 	/* Create the full path */
-	rc = fullpath(&pathbuf, hdl);
+	rc = create_fullpath(&pathbuf, hdl);
 
 	if (rc < 0) {
 		LogDebug(COMPONENT_FSAL,
@@ -234,12 +235,12 @@ static struct pseudo_fsal_obj_handle
 	if ((attrs->valid_mask & ATTR_OWNER) != 0)
 		hdl->attributes.owner = attrs->owner;
 	else
-		hdl->attributes.owner = op_ctx->creds->caller_uid;
+		hdl->attributes.owner = op_ctx->creds.caller_uid;
 
 	if ((attrs->valid_mask & ATTR_GROUP) != 0)
 		hdl->attributes.group = attrs->group;
 	else
-		hdl->attributes.group = op_ctx->creds->caller_gid;
+		hdl->attributes.group = op_ctx->creds.caller_gid;
 
 	/* Use full timer resolution */
 	now(&hdl->attributes.ctime);
@@ -277,6 +278,12 @@ static struct pseudo_fsal_obj_handle
 		hdl->index = (parent->next_i)++;
 		avltree_insert(&hdl->avl_i, &parent->avl_index);
 		hdl->inavl = true;
+
+		now(&parent->attributes.mtime);
+		parent->attributes.ctime = parent->attributes.mtime;
+		parent->attributes.change = timespec_to_nsecs(
+					&parent->attributes.mtime);
+
 		PTHREAD_RWLOCK_unlock(&parent->obj_handle.obj_lock);
 	}
 	return hdl;
@@ -300,7 +307,7 @@ static struct pseudo_fsal_obj_handle
 static fsal_status_t lookup(struct fsal_obj_handle *parent,
 			    const char *path,
 			    struct fsal_obj_handle **handle,
-			    struct attrlist *attrs_out)
+			    struct fsal_attrlist *attrs_out)
 {
 	struct pseudo_fsal_obj_handle *myself, *hdl = NULL;
 	struct pseudo_fsal_obj_handle key[1];
@@ -340,6 +347,7 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 	if (node) {
 		hdl = avltree_container_of(node, struct pseudo_fsal_obj_handle,
 					   avl_n);
+
 		*handle = &hdl->obj_handle;
 		error = ERR_FSAL_NO_ERROR;
 			LogFullDebug(COMPONENT_FSAL,
@@ -348,6 +356,20 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 	}
 
 out:
+	/* op_ctx->pseudo_fsal_internal_lookup signals that this is a lookup
+	 * request from the export update process and so we should answer it
+	 * with the current state */
+	if (is_export_update_in_progress() &&
+		!op_ctx->flags.pseudo_fsal_internal_lookup) {
+		/* An export update is in progress, we can't trust our entries.
+		 * Tell the client to retry.
+		 */
+		LogDebug(COMPONENT_EXPORT,
+			 "PseudoFS LOOKUP of %s failed due to export update",
+			 path);
+
+		error = ERR_FSAL_DELAY;
+	}
 
 	if (op_ctx->fsal_private != parent)
 		PTHREAD_RWLOCK_unlock(&parent->obj_lock);
@@ -384,9 +406,9 @@ out:
  */
 static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 			     const char *name,
-			     struct attrlist *attrs_in,
+			     struct fsal_attrlist *attrs_in,
 			     struct fsal_obj_handle **handle,
-			     struct attrlist *attrs_out)
+			     struct fsal_attrlist *attrs_out)
 {
 	struct pseudo_fsal_obj_handle *myself, *hdl;
 	uint32_t numlinks;
@@ -447,8 +469,9 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 	struct pseudo_fsal_obj_handle *myself, *hdl;
 	struct avltree_node *node;
 	fsal_cookie_t seekloc;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 	enum fsal_dir_result cb_rc;
+	fsal_errors_t error = ERR_FSAL_NO_ERROR;
 
 	if (whence != NULL)
 		seekloc = *whence;
@@ -478,6 +501,12 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		hdl = avltree_container_of(node,
 					   struct pseudo_fsal_obj_handle,
 					   avl_i);
+
+		if (is_export_update_in_progress()) {
+			error = ERR_FSAL_DELAY;
+			goto out;
+		}
+
 		/* skip entries before seekloc */
 		if (hdl->index < seekloc)
 			continue;
@@ -497,15 +526,16 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		}
 	}
 
+out:
 	op_ctx->fsal_private = NULL;
 
 	PTHREAD_RWLOCK_unlock(&dir_hdl->obj_lock);
 
-	return fsalstat(ERR_FSAL_NO_ERROR, 0);
+	return fsalstat(error, 0);
 }
 
 static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
-			      struct attrlist *outattrs)
+			      struct fsal_attrlist *outattrs)
 {
 	struct pseudo_fsal_obj_handle *myself;
 
@@ -648,8 +678,9 @@ static void handle_to_key(struct fsal_obj_handle *obj_hdl,
 	fh_desc->len = V4_FH_OPAQUE_SIZE;
 }
 
-/*
- * release
+/**
+ * @brief release object handle
+ *
  * release our export first so they know we are gone
  */
 
@@ -706,10 +737,10 @@ void pseudofs_handle_ops_init(struct fsal_obj_ops *ops)
 fsal_status_t pseudofs_lookup_path(struct fsal_export *exp_hdl,
 				 const char *path,
 				 struct fsal_obj_handle **handle,
-				 struct attrlist *attrs_out)
+				 struct fsal_attrlist *attrs_out)
 {
 	struct pseudofs_fsal_export *myself;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 
 	myself = container_of(exp_hdl, struct pseudofs_fsal_export, export);
 
@@ -743,7 +774,7 @@ fsal_status_t pseudofs_lookup_path(struct fsal_export *exp_hdl,
 
 /* create_handle
  * Does what original FSAL_ExpandHandle did (sort of)
- * returns a ref counted handle to be later used in cache_inode etc.
+ * returns a ref counted handle to be later used in mdcache etc.
  * NOTE! you must release this thing when done with it!
  * BEWARE! Thanks to some holes in the *AT syscalls implementation,
  * we cannot get an fd on an AF_UNIX socket, nor reliably on block or
@@ -756,7 +787,7 @@ fsal_status_t pseudofs_lookup_path(struct fsal_export *exp_hdl,
 fsal_status_t pseudofs_create_handle(struct fsal_export *exp_hdl,
 				   struct gsh_buffdesc *hdl_desc,
 				   struct fsal_obj_handle **handle,
-				   struct attrlist *attrs_out)
+				   struct fsal_attrlist *attrs_out)
 {
 	struct glist_head *glist;
 	struct fsal_obj_handle *hdl;
@@ -773,7 +804,7 @@ fsal_status_t pseudofs_create_handle(struct fsal_export *exp_hdl,
 		return fsalstat(ERR_FSAL_BADHANDLE, 0);
 	}
 
-	PTHREAD_RWLOCK_rdlock(&exp_hdl->fsal->lock);
+	PTHREAD_RWLOCK_rdlock(&exp_hdl->fsal->fsm_lock);
 
 	glist_for_each(glist, &exp_hdl->fsal->handles) {
 		hdl = glist_entry(glist, struct fsal_obj_handle, handles);
@@ -791,7 +822,7 @@ fsal_status_t pseudofs_create_handle(struct fsal_export *exp_hdl,
 
 			*handle = hdl;
 
-			PTHREAD_RWLOCK_unlock(&exp_hdl->fsal->lock);
+			PTHREAD_RWLOCK_unlock(&exp_hdl->fsal->fsm_lock);
 
 			if (attrs_out != NULL) {
 				fsal_copy_attrs(attrs_out, &my_hdl->attributes,
@@ -802,10 +833,22 @@ fsal_status_t pseudofs_create_handle(struct fsal_export *exp_hdl,
 		}
 	}
 
+	if (is_export_update_in_progress()) {
+		/* An export update may be the cause of the failure. Tell the
+		 * client to retry.
+		 */
+		PTHREAD_RWLOCK_unlock(&exp_hdl->fsal->fsm_lock);
+
+		LogDebug(COMPONENT_EXPORT,
+			 "PseudoFS create handle may have failed due to export update");
+
+		return fsalstat(ERR_FSAL_DELAY, 0);
+	}
+
 	LogDebug(COMPONENT_FSAL,
 		"Could not find handle");
 
-	PTHREAD_RWLOCK_unlock(&exp_hdl->fsal->lock);
+	PTHREAD_RWLOCK_unlock(&exp_hdl->fsal->fsm_lock);
 
 	return fsalstat(ERR_FSAL_STALE, ESTALE);
 }
