@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright (C) Panasas Inc., 2011
  * Author: Jim Lieb jlieb@panasas.com
@@ -39,7 +40,7 @@
 #include "pseudofs_methods.h"
 #include "../fsal_private.h"
 
-const char pseudoname[] = "PSEUDO";
+static const char pseudoname[] = "PSEUDO";
 
 /* my module private storage
  */
