@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
@@ -55,16 +56,17 @@ struct pseudofs_fsal_export {
 fsal_status_t pseudofs_lookup_path(struct fsal_export *exp_hdl,
 				 const char *path,
 				 struct fsal_obj_handle **handle,
-				 struct attrlist *attrs_out);
+				 struct fsal_attrlist *attrs_out);
 
 fsal_status_t pseudofs_create_handle(struct fsal_export *exp_hdl,
 				   struct gsh_buffdesc *hdl_desc,
 				   struct fsal_obj_handle **handle,
-				   struct attrlist *attrs_out);
+				   struct fsal_attrlist *attrs_out);
 
-/*
- * PSEUDOFS internal object handle
- * handle is a pointer because
+/**
+ * @brief PSEUDOFS internal object handle
+ *
+ * The handle is a pointer because
  *  a) the last element of file_handle is a char[] meaning variable len...
  *  b) we cannot depend on it *always* being last or being the only
  *     variable sized struct here...  a pointer is safer.
@@ -72,7 +74,7 @@ fsal_status_t pseudofs_create_handle(struct fsal_export *exp_hdl,
 
 struct pseudo_fsal_obj_handle {
 	struct fsal_obj_handle obj_handle;
-	struct attrlist attributes;
+	struct fsal_attrlist attributes;
 	char *handle;
 	struct pseudo_fsal_obj_handle *parent;
 	struct avltree avl_name;
