@@ -131,8 +131,16 @@ struct rgw_cb_arg {
 	attrmask_t attrmask;
 };
 
-static bool rgw_cb(const char *name, void *arg, uint64_t offset,
-		struct stat *st, uint32_t st_mask, uint32_t flags)
+#if ((LIBRGW_FILE_VER_MAJOR > 1) || \
+	((LIBRGW_FILE_VER_MAJOR == 1) && \
+	 ((LIBRGW_FILE_VER_MINOR > 2) || \
+	  ((LIBRGW_FILE_VER_MINOR == 2) && (LIBRGW_FILE_VER_EXTRA >= 1)))))
+static int
+#else
+static bool
+#endif
+rgw_cb(const char *name, void *arg, uint64_t offset,
+	struct stat *st, uint32_t st_mask, uint32_t flags)
 {
 	struct rgw_cb_arg *rgw_cb_arg = arg;
 	struct fsal_obj_handle *obj = NULL;
@@ -426,12 +434,10 @@ fsal_status_t rgw_fsal_setattr2(struct fsal_obj_handle *obj_hdl,
 
 	fsal_status_t status = {0, 0};
 	int rc = 0;
-	bool has_lock = false;
-	bool closefd = false;
+	bool has_share = false;
 	struct stat st;
 	/* Mask of attributes to set */
 	uint32_t mask = 0;
-	bool reusing_open_state_fd = false;
 
 	struct rgw_export *export =
 		container_of(op_ctx->fsal_export, struct rgw_export, export);
@@ -467,20 +473,24 @@ fsal_status_t rgw_fsal_setattr2(struct fsal_obj_handle *obj_hdl,
 				"Setting size on non-regular file");
 			return fsalstat(ERR_FSAL_INVAL, EINVAL);
 		}
+		if (state == NULL) {
+			/* Check share reservation and if OK, update the
+			 * counters.
+			 */
+			status = check_share_conflict_and_update_locked(
+						obj_hdl, &handle->share,
+						FSAL_O_CLOSED, FSAL_O_WRITE,
+						bypass);
 
-		/* We don't actually need an open fd, we are just doing the
-		 * share reservation checking, thus the NULL parameters.
-		 */
-		status = fsal_find_fd(NULL, obj_hdl, NULL, &handle->share,
-				bypass, state, FSAL_O_WRITE, NULL, NULL,
-				&has_lock, &closefd, false,
-				&reusing_open_state_fd);
+			if (FSAL_IS_ERROR(status)) {
+				LogDebug(COMPONENT_FSAL,
+					 "check_share_conflict_and_update_locked failed with %s",
+					 fsal_err_txt(status));
 
-		if (FSAL_IS_ERROR(status)) {
-			LogFullDebug(COMPONENT_FSAL,
-				"fsal_find_fd status=%s",
-				fsal_err_txt(status));
-			goto out;
+				return status;
+			}
+
+			has_share = true;
 		}
 	}
 
@@ -574,8 +584,12 @@ fsal_status_t rgw_fsal_setattr2(struct fsal_obj_handle *obj_hdl,
 
  out:
 
-	if (has_lock)
-		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+	if (has_share) {
+		/* Release the share reservation now by updating the counters.
+		 */
+		update_share_counters_locked(obj_hdl, &handle->share,
+					     FSAL_O_WRITE, FSAL_O_CLOSED);
+	}
 
 	return status;
 }
@@ -809,25 +823,13 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 			 * caller is a stateless create such as NFS v3 CREATE).
 			 */
 
-			/* This can block over an I/O operation. */
-			PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+			status = check_share_conflict_and_update_locked(
+						obj_hdl, &handle->share,
+						FSAL_O_CLOSED, openflags,
+						false);
 
-			/* Check share reservation conflicts. */
-			status = check_share_conflict(&handle->share,
-						      openflags, false);
-
-			if (FSAL_IS_ERROR(status)) {
-				PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+			if (FSAL_IS_ERROR(status))
 				return status;
-			}
-
-			/* Take the share reservation now by updating the
-			 * counters.
-			 */
-			update_share_counters(&handle->share, FSAL_O_CLOSED,
-					      openflags);
-
-			PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 		} else {
 			/* RGW doesn't have a file descriptor/open abstraction,
 			 * and actually forbids concurrent opens;  This is
@@ -925,11 +927,8 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 		 * and undo the update of the share counters.
 		 * This can block over an I/O operation.
 		 */
-		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
-
-		update_share_counters(&handle->share, openflags, FSAL_O_CLOSED);
-
-		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+		update_share_counters_locked(obj_hdl, &handle->share,
+					     openflags, FSAL_O_CLOSED);
 
 		return status;
 	} /* !name */
@@ -1090,8 +1089,14 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 	 * handy since we can then call setattr2 which WILL take the lock
 	 * without a double locking deadlock.
 	 */
-	if (my_fd == NULL)
+	if (my_fd == NULL) {
+		LogFullDebug(COMPONENT_FSAL, "Using global fd");
 		my_fd = &hdl->fd;
+		/* Need to LRU track global fd including incrementing
+		 * fsal_fd_global_counter.
+		 */
+		insert_fd_lru(&my_fd->fsal_fd);
+	}
 
 	my_fd->fd = fd;
 #endif
@@ -1152,13 +1157,9 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 		 * a stateless create such as NFS v3 CREATE).
 		 */
 
-		/* This can block over an I/O operation. */
-		PTHREAD_RWLOCK_wrlock(&(*new_obj)->obj_lock);
-
 		/* Take the share reservation now by updating the counters. */
-		update_share_counters(&obj->share, FSAL_O_CLOSED, openflags);
-
-		PTHREAD_RWLOCK_unlock(&(*new_obj)->obj_lock);
+		update_share_counters_locked(*new_obj, &obj->share,
+					     FSAL_O_CLOSED, openflags);
 	}
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
@@ -1251,26 +1252,18 @@ fsal_status_t rgw_fsal_reopen2(struct fsal_obj_handle *obj_hdl,
 
 	fsal2posix_openflags(openflags, &posix_flags);
 
-	/* This can block over an I/O operation. */
-	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
-
 	old_openflags = handle->openflags;
 
-	/* We can conflict with old share, so go ahead and check now. */
-	status = check_share_conflict(&handle->share, openflags, false);
-
-	if (FSAL_IS_ERROR(status)) {
-		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
-
-		return status;
-	}
-
-	/* Set up the new share so we can drop the lock and not have a
+	/* We can conflict with old share, so go ahead and check now. If OK
+	 * set up the new share so we can drop the lock and not have a
 	 * conflicting share be asserted, updating the share counters.
 	 */
-	update_share_counters(&handle->share, old_openflags, openflags);
+	status = check_share_conflict_and_update_locked(obj_hdl, &handle->share,
+							old_openflags,
+							openflags, false);
 
-	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+	if (FSAL_IS_ERROR(status))
+		return status;
 
 	/* perform a provider open iff not already open */
 	if (true) {
@@ -1285,14 +1278,10 @@ fsal_status_t rgw_fsal_reopen2(struct fsal_obj_handle *obj_hdl,
 
 		if (rc < 0) {
 			/* We had a failure on open - we need to revert the
-			 * share. This can block over an I/O operation.
+			 * share.
 			 */
-			PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
-
-			update_share_counters(&handle->share, openflags,
-					old_openflags);
-
-			PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+			update_share_counters_locked(obj_hdl, &handle->share,
+						     openflags, old_openflags);
 		}
 
 		status = rgw2fsal_error(rc);
@@ -1499,7 +1488,7 @@ struct state_t *rgw_alloc_state(struct fsal_export *exp_hdl,
 				struct state_t *related_state)
 {
 	return init_state(gsh_calloc(1, sizeof(struct rgw_open_state)),
-			exp_hdl, state_type, related_state);
+			  NULL, state_type, related_state);
 }
 
 /**
@@ -1550,8 +1539,8 @@ fsal_status_t rgw_fsal_close2(struct fsal_obj_handle *obj_hdl,
 			 */
 
 			update_share_counters(&handle->share,
-					handle->openflags,
-					FSAL_O_CLOSED);
+					      handle->openflags,
+					      FSAL_O_CLOSED);
 
 		}
 
@@ -1661,12 +1650,6 @@ static void handle_to_key(struct fsal_obj_handle *obj_hdl,
 
 #ifdef USE_FSAL_RGW_XATTRS
 
-/** @todo - the code below has some serious problems with dereferencing NULL
- *          pointers. The CMake capability detection has been "nerfed" so this
- *          code will never be enabled...
- *
- */
-
 static int getxattr_cb(rgw_xattrlist *attrs, void *arg, uint32_t flags)
 {
 	xattrvalue4 *cb_arg = (xattrvalue4 *)arg;
@@ -1681,7 +1664,7 @@ static int getxattr_cb(rgw_xattrlist *attrs, void *arg, uint32_t flags)
 /**
  * @brief Get Extended Attribute
  *
- * The function fetches the extended attr of the object.
+ * The function fetches an extended attr of the object.
  *
  * @param[in]  obj_hdl  Input object to query
  * @param[in]  xa_name  Input xattr name
@@ -1695,7 +1678,6 @@ static fsal_status_t getxattrs(struct fsal_obj_handle *obj_hdl,
 	int rc = 0;
 	int errsv = 0;
 	fsal_status_t status;
-	rgw_xattrlist attrs = {NULL, 1};
 
 	struct rgw_export *export =
 		container_of(op_ctx->fsal_export,
@@ -1703,13 +1685,13 @@ static fsal_status_t getxattrs(struct fsal_obj_handle *obj_hdl,
 	struct rgw_handle *handle =
 		container_of(obj_hdl, struct rgw_handle, handle);
 
-	/** @todo - deref of NULL pointer set above! */
-	attrs.xattrs->key.val =
-		xa_name->utf8string_val;
-	attrs.xattrs->key.len = xa_name->utf8string_len;
+	rgw_xattrstr xattr_k = {xa_name->utf8string_val, xa_name->utf8string_len};
+	rgw_xattrstr xattr_v = {NULL, 0};
+	rgw_xattr xattrs[1] = {{xattr_k, xattr_v}};
+	rgw_xattrlist xattrlist = {xattrs, 1};
 
-	rc = rgw_getxattrs(export->rgw_fs, handle->rgw_fh, &attrs,
-			   getxattr_cb, &xa_value,
+	rc = rgw_getxattrs(export->rgw_fs, handle->rgw_fh, &xattrlist,
+			   getxattr_cb, xa_value,
 			   RGW_GETXATTR_FLAG_NONE);
 
 	if (rc < 0) {
@@ -1744,7 +1726,7 @@ out:
 /**
  * @brief Set Extended Attribute
  *
- * The function sets the extended attr of the object.
+ * The function sets an extended attr of the object.
  *
  * @param[in]  obj_hdl  The input object
  * @param[in]  sa_type  Input xattr type
@@ -1761,7 +1743,6 @@ static fsal_status_t setxattrs(struct fsal_obj_handle *obj_hdl,
 	int rc = 0;
 	int errsv = 0;
 	fsal_status_t status = {0, 0};
-	rgw_xattrlist attrs = {NULL, 1};
 
 	struct rgw_export *export =
 		container_of(op_ctx->fsal_export,
@@ -1769,15 +1750,12 @@ static fsal_status_t setxattrs(struct fsal_obj_handle *obj_hdl,
 	struct rgw_handle *handle = container_of(obj_hdl, struct rgw_handle,
 						 handle);
 
-	/** @todo - deref of NULL pointer set above! */
-	attrs.xattrs->key.val =
-		xa_name->utf8string_val;
-	attrs.xattrs->key.len = xa_name->utf8string_len;
-	attrs.xattrs->val.val =
-		xa_value->utf8string_val;
-	attrs.xattrs->val.len = xa_value->utf8string_len;
+	rgw_xattrstr xattr_k = {xa_name->utf8string_val, xa_name->utf8string_len};
+	rgw_xattrstr xattr_v = {xa_value->utf8string_val, xa_value->utf8string_len};
+	rgw_xattr xattr = {xattr_k, xattr_v};
+	rgw_xattrlist xattrlist = {&xattr, 1};
 
-	rc = rgw_setxattrs(export->rgw_fs, handle->rgw_fh, &attrs,
+	rc = rgw_setxattrs(export->rgw_fs, handle->rgw_fh, &xattrlist,
 			   RGW_SETXATTR_FLAG_NONE);
 
 	if (rc < 0) {
@@ -1811,7 +1789,6 @@ static fsal_status_t removexattrs(struct fsal_obj_handle *obj_hdl,
 	int rc = 0;
 	int errsv = 0;
 	fsal_status_t status = {0, 0};
-	rgw_xattrlist attrs = {NULL, 1};
 
 	struct rgw_export *export =
 		container_of(op_ctx->fsal_export,
@@ -1819,15 +1796,14 @@ static fsal_status_t removexattrs(struct fsal_obj_handle *obj_hdl,
 	struct rgw_handle *handle =
 		container_of(obj_hdl, struct rgw_handle, handle);
 
-	attrs.xattr_cnt = 1;
-	/** @todo - deref of NULL pointer set above! */
-	attrs.xattrs->key.val =
-		xa_name->utf8string_val;
-	attrs.xattrs->key.len = xa_name->utf8string_len;
+	rgw_xattrstr xattr_k = {xa_name->utf8string_val, xa_name->utf8string_len};
+	rgw_xattrstr xattr_v = {NULL, 0};
+	rgw_xattr xattr = {xattr_k, xattr_v};
+	rgw_xattrlist xattrlist = {&xattr, 1};
 
 	rc = rgw_rmxattrs(export->rgw_fs,
 			  handle->rgw_fh,
-			  &attrs, RGW_RMXATTR_FLAG_NONE);
+			  &xattrlist, RGW_RMXATTR_FLAG_NONE);
 
 	if (rc < 0) {
 		errsv = errno;
@@ -1848,19 +1824,31 @@ struct lxattr_cb_arg {
 	xattrlist4 *names;
 };
 
-static int lsxattr_cb(rgw_xattrlist *attrs, void *cb_arg, uint32_t flag)
+#define XATTR_USER_PREFIX	"user."
+#define XATTR_USER_PREFIX_LEN	(sizeof(XATTR_USER_PREFIX) - 1)
+
+static int lsxattr_cb(rgw_xattrlist *attrs, void *arg, uint32_t flag)
 {
-	struct lxattr_cb_arg *cb = (struct lxattr_cb_arg *)cb_arg;
+	struct lxattr_cb_arg *cb_arg = (struct lxattr_cb_arg *)arg;
 
-	cb->names->xl4_entries[cb->names->xl4_count].utf8string_val =
-		gsh_strldup(attrs->xattrs[0].key.val, attrs->xattrs[0].key.len,
-		&cb->names->xl4_entries[cb->names->xl4_count].utf8string_len);
+	for (uint32_t ix = 0; ix < attrs->xattr_cnt; ++ix) {
+		rgw_xattr *xattr = &(attrs->xattrs[ix]);
+		component4 *entry = &(cb_arg->names->xl4_entries[cb_arg->names->xl4_count]);
 
-	cb->names->xl4_count++;
+		/* defensive checks borrowed from Jeff Layton's helper */
+		if (xattr->key.len < XATTR_USER_PREFIX_LEN + 1)
+			return 0;
 
-	if (cb->names->xl4_count == cb->max) {
-		flag = RGW_LSXATTR_FLAG_STOP;
-		return flag;
+		if (strncmp(xattr->key.val, XATTR_USER_PREFIX, XATTR_USER_PREFIX_LEN))
+			return 0;
+
+		entry->utf8string_val =
+			gsh_strldup(xattr->key.val, xattr->key.len, &entry->utf8string_len);
+		cb_arg->names->xl4_count++;
+
+		if (cb_arg->names->xl4_count == cb_arg->max) {
+			return RGW_LSXATTR_FLAG_STOP;
+		}
 	}
 
 	return 0;
@@ -1891,10 +1879,6 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 	int rc = 0;
 	int errsv = 0;
 	fsal_status_t status = {0, 0};
-	struct lxattr_cb_arg attrs;
-
-	attrs.names = lr_names;
-	attrs.max = la_maxcount;
 
 	struct rgw_export *export =
 		container_of(op_ctx->fsal_export,
@@ -1902,9 +1886,18 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 	struct rgw_handle *handle =
 		container_of(obj_hdl, struct rgw_handle, handle);
 
+	struct lxattr_cb_arg cb_arg;
+
+	/* XXX: ffilz: the doc in fsal_api strongly implies this buffer is pre-allocated,
+	 * but it's not */
+	lr_names->xl4_entries = gsh_calloc(la_maxcount, sizeof(xattrlist4));
+	lr_names->xl4_count = 0; /* defensive */
+	cb_arg.names = lr_names;
+	cb_arg.max = la_maxcount;
+
 	rc = rgw_lsxattrs(export->rgw_fs, handle->rgw_fh,
 			  NULL /* filter_prefix not yet implemented in RGW */,
-			  lsxattr_cb, &attrs, RGW_LSXATTR_FLAG_NONE);
+			  lsxattr_cb, &cb_arg, RGW_LSXATTR_FLAG_NONE);
 
 	if (rc < 0) {
 		errsv = errno;
@@ -1915,7 +1908,8 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 		goto out;
 	}
 
-	/* Setting true unconditionally since cookie isn't implemented yet */
+	/* Setting lr_eof true unconditionally since cookie isn't implemented yet */
+	*la_cookie = cb_arg.names->xl4_count;
 	*lr_eof = true;
 
 	status = fsalstat(ERR_FSAL_NO_ERROR, 0);
