@@ -84,6 +84,9 @@ struct rgw_fsal_module RGWFSM = {
 			#endif
 			.whence_is_name = true,
 			.expire_time_parent = -1,
+#ifdef USE_FSAL_RGW_XATTRS
+			.xattr_support = true,
+#endif
 		}
 	}
 };
@@ -106,12 +109,13 @@ struct config_block rgw_block = {
 	.dbus_interface_name = "org.ganesha.nfsd.config.fsal.rgw",
 	.blk_desc.name = "RGW",
 	.blk_desc.type = CONFIG_BLOCK,
+	.blk_desc.flags = CONFIG_UNIQUE,  /* too risky to have more */
 	.blk_desc.u.blk.init = noop_conf_init,
 	.blk_desc.u.blk.params = rgw_items,
 	.blk_desc.u.blk.commit = noop_conf_commit
 };
 
-static pthread_mutex_t init_mtx = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t init_mtx;
 
 /* Module methods
  */
@@ -401,6 +405,8 @@ MODULE_INIT void init(void)
 {
 	struct fsal_module *myself = &RGWFSM.fsal;
 
+	PTHREAD_MUTEX_init(&init_mtx, NULL);
+
 	LogDebug(COMPONENT_FSAL,
 		 "RGW module registering.");
 
@@ -445,4 +451,6 @@ MODULE_FINI void finish(void)
 	if (RGWFSM.rgw) {
 		librgw_shutdown(RGWFSM.rgw);
 	}
+
+	PTHREAD_MUTEX_destroy(&init_mtx);
 }
