@@ -290,6 +290,30 @@ static fsal_status_t wire_to_host(struct fsal_export *exp_hdl,
 	return vfs_check_handle(exp_hdl, fh_desc, &fs, fh, &dummy);
 }
 
+
+/**
+ * @brief Function to get the fasl_obj_handle that has fsal_fd as its global fd.
+ *
+ * @param[in]     exp_hdl   The export in which the handle exists
+ * @param[in]     fd        File descriptor in question
+ * @param[out]    handle    FSAL object handle
+ *
+ * @return the fsal_obj_handle.
+ */
+void get_fsal_obj_hdl(struct fsal_export *exp_hdl,
+				  struct fsal_fd *fd,
+				  struct fsal_obj_handle **handle)
+{
+	struct vfs_fd *my_fd = NULL;
+	struct vfs_fsal_obj_handle *myself = NULL;
+
+	my_fd = container_of(fd, struct vfs_fd, fsal_fd);
+	myself = container_of(my_fd, struct vfs_fsal_obj_handle, u.file.fd);
+
+	*handle = &myself->obj_handle;
+}
+
+
 /* vfs_export_ops_init
  * overwrite vector entries with the methods that we support
  */
@@ -304,7 +328,7 @@ void vfs_export_ops_init(struct export_ops *ops)
 	ops->get_quota = get_quota;
 	ops->set_quota = set_quota;
 	ops->alloc_state = vfs_alloc_state;
-	ops->free_state = vfs_free_state;
+	ops->get_fsal_obj_hdl = get_fsal_obj_hdl;
 }
 
 int vfs_claim_filesystem(struct fsal_filesystem *fs,
