@@ -99,6 +99,7 @@ void free_vfs_fsal_obj_handle(struct vfs_fsal_obj_handle **hdl_ref)
 
 		handle_to_key(&myself->obj_handle, &key);
 		vfs_state_release(&key);
+		destroy_fsal_fd(&myself->u.file.fd.fsal_fd);
 	} else if (vfs_unopenable_type(type)) {
 		gsh_free(myself->u.unopenable.name);
 		gsh_free(myself->u.unopenable.dir);
@@ -152,8 +153,9 @@ struct vfs_fsal_obj_handle *alloc_handle(int dirfd,
 		     fs, fs->path);
 
 	if (hdl->obj_handle.type == REGULAR_FILE) {
+		init_fsal_fd(&hdl->u.file.fd.fsal_fd, FSAL_FD_GLOBAL,
+			     op_ctx->fsal_export);
 		hdl->u.file.fd.fd = -1;	/* no open on this yet */
-		hdl->u.file.fd.openflags = FSAL_O_CLOSED;
 	} else if (hdl->obj_handle.type == SYMBOLIC_LINK) {
 		ssize_t retlink;
 		size_t len = stat->st_size + 1;
@@ -833,8 +835,10 @@ static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 
 	dir_fd = vfs_fsal_open(myself, flags, &status.major);
 
-	if (dir_fd < 0)
+	if (dir_fd < 0) {
+		retval = -dir_fd;
 		goto errout;
+	}
 
 	retval = vfs_stat_by_handle(dir_fd, &stat);
 
@@ -1196,19 +1200,25 @@ static fsal_status_t linkfile(struct fsal_obj_handle *obj_hdl,
 			      const char *name)
 {
 	struct vfs_fsal_obj_handle *myself, *destdir;
-	int srcfd, destdirfd;
+	int destdirfd;
 	int retval = 0;
 	int flags = O_PATH | O_NOACCESS | O_NOFOLLOW;
-	fsal_errors_t fsal_error = ERR_FSAL_NO_ERROR;
+	fsal_status_t status;
+	fsal_status_t status2 = {ERR_FSAL_NO_ERROR, 0};
+	struct vfs_fd *my_fd;
+	struct vfs_fd temp_fd = { FSAL_FD_INIT, -1 };
+	struct fsal_fd *out_fd;
 
 	LogFullDebug(COMPONENT_FSAL, "link to %s", name);
 
 	if (!op_ctx->fsal_export->exp_ops.fs_supports(
 				op_ctx->fsal_export, fso_link_support)) {
-		fsal_error = ERR_FSAL_NOTSUPP;
+		status = fsalstat(ERR_FSAL_NOTSUPP, 0);
 		goto out;
 	}
+
 	myself = container_of(obj_hdl, struct vfs_fsal_obj_handle, obj_handle);
+
 	if (obj_hdl->fsal != obj_hdl->fs->fsal) {
 		LogDebug(COMPONENT_FSAL,
 			 "FSAL %s operation for handle belonging to FSAL %s, return EXDEV",
@@ -1216,31 +1226,67 @@ static fsal_status_t linkfile(struct fsal_obj_handle *obj_hdl,
 			 obj_hdl->fs->fsal != NULL
 				? obj_hdl->fs->fsal->name
 				: "(none)");
-		retval = EXDEV;
-		fsal_error = posix2fsal_error(retval);
+		status = posix2fsal_status(EXDEV);
 		goto out;
 	}
 
-	/* Take read lock on object to protect file descriptor.
-	 * We only take a read lock because we are not changing the state of
-	 * the file descriptor.
+	/* Get a usable file descriptor (don't need to bypass - FSAL_O_ANY
+	 * won't conflict with any share reservation).
 	 */
-	PTHREAD_RWLOCK_rdlock(&obj_hdl->obj_lock);
-
-	if (obj_hdl->type == REGULAR_FILE &&
-	    myself->u.file.fd.openflags != FSAL_O_CLOSED) {
-		srcfd = myself->u.file.fd.fd;
+	if (obj_hdl->type == REGULAR_FILE) {
+		/* For regular files, use fsal_start_global_io so that we use
+		 * the global fd if open otherwise we use temp_fd. We do this
+		 * instead of the code from before the re-work that just
+		 * used the global fd protected by the obj_lock which is
+		 * no longer the way to protect use of the global fd, instead
+		 * fsal_start_io uses the new protection mechanism.
+		 */
+		status = fsal_start_global_io(&out_fd, obj_hdl,
+					      &myself->u.file.fd.fsal_fd,
+					      &temp_fd.fsal_fd, FSAL_O_ANY,
+					      false, &myself->u.file.share);
 	} else {
-		srcfd = vfs_fsal_open(myself, flags, &fsal_error);
-		if (srcfd < 0) {
-			retval = -srcfd;
-			fsal_error = posix2fsal_error(retval);
+		/* For all other objects, we open the object itself. We don't
+		 * use find_fd because that doesn't always return an fd for the
+		 * object itself, For some object types, it returns an open fd
+		 * for the object's parent. For doing a link though, we CAN
+		 * open the actual object by handle and get an fd that works
+		 * for link. This matches the code from before the fsal_fd
+		 * re-work.
+		 */
+		temp_fd.fd = vfs_fsal_open(myself, flags, &status.major);
+		out_fd = &temp_fd.fsal_fd;
+
+		if (temp_fd.fd < 0) {
+			status = posix2fsal_status(-temp_fd.fd);
 			LogDebug(COMPONENT_FSAL,
-				 "open myself returned %d", retval);
-			goto out_unlock;
+				 "open myself returned %s (%d)",
+				 strerror(-temp_fd.fd), -temp_fd.fd);
+		} else {
+			status = fsalstat(ERR_FSAL_NO_ERROR, 0);
 		}
 	}
 
+	if (FSAL_IS_ERROR(status)) {
+		if (obj_hdl->type == SYMBOLIC_LINK &&
+		    status.major == ERR_FSAL_PERM) {
+			/* You cannot open_by_handle (XFS on linux) a symlink
+			 * and it throws an EPERM error for it.
+			 * open_by_handle_at does not throw that error for
+			 * symlinks so we play a game here.  Since there is
+			 * not much we can do with symlinks anyway,
+			 * say that we did it but don't actually
+			 * do anything.  In this case, return the stat we got
+			 * at lookup time.  If you *really* want to tweek things
+			 * like owners, get a modern linux kernel...
+			 */
+			status = fsalstat(ERR_FSAL_NO_ERROR, 0);
+		}
+		goto out;
+	}
+
+	my_fd = container_of(out_fd, struct vfs_fd, fsal_fd);
+
 	destdir =
 	    container_of(destdir_hdl, struct vfs_fsal_obj_handle, obj_handle);
 
@@ -1251,43 +1297,55 @@ static fsal_status_t linkfile(struct fsal_obj_handle *obj_hdl,
 			 destdir_hdl->fs->fsal != NULL
 				? destdir_hdl->fs->fsal->name
 				: "(none)");
-		retval = EXDEV;
-		fsal_error = posix2fsal_error(retval);
-		goto fileerr;
+		status = posix2fsal_status(EXDEV);
+		goto no_destdir;
 	}
 
-	destdirfd = vfs_fsal_open(destdir, flags, &fsal_error);
+	destdirfd = vfs_fsal_open(destdir, flags, &status.major);
 
 	if (destdirfd < 0) {
-		retval = destdirfd;
-		fsal_error = posix2fsal_error(retval);
+		status = posix2fsal_status(-destdirfd);
 		LogDebug(COMPONENT_FSAL,
 			 "open destdir returned %d", retval);
-		goto fileerr;
+		goto no_destdir;
 	}
 
-	retval = vfs_link_by_handle(myself->handle, srcfd, destdirfd, name);
+	retval = vfs_link_by_handle(myself->handle, my_fd->fd, destdirfd, name);
 
 	if (retval < 0) {
 		retval = errno;
+
 		LogFullDebug(COMPONENT_FSAL,
 			     "link returned %d", retval);
-		fsal_error = posix2fsal_error(retval);
+
+		status = posix2fsal_status(retval);
 	}
 
 	close(destdirfd);
 
- fileerr:
-	if (!(obj_hdl->type == REGULAR_FILE && myself->u.file.fd.fd >= 0))
-		close(srcfd);
+ no_destdir:
 
- out_unlock:
+	if (obj_hdl->type == REGULAR_FILE) {
+		status2 = fsal_complete_io(obj_hdl, out_fd);
 
-	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+		LogFullDebug(COMPONENT_FSAL,
+			     "fsal_complete_io returned %s",
+			     fsal_err_txt(status2));
+	} else {
+		retval = close(temp_fd.fd);
+
+		if (retval < 0)
+			status2 = posix2fsal_status(errno);
+
+		LogFullDebug(COMPONENT_FSAL,
+			     "close returned %s",
+			     fsal_err_txt(status2));
+	}
 
  out:
-	LogFullDebug(COMPONENT_FSAL, "returning %d, %d", fsal_error, retval);
-	return fsalstat(fsal_error, retval);
+
+	LogFullDebug(COMPONENT_FSAL, "returning %s", fsal_err_txt(status));
+	return status;
 }
 
 /*
@@ -1532,128 +1590,6 @@ static fsal_status_t renamefile(struct fsal_obj_handle *obj_hdl,
 	return fsalstat(fsal_error, retval);
 }
 
-/**
- * @brief Open file and get attributes.
- *
- * This function opens a file and returns the file descriptor and fetches
- * the attributes. The obj_hdl->obj_lock MUST be held for this call.
- *
- * @param[in]     exp         The fsal_export the file belongs to
- * @param[in]     myself      File to access
- * @param[in/out] stat        The struct stat to fetch attributes into
- * @param[in]     open_flags  The mode to open the file in
- * @param[out]    fsal_error  Place to return an error
- *
- * @return The file descriptor plus indication if it needs to be closed.
- *
- */
-struct closefd vfs_fsal_open_and_stat(struct fsal_export *exp,
-				      struct vfs_fsal_obj_handle *myself,
-				      struct stat *stat,
-				      fsal_openflags_t flags,
-				      fsal_errors_t *fsal_error)
-{
-	struct fsal_obj_handle *obj_hdl = &myself->obj_handle;
-	struct closefd cfd = { .fd = -1, .close_fd = false };
-	int retval = 0;
-	const char *func = "unknown";
-	int open_flags;
-
-	fsal2posix_openflags(flags, &open_flags);
-
-	switch (obj_hdl->type) {
-	case SOCKET_FILE:
-	case CHARACTER_FILE:
-	case BLOCK_FILE:
-		cfd.fd = vfs_open_by_handle(myself->obj_handle.fs,
-					    myself->u.unopenable.dir,
-					    O_PATH | O_NOACCESS,
-					    fsal_error);
-		if (cfd.fd < 0) {
-			LogDebug(COMPONENT_FSAL,
-				 "Failed with %s open_flags 0x%08x",
-				 strerror(-cfd.fd), O_PATH | O_NOACCESS);
-			return cfd;
-		}
-		cfd.close_fd = true;
-		retval =
-		    fstatat(cfd.fd, myself->u.unopenable.name, stat,
-			    AT_SYMLINK_NOFOLLOW);
-
-		func = "fstatat";
-		break;
-	case REGULAR_FILE:
-		/* Check if the file descriptor happens to be in a compatible
-		 * mode. If not, open a temporary file descriptor.
-		 *
-		 * Note that FSAL_O_REOPEN will never be set in
-		 * myself->u.file.fd.openflags and thus forces a re-open.
-		 */
-		if (((flags & FSAL_O_ANY) != 0 &&
-		     (myself->u.file.fd.openflags & FSAL_O_RDWR) == 0) ||
-		    ((myself->u.file.fd.openflags & flags) != flags)) {
-			/* no file open at the moment */
-			cfd.fd = vfs_fsal_open(myself, open_flags, fsal_error);
-			if (cfd.fd < 0) {
-				LogDebug(COMPONENT_FSAL,
-					 "Failed with %s open_flags 0x%08x",
-					 strerror(-cfd.fd), open_flags);
-				return cfd;
-			}
-			cfd.close_fd = true;
-		} else {
-			cfd.fd = myself->u.file.fd.fd;
-		}
-		retval = fstat(cfd.fd, stat);
-		func = "fstat";
-		break;
-	case SYMBOLIC_LINK:
-		open_flags |= (O_PATH | O_RDWR | O_NOFOLLOW);
-		goto vfos_open;
-	case FIFO_FILE:
-		open_flags |= O_NONBLOCK;
-		/* fall through */
-	case DIRECTORY:
-	default:
- vfos_open:
-		cfd.fd = vfs_fsal_open(myself, open_flags, fsal_error);
-		if (cfd.fd < 0) {
-			LogDebug(COMPONENT_FSAL,
-				 "Failed with %s open_flags 0x%08x",
-				 strerror(-cfd.fd), open_flags);
-			return cfd;
-		}
-		cfd.close_fd = true;
-		retval = vfs_stat_by_handle(cfd.fd, stat);
-		func = "vfs_stat_by_handle";
-		break;
-	}
-
-	if (retval < 0) {
-		retval = errno;
-		if (cfd.close_fd) {
-			int rc;
-
-			rc = close(cfd.fd);
-
-			if (rc < 0) {
-				rc = errno;
-				LogDebug(COMPONENT_FSAL, "close failed with %s",
-					 strerror(rc));
-			}
-		}
-		if (retval == ENOENT)
-			retval = ESTALE;
-		*fsal_error = posix2fsal_error(retval);
-		LogDebug(COMPONENT_FSAL, "%s failed with %s", func,
-			 strerror(retval));
-		cfd.fd = -retval;
-		cfd.close_fd = false;
-		return cfd;
-	}
-	return cfd;
-}
-
 /* file_unlink
  * unlink the named file in the directory
  */
@@ -1782,19 +1718,13 @@ static void release(struct fsal_obj_handle *obj_hdl)
 	if (type == REGULAR_FILE) {
 		fsal_status_t st;
 
-		/* Take write lock on object to protect file descriptor.
-		 * This can block over an I/O operation.
-		 */
-		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
-
-		st = vfs_close_my_fd(&myself->u.file.fd);
-
-		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+		st = close_fsal_fd(obj_hdl, &myself->u.file.fd.fsal_fd, false);
 
 		if (FSAL_IS_ERROR(st)) {
 			LogCrit(COMPONENT_FSAL,
-				"Could not close hdl 0x%p, error %s(%d)",
-				obj_hdl, strerror(st.minor), st.minor);
+				"Could not close hdl 0x%p, status %s error %s(%d)",
+				obj_hdl, fsal_err_txt(st),
+				strerror(st.minor), st.minor);
 		}
 	}
 
@@ -1825,6 +1755,7 @@ void vfs_handle_ops_init(struct fsal_obj_ops *ops)
 	ops->handle_to_wire = handle_to_wire;
 	ops->handle_to_key = handle_to_key;
 	ops->open2 = vfs_open2;
+	ops->status2 = vfs_status2;
 	ops->reopen2 = vfs_reopen2;
 	ops->read2 = vfs_read2;
 	ops->write2 = vfs_write2;
@@ -1837,6 +1768,8 @@ void vfs_handle_ops_init(struct fsal_obj_ops *ops)
 #endif
 	ops->setattr2 = vfs_setattr2;
 	ops->close2 = vfs_close2;
+	ops->close_func = vfs_close_func;
+	ops->reopen_func = vfs_reopen_func;
 
 	/* xattr related functions */
 	ops->list_ext_attrs = vfs_list_ext_attrs;
