@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  *   Copyright (C) Panasas, Inc. 2011
  *   Author(s): Brent Welch <welch@panasas.com>
@@ -51,9 +52,9 @@ static inline size_t vfs_sizeof_handle(struct v_fhandle *fh)
  * sizeof(fsid_t) == 8
  * sizeof(fid_len) == 2
  * sizeof(fid_reserved) == 2
- * fid_len == MAXFIDSIZE
+ * fid_len == MAXFIDSZ
  */
-#if VFS_HANDLE_LEN < (1 + 8 + 2 + 2 + MAXFIDSIZE)
+#if VFS_HANDLE_LEN < (1 + 8 + 2 + 2 + MAXFIDSZ)
 #error "VFS_HANDLE_LEN is too small"
 #endif
 
@@ -126,7 +127,7 @@ int vfs_name_to_handle(int atfd,
 	return error;
 }
 
-int vfs_open_by_handle(struct vfs_filesystem *fs,
+int vfs_open_by_handle(struct fsal_filesystem *fs,
 		       vfs_file_handle_t *fh, int openflags,
 		       fsal_errors_t *fsal_error)
 {
@@ -152,25 +153,6 @@ int vfs_extract_fsid(vfs_file_handle_t *fh,
 
 	LogVFSHandle(fh);
 
-#ifdef __PanFS__
-	if (hdl->fh_fid.fid_reserved != 0) {
-		int rc;
-
-		*fsid_type = (enum fsid_type) (hdl->fh_fid.fid_reserved - 1);
-
-		rc = decode_fsid(fh->handle_data,
-				 sizeof(fh->handle_data),
-				 fsid,
-				 *fsid_type);
-		if (rc < 0) {
-			errno = EINVAL;
-			return rc;
-		}
-
-		return 0;
-	}
-#endif
-
 	*fsid_type = FSID_TWO_UINT32;
 	fsid->major = hdl->fh_fsid.val[0];
 	fsid->minor = hdl->fh_fsid.val[1];
@@ -249,37 +231,13 @@ bool vfs_valid_handle(struct gsh_buffdesc *desc)
 		LogMidDebug(COMPONENT_FSAL, "%s", buf);
 	}
 
-#ifdef __PanFS__
-	if (hdl->fh_fid.fid_reserved != 0) {
-		bool fsid_type_ok = false;
-
-		switch ((enum fsid_type) (hdl->fh_fid.fid_reserved - 1)) {
-		case FSID_NO_TYPE:
-		case FSID_ONE_UINT64:
-		case FSID_MAJOR_64:
-		case FSID_TWO_UINT64:
-		case FSID_TWO_UINT32:
-		case FSID_DEVICE:
-			fsid_type_ok = true;
-			break;
-		}
-
-		if (!fsid_type_ok) {
-			LogDebug(COMPONENT_FSAL,
-				 "FSID Type %02"PRIu16" invalid",
-				 hdl->fh_fid.fid_reserved - 1);
-			return false;
-		}
-	}
-#endif
-
 	return (desc->len >= (sizeof(fsid_t) +
 			  sizeof(hdl->fh_fid.fid_len) +
 			  sizeof(hdl->fh_fid.fid_reserved))) &&
 	       (desc->len == vfs_sizeof_handle(hdl));
 }
 
-int vfs_re_index(struct vfs_filesystem *vfs_fs,
+int vfs_re_index(struct fsal_filesystem *fs,
 		 struct vfs_fsal_export *exp)
 {
 	enum fsid_type fsid_type;
@@ -289,13 +247,13 @@ int vfs_re_index(struct vfs_filesystem *vfs_fs,
 
 	vfs_alloc_handle(fh);
 
-	retval = vfs_fd_to_handle(vfs_fs->root_fd, vfs_fs->fs, fh);
+	retval = vfs_fd_to_handle(root_fd(fs), fs, fh);
 
 	if (retval != 0) {
 		retval = errno;
 		LogMajor(COMPONENT_FSAL,
 			 "Get root handle for %s failed with %s (%d)",
-			 vfs_fs->fs->path, strerror(retval), retval);
+			 fs->path, strerror(retval), retval);
 		goto errout;
 	}
 
@@ -305,12 +263,12 @@ int vfs_re_index(struct vfs_filesystem *vfs_fs,
 	 */
 	(void) vfs_extract_fsid(fh, &fsid_type, &fsid);
 
-	retval = re_index_fs_fsid(vfs_fs->fs, fsid_type, &fsid);
+	retval = re_index_fs_fsid(fs, fsid_type, &fsid);
 
 	if (retval < 0) {
 		LogCrit(COMPONENT_FSAL,
 			"Could not re-index VFS file system fsid for %s",
-			vfs_fs->fs->path);
+			fs->path);
 		retval = -retval;
 	}
 
