@@ -95,7 +95,6 @@ static void mdcache_unexport(struct fsal_export *exp_hdl,
 						   obj_handle);
 	mdcache_entry_t *entry;
 	struct entry_export_map *expmap;
-	fsal_status_t status;
 
 	/* Indicate this export is going away so we don't create any new
 	 * export map entries.
@@ -130,17 +129,9 @@ static void mdcache_unexport(struct fsal_export *exp_hdl,
 		/* Get a ref across cleanup.  This must be an initial ref, so
 		 * that it takes the LRU lane lock, keeping it from racing with
 		 * lru_lane_run() */
-		status = mdcache_lru_ref(entry, LRU_REQ_INITIAL);
+		mdcache_lru_ref(entry, LRU_ACTIVE_REF | LRU_PROMOTE);
 		PTHREAD_RWLOCK_unlock(&exp->mdc_exp_lock);
 
-		if (FSAL_IS_ERROR(status)) {
-			/* Entry was stale; skip it */
-			LogFullDebug(COMPONENT_EXPORT,
-				     "Error %s on entry %p",
-				     msg_fsal_err(status.major), entry);
-			continue;
-		}
-
 		/* Must get attr_lock before mdc_exp_lock */
 		PTHREAD_RWLOCK_wrlock(&entry->attr_lock);
 		PTHREAD_RWLOCK_wrlock(&exp->mdc_exp_lock);
@@ -186,7 +177,7 @@ static void mdcache_unexport(struct fsal_export *exp_hdl,
 		}
 
 		/* Release above ref */
-		mdcache_put(entry);
+		mdcache_lru_unref(entry, LRU_ACTIVE_REF);
 	};
 
 	/* Last unexport for the sub-FSAL */
@@ -340,6 +331,9 @@ static void mdcache_exp_release(struct fsal_export *exp_hdl)
 
 	gsh_free(exp->name);
 
+	PTHREAD_RWLOCK_destroy(&exp->mdc_exp_lock);
+	PTHREAD_MUTEX_destroy(&exp->dirent_map.dm_mtx);
+
 	gsh_free(exp);	/* elvis has left the building */
 }
 
@@ -899,23 +893,6 @@ static struct state_t *mdcache_alloc_state(struct fsal_export *exp_hdl,
 	return state;
 }
 
-/**
- * @brief Free state_t structure
- *
- * @param[in] exp_hdl	Export state is associated with
- * @param[in] state	State to free
- */
-static void mdcache_free_state(struct fsal_export *exp_hdl,
-			       struct state_t *state)
-{
-	struct mdcache_fsal_export *exp = mdc_export(exp_hdl);
-	struct fsal_export *sub_export = exp->mfe_exp.sub_export;
-
-	subcall_raw(exp,
-		sub_export->exp_ops.free_state(sub_export, state)
-	       );
-}
-
 /**
  * @brief Check to see if a user is superuser
  *
@@ -992,7 +969,6 @@ void mdcache_export_ops_init(struct export_ops *ops)
 	ops->fs_loc_body_size = mdcache_fs_loc_body_size;
 	ops->get_write_verifier = mdcache_get_write_verifier;
 	ops->alloc_state = mdcache_alloc_state;
-	ops->free_state = mdcache_free_state;
 	ops->is_superuser = mdcache_is_superuser;
 	ops->fs_expiretimeparent = mdcache_fs_expiretimeparent;
 }
