@@ -78,9 +78,21 @@ struct mdcache_parameter {
 	/** High water mark for chunks.  Defaults to 100000,
 	    settable by Chunks_HWMark. */
 	uint32_t chunks_hwmark;
+	/** Low water mark for chunks.  Defaults to 10000,
+	    settable by Chunks_HWMark. */
+	uint32_t chunks_lwmark;
 	/** Base interval in seconds between runs of the LRU cleaner
 	    thread. Defaults to 60, settable with LRU_Run_Interval. */
 	uint32_t lru_run_interval;
+	/** If "Cache_FDs" is set to false, the reaper thread aggressively
+	 *  closes FDs , significantly reducing the number of open FDs.
+	 *  This will help to maintain a minimal number of open FDs.
+	 *
+	 *  If "Cache_FDs" is set to true (default), FDs are cached, and the
+	 *  LRU reaper thread closes FDs only when the current open FD count
+	 *  reaches or exceeds the "fds_lowat" threshold.
+	 */
+	bool Cache_FDs;
 	/** The percentage of the system-imposed maximum of file
 	    descriptors at which Ganesha will deny requests.
 	    Defaults to 99, settable with FD_Limit_Percent. */
@@ -92,12 +104,11 @@ struct mdcache_parameter {
 	uint32_t fd_hwmark_percent;
 	/** The percentage of the system-imposed maximum of file
 	    descriptors below which Ganesha will not reap file
-	    descriptonot reap file descriptors.  Defaults to 50,
-	    settable with FD_LWMark_Percent. */
+	    descriptors.  Defaults to 50, settable with FD_LWMark_Percent. */
 	uint32_t fd_lwmark_percent;
 	/** Roughly, the amount of work to do on each pass through the
 	    thread under normal conditions.  (Ideally, a multiple of
-	    the number of lanes.)  Defaults to 1000, settable with
+	    the number of lanes.)  Defaults to 0, settable with
 	    Reaper_Work. */
 	uint32_t reaper_work;
 	/** The amount of work for the reaper thread to do per-lane
