@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -126,8 +127,8 @@ static fsal_status_t mdc_open2_by_name(mdcache_entry_t *mdc_parent,
 				       fsal_openflags_t openflags,
 				       enum fsal_create_mode createmode,
 				       const char *name,
-				       struct attrlist *attrib_set,
-				       struct attrlist *attrs_out,
+				       struct fsal_attrlist *attrib_set,
+				       struct fsal_attrlist *attrs_out,
 				       fsal_verifier_t verifier,
 				       mdcache_entry_t **new_entry,
 				       bool *caller_perm_check)
@@ -148,23 +149,23 @@ static fsal_status_t mdc_open2_by_name(mdcache_entry_t *mdc_parent,
 		/* Does not exist, or other error, return to open2 to
 		 * proceed if not found, otherwise to return the error.
 		 */
-		LogFullDebug(COMPONENT_CACHE_INODE,
+		LogFullDebug(COMPONENT_MDCACHE,
 			     "Lookup failed");
 		return status;
 	}
 
 	/* Found to exist */
 	if (createmode == FSAL_GUARDED) {
-		mdcache_put(entry);
-		return fsalstat(ERR_FSAL_EXIST, 0);
+		status = fsalstat(ERR_FSAL_EXIST, 0);
+		goto out_unref;
 	} else if (createmode == FSAL_EXCLUSIVE) {
 		/* Exclusive create with entry found, check verifier */
 		if (!mdcache_check_verifier(&entry->obj_handle, verifier)) {
 			/* Verifier check failed. */
-			LogFullDebug(COMPONENT_CACHE_INODE,
+			LogFullDebug(COMPONENT_MDCACHE,
 				     "Verifier check failed.");
-			mdcache_put(entry);
-			return fsalstat(ERR_FSAL_EXIST, 0);
+			status = fsalstat(ERR_FSAL_EXIST, 0);
+			goto out_unref;
 		}
 
 		/* Verifier matches, go ahead and open the file. */
@@ -173,16 +174,16 @@ static fsal_status_t mdc_open2_by_name(mdcache_entry_t *mdc_parent,
 
 	/* Check if the object type is REGULAR_FILE. If not then give error. */
 	if (entry->obj_handle.type != REGULAR_FILE) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "Trying to open a non-regular file");
 		if (entry->obj_handle.type == DIRECTORY) {
 			/* Trying to open2 a directory */
-			mdcache_put(entry);
-			return fsalstat(ERR_FSAL_ISDIR, 0);
+			status = fsalstat(ERR_FSAL_ISDIR, 0);
+			goto out_unref;
 		} else {
 			/* Trying to open2 any other non-regular file */
-			mdcache_put(entry);
-			return fsalstat(ERR_FSAL_SYMLINK, 0);
+			status = fsalstat(ERR_FSAL_SYMLINK, 0);
+			goto out_unref;
 		}
 	}
 	subcall(
@@ -194,14 +195,13 @@ static fsal_status_t mdc_open2_by_name(mdcache_entry_t *mdc_parent,
 
 	if (FSAL_IS_ERROR(status)) {
 		/* Open failed. */
-		LogFullDebug(COMPONENT_CACHE_INODE,
+		LogFullDebug(COMPONENT_MDCACHE,
 			     "Open failed %s",
 			     msg_fsal_err(status.major));
-		mdcache_put(entry);
-		return status;
+		goto out_unref;
 	}
 
-	LogFullDebug(COMPONENT_CACHE_INODE,
+	LogFullDebug(COMPONENT_MDCACHE,
 		     "Opened entry %p, sub_handle %p",
 		     entry, entry->sub_handle);
 
@@ -214,7 +214,7 @@ static fsal_status_t mdc_open2_by_name(mdcache_entry_t *mdc_parent,
 	if (attrs_out) {
 		/* Handle attribute request */
 		if (!(attrs_out->valid_mask & ATTR_RDATTR_ERR)) {
-			struct attrlist attrs;
+			struct fsal_attrlist attrs;
 
 			/* open2() gave us attributes.  Update the cache */
 			fsal_prepare_attrs(
@@ -236,7 +236,7 @@ static fsal_status_t mdc_open2_by_name(mdcache_entry_t *mdc_parent,
 			status = entry->obj_handle.obj_ops->getattrs(
 					    &entry->obj_handle, attrs_out);
 			if (FSAL_IS_ERROR(status)) {
-				LogFullDebug(COMPONENT_CACHE_INODE,
+				LogFullDebug(COMPONENT_MDCACHE,
 					     "getattrs failed status=%s",
 					     fsal_err_txt(status));
 			}
@@ -245,6 +245,11 @@ static fsal_status_t mdc_open2_by_name(mdcache_entry_t *mdc_parent,
 	*new_entry = entry;
 
 	return status;
+
+out_unref:
+
+	mdcache_lru_unref(entry, LRU_ACTIVE_REF);
+	return status;
 }
 
 /**
@@ -318,10 +323,10 @@ fsal_status_t mdcache_open2(struct fsal_obj_handle *obj_hdl,
 			   fsal_openflags_t openflags,
 			   enum fsal_create_mode createmode,
 			   const char *name,
-			   struct attrlist *attrs_in,
+			   struct fsal_attrlist *attrs_in,
 			   fsal_verifier_t verifier,
 			   struct fsal_obj_handle **new_obj,
-			   struct attrlist *attrs_out,
+			   struct fsal_attrlist *attrs_out,
 			   bool *caller_perm_check)
 {
 	mdcache_entry_t *mdc_parent =
@@ -329,22 +334,15 @@ fsal_status_t mdcache_open2(struct fsal_obj_handle *obj_hdl,
 	mdcache_entry_t *new_entry = NULL;
 	struct fsal_obj_handle *sub_handle = NULL;
 	fsal_status_t status;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 	const char *dispname = name != NULL ? name : "<by-handle>";
 	struct mdcache_fsal_export *export = mdc_cur_export();
 	bool invalidate;
 
-	LogAttrlist(COMPONENT_CACHE_INODE, NIV_FULL_DEBUG,
+	LogAttrlist(COMPONENT_MDCACHE, NIV_FULL_DEBUG,
 		    "attrs_in ", attrs_in, false);
 
 	if (name) {
-		if (!state && !mdcache_lru_fds_available()) {
-			/* This seems the best idea, let the
-			 * client try again later after the reap.
-			 */
-			return fsalstat(ERR_FSAL_DELAY, 0);
-		}
-
 		/* Check if we have the file already cached, in which case
 		 * we can open by object instead of by name.
 		 */
@@ -388,7 +386,7 @@ fsal_status_t mdcache_open2(struct fsal_obj_handle *obj_hdl,
 	       );
 
 	if (unlikely(FSAL_IS_ERROR(status))) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "open2 %s failed with %s",
 			 dispname, fsal_err_txt(status));
 		if (status.major == ERR_FSAL_STALE) {
@@ -412,7 +410,7 @@ fsal_status_t mdcache_open2(struct fsal_obj_handle *obj_hdl,
 						   MDCACHE_TRUST_ATTRS);
 		}
 
-		LogFullDebug(COMPONENT_CACHE_INODE,
+		LogFullDebug(COMPONENT_MDCACHE,
 			     "Open2 of object succeeded.");
 		*new_obj = obj_hdl;
 		/* We didn't actually get any attributes, but release anyway
@@ -557,14 +555,14 @@ static void mdc_read_super_cb(struct fsal_obj_handle *obj, fsal_status_t ret,
 	 * cb will drop the initial ref, take an additional ref to prevent the
 	 * entry from getting freed/reaped.
 	 */
-	mdcache_get(entry);
+	mdcache_lru_ref(entry, LRU_ACTIVE_REF);
 	arg->cb(arg->obj_hdl, ret, obj_data, arg->cb_arg);
 
 	if (!FSAL_IS_ERROR(ret))
 		mdc_set_time_current(&entry->attrs.atime);
-	else if (ret.major == ERR_FSAL_DELAY)
+	else if (ret.major == ERR_FSAL_STALE)
 		mdcache_kill_entry(entry);
-	mdcache_put(entry);
+	mdcache_lru_unref(entry, LRU_ACTIVE_REF);
 	gsh_free(arg);
 }
 
@@ -644,7 +642,7 @@ static void mdc_write_super_cb(struct fsal_obj_handle *obj, fsal_status_t ret,
 		 * killing the entry might drop the sentinel ref. Take an
 		 * extra ref to prevent this entry for being reused.
 		 */
-		mdcache_get(entry);
+		mdcache_lru_ref(entry, LRU_ACTIVE_REF);
 		mdcache_kill_entry(entry);
 	} else {
 		/*
@@ -660,7 +658,7 @@ static void mdc_write_super_cb(struct fsal_obj_handle *obj, fsal_status_t ret,
 	arg->cb(arg->obj_hdl, ret, obj_data, arg->cb_arg);
 
 	if (ret.major == ERR_FSAL_STALE)
-		mdcache_put(entry);
+		mdcache_lru_unref(entry, LRU_ACTIVE_REF);
 	gsh_free(arg);
 }
 
@@ -738,7 +736,7 @@ fsal_status_t mdcache_seek2(struct fsal_obj_handle *obj_hdl,
 			entry->sub_handle, state, info)
 	       );
 
-	if (status.major == ERR_FSAL_DELAY)
+	if (status.major == ERR_FSAL_STALE)
 		mdcache_kill_entry(entry);
 
 	return status;
@@ -767,7 +765,7 @@ fsal_status_t mdcache_io_advise2(struct fsal_obj_handle *obj_hdl,
 			entry->sub_handle, state, hints)
 	       );
 
-	if (status.major == ERR_FSAL_DELAY)
+	if (status.major == ERR_FSAL_STALE)
 		mdcache_kill_entry(entry);
 
 	return status;
