@@ -45,6 +45,10 @@
 #include "mdcache_hash.h"
 #include "mdcache_avl.h"
 
+#ifdef USE_MONITORING
+#include "monitoring.h"
+#endif  /* USE_MONITORING */
+
 /*
  * handle methods
  */
@@ -100,14 +104,14 @@ fsal_status_t mdcache_alloc_and_check_handle(
 
 	status = mdcache_new_entry(export, sub_handle, attrs_in, false,
 				   attrs_out, new_directory, &new_entry, state,
-				   MDC_REASON_DEFAULT);
+				   LRU_ACTIVE_REF | LRU_PROMOTE);
 
 	if (FSAL_IS_ERROR(status)) {
 		*new_obj = NULL;
 		return status;
 	}
 
-	LogFullDebug(COMPONENT_CACHE_INODE,
+	LogFullDebug(COMPONENT_MDCACHE,
 		     "%sCreated entry %p FSAL %s for %s",
 		     tag, new_entry, new_entry->sub_handle->fsal->name,
 		     name);
@@ -127,11 +131,11 @@ fsal_status_t mdcache_alloc_and_check_handle(
 					    invalidate);
 
 		if (FSAL_IS_ERROR(status)) {
-			LogDebug(COMPONENT_CACHE_INODE,
+			LogDebug(COMPONENT_MDCACHE,
 				 "%s%s failed because add dirent failed",
 				 tag, name);
 
-			mdcache_put(new_entry);
+			mdcache_lru_unref(new_entry, LRU_ACTIVE_REF);
 			*new_obj = NULL;
 			return status;
 		}
@@ -147,7 +151,7 @@ fsal_status_t mdcache_alloc_and_check_handle(
 	*new_obj = &new_entry->obj_handle;
 
 	if (attrs_out != NULL) {
-		LogAttrlist(COMPONENT_CACHE_INODE, NIV_FULL_DEBUG,
+		LogAttrlist(COMPONENT_MDCACHE, NIV_FULL_DEBUG,
 			    tag, attrs_out, true);
 	}
 
@@ -226,14 +230,14 @@ static fsal_status_t mdcache_mkdir(struct fsal_obj_handle *dir_hdl,
 	       );
 
 	if (unlikely(FSAL_IS_ERROR(status))) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "mkdir %s failed with %s",
 			 name, fsal_err_txt(status));
 		if (status.major == ERR_FSAL_STALE) {
 			/* If we got ERR_FSAL_STALE, the previous FSAL call
 			 * must have failed with a bad parent.
 			 */
-			LogEvent(COMPONENT_CACHE_INODE,
+			LogEvent(COMPONENT_MDCACHE,
 				 "FSAL returned STALE on mkdir");
 			mdcache_kill_entry(parent);
 		}
@@ -306,14 +310,14 @@ static fsal_status_t mdcache_mknode(struct fsal_obj_handle *dir_hdl,
 	       );
 
 	if (unlikely(FSAL_IS_ERROR(status))) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "mknod %s failed with %s",
 			 name, fsal_err_txt(status));
 		if (status.major == ERR_FSAL_STALE) {
 			/* If we got ERR_FSAL_STALE, the previous FSAL call
 			 * must have failed with a bad parent.
 			 */
-			LogEvent(COMPONENT_CACHE_INODE,
+			LogEvent(COMPONENT_MDCACHE,
 				 "FSAL returned STALE on mknod");
 			mdcache_kill_entry(parent);
 		}
@@ -386,14 +390,14 @@ static fsal_status_t mdcache_symlink(struct fsal_obj_handle *dir_hdl,
 	       );
 
 	if (unlikely(FSAL_IS_ERROR(status))) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "symlink %s failed with %s",
 			 name, fsal_err_txt(status));
 		if (status.major == ERR_FSAL_STALE) {
 			/* If we got ERR_FSAL_STALE, the previous FSAL call
 			 * must have failed with a bad parent.
 			 */
-			LogEvent(COMPONENT_CACHE_INODE,
+			LogEvent(COMPONENT_MDCACHE,
 				 "FSAL returned STALE on symlink");
 			mdcache_kill_entry(parent);
 		}
@@ -490,7 +494,7 @@ static fsal_status_t mdcache_link(struct fsal_obj_handle *obj_hdl,
 	       );
 
 	if (FSAL_IS_ERROR(status)) {
-		LogFullDebug(COMPONENT_CACHE_INODE,
+		LogFullDebug(COMPONENT_MDCACHE,
 			     "link failed %s",
 			     fsal_err_txt(status));
 		return status;
@@ -553,7 +557,7 @@ static fsal_status_t mdcache_readdir(struct fsal_obj_handle *dir_hdl,
 						cb, attrmask, eod_met);
 	} else {
 		/* Dirent chunking is enabled. */
-		LogDebugAlt(COMPONENT_NFS_READDIR, COMPONENT_CACHE_INODE,
+		LogDebugAlt(COMPONENT_NFS_READDIR, COMPONENT_MDCACHE,
 			    "Calling mdcache_readdir_chunked whence=%"PRIx64,
 			    whence ? *whence : (uint64_t) 0);
 
@@ -637,19 +641,28 @@ static fsal_status_t mdcache_rename(struct fsal_obj_handle *obj_hdl,
 
 	status = mdc_lookup(mdc_newdir, new_name, true, &mdc_lookup_dst, NULL);
 	if (!FSAL_IS_ERROR(status)) {
-		if (mdc_obj == mdc_lookup_dst) {
+		if ((mdc_obj == mdc_lookup_dst) &&
+		    !strcmp(old_name, new_name) &&
+		    olddir_hdl->obj_ops->handle_cmp(olddir_hdl, newdir_hdl)) {
+			LogDebug(COMPONENT_MDCACHE,
+				 "Rename (%p,%s)->(%p,%s): same source and destination",
+				 mdc_olddir, old_name, mdc_newdir, new_name);
 			/* Same source and destination */
 			goto out;
 		}
 		if (obj_is_junction(&mdc_lookup_dst->obj_handle)) {
+			LogDebug(COMPONENT_MDCACHE,
+				 "Rename (%p,%s)->(%p,%s): Cannot rename on top of junction",
+				 mdc_olddir, old_name, mdc_newdir, new_name);
 			/* Cannot rename on top of junction */
 			status = fsalstat(ERR_FSAL_XDEV, 0);
 			goto out;
 		}
 
 		if (state_deleg_conflict(&mdc_lookup_dst->obj_handle, true)) {
-			LogDebug(COMPONENT_CACHE_INODE, "Found an existing delegation for %s",
-				  new_name);
+			LogDebug(COMPONENT_MDCACHE,
+				 "Found an existing delegation for %s",
+				 new_name);
 			status = fsalstat(ERR_FSAL_DELAY, 0);
 			goto out;
 		}
@@ -707,7 +720,7 @@ static fsal_status_t mdcache_rename(struct fsal_obj_handle *obj_hdl,
 				sub_export, fso_rename_changes_key)
 	       );
 	if (rename_change_key) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "Rename (%p,%s)->(%p,%s) : key changing", mdc_olddir,
 			 old_name, mdc_newdir, new_name);
 
@@ -733,7 +746,7 @@ static fsal_status_t mdcache_rename(struct fsal_obj_handle *obj_hdl,
 	} else if (mdcache_param.dir.avl_chunk != 0) {
 		bool invalidate = true;
 
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "Rename (%p,%s)->(%p,%s) : moving entry", mdc_olddir,
 			 old_name, mdc_newdir, new_name);
 
@@ -750,7 +763,7 @@ static fsal_status_t mdcache_rename(struct fsal_obj_handle *obj_hdl,
 		if (FSAL_IS_ERROR(status)) {
 			/* We're obviously out of date.  Throw out the cached
 			   directory */
-			LogCrit(COMPONENT_CACHE_INODE, "Add dirent returned %s",
+			LogCrit(COMPONENT_MDCACHE, "Add dirent returned %s",
 				fsal_err_txt(status));
 			/* Protected by mdcache_src_dst_lock() above */
 			mdcache_dirent_invalidate_all(mdc_newdir);
@@ -786,7 +799,7 @@ out:
 	}
 
 	if (mdc_lookup_dst)
-		mdcache_put(mdc_lookup_dst);
+		mdcache_lru_unref(mdc_lookup_dst, LRU_ACTIVE_REF);
 
 	return status;
 }
@@ -914,7 +927,7 @@ out:
 		cbgetattr->filesize = entry->attrs.filesize;
 	}
 
-	LogAttrlist(COMPONENT_CACHE_INODE, NIV_FULL_DEBUG,
+	LogAttrlist(COMPONENT_MDCACHE, NIV_FULL_DEBUG,
 		    "attrs ", &entry->attrs, true);
 
 	if (invalidate && entry->obj_handle.type == DIRECTORY &&
@@ -945,10 +958,19 @@ static fsal_status_t mdcache_getattrs(struct fsal_obj_handle *obj_hdl,
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 	fsal_status_t status = {0, 0};
 
+#ifdef USE_MONITORING
+	const char *OPERATION = "getattr";
+	struct fsal_export *export = op_ctx->fsal_export;
+	const uint16_t export_id = (export == NULL ? 0 : export->export_id);
+#endif  /* USE_MONITORING */
+
 	PTHREAD_RWLOCK_rdlock(&entry->attr_lock);
 
 	if (mdcache_is_attrs_valid(entry, attrs_out->request_mask)) {
 		/* Up-to-date */
+#ifdef USE_MONITORING
+		monitoring_mdcache_cache_hit(OPERATION, export_id);
+#endif  /* USE_MONITORING */
 		goto unlock;
 	}
 
@@ -958,9 +980,16 @@ static fsal_status_t mdcache_getattrs(struct fsal_obj_handle *obj_hdl,
 
 	if (mdcache_is_attrs_valid(entry, attrs_out->request_mask)) {
 		/* Someone beat us to it */
+#ifdef USE_MONITORING
+		monitoring_mdcache_cache_hit(OPERATION, export_id);
+#endif  /* USE_MONITORING */
 		goto unlock;
 	}
 
+#ifdef USE_MONITORING
+	monitoring_mdcache_cache_miss(OPERATION, export_id);
+#endif  /* USE_MONITORING */
+
 	status = mdcache_refresh_attrs(
 			entry, (attrs_out->request_mask & ATTR_ACL) != 0,
 			(attrs_out->request_mask & ATTR4_FS_LOCATIONS) != 0,
@@ -989,7 +1018,7 @@ unlock_no_attrs:
 	if (FSAL_IS_ERROR(status) && (status.major == ERR_FSAL_STALE))
 		mdcache_kill_entry(entry);
 
-	LogAttrlist(COMPONENT_CACHE_INODE, NIV_FULL_DEBUG,
+	LogAttrlist(COMPONENT_MDCACHE, NIV_FULL_DEBUG,
 		    "attrs ", attrs_out, true);
 
 	return status;
@@ -1048,7 +1077,7 @@ static fsal_status_t mdcache_setattr2(struct fsal_obj_handle *obj_hdl,
 		if (status2.major == ERR_FSAL_STALE)
 			kill_entry = true;
 	} else if (change == entry->attrs.change) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "setattr2 did not change attribute before %lld after = %lld",
 			 (long long) change,
 			 (long long) entry->attrs.change);
@@ -1082,7 +1111,7 @@ static fsal_status_t mdcache_unlink(struct fsal_obj_handle *dir_hdl,
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 	fsal_status_t status;
 
-	LogFullDebug(COMPONENT_CACHE_INODE,
+	LogFullDebug(COMPONENT_MDCACHE,
 		     "Unlink %p/%s (%p)",
 		     parent, name, entry);
 
@@ -1097,7 +1126,7 @@ static fsal_status_t mdcache_unlink(struct fsal_obj_handle *dir_hdl,
 	       );
 
 	if (FSAL_IS_ERROR(status)) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "unlink %s returned %s",
 			  name, fsal_err_txt(status));
 		if (status.major == ERR_FSAL_STALE)
@@ -1131,7 +1160,7 @@ static fsal_status_t mdcache_unlink(struct fsal_obj_handle *dir_hdl,
 		mdc_unreachable(entry);
 	}
 
-	LogFullDebug(COMPONENT_CACHE_INODE,
+	LogFullDebug(COMPONENT_MDCACHE,
 		     "Unlink %s %p/%s (%p)",
 		     FSAL_IS_ERROR(status) ? "failed" : "done",
 		     parent, name, entry);
@@ -1316,28 +1345,26 @@ static nfsstat4 mdcache_layoutcommit(struct fsal_obj_handle *obj_hdl,
  * @brief Get a reference to the handle
  *
  * @param[in] obj_hdl	Handle to ref
- * @return FSAL status
  */
 static void mdcache_get_ref(struct fsal_obj_handle *obj_hdl)
 {
 	mdcache_entry_t *entry =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 
-	mdcache_get(entry);
+	mdcache_lru_ref(entry, LRU_ACTIVE_REF | LRU_PROMOTE);
 }
 
 /**
  * @brief Put a reference to the handle
  *
  * @param[in] obj_hdl	Handle to unref
- * @return FSAL status
  */
 static void mdcache_put_ref(struct fsal_obj_handle *obj_hdl)
 {
 	mdcache_entry_t *entry =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 
-	mdcache_put(entry);
+	mdcache_lru_unref(entry, LRU_ACTIVE_REF);
 }
 
 /**
@@ -1353,7 +1380,7 @@ static void mdcache_hdl_release(struct fsal_obj_handle *obj_hdl)
 	mdcache_entry_t *entry =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 
-	LogDebug(COMPONENT_CACHE_INODE,
+	LogDebug(COMPONENT_MDCACHE,
 		 "Releasing obj_hdl=%p, entry=%p",
 		 obj_hdl, entry);
 
@@ -1578,7 +1605,7 @@ fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 	       );
 
 	if (unlikely(FSAL_IS_ERROR(status))) {
-		LogDebug(COMPONENT_CACHE_INODE,
+		LogDebug(COMPONENT_MDCACHE,
 			 "lookup_path %s failed with %s",
 			 path, fsal_err_txt(status));
 		fsal_release_attrs(&attrs);
@@ -1586,12 +1613,13 @@ fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 	}
 
 	status = mdcache_new_entry(export, sub_handle, &attrs, false, attrs_out,
-				   false, &new_entry, NULL, MDC_REASON_DEFAULT);
+				   false, &new_entry, NULL,
+				   LRU_ACTIVE_REF | LRU_PROMOTE);
 
 	fsal_release_attrs(&attrs);
 
 	if (!FSAL_IS_ERROR(status)) {
-		LogFullDebug(COMPONENT_CACHE_INODE,
+		LogFullDebug(COMPONENT_MDCACHE,
 			     "lookup_path Created entry %p FSAL %s",
 			     new_entry, new_entry->sub_handle->fsal->name);
 		/* Make sure this entry has a parent pointer */
@@ -1601,7 +1629,7 @@ fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 	}
 
 	if (attrs_out != NULL) {
-		LogAttrlist(COMPONENT_CACHE_INODE, NIV_FULL_DEBUG,
+		LogAttrlist(COMPONENT_MDCACHE, NIV_FULL_DEBUG,
 			    "lookup_path ", attrs_out, true);
 	}
 
@@ -1640,7 +1668,7 @@ fsal_status_t mdcache_create_handle(struct fsal_export *exp_hdl,
 	mdc_get_parent(export, entry, NULL);
 
 	if (attrs_out != NULL) {
-		LogAttrlist(COMPONENT_CACHE_INODE, NIV_FULL_DEBUG,
+		LogAttrlist(COMPONENT_MDCACHE, NIV_FULL_DEBUG,
 			    "create_handle ", attrs_out, true);
 	}
 
