@@ -42,8 +42,8 @@
  * implementation.
  */
 
-#ifndef CACHE_INODE_HASH_H
-#define CACHE_INODE_HASH_H
+#ifndef MDCACHE_HASH_H
+#define MDCACHE_HASH_H
 
 #include "config.h"
 #include "log.h"
@@ -65,7 +65,7 @@
  */
 typedef struct cih_partition {
 	uint32_t part_ix;
-	pthread_rwlock_t lock;
+	pthread_rwlock_t cih_lock;
 	struct avltree t;
 	struct avltree_node **cache;
 #ifdef ENABLE_LOCKTRACE
@@ -234,7 +234,7 @@ typedef struct cih_latch {
 static inline void
 cih_hash_release(cih_latch_t *latch)
 {
-	PTHREAD_RWLOCK_unlock(&(latch->cp->lock));
+	PTHREAD_RWLOCK_unlock(&(latch->cp->cih_lock));
 }
 
 /**
@@ -254,9 +254,9 @@ cih_latch_entry(mdcache_key_t *key, cih_latch_t *latch, uint32_t flags,
 	    cih_partition_of_scalar(&cih_fhcache, key->hk);
 
 	if (flags & CIH_GET_WLOCK)
-		PTHREAD_RWLOCK_wrlock(&cp->lock);	/* SUBTREE_WLOCK */
+		PTHREAD_RWLOCK_wrlock(&cp->cih_lock);	/* SUBTREE_WLOCK */
 	else
-		PTHREAD_RWLOCK_rdlock(&cp->lock);	/* SUBTREE_RLOCK */
+		PTHREAD_RWLOCK_rdlock(&cp->cih_lock);	/* SUBTREE_RLOCK */
 
 #ifdef ENABLE_LOCKTRACE
 	cp->locktrace.func = (char *)func;
@@ -394,10 +394,10 @@ cih_remove_checked(mdcache_entry_t *entry)
 	bool unref = false;
 	bool freed = false;
 
-	PTHREAD_RWLOCK_wrlock(&cp->lock);
+	PTHREAD_RWLOCK_wrlock(&cp->cih_lock);
 	node = cih_fhcache_inline_lookup(&cp->t, &entry->fh_hk.node_k);
 	if (entry->fh_hk.inavl && node) {
-		LogFullDebug(COMPONENT_CACHE_INODE,
+		LogFullDebug(COMPONENT_MDCACHE,
 			     "Unhashing entry %p", entry);
 #ifdef USE_LTTNG
 		tracepoint(mdcache, mdc_lru_remove, __func__, __LINE__,
@@ -410,13 +410,13 @@ cih_remove_checked(mdcache_entry_t *entry)
 		/* return sentinel ref */
 		unref = true;
 	}
-	PTHREAD_RWLOCK_unlock(&cp->lock);
+	PTHREAD_RWLOCK_unlock(&cp->cih_lock);
 
 	if (unref) {
 		/* We can't unref with the lock held, in case this is the last
 		 * ref.  That will recurse into here, and try to take the lock
 		 * again. */
-		freed = mdcache_lru_unref(entry);
+		freed = mdcache_lru_unref(entry, LRU_FLAG_SENTINEL);
 	}
 
 	return freed;
@@ -443,7 +443,7 @@ cih_remove_latched(mdcache_entry_t *entry, cih_latch_t *latch, uint32_t flags)
 	    cih_partition_of_scalar(&cih_fhcache, entry->fh_hk.key.hk);
 
 	if (entry->fh_hk.inavl) {
-		LogFullDebug(COMPONENT_CACHE_INODE,
+		LogFullDebug(COMPONENT_MDCACHE,
 			     "Unhashing entry %p", entry);
 #ifdef USE_LTTNG
 		tracepoint(mdcache, mdc_lru_remove, __func__, __LINE__,
@@ -453,7 +453,7 @@ cih_remove_latched(mdcache_entry_t *entry, cih_latch_t *latch, uint32_t flags)
 		cp->cache[cih_cache_offsetof(&cih_fhcache,
 					     entry->fh_hk.key.hk)] = NULL;
 		entry->fh_hk.inavl = false;
-		mdcache_lru_unref(entry);
+		mdcache_lru_unref(entry, LRU_FLAG_SENTINEL);
 		if (flags & CIH_REMOVE_UNLOCK)
 			cih_hash_release(latch);
 		return true;
@@ -464,5 +464,5 @@ cih_remove_latched(mdcache_entry_t *entry, cih_latch_t *latch, uint32_t flags)
 	return false;
 }
 
-#endif				/* CACHE_INODE_HASH_H */
+#endif				/* MDCACHE_HASH_H */
 /** @} */
