@@ -41,6 +41,7 @@
 #include "fsal_up.h"
 #include "fsal_convert.h"
 #include "display.h"
+#include "common_utils.h"
 
 typedef struct mdcache_fsal_obj_handle mdcache_entry_t;
 
@@ -53,14 +54,6 @@ extern struct mdcache_fsal_module MDCACHE;
 
 #define MDC_UNEXPORT 1
 
-/**
- * @brief Reason an entry is being inserted/looked up
- */
-typedef enum {
-	MDC_REASON_DEFAULT,	/**< Default insertion */
-	MDC_REASON_SCAN		/**< Is being inserted by a scan */
-} mdc_reason_t;
-
 typedef struct mdcache_dmap_entry__ {
 	/** AVL node in tree by cookie */
 	struct avltree_node node;
@@ -76,7 +69,7 @@ typedef struct mdcache_dmap_entry__ {
 
 typedef struct {
 	/** Lock protecting this structure */
-	pthread_mutex_t mtx;
+	pthread_mutex_t dm_mtx;
 	/** Mapping of ck -> name for whence-is-name */
 	struct avltree map;
 	/** LRU of dirent map entries */
@@ -158,11 +151,16 @@ enum lru_q_id {
 	LRU_ENTRY_NONE = 0, /* entry not queued */
 	LRU_ENTRY_L1,
 	LRU_ENTRY_L2,
-	LRU_ENTRY_CLEANUP
+	LRU_ENTRY_CLEANUP,
+	LRU_ENTRY_ACTIVE,
 };
 
 #define LRU_CLEANUP 0x00000001 /* Entry is on cleanup queue */
 #define LRU_CLEANED 0x00000002 /* Entry has been cleaned */
+#define LRU_EVER_PROMOTED 0x00000004 /* Entry will be promoted after releasing
+				   * last active reference (never cleared).
+				   */
+#define LRU_SENTINEL_HELD 0x00000008 /* true if sentinel reference is held */
 
 typedef struct mdcache_lru__ {
 	struct glist_head q;	/*< Link in the physical deque
@@ -171,6 +169,8 @@ typedef struct mdcache_lru__ {
 	enum lru_q_id qid;	/*< Queue identifier */
 	int32_t refcnt;		/*< Reference count.  This is signed to make
 				   mistakes easy to see. */
+	int32_t active_refcnt;	/*< Active Reference count.  This is signed
+				    to make mistakes easy to see. */
 	uint32_t flags;		/*< Status flags; MUST use atomic ops */
 	uint32_t lane;		/*< The lane in which an entry currently
 				 *< resides, so we can lock the deque and
@@ -321,7 +321,7 @@ struct mdcache_fsal_obj_handle {
 			/** List of detached directory entries. */
 			struct glist_head detached;
 			/** Spin lock to protect the detached list. */
-			pthread_spinlock_t spin;
+			pthread_spinlock_t fsd_spin;
 			/** Count of detached directory entries. */
 			int detached_count;
 			/** @todo FSF
@@ -422,7 +422,7 @@ typedef struct mdcache_dir_entry__ {
 	/** Temporary entry pointer
 	 * Only valid while the entry is ref'd.  Must be NULL otherwise.
 	 * Protected by the parent content_lock */
-	mdcache_entry_t *entry;
+	mdcache_entry_t *mde_entry;
 	const char *name;
 	/** The NUL-terminated filename */
 	char name_buffer[];
@@ -438,13 +438,13 @@ typedef struct mdcache_dir_entry__ {
 static inline void bump_detached_dirent(mdcache_entry_t *parent,
 					mdcache_dir_entry_t *dirent)
 {
-	pthread_spin_lock(&parent->fsobj.fsdir.spin);
+	PTHREAD_SPIN_lock(&parent->fsobj.fsdir.fsd_spin);
 	if (glist_first_entry(&parent->fsobj.fsdir.detached,
 			      mdcache_dir_entry_t, chunk_list) != dirent) {
 		glist_del(&dirent->chunk_list);
 		glist_add(&parent->fsobj.fsdir.detached, &dirent->chunk_list);
 	}
-	pthread_spin_unlock(&parent->fsobj.fsdir.spin);
+	PTHREAD_SPIN_unlock(&parent->fsobj.fsdir.fsd_spin);
 }
 
 /**
@@ -457,7 +457,7 @@ static inline void bump_detached_dirent(mdcache_entry_t *parent,
 static inline void rmv_detached_dirent(mdcache_entry_t *parent,
 				       mdcache_dir_entry_t *dirent)
 {
-	pthread_spin_lock(&parent->fsobj.fsdir.spin);
+	PTHREAD_SPIN_lock(&parent->fsobj.fsdir.fsd_spin);
 	/* Note that the dirent might not be on the detached list if it
 	 * was being reaped by another thread. All is well here...
 	 */
@@ -465,7 +465,7 @@ static inline void rmv_detached_dirent(mdcache_entry_t *parent,
 		glist_del(&dirent->chunk_list);
 		parent->fsobj.fsdir.detached_count--;
 	}
-	pthread_spin_unlock(&parent->fsobj.fsdir.spin);
+	PTHREAD_SPIN_unlock(&parent->fsobj.fsdir.fsd_spin);
 }
 
 /* Helpers */
@@ -494,12 +494,10 @@ fsal_status_t mdcache_new_entry(struct mdcache_fsal_export *exp,
 				bool new_directory,
 				mdcache_entry_t **entry,
 				struct state_t *state,
-				mdc_reason_t reason);
+				uint32_t flags);
 fsal_status_t mdcache_find_keyed_reason(mdcache_key_t *key,
 					mdcache_entry_t **entry,
-					mdc_reason_t reason);
-#define mdcache_find_keyed(key, entry) mdcache_find_keyed_reason((key), \
-					(entry), MDC_REASON_DEFAULT)
+					uint32_t flags);
 fsal_status_t mdcache_locate_host(struct gsh_buffdesc *fh_desc,
 				  struct mdcache_fsal_export *exp,
 				  mdcache_entry_t **entry,
@@ -610,9 +608,17 @@ extern struct config_block mdcache_param_blk;
 	subcall_raw(__export, call); \
 } while (0)
 
+/* Call an async sub-FSAL function using it's export.
+ * op_ctx is undefined after this call.
+ */
+#define subcall_async_raw(myexp, call) do { \
+	op_ctx->fsal_export = (myexp)->mfe_exp.sub_export; \
+	call; \
+} while (0)
+
 /* During a callback from a sub-FSAL, call using MDCACHE's export */
 #define supercall_raw(myexp, call) do { \
-	LogFullDebug(COMPONENT_CACHE_INODE, "supercall %s", myexp->name); \
+	LogFullDebug(COMPONENT_MDCACHE, "supercall %s", myexp->name); \
 	op_ctx->fsal_export = &(myexp)->mfe_exp; \
 	call; \
 	op_ctx->fsal_export = (myexp)->mfe_exp.sub_export; \
@@ -977,8 +983,8 @@ static inline void
 _mdc_unreachable(mdcache_entry_t *entry,
 		 char *file, int line, char *function)
 {
-	if (isDebug(COMPONENT_CACHE_INODE)) {
-		DisplayLogComponentLevel(COMPONENT_CACHE_INODE,
+	if (isDebug(COMPONENT_MDCACHE)) {
+		DisplayLogComponentLevel(COMPONENT_MDCACHE,
 					 file, line, function, NIV_DEBUG,
 					 "Unreachable %s entry %p %s state",
 					 object_file_type_to_str(
@@ -1152,10 +1158,10 @@ fsal_status_t mdcache_export_up_ops_init(struct fsal_up_vector *my_up_ops,
 
 /* Debug functions */
 #define MDC_LOG_KEY(key) do { \
-	LogFullDebugOpaque(COMPONENT_CACHE_INODE, \
+	LogFullDebugOpaque(COMPONENT_MDCACHE, \
 			   "FSAL key: %s", 128, (key)->kv.addr, \
 			   (key)->kv.len); \
-	LogFullDebug(COMPONENT_CACHE_INODE, "hash key: %lx", (key)->hk); \
+	LogFullDebug(COMPONENT_MDCACHE, "hash key: %lx", (key)->hk); \
 } while (0)
 
 static inline
@@ -1170,7 +1176,7 @@ fsal_status_t mdcache_refresh_attrs_no_invalidate(mdcache_entry_t *entry)
 	PTHREAD_RWLOCK_unlock(&entry->attr_lock);
 
 	if (FSAL_IS_ERROR(status)) {
-		LogDebug(COMPONENT_CACHE_INODE, "Refresh attributes failed %s",
+		LogDebug(COMPONENT_MDCACHE, "Refresh attributes failed %s",
 			 fsal_err_txt(status));
 		if (status.major == ERR_FSAL_STALE)
 			mdcache_kill_entry(entry);
