@@ -40,13 +40,6 @@
 #include "log.h"
 #include "mdcache_int.h"
 
-enum fd_states {
-	FD_LOW,
-	FD_MIDDLE,
-	FD_HIGH,
-	FD_LIMIT,
-};
-
 /**
  * @file mdcache_lru.h
  * @author Matt Benjamin
@@ -77,20 +70,10 @@ struct lru_state {
 	uint64_t entries_used;
 	uint32_t entries_release_size;
 	uint64_t chunks_hiwat;
+	uint64_t chunks_lowat;
 	uint64_t chunks_used;
-	uint32_t fds_system_imposed;
-	uint32_t fds_hard_limit;
-	uint32_t fds_hiwat;
-	uint32_t fds_lowat;
-	/** This is the actual counter of 'futile' attempts at reaping
-	    made  in a given time period.  When it reaches the futility
-	    count, we turn off caching of file descriptors. */
-	uint32_t futility;
 	uint32_t per_lane_work;
-	uint32_t biggest_window;
-	uint64_t prev_fd_count;	/* previous # of open fds */
 	time_t prev_time;	/* previous time the gc thread was run. */
-	uint32_t fd_state;
 };
 
 extern struct lru_state lru_state;
@@ -99,28 +82,13 @@ extern struct lru_state lru_state;
 extern pool_t *mdcache_entry_pool;
 
 /**
- * Flags for functions in the LRU package
+ * Reference type Flags for functions in the LRU package
  */
 
-/**
- * No flag at all.
- */
-#define LRU_FLAG_NONE  0x0000
-
-/**
- * The caller holds the lock on the LRU entry.
- */
-#define LRU_FLAG_LOCKED  0x0001
-
-/**
- * The caller is fetching an initial reference
- */
-#define LRU_REQ_INITIAL  0x0002
-
-/**
- * qlane is locked
- */
-#define LRU_UNREF_QLOCKED 0x0008
+#define LRU_ACTIVE_REF 0x0004
+#define LRU_PROMOTE 0x0008
+#define LRU_FLAG_SENTINEL 0x0001
+#define LRU_TEMP_REF 0x0002
 
 /**
  * The minimum reference count for a cache entry not being recycled.
@@ -137,13 +105,21 @@ extern pool_t *mdcache_entry_pool;
 fsal_status_t mdcache_lru_pkginit(void);
 fsal_status_t mdcache_lru_pkgshutdown(void);
 
-extern size_t open_fd_count;
+mdcache_entry_t *mdcache_lru_get(struct fsal_obj_handle *sub_handle,
+				 uint32_t flags);
+void mdcache_lru_insert_active(mdcache_entry_t *entry);
 
-mdcache_entry_t *mdcache_lru_get(struct fsal_obj_handle *sub_handle);
-void mdcache_lru_insert(mdcache_entry_t *entry, mdc_reason_t reason);
 #define mdcache_lru_ref(e, f) _mdcache_lru_ref(e, f, __func__, __LINE__)
-fsal_status_t _mdcache_lru_ref(mdcache_entry_t *entry, uint32_t flags,
-			       const char *func, int line);
+
+/**
+ *
+ * @brief Get a logical reference to a cache entry
+ *
+ * @param[in] entry   Cache entry being returned
+ * @param[in] flags   Set of flags to specify type of reference
+ */
+void _mdcache_lru_ref(mdcache_entry_t *entry, uint32_t flags,
+		      const char *func, int line);
 
 /* XXX */
 void mdcache_lru_kill(mdcache_entry_t *entry);
@@ -152,22 +128,7 @@ void mdcache_lru_cleanup_try_push(mdcache_entry_t *entry);
 
 size_t mdcache_lru_release_entries(int32_t want_release);
 
-#define mdcache_lru_unref(e) _mdcache_lru_unref(e, LRU_FLAG_NONE, \
-						__func__, __LINE__)
-bool _mdcache_lru_unref(mdcache_entry_t *entry, uint32_t flags,
-			const char *func, int line);
-void mdcache_lru_kill_for_shutdown(mdcache_entry_t *entry);
-
-/**
- *
- * @brief Get a logical reference to a cache entry
- *
- * @param[in] entry Cache entry being returned
- */
-static inline fsal_status_t mdcache_get(mdcache_entry_t *entry)
-{
-	return mdcache_lru_ref(entry, LRU_FLAG_NONE);
-}
+#define mdcache_lru_unref(e, f) _mdcache_lru_unref(e, f, __func__, __LINE__)
 
 /**
  *
@@ -184,11 +145,12 @@ static inline fsal_status_t mdcache_get(mdcache_entry_t *entry)
  * accesses to the memory pointed to by entry.
  *
  * @param[in] entry Cache entry being returned
+ * @param[in] flags Flag to indicate what kind of reference is to be released
  */
-static inline void mdcache_put(mdcache_entry_t *entry)
-{
-	mdcache_lru_unref(entry);
-}
+bool _mdcache_lru_unref(mdcache_entry_t *entry, uint32_t flags,
+			const char *func, int line);
+
+void mdcache_lru_kill_for_shutdown(mdcache_entry_t *entry);
 
 #define mdcache_lru_ref_chunk(chunk) \
 	_mdcache_lru_ref_chunk(chunk, __func__, __LINE__)
