@@ -147,6 +147,7 @@ void mdcache_export_uninit(void)
 	fsal_detach_export(op_ctx->fsal_export->fsal,
 			   &op_ctx->fsal_export->exports);
 	free_export_ops(op_ctx->fsal_export);
+	up_ready_destroy(&exp->up_ops);
 
 	gsh_free(exp);
 
@@ -179,7 +180,6 @@ mdcache_fsal_create_export(struct fsal_module *sub_fsal, void *parse_node,
 {
 	fsal_status_t status = {0, 0};
 	struct mdcache_fsal_export *myself;
-	pthread_rwlockattr_t attrs;
 
 	myself = gsh_calloc(1, sizeof(struct mdcache_fsal_export));
 	myself->name = gsh_concat(sub_fsal->name, "/MDC");
@@ -195,14 +195,8 @@ mdcache_fsal_create_export(struct fsal_module *sub_fsal, void *parse_node,
 	myself->mfe_exp.fsal = &MDCACHE.module;
 
 	glist_init(&myself->entry_list);
-	pthread_rwlockattr_init(&attrs);
-#ifdef GLIBC
-	pthread_rwlockattr_setkind_np(&attrs,
-		PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP);
-#endif
-	PTHREAD_RWLOCK_init(&myself->mdc_exp_lock, &attrs);
-	PTHREAD_MUTEX_init(&myself->dirent_map.mtx, NULL);
-	pthread_rwlockattr_destroy(&attrs);
+	PTHREAD_RWLOCK_init(&myself->mdc_exp_lock, NULL);
+	PTHREAD_MUTEX_init(&myself->dirent_map.dm_mtx, NULL);
 
 	status = sub_fsal->m_ops.create_export(sub_fsal,
 						 parse_node,
@@ -437,11 +431,11 @@ void mdcache_utilization(DBusMessageIter *iter)
 	dbus_message_iter_open_container(iter, DBUS_TYPE_STRUCT, NULL,
 					 &struct_iter);
 	/* Gather the data */
-	open_fds = atomic_fetch_size_t(&open_fd_count);
+	open_fds = atomic_fetch_int32_t(&fsal_fd_global_counter);
 	entries_used = atomic_fetch_uint64_t(&lru_state.entries_used);
 	chunks_used = atomic_fetch_uint64_t(&lru_state.chunks_used);
-	fd_state = atomic_fetch_uint32_t(&lru_state.fd_state);
-	fd_limit = atomic_fetch_uint32_t(&lru_state.fds_system_imposed);
+	fd_state = atomic_fetch_uint32_t(&fd_lru_state.fd_state);
+	fd_limit = atomic_fetch_uint32_t(&fd_lru_state.fds_system_imposed);
 
 	type = " FSAL opened FD count : ";
 	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING, &type);
