@@ -319,18 +319,6 @@ static struct state_t *nullfs_alloc_state(struct fsal_export *exp_hdl,
 	return state;
 }
 
-static void nullfs_free_state(struct fsal_export *exp_hdl,
-			      struct state_t *state)
-{
-	struct nullfs_fsal_export *exp = container_of(exp_hdl,
-					struct nullfs_fsal_export, export);
-
-	op_ctx->fsal_export = exp->export.sub_export;
-	exp->export.sub_export->exp_ops.free_state(exp->export.sub_export,
-						   state);
-	op_ctx->fsal_export = &exp->export;
-}
-
 static bool nullfs_is_superuser(struct fsal_export *exp_hdl,
 				const struct user_cred *creds)
 {
@@ -423,7 +411,6 @@ void nullfs_export_ops_init(struct export_ops *ops)
 	ops->get_quota = get_quota;
 	ops->set_quota = set_quota;
 	ops->alloc_state = nullfs_alloc_state;
-	ops->free_state = nullfs_free_state;
 	ops->is_superuser = nullfs_is_superuser;
 }
 
