@@ -242,8 +242,6 @@ static const char *fsal_ace_flag(char *buf, fsal_aceflag_t flag)
 		*c++ = 'X';
 	if (flag & FSAL_ACE_IFLAG_SPECIAL_ID)
 		*c++ = 'S';
-	if (flag & FSAL_ACE_IFLAG_MODE_GEN)
-		*c++ = 'G';
 	*c = '\0';
 
 	return buf;
@@ -490,7 +488,8 @@ static fsal_status_t fsal_check_access_acl(
 	}
 
 	/* unsatisfied flags */
-	missing_access = v4mask & ~FSAL_ACE4_PERM_CONTINUE;
+	missing_access = v4mask & ~(FSAL_ACE4_PERM_CONTINUE |
+			FSAL_ACE4_REQ_FLAG);
 	if (!missing_access) {
 		LogFullDebug(COMPONENT_NFS_V4_ACL, "Nothing was requested");
 		return fsalstat(ERR_FSAL_NO_ERROR, 0);
@@ -677,10 +676,7 @@ static fsal_status_t fsal_check_access_acl(
 		}
 	}
 
-	if (IS_FSAL_ACE4_REQ(v4mask) && missing_access) {
-		LogDebug(COMPONENT_NFS_V4_ACL, "final access unknown (NO_ACE)");
-		return fsalstat(ERR_FSAL_NO_ACE, 0);
-	} else if (missing_access || (denied != NULL && *denied != 0)) {
+	if (missing_access || (denied != NULL && *denied != 0)) {
 #ifndef ENABLE_RFC_ACL
 		if ((missing_access &
 		     (FSAL_ACE_PERM_WRITE_ATTR | FSAL_ACE_PERM_WRITE_ACL |
