@@ -109,14 +109,14 @@ static int unload_fsal(struct fsal_module *fsal_hdl)
 	}
 
 	glist_del(&fsal_hdl->fsals);
-	PTHREAD_RWLOCK_destroy(&fsal_hdl->lock);
+	PTHREAD_RWLOCK_destroy(&fsal_hdl->fsm_lock);
 
 	retval = dlclose(fsal_hdl->dl_handle);
 	PTHREAD_MUTEX_unlock(&fsal_lock);
 	return retval;
 
  err:
-	PTHREAD_RWLOCK_unlock(&fsal_hdl->lock);
+	PTHREAD_RWLOCK_unlock(&fsal_hdl->fsm_lock);
 	PTHREAD_MUTEX_unlock(&fsal_lock);
 	return retval;
 }
@@ -685,21 +685,8 @@ static struct state_t *alloc_state(struct fsal_export *exp_hdl,
 				   enum state_type state_type,
 				   struct state_t *related_state)
 {
-	return init_state(gsh_calloc(1, sizeof(struct state_t)),
-			  exp_hdl, state_type, related_state);
-}
-
-/**
- * @brief Free a state_t structure
- *
- * @param[in] state                 state_t structure to free.
- *
- * @returns NULL on failure otherwise a state structure.
- */
-
-void free_state(struct fsal_export *exp_hdl, struct state_t *state)
-{
-	gsh_free(state);
+	/* MUST be overridden. */
+	return NULL;
 }
 
 /**
@@ -752,7 +739,6 @@ struct export_ops def_export_ops = {
 	.fs_loc_body_size = fs_loc_body_size,
 	.get_write_verifier = global_verifier,
 	.alloc_state = alloc_state,
-	.free_state = free_state,
 	.is_superuser = is_superuser,
 	.fs_expiretimeparent = fs_expiretimeparent,
 };
@@ -1028,18 +1014,6 @@ static fsal_status_t file_unlink(struct fsal_obj_handle *dir_hdl,
 	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 }
 
-/* seek
- * default case not supported
- */
-
-static fsal_status_t file_seek(struct fsal_obj_handle *obj_hdl,
-				struct io_info *info)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
 /* io advise
  * default case not supported
  */
@@ -1356,9 +1330,8 @@ static bool check_verifier(struct fsal_obj_handle *obj_hdl,
 static fsal_openflags_t status2(struct fsal_obj_handle *obj_hdl,
 				struct state_t *state)
 {
-	struct fsal_fd *fd = (struct fsal_fd *)(state + 1);
-
-	return fd->openflags;
+	/* default does nothing, MUST be overridden */
+	return 0;
 }
 
 /* reopen2
@@ -1533,7 +1506,6 @@ struct fsal_obj_ops def_handle_ops = {
 	.link = linkfile,
 	.rename = renamefile,
 	.unlink = file_unlink,
-	.seek = file_seek,
 	.io_advise = file_io_advise,
 	.fallocate = file_fallocate,
 	.close = file_close,
