@@ -54,18 +54,6 @@
 #include "FSAL/fsal_commonlib.h"
 #include "sal_functions.h"
 
-/**
- * This is a global counter of files opened.
- *
- * This is preliminary expected to go away.  Problems with this method are that
- * it overcounts file descriptors for FSALs that don't use them for open files,
- * and, under the Lieb Rearchitecture, FSALs will be responsible for caching
- * their own file descriptors, with interfaces for MDCACHE to interrogate
- * them as to usage or instruct them to close them.
- */
-
-size_t open_fd_count;
-
 static bool fsal_not_in_group_list(gid_t gid)
 {
 	int i;
@@ -460,10 +448,6 @@ fsal_status_t open2_by_name(struct fsal_obj_handle *in_obj,
 		return status;
 	}
 
-	if (!state) {
-		(void) atomic_inc_size_t(&open_fd_count);
-	}
-
 	LogFullDebug(COMPONENT_FSAL,
 		     "Created entry %p FSAL %s for %s",
 		     *obj, (*obj)->fsal->name, name);
@@ -1846,13 +1830,13 @@ static void sync_cb(struct fsal_obj_handle *obj, fsal_status_t ret,
 
 	/* Let caller know we are done. */
 
-	PTHREAD_MUTEX_lock(data->mutex);
+	PTHREAD_MUTEX_lock(data->fsa_mutex);
 
 	data->done = true;
 
-	pthread_cond_signal(data->cond);
+	pthread_cond_signal(data->fsa_cond);
 
-	PTHREAD_MUTEX_unlock(data->mutex);
+	PTHREAD_MUTEX_unlock(data->fsa_mutex);
 }
 
 void fsal_read(struct fsal_obj_handle *obj_hdl,
@@ -1860,17 +1844,24 @@ void fsal_read(struct fsal_obj_handle *obj_hdl,
 	       struct fsal_io_arg *arg,
 	       struct async_process_data *data)
 {
+again:
+
 	obj_hdl->obj_ops->read2(obj_hdl, bypass, sync_cb, arg, data);
 
-	PTHREAD_MUTEX_lock(data->mutex);
+	PTHREAD_MUTEX_lock(data->fsa_mutex);
 
 	while (!data->done) {
-		int rc = pthread_cond_wait(data->cond, data->mutex);
+		int rc = pthread_cond_wait(data->fsa_cond, data->fsa_mutex);
 
 		assert(rc == 0);
 	}
 
-	PTHREAD_MUTEX_unlock(data->mutex);
+	PTHREAD_MUTEX_unlock(data->fsa_mutex);
+
+	if (arg->fsal_resume) {
+		data->done = false;
+		goto again;
+	}
 }
 
 void fsal_write(struct fsal_obj_handle *obj_hdl,
@@ -1878,17 +1869,24 @@ void fsal_write(struct fsal_obj_handle *obj_hdl,
 		struct fsal_io_arg *arg,
 		struct async_process_data *data)
 {
+again:
+
 	obj_hdl->obj_ops->write2(obj_hdl, bypass, sync_cb, arg, data);
 
-	PTHREAD_MUTEX_lock(data->mutex);
+	PTHREAD_MUTEX_lock(data->fsa_mutex);
 
 	while (!data->done) {
-		int rc = pthread_cond_wait(data->cond, data->mutex);
+		int rc = pthread_cond_wait(data->fsa_cond, data->fsa_mutex);
 
 		assert(rc == 0);
 	}
 
-	PTHREAD_MUTEX_unlock(data->mutex);
+	PTHREAD_MUTEX_unlock(data->fsa_mutex);
+
+	if (arg->fsal_resume) {
+		data->done = false;
+		goto again;
+	}
 }
 
 #define XATTR_USER_PREFIX	"user."
