@@ -174,7 +174,7 @@ int open_dir_by_path_walk(int first_fd, const char *path, struct stat *stat)
 	return fd;
 }
 
-pthread_rwlock_t fs_lock = PTHREAD_RWLOCK_INITIALIZER;
+pthread_rwlock_t fs_lock;
 
 static struct glist_head posix_file_systems = {
 	&posix_file_systems, &posix_file_systems
@@ -554,7 +554,7 @@ static void posix_create_fs_btrfs_subvols(struct fsal_filesystem *fs)
 	char *path;
 
 	LogFullDebug(COMPONENT_FSAL,
-		     "Attempting to add subvols for btfs filesystem %s",
+		     "Attempting to add subvols for btrfs filesystem %s",
 		     fs->path);
 
 	/* Setup common fields for fake mntent, many are NULL or 0.
@@ -636,7 +636,7 @@ out:
 	btrfs_util_destroy_subvolume_iterator(iter);
 #else
 	LogWarn(COMPONENT_FSAL,
-		"btfs filesystem %s may have unsupported subvols",
+		"btrfs filesystem %s may have unsupported subvols",
 		fs->path);
 #endif
 }
@@ -971,7 +971,8 @@ int populate_posix_file_systems(const char *path)
 		    strcasecmp(mnt->mnt_type, "configfs") == 0 ||
 		    strcasecmp(mnt->mnt_type, "binfmt_misc") == 0 ||
 		    strcasecmp(mnt->mnt_type, "rpc_pipefs") == 0 ||
-		    strcasecmp(mnt->mnt_type, "vboxsf") == 0) {
+		    strcasecmp(mnt->mnt_type, "vboxsf") == 0 ||
+		    strcasecmp(mnt->mnt_type, "tmpfs") == 0) {
 			LogDebug(COMPONENT_FSAL,
 				 "Ignoring %s because type %s",
 				 mnt->mnt_dir,
@@ -1014,18 +1015,55 @@ int resolve_posix_filesystem(const char *path,
 {
 	int retval = EAGAIN;
 	struct stat statbuf;
+	uint32_t retries_left = nfs_param.core_param.resolve_fs_retries;
+
+	while (retval == EAGAIN && retries_left > 0) {
+		struct timespec how_long = {
+			/* 1M micros per second */
+			.tv_sec  = nfs_param.core_param.resolve_fs_delay / 1000,
+			/* Remainder => nanoseconds */
+			.tv_nsec = (nfs_param.core_param.resolve_fs_delay %
+				    1000) * 1000000
+		};
 
-	while (retval == EAGAIN) {
 		/* Need to retry stat on path until we don't get EAGAIN in case
 		 * autofs needed to mount the file system.
 		 */
 		retval = stat(path, &statbuf);
-		if (retval != 0) {
-			retval = errno;
-			LogDebug(COMPONENT_FSAL,
-				 "stat returned %s (%d) while resolving export path %s %s",
-				 strerror(retval), retval, path,
-				 retval == EAGAIN ? "(may retry)" : "(failed)");
+
+		if (retval == 0)
+			break;
+
+		retval = errno;
+		LogDebug(COMPONENT_FSAL,
+			 "stat returned %s (%d) while resolving export path %s %s",
+			 strerror(retval), retval, path,
+			 retval == EAGAIN ? "(may retry)" : "(failed)");
+
+		retries_left--;
+
+		if (retries_left == 0) {
+			LogCrit(COMPONENT_FSAL,
+				"Stat on filesystem %s failed, no retries remaining.",
+				path);
+			break;
+		}
+
+		if (nanosleep(&how_long, NULL) != 0) {
+			/*
+			 * Let interrupts wake us up and not care. Anything else
+			 * should be fatal.
+			 */
+
+			if (errno != EINTR) {
+				retval = errno;
+				LogCrit(COMPONENT_FSAL,
+					"nanosleep failed. Asked for %"PRIu32
+					" msec. Errno %d (%s)",
+					nfs_param.core_param.resolve_fs_delay,
+					retval, strerror(retval));
+				return retval;
+			}
 		}
 	}
 
@@ -1273,6 +1311,30 @@ void unclaim_all_filesystem_maps(struct fsal_filesystem *this)
 	}
 }
 
+void get_fs_first_export_ref(struct fsal_filesystem *this,
+			     struct gsh_export **gsh_export,
+			     struct fsal_export **fsal_export)
+{
+	struct fsal_filesystem_export_map *map;
+
+	PTHREAD_RWLOCK_wrlock(&fs_lock);
+
+	map = glist_first_entry(&this->exports,
+				struct fsal_filesystem_export_map,
+				on_exports);
+
+	if (map != NULL) {
+		*fsal_export = map->exp;
+		*gsh_export = (*fsal_export)->owning_export;
+		get_gsh_export_ref(*gsh_export);
+	} else {
+		*gsh_export = NULL;
+		*fsal_export = NULL;
+	}
+
+	PTHREAD_RWLOCK_unlock(&fs_lock);
+}
+
 void unclaim_all_export_maps(struct fsal_export *exp)
 {
 	PTHREAD_RWLOCK_wrlock(&fs_lock);
