@@ -1470,6 +1470,7 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 	state_owner_t *owner;
 	struct delegrecall_context *drc_ctx;
 	struct req_op_context op_context;
+	nfs_client_id_t *client_id = NULL;
 
 	LogDebug(COMPONENT_FSAL_UP,
 		 "FSAL_UP_DELEG: obj %p type %u",
@@ -1512,9 +1513,53 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 			LogDebug(COMPONENT_FSAL_UP,
 				 "Something is going stale, no need to recall delegation");
 			gsh_free(drc_ctx);
+			*deleg_state = DELEG_GRANTED;
 			continue;
 		}
 
+		client_id = owner->so_owner.so_nfs4_owner.so_clientrec;
+		inc_client_id_ref(client_id);
+		dec_state_owner_ref(owner);
+
+		/* Being an area in which there are/could be a lot of races,
+		 * updating the logic so that multiple ASync recalls as part of
+		 * conflciting IOs on same or different objects could happen
+		 * concurrently and in parallel the reaper wiping off the
+		 * expired clients with its states.
+		 * Steps:
+		 * 1) For an expired client, proceed directly to recall off the
+		 *    conflciting state as part of delegrecall_one() below.
+		 * 2) For a non-expired client -
+		 *     a) Prevent client's lease expiring until we complete this
+		 *        recall/revoke operation, be renewing/reserving lease.
+		 *     b) If reserve lease fails, then we cleanup and set the
+		 *        deleg state back to GRANTED, so that reaper could
+		 *        clean this up proper later, else below path ignores
+		 *        the deleg state to be revoked.
+		 *            nfs_client_id_expire -> revoke_owner_delegs ->
+		 *            state_deleg_revoke -> state_deleg_revoke
+		 * 3) Once deleg state is marked properly, then further ops on
+		 *    this state would return NFS4ERR_REVOKED or NFS4ERR_EXPIRED
+		 * 4) Continue to check for other conflicting delegation holders
+		 *    holding the state lock
+		 */
+		PTHREAD_MUTEX_lock(&client_id->cid_mutex);
+		if (client_id->cid_confirmed != EXPIRED_CLIENT_ID &&
+		    !reserve_lease(client_id)) {
+			PTHREAD_MUTEX_unlock(&client_id->cid_mutex);
+			/* Let's release the ref &*/
+			dec_client_id_ref(client_id);
+
+			LogDebug(COMPONENT_FSAL_UP,
+				"Failed to reserve client's lease.");
+			gsh_free(drc_ctx);
+			/* Reset the state for reaper to clean  */
+			*deleg_state = DELEG_GRANTED;
+
+			continue;
+		}
+		PTHREAD_MUTEX_unlock(&client_id->cid_mutex);
+
 		/* Get a ref to the drc_ctx->drc_exp and initialize op_context
 		 * in case state_del_locks and others need it.
 		 */
@@ -1522,29 +1567,11 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 		init_op_context_simple(&op_context, drc_ctx->drc_exp,
 				       drc_ctx->drc_exp->fsal_export);
 
-		drc_ctx->drc_clid = owner->so_owner.so_nfs4_owner.so_clientrec;
+		drc_ctx->drc_clid = client_id;
 		COPY_STATEID(&drc_ctx->drc_stateid, state);
-		inc_client_id_ref(drc_ctx->drc_clid);
-		dec_state_owner_ref(owner);
 
 		obj->state_hdl->file.fdeleg_stats.fds_last_recall = time(NULL);
 
-		/* Prevent client's lease expiring until we complete
-		 * this recall/revoke operation. If the client's lease
-		 * has already expired, let the reaper thread handling
-		 * expired clients revoke this delegation, and we just
-		 * skip it here.
-		 */
-		PTHREAD_MUTEX_lock(&drc_ctx->drc_clid->cid_mutex);
-		if (!reserve_lease(drc_ctx->drc_clid)) {
-			PTHREAD_MUTEX_unlock(&drc_ctx->drc_clid->cid_mutex);
-			dec_client_id_ref(drc_ctx->drc_clid);
-			gsh_free(drc_ctx);
-			release_op_context();
-			continue;
-		}
-		PTHREAD_MUTEX_unlock(&drc_ctx->drc_clid->cid_mutex);
-
 		delegrecall_one(obj, state, drc_ctx);
 		release_op_context();
 	}
@@ -1937,6 +1964,12 @@ void up_ready_init(struct fsal_up_vector *up_ops)
 	PTHREAD_COND_init(&up_ops->up_cond, NULL);
 }
 
+void up_ready_destroy(struct fsal_up_vector *up_ops)
+{
+	PTHREAD_MUTEX_destroy(&up_ops->up_mutex);
+	PTHREAD_COND_destroy(&up_ops->up_cond);
+}
+
 /* This should be called when a module is ready to take upcalls */
 void up_ready_set(struct fsal_up_vector *up_ops)
 {
