@@ -38,6 +38,9 @@
 #ifdef LINUX
 #include <mcheck.h>		/* For mtrace/muntrace */
 #endif
+#ifndef __APPLE__
+#include <malloc.h>
+#endif
 
 #include "nfs_core.h"
 #include "log.h"
@@ -54,18 +57,19 @@
 #include "mdcache.h"
 #endif
 #include "conf_url.h"
+#include "nfs_rpc_callback.h"
 
 /**
  * @brief Mutex protecting shutdown flag.
  */
 
-static pthread_mutex_t admin_control_mtx = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t admin_control_mtx;
 
 /**
  * @brief Condition variable to signal change in shutdown flag.
  */
 
-static pthread_cond_t admin_control_cv = PTHREAD_COND_INITIALIZER;
+static pthread_cond_t admin_control_cv;
 
 /**
  * @brief Flag to indicate shutdown Ganesha.
@@ -480,6 +484,136 @@ static struct gsh_dbus_method method_malloc_untrace = {
 		 END_ARG_LIST}
 };
 
+/**
+ * @brief Dbus method for enabling malloc trim
+ *
+ * @param[in]  args
+ * @param[out] reply
+ */
+static bool admin_dbus_trim_enable(DBusMessageIter *args,
+				   DBusMessage *reply,
+				   DBusError *error)
+{
+	char *errormsg = "Malloc trim enabled";
+	bool success = true;
+	DBusMessageIter iter;
+
+	dbus_message_iter_init_append(reply, &iter);
+	LogEvent(COMPONENT_MEMLEAKS, "enabling malloc_trim");
+	nfs_param.core_param.malloc_trim = true;
+	gsh_dbus_status_reply(&iter, success, errormsg);
+
+	return success;
+}
+
+static struct gsh_dbus_method method_trim_enable = {
+	.name = "trim_enable",
+	.method = admin_dbus_trim_enable,
+	.args = {STATUS_REPLY,
+		 END_ARG_LIST}
+};
+
+/**
+ * @brief Dbus method for disabling malloc trim
+ *
+ * @param[in]  args
+ * @param[out] reply
+ */
+static bool admin_dbus_trim_disable(DBusMessageIter *args,
+				    DBusMessage *reply,
+				    DBusError *error)
+{
+	char *errormsg = "Malloc trim disabled";
+	bool success = true;
+	DBusMessageIter iter;
+
+	dbus_message_iter_init_append(reply, &iter);
+	LogEvent(COMPONENT_MEMLEAKS, "disabling malloc_trim");
+	nfs_param.core_param.malloc_trim = false;
+	gsh_dbus_status_reply(&iter, success, errormsg);
+
+	return success;
+}
+
+static struct gsh_dbus_method method_trim_disable = {
+	.name = "trim_disable",
+	.method = admin_dbus_trim_disable,
+	.args = {STATUS_REPLY,
+		 END_ARG_LIST}
+};
+
+/**
+ * @brief Dbus method for calling malloc_trim()
+ *
+ * @param[in]  args
+ * @param[out] reply
+ */
+static bool admin_dbus_trim_call(DBusMessageIter *args,
+				 DBusMessage *reply,
+				 DBusError *error)
+{
+	char *errormsg = "malloc_trim() called";
+	bool success = true;
+	DBusMessageIter iter;
+
+	dbus_message_iter_init_append(reply, &iter);
+	LogEvent(COMPONENT_MEMLEAKS, "Calling malloc_trim");
+	malloc_trim(0);
+	gsh_dbus_status_reply(&iter, success, errormsg);
+
+	return success;
+}
+
+static struct gsh_dbus_method method_trim_call = {
+	.name = "trim_call",
+	.method = admin_dbus_trim_call,
+	.args = {STATUS_REPLY,
+		 END_ARG_LIST}
+};
+
+/**
+ * @brief Dbus method for getting trim status
+ *
+ * @param[in]  args
+ * @param[out] reply
+ */
+static bool admin_dbus_trim_status(DBusMessageIter *args,
+				   DBusMessage *reply,
+				   DBusError *error)
+{
+	char *errormsg = "Malloc trim status: enabled";
+	bool success = true;
+	DBusMessageIter iter;
+	char hostname[64+1] = {0};
+	char name[100];
+	FILE *fp;
+
+	/* log malloc_info() as a side effect! */
+	(void)gethostname(hostname, sizeof(hostname));
+	snprintf(name, sizeof(name), "/tmp/mallinfo-%s.%d.txt",
+		 hostname, getpid());
+	fp = fopen(name, "w");
+	if (fp != NULL) {
+		malloc_info(0, fp);
+		fclose(fp);
+	}
+
+	dbus_message_iter_init_append(reply, &iter);
+	if (!nfs_param.core_param.malloc_trim)
+		errormsg = "Malloc trim status: disabled";
+	gsh_dbus_status_reply(&iter, success, errormsg);
+
+	return success;
+}
+
+static struct gsh_dbus_method method_trim_status = {
+	.name = "trim_status",
+	.method = admin_dbus_trim_status,
+	.args = {STATUS_REPLY,
+		 END_ARG_LIST}
+};
+
+
 static struct gsh_dbus_method *admin_methods[] = {
 	&method_shutdown,
 	&method_grace_period,
@@ -490,6 +624,10 @@ static struct gsh_dbus_method *admin_methods[] = {
 	&method_purge_idmapper_cache,
 	&method_malloc_trace,
 	&method_malloc_untrace,
+	&method_trim_enable,
+	&method_trim_disable,
+	&method_trim_call,
+	&method_trim_status,
 	NULL
 };
 
@@ -568,6 +706,8 @@ static struct gsh_dbus_interface *admin_interfaces[] = {
 
 void nfs_Init_admin_thread(void)
 {
+	PTHREAD_MUTEX_init(&admin_control_mtx, NULL);
+	PTHREAD_COND_init(&admin_control_cv, NULL);
 #ifdef USE_DBUS
 	gsh_dbus_register_path("admin", admin_interfaces);
 #endif				/* USE_DBUS */
@@ -588,6 +728,7 @@ void admin_halt(void)
 	}
 
 	PTHREAD_MUTEX_unlock(&admin_control_mtx);
+	LogEvent(COMPONENT_MAIN, "NFS EXIT: %s done", __func__);
 }
 
 static void do_shutdown(void)
@@ -674,6 +815,7 @@ static void do_shutdown(void)
 	remove_all_exports();
 
 	nfs4_recovery_shutdown();
+	nfs_rpc_cb_pkgshutdown();
 
 	if (disorderly) {
 		LogMajor(COMPONENT_MAIN,
@@ -691,6 +833,9 @@ static void do_shutdown(void)
 	}
 
 	unlink(nfs_pidfile_path);
+	PTHREAD_MUTEX_destroy(&admin_control_mtx);
+	PTHREAD_COND_destroy(&admin_control_cv);
+	LogEvent(COMPONENT_MAIN, "NFS EXIT: %s done", __func__);
 }
 
 void *admin_thread(void *UnusedArg)
