@@ -56,6 +56,9 @@
 #include <string.h>
 #include <signal.h>
 #include <math.h>
+#ifndef __APPLE__
+#include <malloc.h>
+#endif
 #ifdef _USE_NLM
 #include "nlm_util.h"
 #endif /* _USE_NLM */
@@ -79,6 +82,11 @@
 #include "conf_url.h"
 #include "FSAL/fsal_localfs.h"
 
+pthread_mutexattr_t default_mutex_attr;
+pthread_rwlockattr_t default_rwlock_attr;
+
+unsigned long PTHREAD_stack_size;
+
 /**
  * @brief init_complete used to indicate if ganesha is during
  * startup or not
@@ -134,6 +142,9 @@ char *nfs_config_path = GANESHA_CONFIG_PATH;
 
 char *nfs_pidfile_path = GANESHA_PIDFILE_PATH;
 
+char cid_server_owner[MAXNAMLEN+1]; /* max hostname length */
+char *cid_server_scope;
+
 /**
  * @brief Reread the configuration file to accomplish update of options.
  *
@@ -150,7 +161,7 @@ char *nfs_pidfile_path = GANESHA_PIDFILE_PATH;
 
 struct config_error_type err_type;
 
-void reread_config(void)
+bool reread_config(void)
 {
 	int status = 0;
 	config_file_t config_struct;
@@ -161,12 +172,12 @@ void reread_config(void)
 	if (nfs_config_path[0] == '\0') {
 		LogCrit(COMPONENT_CONFIG,
 			"No configuration file was specified for reloading log config.");
-		return;
+		return false;
 	}
 
 	/* Create a memstream for parser+processing error messages */
 	if (!init_error_type(&err_type))
-		return;
+		return false;
 	/* Attempt to parse the new configuration file */
 	config_struct = config_ParseFile(nfs_config_path, &err_type);
 	if (!config_error_no_error(&err_type)) {
@@ -174,22 +185,27 @@ void reread_config(void)
 		LogCrit(COMPONENT_CONFIG,
 			"Error while parsing new configuration file %s",
 			nfs_config_path);
-		report_config_errors(&err_type, NULL, config_errs_to_log);
-		return;
+		(void) report_config_errors(&err_type, NULL, config_errs_to_log);
+		return false;
 	}
 
 	/* Update the logging configuration */
 	status = read_log_config(config_struct, &err_type);
-	if (status < 0)
+	if (status < 0) {
 		LogCrit(COMPONENT_CONFIG, "Error while parsing LOG entries");
+		return false;
+	}
 
 	/* Update the export configuration */
 	status = reread_exports(config_struct, &err_type);
-	if (status < 0)
+	if (status < 0) {
 		LogCrit(COMPONENT_CONFIG, "Error while parsing EXPORT entries");
+		return false;
+	}
 
-	report_config_errors(&err_type, NULL, config_errs_to_log);
+	(void) report_config_errors(&err_type, NULL, config_errs_to_log);
 	config_Free(config_struct);
+	return true;
 }
 
 /**
@@ -239,7 +255,11 @@ static void *sigmgr_thread(void *UnusedArg)
 
 static void crash_handler(int signo, siginfo_t *info, void *ctx)
 {
+#ifdef USE_UNWIND
+	gsh_libunwind();
+#else
 	gsh_backtrace();
+#endif
 	/* re-raise the signal for the default signal handler to dump core */
 	raise(signo);
 }
@@ -284,8 +304,24 @@ static void init_crash_handlers(void)
  * @param[in] dump_trace   Dump trace when segfault
  */
 void nfs_prereq_init(const char *program_name, const char *host_name,
-		     int debug_level, const char *log_path, bool dump_trace)
+		     int debug_level, const char *log_path, bool dump_trace,
+		     unsigned long stack_size)
 {
+	PTHREAD_MUTEXATTR_init(&default_mutex_attr);
+#if defined(__linux__)
+	PTHREAD_MUTEXATTR_settype(&default_mutex_attr,
+				  PTHREAD_MUTEX_ADAPTIVE_NP);
+#endif
+
+	PTHREAD_RWLOCKATTR_init(&default_rwlock_attr);
+#if defined(__linux__)
+	PTHREAD_RWLOCKATTR_setkind_np(
+				&default_rwlock_attr,
+				PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP);
+#endif
+
+	PTHREAD_stack_size = stack_size;
+
 	healthstats.enqueued_reqs = nfs_health_.enqueued_reqs = 0;
 	healthstats.dequeued_reqs = nfs_health_.dequeued_reqs = 0;
 
@@ -305,6 +341,12 @@ void nfs_prereq_init(const char *program_name, const char *host_name,
 	}
 }
 
+void nfs_prereq_destroy(void)
+{
+	PTHREAD_MUTEXATTR_destroy(&default_mutex_attr);
+	PTHREAD_RWLOCKATTR_destroy(&default_rwlock_attr);
+}
+
 /**
  * @brief Print the nfs_parameter_structure
  */
@@ -356,6 +398,11 @@ void nfs_print_param_config(void)
 	printf("}\n\n");
 }
 
+static inline void core_pkginit(void)
+{
+	glist_init(&nfs_param.core_param.haproxy_hosts);
+}
+
 /**
  * @brief Load parameters from config file
  *
@@ -373,6 +420,7 @@ int nfs_set_param_from_conf(config_file_t parse_tree,
 	 * Initialize exports and clients so config parsing can use them
 	 * early.
 	 */
+	core_pkginit();
 	client_pkginit();
 	export_pkginit();
 	server_pkginit();
@@ -512,21 +560,15 @@ static void nfs_Start_threads(void)
 	LogDebug(COMPONENT_THREAD, "Starting threads");
 
 	/* Init for thread parameter (mostly for scheduling) */
-	if (pthread_attr_init(&attr_thr) != 0)
-		LogDebug(COMPONENT_THREAD, "can't init pthread's attributes");
-
-	if (pthread_attr_setscope(&attr_thr, PTHREAD_SCOPE_SYSTEM) != 0)
-		LogDebug(COMPONENT_THREAD, "can't set pthread's scope");
-
-	if (pthread_attr_setdetachstate(&attr_thr,
-					PTHREAD_CREATE_JOINABLE) != 0)
-		LogDebug(COMPONENT_THREAD, "can't set pthread's join state");
+	PTHREAD_ATTR_init(&attr_thr);
+	PTHREAD_ATTR_setscope(&attr_thr, PTHREAD_SCOPE_SYSTEM);
+	PTHREAD_ATTR_setdetachstate(&attr_thr, PTHREAD_CREATE_JOINABLE);
 
 	LogEvent(COMPONENT_THREAD, "Starting delayed executor.");
 	delayed_start();
 
 	/* Starting the thread dedicated to signal handling */
-	rc = pthread_create(&sigmgr_thrid, &attr_thr, sigmgr_thread, NULL);
+	rc = PTHREAD_create(&sigmgr_thrid, &attr_thr, sigmgr_thread, NULL);
 	if (rc != 0) {
 		LogFatal(COMPONENT_THREAD,
 			 "Could not create sigmgr_thread, error = %d (%s)",
@@ -544,7 +586,7 @@ static void nfs_Start_threads(void)
 		}
 
 		/* Starting the 9P/TCP dispatcher thread */
-		rc = pthread_create(&_9p_dispatcher_thrid, &attr_thr,
+		rc = PTHREAD_create(&_9p_dispatcher_thrid, &attr_thr,
 				    _9p_dispatcher_thread, NULL);
 		if (rc != 0) {
 			LogFatal(COMPONENT_THREAD,
@@ -559,7 +601,8 @@ static void nfs_Start_threads(void)
 #ifdef _USE_9P_RDMA
 	/* Starting the 9P/RDMA dispatcher thread */
 	if (nfs_param.core_param.core_options & CORE_OPTION_9P) {
-		rc = pthread_create(&_9p_rdma_dispatcher_thrid, &attr_thr,
+		/** @todo - this thread is never cancelled or cleaned up... */
+		rc = PTHREAD_create(&_9p_rdma_dispatcher_thrid, &attr_thr,
 				    _9p_rdma_dispatcher_thread, NULL);
 		if (rc != 0) {
 			LogFatal(COMPONENT_THREAD,
@@ -573,7 +616,7 @@ static void nfs_Start_threads(void)
 
 #ifdef USE_DBUS
 	/* DBUS event thread */
-	rc = pthread_create(&gsh_dbus_thrid, &attr_thr, gsh_dbus_thread, NULL);
+	rc = PTHREAD_create(&gsh_dbus_thrid, &attr_thr, gsh_dbus_thread, NULL);
 	if (rc != 0) {
 		LogFatal(COMPONENT_THREAD,
 			 "Could not create gsh_dbus_thread, error = %d (%s)",
@@ -583,7 +626,7 @@ static void nfs_Start_threads(void)
 #endif
 
 	/* Starting the admin thread */
-	rc = pthread_create(&admin_thrid, &attr_thr, admin_thread, NULL);
+	rc = PTHREAD_create(&admin_thrid, &attr_thr, admin_thread, NULL);
 	if (rc != 0) {
 		LogFatal(COMPONENT_THREAD,
 			 "Could not create admin_thread, error = %d (%s)",
@@ -609,7 +652,60 @@ static void nfs_Start_threads(void)
 	}
 	LogEvent(COMPONENT_THREAD, "General fridge was started successfully");
 
-	pthread_attr_destroy(&attr_thr);
+	PTHREAD_ATTR_destroy(&attr_thr);
+}
+
+
+/**
+ * @brief Initialise NFSv4 specific parameters.
+ *
+ * @retval 0 if successful.
+ * @retval -1 otherwise
+ *
+ */
+
+int nfsv4_init_params(void)
+{
+	int rc;
+	int owner_len, scope_len, ss_suffix_len;
+	char *cid_server_scope_suffix = "_NFS-Ganesha";
+
+	/* Set up the server owner string */
+	if (nfs_param.nfsv4_param.server_owner == NULL) {
+		/* If the server owner param is NULL, set it to hostname */
+		if (gsh_gethostname(
+			cid_server_owner, sizeof(cid_server_owner),
+			nfs_param.core_param.enable_AUTHSTATS) == -1) {
+			LogCrit(COMPONENT_NFS_V4,
+				"gsh_gethostname failed");
+			return -1;
+		}
+	} else {
+		rc = snprintf(cid_server_owner,
+				sizeof(cid_server_owner), "%s",
+				nfs_param.nfsv4_param.server_owner);
+		/* Assert that server owner conf param is not too long.
+		 * this should never happen since it is validated during
+		 * conf parsing */
+		assert(rc >= 0 && rc < sizeof(cid_server_owner));
+	}
+
+	/* use server_owner as server_scope if server_scope not
+	 * mentioned in main config file
+	 */
+	if (nfs_param.nfsv4_param.server_scope == NULL) {
+		owner_len = strlen(cid_server_owner);
+		ss_suffix_len = strlen(cid_server_scope_suffix);
+		scope_len = owner_len + ss_suffix_len;
+		cid_server_scope = gsh_malloc(scope_len + 1);
+		memcpy(cid_server_scope, cid_server_owner, owner_len);
+		memcpy(cid_server_scope + owner_len,
+				cid_server_scope_suffix,
+				ss_suffix_len + 1);
+	} else {
+		cid_server_scope = nfs_param.nfsv4_param.server_scope;
+	}
+	return 0;
 }
 
 /**
@@ -777,12 +873,20 @@ static void nfs_Init(const nfs_start_info_t *p_start_info)
 	LogInfo(COMPONENT_INIT,
 		"NFSv4 Session Id cache successfully initialized");
 
+	if (nfs_param.core_param.core_options & CORE_OPTION_NFSV4) {
+		/* Initialise NFSv4 specific parameters */
+		if (nfsv4_init_params() != 0) {
+			LogFatal(COMPONENT_INIT,
+			  "Error while initializing NFSv4 specific parameter");
+		}
+		LogInfo(COMPONENT_INIT,
+			"NFSv4 specific parameter initialized");
+	}
 #ifdef _USE_9P
 	LogDebug(COMPONENT_INIT, "Now building 9P resources");
 	if (_9p_init()) {
-		LogCrit(COMPONENT_INIT,
-			"Error while initializing 9P Resources");
-		exit(1);
+		LogFatal(COMPONENT_INIT,
+			 "Error while initializing 9P Resources");
 	}
 	LogInfo(COMPONENT_INIT, "9P resources successfully initialized");
 #endif				/* _USE_9P */
@@ -884,6 +988,18 @@ static void lower_my_caps(void)
 }
 #endif
 
+#if defined(M_TRIM_THRESHOLD)
+#define THIRTY_MIN 1800000000000UL
+static void do_malloc_trim(void *param)
+{
+	LogDebug(COMPONENT_MAIN, malloc_trim(0) ?
+		 "malloc_trim() released some memory" :
+		 "malloc_trim() was not able to release memory");
+	(void) delayed_submit(do_malloc_trim, 0, THIRTY_MIN);
+}
+#endif
+
+
 /**
  * @brief Start NFS service
  *
@@ -926,6 +1042,10 @@ void nfs_start(nfs_start_info_t *p_start_info)
 	nfs_Init(p_start_info);
 	nfs_Start_threads(); /* Spawns service threads */
 
+#if defined(M_TRIM_THRESHOLD)
+	(void) delayed_submit(do_malloc_trim, 0, THIRTY_MIN);
+#endif
+
 	nfs_init_complete();
 
 #ifdef _USE_NLM
@@ -951,6 +1071,8 @@ void nfs_start(nfs_start_info_t *p_start_info)
 	/* Regular exit */
 	LogEvent(COMPONENT_MAIN, "NFS EXIT: regular exit");
 
+	nfs_init_cleanup();
+
 	Cleanup();
 	/* let main return 0 to exit */
 }
@@ -962,6 +1084,12 @@ void nfs_init_init(void)
 	nfs_init.init_complete = false;
 }
 
+void nfs_init_cleanup(void)
+{
+	PTHREAD_MUTEX_destroy(&nfs_init.init_mutex);
+	PTHREAD_COND_destroy(&nfs_init.init_cond);
+}
+
 void nfs_init_complete(void)
 {
 	PTHREAD_MUTEX_lock(&nfs_init.init_mutex);
