@@ -86,6 +86,16 @@ nfs_start_info_t my_nfs_start_info = {
 config_file_t nfs_config_struct;
 char *nfs_host_name = "localhost";
 
+/* Cleanup on shutdown */
+void export_cleanup(void)
+{
+	PTHREAD_RWLOCK_destroy(&export_opt_lock);
+}
+
+struct cleanup_list_element export_cleanup_element = {
+	.clean = export_cleanup,
+};
+
 /**
  * nfs_libmain: library initializer
  *
@@ -129,7 +139,8 @@ int nfs_libmain(const char *ganesha_conf,
 	}
 
 	/* initialize memory and logging */
-	nfs_prereq_init(exec_name, nfs_host_name, debug_level, log_path, false);
+	nfs_prereq_init(exec_name, nfs_host_name, debug_level, log_path,
+			false, 8192 * 1024);
 #if GANESHA_BUILD_RELEASE
 	LogEvent(COMPONENT_MAIN, "%s Starting: Ganesha Version %s",
 		 exec_name, GANESHA_VERSION);
@@ -204,7 +215,11 @@ int nfs_libmain(const char *ganesha_conf,
 	/* We need all the fsal modules loaded so we can have
 	 * the list available at exports parsing time.
 	 */
-	start_fsals();
+	if (start_fsals(nfs_config_struct, &err_type) < 0) {
+		LogCrit(COMPONENT_INIT,
+			 "Error starting FSALs.");
+		goto fatal_die;
+	}
 
 	/* parse configuration file */
 
@@ -250,6 +265,9 @@ int nfs_libmain(const char *ganesha_conf,
 	/* Wait for enforcement to begin */
 	nfs_wait_for_grace_enforcement();
 
+	PTHREAD_RWLOCK_init(&export_opt_lock, NULL);
+	RegisterCleanup(&export_cleanup_element);
+
 	/* Load export entries from parsed file
 	 * returns the number of export entries.
 	 */
@@ -262,7 +280,7 @@ int nfs_libmain(const char *ganesha_conf,
 	if (rc == 0 && dsc == 0)
 		LogWarn(COMPONENT_INIT,
 			"No export entries found in configuration file !!!");
-	report_config_errors(&err_type, NULL, config_errs_to_log);
+	(void) report_config_errors(&err_type, NULL, config_errs_to_log);
 
 	/* freeing syntax tree : */
 
@@ -271,10 +289,12 @@ int nfs_libmain(const char *ganesha_conf,
 	/* Everything seems to be OK! We can now start service threads */
 	nfs_start(&my_nfs_start_info);
 
+	nfs_prereq_destroy();
+
 	return 0;
 
 fatal_die:
-	report_config_errors(&err_type, NULL, config_errs_to_log);
+	(void) report_config_errors(&err_type, NULL, config_errs_to_log);
 	LogFatal(COMPONENT_INIT,
 		 "Fatal errors.  Server exiting...");
 	/* NOT REACHED */
