@@ -77,6 +77,10 @@
 #include "gsh_lttng/fsal_ceph.h"
 #endif /* USE_LTTNG */
 
+#ifdef USE_MONITORING
+#include "monitoring.h"
+#endif  /* USE_MONITORING */
+
 /* parameters for NFSd startup and default values */
 
 static nfs_start_info_t my_nfs_start_info = {
@@ -87,10 +91,11 @@ static nfs_start_info_t my_nfs_start_info = {
 
 config_file_t nfs_config_struct;
 char *nfs_host_name = "localhost";
+bool config_errors_fatal;
 
 /* command line syntax */
 
-static const char options[] = "v@L:N:f:p:FRTE:ChI:";
+static const char options[] = "v@L:N:S:f:p:FRTE:ChI:x";
 static const char usage[] =
 	"Usage: %s [-hd][-L <logfile>][-N <dbg_lvl>][-f <config_file>]\n"
 	"\t[-v]                display version information\n"
@@ -100,10 +105,12 @@ static const char usage[] =
 	"\t[-p <pid_file>]     set the pid file\n"
 	"\t[-F]                the program stays in foreground\n"
 	"\t[-R]                daemon will manage RPCSEC_GSS (default is no RPCSEC_GSS)\n"
+	"\t[-S <size>]         set the default thread stack size (in K) to be used\n"
 	"\t[-T]                dump the default configuration on stdout\n"
 	"\t[-E <epoch>]        overrides ServerBootTime for ServerEpoch\n"
 	"\t[-I <nodeid>]       cluster nodeid\n"
 	"\t[-C]                dump trace when segfault\n"
+	"\t[-x]                fatal exit if there are config errors on startup\n"
 	"\t[-h]                display this help\n"
 	"----------------- Signals ----------------\n"
 	"SIGHUP     : Reload LOG and EXPORT config\n"
@@ -125,6 +132,17 @@ static inline char *main_strdup(const char *var, const char *str)
 	return s;
 }
 
+static int valid_stack_size(unsigned long stack_size)
+{
+	static unsigned long valid_sizes[] = {
+		16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192 };
+	for (unsigned int i = 0;
+	     i < sizeof(valid_sizes) / sizeof(valid_sizes[0]); i++)
+		if (valid_sizes[i] == stack_size)
+			return 1;
+	return 0;
+}
+
 /**
  * main: simply the main function.
  *
@@ -145,6 +163,7 @@ int main(int argc, char *argv[])
 	int dsc;
 	int rc;
 	int pidfile = -1;               /* fd for file to store pid */
+	unsigned long stack_size = 8388608; /* 8M, glibc's default */
 	char *log_path = NULL;
 	char *exec_name = "nfs-ganesha";
 	int debug_level = -1;
@@ -209,6 +228,17 @@ int main(int argc, char *argv[])
 			}
 			break;
 
+		case 'S':
+			/* default thread stack size */
+			stack_size = strtoul(optarg, NULL, 10);
+			if (!valid_stack_size(stack_size)) {
+				fprintf(stderr,
+					"Invalid value for option 'S': valid choices are 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192\n");
+				exit(1);
+			}
+			stack_size *= 1024;
+			break;
+
 		case 'f':
 			/* config file */
 
@@ -256,6 +286,10 @@ int main(int argc, char *argv[])
 			g_nodeid = atoi(optarg);
 			break;
 
+		case 'x':
+			config_errors_fatal = true;
+			break;
+
 		case 'h':
 			fprintf(stderr, usage, exec_name);
 			exit(0);
@@ -268,7 +302,7 @@ int main(int argc, char *argv[])
 
 	/* initialize memory and logging */
 	nfs_prereq_init(exec_name, nfs_host_name, debug_level, log_path,
-			dump_trace);
+			dump_trace, stack_size);
 #if GANESHA_BUILD_RELEASE
 	LogEvent(COMPONENT_MAIN, "%s Starting: Ganesha Version %s",
 		 exec_name, GANESHA_VERSION);
@@ -474,7 +508,11 @@ int main(int argc, char *argv[])
 	/* We need all the fsal modules loaded so we can have
 	 * the list available at exports parsing time.
 	 */
-	start_fsals();
+	if (start_fsals(nfs_config_struct, &err_type) < 0) {
+		LogCrit(COMPONENT_INIT,
+			 "Error starting FSALs.");
+		goto fatal_die;
+	}
 
 	/* parse configuration file */
 
@@ -486,6 +524,10 @@ int main(int argc, char *argv[])
 		goto fatal_die;
 	}
 
+#ifdef USE_MONITORING
+	monitoring_init(nfs_param.core_param.monitoring_port);
+#endif  /* USE_MONITORING */
+
 	/* initialize core subsystems and data structures */
 	if (init_server_pkgs() != 0) {
 		LogCrit(COMPONENT_INIT,
@@ -530,7 +572,13 @@ int main(int argc, char *argv[])
 	if (rc == 0 && dsc == 0)
 		LogWarn(COMPONENT_INIT,
 			"No export entries found in configuration file !!!");
-	report_config_errors(&err_type, NULL, config_errs_to_log);
+
+	find_unused_blocks(nfs_config_struct, &err_type);
+
+	rc = report_config_errors(&err_type, NULL, config_errs_to_log);
+
+	if (config_errors_fatal && rc > 0)
+		goto fatal_die;
 
 	/* freeing syntax tree : */
 
@@ -547,10 +595,12 @@ int main(int argc, char *argv[])
 	if (pidfile != -1)
 		close(pidfile);
 
+	nfs_prereq_destroy();
+
 	return 0;
 
 fatal_die:
-	report_config_errors(&err_type, NULL, config_errs_to_log);
+	(void) report_config_errors(&err_type, NULL, config_errs_to_log);
 
 	if (tempo_exec_name)
 		free(exec_name);
