@@ -73,6 +73,10 @@
 #define TCP_KEEPIDLE TCPCTL_KEEPIDLE
 #endif
 
+#ifdef USE_MONITORING
+#include "monitoring.h"
+#endif /* USE_MONITORING */
+
 #define NFS_options nfs_param.core_param.core_options
 #define NFS_program nfs_param.core_param.program
 
@@ -1355,6 +1359,9 @@ void nfs_Init_svc(void)
 		nfs_param.core_param.rpc.gss.max_ctx;
 	svc_params.gss_max_gc =
 		nfs_param.core_param.rpc.gss.max_gc;
+#ifdef SVC_PARAM_HAS_THR_STACK_SIZE
+	svc_params.thr_stack_size = PTHREAD_stack_size;
+#endif
 
 	/* Only after TI-RPC allocators, log channel are setup */
 	if (!svc_init(&svc_params))
@@ -1544,6 +1551,12 @@ static struct svc_req *alloc_nfs_request(SVCXPRT *xprt, XDR *xdrs)
 
 	(void) atomic_inc_uint64_t(&nfs_health_.enqueued_reqs);
 
+#ifdef USE_MONITORING
+	monitoring_rpc_received();
+	monitoring_rpcs_in_flight(
+		nfs_health_.enqueued_reqs - nfs_health_.dequeued_reqs);
+#endif /* USE_MONITORING*/
+
 	/* set up req */
 	SVC_REF(xprt, SVC_REF_FLAG_NONE);
 	reqdata->svc.rq_xprt = xprt;
@@ -1592,4 +1605,8 @@ static void free_nfs_request(struct svc_req *req, enum xprt_stat stat)
 	SVC_RELEASE(xprt, SVC_REF_FLAG_NONE);
 
 	(void) atomic_inc_uint64_t(&nfs_health_.dequeued_reqs);
+
+#ifdef USE_MONITORING
+	monitoring_rpc_completed();
+#endif /* USE_MONITORING*/
 }
