@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -59,7 +60,7 @@ int _9p_mkdir(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	struct fsal_obj_handle *pentry_newdir = NULL;
 	char dir_name[MAXNAMLEN+1];
 	fsal_status_t fsal_status;
-	struct attrlist sattr;
+	struct fsal_attrlist sattr;
 
 	/* Get data */
 	_9p_getptr(cursor, msgtag, u16);
@@ -86,8 +87,7 @@ int _9p_mkdir(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 
 	_9p_init_opctx(pfid, req9p);
 
-	if ((op_ctx->export_perms->options &
-				 EXPORT_OPTION_WRITE_ACCESS) == 0)
+	if ((op_ctx->export_perms.options & EXPORT_OPTION_WRITE_ACCESS) == 0)
 		return _9p_rerror(req9p, msgtag, EROFS, plenout, preply);
 
 	if (*name_len >= sizeof(dir_name)) {
