@@ -54,10 +54,10 @@ static bool proc_export(struct gsh_export *export, void *arg)
 	struct proc_state *state = arg;
 	struct exportnode *new_expnode;
 	struct glist_head *glist_item;
-	exportlist_client_entry_t *client;
 	struct groupnode *group, *grp_tail = NULL;
 	char *grp_name;
 	bool free_grp_name;
+	struct glist_head *clients = &export->clients;
 
 	state->retval = 0;
 
@@ -86,12 +86,18 @@ static bool proc_export(struct gsh_export *export, void *arg)
 
 	new_expnode = gsh_calloc(1, sizeof(struct exportnode));
 
-	PTHREAD_RWLOCK_rdlock(&op_ctx->ctx_export->lock);
+	PTHREAD_RWLOCK_rdlock(&op_ctx->ctx_export->exp_lock);
+	PTHREAD_RWLOCK_rdlock(&export_opt_lock);
+
+	if (glist_empty(clients))
+		clients = &export_opt.clients;
+
+	glist_for_each(glist_item, clients) {
+		struct base_client_entry *client =
+					glist_entry(glist_item,
+						    struct base_client_entry,
+						    cle_list);
 
-	glist_for_each(glist_item, &export->clients) {
-		client =
-		    glist_entry(glist_item, exportlist_client_entry_t,
-				cle_list);
 		group = gsh_calloc(1, sizeof(struct groupnode));
 
 		if (grp_tail == NULL)
@@ -135,7 +141,8 @@ static bool proc_export(struct gsh_export *export, void *arg)
 			gsh_free(grp_name);
 	}
 
-	PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->lock);
+	PTHREAD_RWLOCK_unlock(&export_opt_lock);
+	PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->exp_lock);
 
 	/* Now that we are almost done, get a gsh_refstr to the path for ex_dir.
 	 * The op context has a reference but we don't want to play games with
