@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -56,7 +57,7 @@
  * @param[in]  req     SVC request related to this call
  * @param[out] res     Structure to contain the result of the call
  *
- * @retval NFS_REQ_OK if successfull
+ * @retval NFS_REQ_OK if successful
  * @retval NFS_REQ_DROP if failed but retryable
  * @retval NFS_REQ_FAILED if failed and not retryable
  *
@@ -70,7 +71,7 @@ int nfs3_create(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	pre_op_attr pre_parent = {
 		.attributes_follow = false
 	};
-	struct attrlist sattr, attrs;
+	struct fsal_attrlist sattr, attrs;
 	fsal_status_t fsal_status = {0, 0};
 	int rc = NFS_REQ_OK;
 	fsal_verifier_t verifier;
@@ -84,7 +85,7 @@ int nfs3_create(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	 */
 	fsal_prepare_attrs(&attrs, ATTRS_NFS3 | ATTR_RDATTR_ERR);
 
-	memset(&sattr, 0, sizeof(struct attrlist));
+	memset(&sattr, 0, sizeof(struct fsal_attrlist));
 
 	/* to avoid setting it on each error case */
 	res->res_create3.CREATE3res_u.resfail.dir_wcc.before.attributes_follow =
@@ -114,10 +115,10 @@ int nfs3_create(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	/* if quota support is active, then we should check is the
 	   FSAL allows inode creation or not */
-	fsal_status =
-	    op_ctx->fsal_export->exp_ops.check_quota(op_ctx->fsal_export,
-						   op_ctx->ctx_export->fullpath,
-						   FSAL_QUOTA_INODES);
+	fsal_status = op_ctx->fsal_export->exp_ops.check_quota(
+							op_ctx->fsal_export,
+							CTX_FULLPATH(op_ctx),
+							FSAL_QUOTA_INODES);
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		res->res_create3.status = NFS3ERR_DQUOT;
@@ -234,7 +235,7 @@ int nfs3_create(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 /**
  * @brief Free the result structure allocated for nfs3_create.
  *
- * Thsi function frees the result structure allocated for nfs3_create.
+ * This function frees the result structure allocated for nfs3_create.
  *
  * @param[in,out] res Result structure
  *
