@@ -102,15 +102,48 @@ static int nfs3_complete_write(struct nfs3_write_data *data)
 	return data->rc;
 }
 
+static void nfs3_write_cb(struct fsal_obj_handle *obj, fsal_status_t ret,
+			  void *write_data, void *caller_data);
+
 static enum xprt_stat nfs3_write_resume(struct svc_req *req)
 {
 	nfs_request_t *reqdata = container_of(req, nfs_request_t, svc);
 	struct nfs3_write_data *data = reqdata->proc_data;
 	int rc;
+	uint32_t flags;
 
 	/* Restore the op_ctx */
 	resume_op_context(&reqdata->op_context);
 
+	if (data->write_arg.fsal_resume) {
+		/* FSAL is requesting another write2 call on resume */
+		atomic_postclear_uint32_t_bits(&data->flags,
+					       ASYNC_PROC_EXIT |
+					       ASYNC_PROC_DONE);
+
+		data->obj->obj_ops->write2(data->obj, true, nfs3_write_cb,
+					   &data->write_arg, data);
+
+		/* Only atomically set the flags if we actually call write2,
+		 * otherwise we will have indicated as having been DONE.
+		 */
+		flags =
+		    atomic_postset_uint32_t_bits(&data->flags, ASYNC_PROC_EXIT);
+
+		if ((flags & ASYNC_PROC_DONE) != ASYNC_PROC_DONE) {
+			/* The write was not finished before we got here. When
+			 * the write completes, nfs3_write_cb() will have to
+			 * reschedule the request for completion. The resume
+			 * will be resolved by nfs3_write_resume() which will
+			 * free write_data and return the appropriate return
+			 * result. We will NOT go async again for the write op
+			 * (but could for a subsequent op in the compound).
+			 */
+			suspend_op_context();
+			return XPRT_SUSPEND;
+		}
+	}
+
 	/* Complete the write */
 	rc = nfs3_complete_write(data);
 
@@ -240,8 +273,7 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		res->res_write3.status = nfs3_Errno_status(fsal_status);
-		rc = NFS_REQ_OK;
-		goto putref;
+		goto return_ok;
 	}
 
 	/* Sanity check: write only a regular file */
@@ -251,8 +283,7 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		else
 			res->res_write3.status = NFS3ERR_INVAL;
 
-		rc = NFS_REQ_OK;
-		goto putref;
+		goto return_ok;
 	}
 
 	/* if quota support is active, then we should check is the
@@ -264,15 +295,13 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		res->res_write3.status = NFS3ERR_DQUOT;
-		rc = NFS_REQ_OK;
-		goto putref;
+		goto return_ok;
 	}
 
 	if (size > arg->arg_write3.data.data_len) {
 		/* should never happen */
 		res->res_write3.status = NFS3ERR_INVAL;
-		rc = NFS_REQ_OK;
-		goto putref;
+		goto return_ok;
 	}
 
 	/* Do not exceed maximum WRITE offset if set */
@@ -293,8 +322,7 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 			nfs_SetWccData(NULL, obj, &resfail->file_wcc);
 
-			rc = NFS_REQ_OK;
-			goto putref;
+			goto return_ok;
 		}
 	}
 
@@ -308,14 +336,13 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		fsal_status = fsalstat(ERR_FSAL_NO_ERROR, 0);
 		res->res_write3.status = NFS3_OK;
 		nfs_SetWccData(NULL, obj, &resfail->file_wcc);
-		rc = NFS_REQ_OK;
 		if ((arg->arg_write3.stable == DATA_SYNC) ||
 		    (arg->arg_write3.stable == FILE_SYNC))
 			resok->committed = FILE_SYNC;
 		else
 			resok->committed = UNSTABLE;
 		memcpy(resok->verf, NFS3_write_verifier, sizeof(writeverf3));
-		goto putref;
+		goto return_ok;
 	}
 
 	/* An actual write is to be made, prepare it */
@@ -323,8 +350,7 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	/* Check for delegation conflict. */
 	if (state_deleg_conflict(obj, true)) {
 		res->res_write3.status = NFS3ERR_JUKEBOX;
-		rc = NFS_REQ_OK;
-		goto putref;
+		goto return_ok;
 	}
 
 	/* Set up args, allocate from heap, iov_count will be 1 */
@@ -348,6 +374,8 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	reqdata->proc_data = write_data;
 
+again:
+
 	obj->obj_ops->write2(obj, true, nfs3_write_cb, write_arg, write_data);
 
 	/* Only atomically set the flags if we actually call write2, otherwise
@@ -361,12 +389,22 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		 * write completes, nfs3_write_cb() will have to reschedule the
 		 * request for completion. The resume will be resolved by
 		 * nfs3_write_resume() which will free write_data and return
-		 * the appropriate return result. We will NOT go async again for
-		 * the write op (but could for a subsequent op in the compound).
+		 * the appropriate return result.
 		 */
 		return NFS_REQ_ASYNC_WAIT;
 	}
 
+	if (write_arg->fsal_resume) {
+		/* FSAL is requesting another write2 call */
+		atomic_postclear_uint32_t_bits(&write_data->flags,
+					       ASYNC_PROC_EXIT |
+					       ASYNC_PROC_DONE);
+		/* Make the call with the same params, though the FSAL will be
+		 * signaled by fsal_resume being set.
+		 */
+		goto again;
+	}
+
 	/* Complete the write */
 	rc = nfs3_complete_write(write_data);
 
@@ -376,13 +414,14 @@ int nfs3_write(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	return rc;
 
- putref:
+ return_ok:
+
 	/* return references */
 	obj->obj_ops->put_ref(obj);
 
-	server_stats_io_done(size, 0, (rc == NFS_REQ_OK ? true : false), true);
+	server_stats_io_done(size, 0, true, true);
 
-	return rc;
+	return NFS_REQ_OK;
 }				/* nfs3_write */
 
 /**
