@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
  /*
   * vim:noexpandtab:shiftwidth=8:tabstop=8:
   *
@@ -61,17 +62,16 @@
  * @return per RFC5661, p. 363
  */
 
-int nfs4_op_create(struct nfs_argop4 *op, compound_data_t *data,
-		   struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_create(struct nfs_argop4 *op, compound_data_t *data,
+				   struct nfs_resop4 *resp)
 {
 	CREATE4args * const arg_CREATE4 = &op->nfs_argop4_u.opcreate;
 	CREATE4res * const res_CREATE4 = &resp->nfs_resop4_u.opcreate;
 
 	struct fsal_obj_handle *obj_parent = NULL;
 	struct fsal_obj_handle *obj_new = NULL;
-	struct attrlist sattr;
+	struct fsal_attrlist sattr;
 	int convrc = 0;
-	char *name = NULL;
 	char *link_content = NULL;
 	struct fsal_export *exp_hdl;
 	fsal_status_t fsal_status;
@@ -92,8 +92,8 @@ int nfs4_op_create(struct nfs_argop4 *op, compound_data_t *data,
 	exp_hdl = op_ctx->fsal_export;
 
 	fsal_status = exp_hdl->exp_ops.check_quota(exp_hdl,
-						op_ctx->ctx_export->fullpath,
-						FSAL_QUOTA_INODES);
+						   CTX_FULLPATH(op_ctx),
+						   FSAL_QUOTA_INODES);
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		res_CREATE4->status = NFS4ERR_DQUOT;
@@ -126,15 +126,14 @@ int nfs4_op_create(struct nfs_argop4 *op, compound_data_t *data,
 	 */
 
 	/* Validate and convert the UFT8 objname to a regular string */
-	res_CREATE4->status = nfs4_utf8string2dynamic(&arg_CREATE4->objname,
-						      UTF8_SCAN_ALL,
-						      &name);
+	res_CREATE4->status = nfs4_utf8string_scan(&arg_CREATE4->objname,
+						   UTF8_SCAN_PATH_COMP);
 
 	if (res_CREATE4->status != NFS4_OK)
 		goto out;
 
 	/* Convert current FH into a obj, the current_obj
-	   (assocated with the current FH will be used for this */
+	   (associated with the current FH will be used for this */
 	obj_parent = data->current_obj;
 
 	/* The currentFH must point to a directory
@@ -166,15 +165,19 @@ int nfs4_op_create(struct nfs_argop4 *op, compound_data_t *data,
 	/* Create either a symbolic link or a directory */
 	switch (arg_CREATE4->objtype.type) {
 	case NF4LNK:
-		/* Convert the name to link from into a regular string */
+		/* Validate the symbolic link length, specifically do NOT
+		 * validate the content (per RFC 7530 Section 12.4)
+		 */
 		type = SYMBOLIC_LINK;
-		res_CREATE4->status = nfs4_utf8string2dynamic(
+		res_CREATE4->status = nfs4_utf8string_scan(
 				&arg_CREATE4->objtype.createtype4_u.linkdata,
-				UTF8_SCAN_SYMLINK,
-				&link_content);
+				UTF8_SCAN_PATH);
 
 		if (res_CREATE4->status != NFS4_OK)
 			goto out;
+
+		link_content =
+		    arg_CREATE4->objtype.createtype4_u.linkdata.utf8string_val;
 		break;
 
 	case NF4DIR:
@@ -229,8 +232,13 @@ int nfs4_op_create(struct nfs_argop4 *op, compound_data_t *data,
 		sattr.valid_mask |= ATTR_MODE;
 	}
 
-	fsal_status = fsal_create(obj_parent, name, type, &sattr, link_content,
-				  &obj_new, NULL);
+	fsal_status = fsal_create(obj_parent,
+				  arg_CREATE4->objname.utf8string_val,
+				  type,
+				  &sattr,
+				  link_content,
+				  &obj_new,
+				  NULL);
 
 	/* Release the attributes (may release an inherited ACL) */
 	fsal_release_attrs(&sattr);
@@ -279,12 +287,12 @@ int nfs4_op_create(struct nfs_argop4 *op, compound_data_t *data,
 		     res_CREATE4->CREATE4res_u.resok4.cinfo.after,
 		     res_CREATE4->CREATE4res_u.resok4.cinfo.atomic);
 
-	/* @todo : BUGAZOMEU: fair ele free dans cette fonction */
+	/* @todo : BUGAZOMEU: faire le free dans cette fonction */
 
 	/* Keep the vnode entry for the file in the compound data */
 	set_current_entry(data, obj_new);
 
-	/* If you reach this point, then no error occured */
+	/* If you reach this point, then no error occurred */
 	res_CREATE4->status = NFS4_OK;
 
  out:
@@ -294,10 +302,7 @@ int nfs4_op_create(struct nfs_argop4 *op, compound_data_t *data,
 		obj_new->obj_ops->put_ref(obj_new);
 	}
 
-	gsh_free(name);
-	gsh_free(link_content);
-
-	return res_CREATE4->status;
+	return nfsstat4_to_nfs_req_result(res_CREATE4->status);
 }				/* nfs4_op_create */
 
 /**
