@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -70,7 +71,7 @@
  * @param[in]     layout_type      Type of layout being requested
  * @param[out]    layout_state     The layout state
  *
- * @return NFS4_OK if successfull, other values on error
+ * @return NFS4_OK if successful, other values on error
  */
 
 static nfsstat4 acquire_layout_state(compound_data_t *data,
@@ -131,8 +132,7 @@ static nfsstat4 acquire_layout_state(compound_data_t *data,
 
 		memset(&layout_data, 0, sizeof(layout_data));
 
-		PTHREAD_RWLOCK_wrlock(
-			&data->current_obj->state_hdl->state_lock);
+		STATELOCK_lock(data->current_obj);
 		lock_held = true;
 
 		/* See if a layout state already exists */
@@ -204,7 +204,7 @@ static nfsstat4 acquire_layout_state(compound_data_t *data,
 
 		glist_init(&(*layout_state)->state_data.layout.state_segments);
 	} else {
-		/* A state eixsts but is of an invalid type. */
+		/* A state exists but is of an invalid type. */
 		nfs_status = NFS4ERR_BAD_STATEID;
 		goto out;
 	}
@@ -215,8 +215,7 @@ static nfsstat4 acquire_layout_state(compound_data_t *data,
 	dec_state_t_ref(supplied_state);
 
 	if (lock_held)
-		PTHREAD_RWLOCK_unlock(
-			&data->current_obj->state_hdl->state_lock);
+		STATELOCK_unlock(data->current_obj);
 
 	return nfs_status;
 }
@@ -251,9 +250,9 @@ void free_layouts(layout4 *layouts, uint32_t numlayouts)
  * @param[in]     obj     File handle
  * @param[in]     arg     Input arguments to the FSAL
  * @param[in,out] res     Input/output and output arguments to the FSAL
- * @param[out]    current Current entry in the logr_layout array.
+ * @param[out]    current The current entry in the logr_layout array.
  *
- * @return NFS4_OK if successfull, other values show an error.
+ * @return NFS4_OK if successful, other values show an error.
  */
 
 static nfsstat4 one_segment(struct fsal_obj_handle *obj,
@@ -295,7 +294,7 @@ static nfsstat4 one_segment(struct fsal_obj_handle *obj,
 
 	++layout_state->state_data.layout.granting;
 
-	nfs_status = obj->obj_ops->layoutget(obj, op_ctx, &loc_body, arg, res);
+	nfs_status = obj->obj_ops->layoutget(obj, &loc_body, arg, res);
 
 	--layout_state->state_data.layout.granting;
 
@@ -325,7 +324,7 @@ static nfsstat4 one_segment(struct fsal_obj_handle *obj,
 	 * @todo This is where you would want to record layoutget
 	 * operation.  You can get the details of every segment added
 	 * here, including the segment description in
-	 * res->fsal_seg_data and clientid in *req_ctx->clientid.
+	 * res->fsal_seg_data and clientid in *op_ctx->clientid.
 	 */
 
  out:
