@@ -123,9 +123,10 @@ static nfsstat4 open4_validate_claim(compound_data_t *data,
 	nfsstat4 status = NFS4_OK;
 	/* does this claim require a grace period? */
 	bool want_grace = false;
-	/* do we need a reference on the current state of grace? */
-	bool take_ref = !op_ctx->fsal_export->exp_ops.fs_supports(
+	bool fsal_grace_support = op_ctx->fsal_export->exp_ops.fs_supports(
 					op_ctx->fsal_export, fso_grace_method);
+	/* do we need a reference on the current state of grace? */
+	bool take_ref = !fsal_grace_support;
 
 	switch (claim) {
 	case CLAIM_NULL:
@@ -145,7 +146,7 @@ static nfsstat4 open4_validate_claim(compound_data_t *data,
 
 	case CLAIM_PREVIOUS:
 		want_grace = true;
-		if (!clientid->cid_allow_reclaim ||
+		if ((!clientid->cid_allow_reclaim && !fsal_grace_support) ||
 		    ((data->minorversion > 0) &&
 		    clientid->cid_cb.v41.cid_reclaim_complete))
 			status = NFS4ERR_NO_GRACE;
@@ -1178,6 +1179,12 @@ static void open4_ex(OPEN4args *arg,
 		st_lock_held = true;
 	}
 
+	/* At this point open has succeeded and we are holding the state
+	 * lock.
+	 */
+	if (arg->share_access & OPEN4_SHARE_ACCESS_WRITE)
+		file_obj->state_hdl->file.fdeleg_stats.fds_num_write_opens++;
+
 	do_delegation(arg, res_OPEN4, data, owner, *file_state, clientid);
  out:
 
@@ -1191,8 +1198,7 @@ static void open4_ex(OPEN4args *arg,
 	if (res_OPEN4->status != NFS4_OK) {
 		/* Cleanup state on error */
 		if (*new_state)
-			op_ctx->fsal_export->exp_ops.free_state(
-					op_ctx->fsal_export, *file_state);
+			free_state(*file_state);
 		else if (*file_state != NULL)
 			dec_state_t_ref(*file_state);
 		*file_state = NULL;
@@ -1326,17 +1332,13 @@ enum nfs_req_result nfs4_op_open(struct nfs_argop4 *op, compound_data_t *data,
 	}
 
 	/* Check if lease is expired and reserve it */
-	PTHREAD_MUTEX_lock(&clientid->cid_mutex);
-
-	if (data->minorversion == 0 && !reserve_lease(clientid)) {
-		PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
+	if (data->minorversion == 0 &&
+	    !reserve_lease_or_expire(clientid, false)) {
 		res_OPEN4->status = NFS4ERR_EXPIRED;
 		LogDebug(COMPONENT_NFS_V4, "Lease expired");
 		goto out3;
 	}
 
-	PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
-
 	/* Get the open owner */
 	if (!open4_open_owner(op, data, resp, clientid, &owner)) {
 		LogDebug(COMPONENT_NFS_V4, "open4_open_owner failed");
