@@ -173,51 +173,6 @@ static void nfs4_complete_read_plus(struct nfs_resop4 *resp,
 	}
 }
 
-enum nfs_req_result nfs4_op_read_resume(struct nfs_argop4 *op,
-					compound_data_t *data,
-					struct nfs_resop4 *resp)
-{
-	enum nfs_req_result rc = nfs4_complete_read(data->op_data);
-
-	if (rc != NFS_REQ_ASYNC_WAIT) {
-		/* We are completely done with the request. This test wasn't
-		 * strictly necessary since nfs4_complete_read doesn't async but
-		 * at some future time, the getattr it does might go async so we
-		 * might as well be prepared here. Our caller is already
-		 * prepared for such a scenario.
-		 */
-		gsh_free(data->op_data);
-		data->op_data = NULL;
-	}
-
-	return rc;
-}
-
-enum nfs_req_result nfs4_op_read_plus_resume(struct nfs_argop4 *op,
-					     compound_data_t *data,
-					     struct nfs_resop4 *resp)
-{
-	struct nfs4_read_data *read_data = data->op_data;
-	enum nfs_req_result rc = nfs4_complete_read(read_data);
-
-	if (rc == NFS_REQ_OK) {
-		nfs4_complete_read_plus(resp, &read_data->info);
-	}
-
-	if (rc != NFS_REQ_ASYNC_WAIT) {
-		/* We are completely done with the request. This test wasn't
-		 * strictly necessary since nfs4_complete_read doesn't async but
-		 * at some future time, the getattr it does might go async so we
-		 * might as well be prepared here. Our caller is already
-		 * prepared for such a scenario.
-		 */
-		gsh_free(read_data);
-		data->op_data = NULL;
-	}
-
-	return rc;
-}
-
 /**
  * @brief Callback for NFS4 read done
  *
@@ -249,6 +204,118 @@ static void nfs4_read_cb(struct fsal_obj_handle *obj, fsal_status_t ret,
 	}
 }
 
+enum nfs_req_result nfs4_op_read_resume(struct nfs_argop4 *op,
+					compound_data_t *data,
+					struct nfs_resop4 *resp)
+{
+	struct nfs4_read_data *read_data = data->op_data;
+	enum nfs_req_result rc;
+	uint32_t flags;
+
+	if (read_data->read_arg.fsal_resume) {
+		/* FSAL is requesting another read2 call on resume */
+		atomic_postclear_uint32_t_bits(&read_data->flags,
+					       ASYNC_PROC_EXIT |
+					       ASYNC_PROC_DONE);
+
+		read_data->obj->obj_ops->read2(read_data->obj, true,
+					       nfs4_read_cb,
+					       &read_data->read_arg, read_data);
+
+		/* Only atomically set the flags if we actually call read2,
+		 * otherwise we will have indicated as having been DONE.
+		 */
+		flags =
+		    atomic_postset_uint32_t_bits(&read_data->flags,
+						 ASYNC_PROC_EXIT);
+
+		if ((flags & ASYNC_PROC_DONE) != ASYNC_PROC_DONE) {
+			/* The read was not finished before we got here. When
+			 * the read completes, nfs4_read_cb() will have to
+			 * reschedule the request for completion. The resume
+			 * will be resolved by nfs4_op_read_resume() which will
+			 * free read_data and return the appropriate return
+			 * result. We will NOT go async again for the read op
+			 * (but could for a subsequent op in the compound).
+			 */
+			return NFS_REQ_ASYNC_WAIT;
+		}
+	}
+
+	rc = nfs4_complete_read(data->op_data);
+
+	if (rc != NFS_REQ_ASYNC_WAIT) {
+		/* We are completely done with the request. This test wasn't
+		 * strictly necessary since nfs4_complete_read doesn't async but
+		 * at some future time, the getattr it does might go async so we
+		 * might as well be prepared here. Our caller is already
+		 * prepared for such a scenario.
+		 */
+		gsh_free(data->op_data);
+		data->op_data = NULL;
+	}
+
+	return rc;
+}
+
+enum nfs_req_result nfs4_op_read_plus_resume(struct nfs_argop4 *op,
+					     compound_data_t *data,
+					     struct nfs_resop4 *resp)
+{
+	struct nfs4_read_data *read_data = data->op_data;
+	enum nfs_req_result rc;
+	uint32_t flags;
+
+	if (read_data->read_arg.fsal_resume) {
+		/* FSAL is requesting another read2 call on resume */
+		atomic_postclear_uint32_t_bits(&read_data->flags,
+					       ASYNC_PROC_EXIT |
+					       ASYNC_PROC_DONE);
+
+		read_data->obj->obj_ops->read2(read_data->obj, true,
+					       nfs4_read_cb,
+					       &read_data->read_arg, read_data);
+
+		/* Only atomically set the flags if we actually call read2,
+		 * otherwise we will have indicated as having been DONE.
+		 */
+		flags =
+		    atomic_postset_uint32_t_bits(&read_data->flags,
+						 ASYNC_PROC_EXIT);
+
+		if ((flags & ASYNC_PROC_DONE) != ASYNC_PROC_DONE) {
+			/* The read was not finished before we got here. When
+			 * the read completes, nfs4_read_cb() will have to
+			 * reschedule the request for completion. The resume
+			 * will be resolved by nfs4_op_read_resume() which will
+			 * free read_data and return the appropriate return
+			 * result. We will NOT go async again for the read op
+			 * (but could for a subsequent op in the compound).
+			 */
+			return NFS_REQ_ASYNC_WAIT;
+		}
+	}
+
+	rc = nfs4_complete_read(read_data);
+
+	if (rc == NFS_REQ_OK) {
+		nfs4_complete_read_plus(resp, &read_data->info);
+	}
+
+	if (rc != NFS_REQ_ASYNC_WAIT) {
+		/* We are completely done with the request. This test wasn't
+		 * strictly necessary since nfs4_complete_read doesn't async but
+		 * at some future time, the getattr it does might go async so we
+		 * might as well be prepared here. Our caller is already
+		 * prepared for such a scenario.
+		 */
+		gsh_free(read_data);
+		data->op_data = NULL;
+	}
+
+	return rc;
+}
+
 /**
  * @brief Read on a pNFS pNFS data server
  *
@@ -417,7 +484,7 @@ static enum nfs_req_result nfs4_read(struct nfs_argop4 *op,
 	state_owner_t *owner = NULL;
 	bool bypass = false;
 	struct nfs4_read_data *read_data = NULL;
-	struct fsal_io_arg *read_arg;
+	struct fsal_io_arg *read_arg = NULL;
 	uint32_t resp_size;
 	/* In case we don't call read2, we indicate the I/O as already done
 	 * since in that case we should go ahead and exit as expected.
@@ -687,6 +754,8 @@ static enum nfs_req_result nfs4_read(struct nfs_argop4 *op,
 		read_data->info.io_advise = info->io_advise;
 	}
 
+again:
+
 	/* Do the actual read */
 	obj->obj_ops->read2(obj, bypass, nfs4_read_cb, read_arg, read_data);
 
@@ -698,8 +767,10 @@ static enum nfs_req_result nfs4_read(struct nfs_argop4 *op,
 
  out:
 
-	if (state_open != NULL)
+	if (state_open != NULL) {
 		dec_state_t_ref(state_open);
+		state_open = NULL;
+	}
 
 	if ((flags & ASYNC_PROC_DONE) != ASYNC_PROC_DONE) {
 		/* The read was not finished before we got here. When the
@@ -712,6 +783,17 @@ static enum nfs_req_result nfs4_read(struct nfs_argop4 *op,
 		return NFS_REQ_ASYNC_WAIT;
 	}
 
+	if (read_data != NULL && read_arg->fsal_resume) {
+		/* FSAL is requesting another read2 call */
+		atomic_postclear_uint32_t_bits(&read_data->flags,
+					       ASYNC_PROC_EXIT |
+					       ASYNC_PROC_DONE);
+		/* Make the call with the same params, though the FSAL will be
+		 * signaled by fsal_resume being set.
+		 */
+		goto again;
+	}
+
 	return nfsstat4_to_nfs_req_result(res_READ4->status);
 }				/* nfs4_op_read */
 
