@@ -44,9 +44,11 @@
  */
 struct nfs4_readdir_cb_data {
 	XDR xdr;	/*< The xdrmem to serialize the entries */
-	char *entries;	/*< The array holding individual entries */
+	uint8_t *entries;	/*< The array holding individual entries */
 	size_t mem_avail;	/*< The amount of memory remaining before we
 				   hit maxcount */
+	int count;		/*< Number of entries accumulated so far. */
+	uint32_t max_count;	/*< Maximum number of entries allowed. */
 	bool has_entries;	/*< Track if at least one entry fit  */
 	nfsstat4 error;		/*< Set to a value other than NFS4_OK if the
 				   callback function finds a fatal error. */
@@ -116,6 +118,7 @@ fsal_errors_t nfs4_readdir_callback(void *opaque,
 	component4 name;
 	bool_t res_false = false;
 	bool_t lock_dir = false;
+	struct fsal_obj_handle *saved_current_obj = NULL;
 
 	LogFullDebug(COMPONENT_NFS_READDIR,
 		     "Entry %s pos %d mem_left %d",
@@ -292,6 +295,14 @@ not_junction:
 	/* Now process the entry */
 	memset(val_fh, 0, NFS4_FHSIZE);
 
+	/* See if we have space based on max_count. */
+	if (tracker->count == tracker->max_count) {
+		LogDebug(COMPONENT_NFS_READDIR,
+			 "Skipping because we already have %d entries",
+			 tracker->count);
+		goto failure;
+	}
+
 	/* Bits that don't require allocation */
 	if (mem_left < BASE_ENTRY_SIZE) {
 		if (!tracker->has_entries) {
@@ -387,6 +398,21 @@ not_junction:
 		goto skip;
 	}
 
+	/*
+	 * At this point, current_obj points to what is setup by PUTFH.
+	 * If readdir involves junctions, then it is possible that the
+	 * FSAL for current_obj and obj could be different. If attributes
+	 * requested includes Filesystem attributes, then a call will be
+	 * made to the FSAL that owns the object as populate_dirent would
+	 * have changed fsal_export to the new export. So, a call to
+	 * get_dynamic_info of the FSAL will be made with data->current_obj
+	 * which is actually the parent's FSAL object.
+	 * This normally happens if NFS clients tries to query FS attributes
+	 * in READDIR of a directory that contains junctions (ex:- pseudo
+	 * namespace)
+	 */
+	saved_current_obj = data->current_obj;
+	data->current_obj = obj;
 	if (!xdr_encode_entry4(&tracker->xdr, &args, tracker->req_attr,
 			       cookie, &name) ||
 	    (xdr_getpos(&tracker->xdr) + BYTES_PER_XDR_UNIT)
@@ -395,8 +421,10 @@ not_junction:
 		LogFullDebug(COMPONENT_NFS_READDIR,
 			     "Overflow of buffer after xdr_encode_entry4 - pos = %d",
 			     xdr_getpos(&tracker->xdr));
+		data->current_obj = saved_current_obj;
 		goto failure;
 	}
+	data->current_obj = saved_current_obj;
 
  skip:
 
@@ -428,6 +456,7 @@ not_junction:
 
 	tracker->has_entries = true;
 	cb_parms->in_result = true;
+	tracker->count++;
 	goto out;
 
  server_fault:
@@ -527,17 +556,18 @@ enum nfs_req_result nfs4_op_readdir(struct nfs_argop4 *op,
 
 	/* Dont over flow V4.1 maxresponsesize or maxcachedsize */
 	maxcount = resp_room(data);
-	if (nfs_param.core_param.readdir_res_size != 0 &&
-		nfs_param.core_param.readdir_res_size < maxcount) {
+
+	if (nfs_param.core_param.readdir_res_size < maxcount)
 		maxcount = nfs_param.core_param.readdir_res_size;
-	}
 
 	if (maxcount > (arg_READDIR4->maxcount + sizeof(nfsstat4)))
 		maxcount = arg_READDIR4->maxcount + sizeof(nfsstat4);
 
-	/* Dircount is considered meaningless by many nfsv4 client (like the
-	 * CITI one).  we use maxcount instead.
+	/* Use the dircount from the request as the max number of entries if
+	 * lower than the configured max.
 	 */
+	if (dircount > nfs_param.core_param.readdir_max_count)
+		dircount = nfs_param.core_param.readdir_max_count;
 
 	LogDebug(COMPONENT_NFS_READDIR,
 		 "dircount=%lu maxcount=%lu cookie=%" PRIu64,
@@ -552,6 +582,24 @@ enum nfs_req_result nfs4_op_readdir(struct nfs_argop4 *op,
 		goto out;
 	}
 
+	/* Make sure requested attributes are valid and initialize attrmask */
+	res_READDIR4->status =
+	    bitmap4_to_attrmask_t(&arg_READDIR4->attr_request, &attrmask);
+	if (res_READDIR4->status != NFS4_OK) {
+		LogDebug(COMPONENT_NFS_READDIR, "Requested invalid attributes");
+		goto out;
+	}
+
+	/* Make sure the FSAL supports all the requested attributes */
+	attrmask_t supported_attrs =
+	    op_ctx->fsal_export->exp_ops.fs_supported_attrs(
+		op_ctx->fsal_export);
+	if ((attrmask & ~supported_attrs) != 0) {
+		res_READDIR4->status = NFS4ERR_INVAL;
+		LogDebug(COMPONENT_NFS_READDIR, "Requested invalid attributes");
+		goto out;
+	}
+
 	/* Get only attributes that are allowed to be read */
 	if (!nfs4_Fattr_Check_Access_Bitmap
 	    (&arg_READDIR4->attr_request, FATTR4_ATTR_READ)) {
@@ -623,27 +671,28 @@ enum nfs_req_result nfs4_op_readdir(struct nfs_argop4 *op,
 
 	/* Prepare to read the entries */
 	tracker.mem_avail = maxcount - READDIR_RESP_BASE_SIZE;
+	tracker.max_count = dircount;
 	tracker.entries = gsh_malloc(tracker.mem_avail);
 	tracker.error = NFS4_OK;
 	tracker.req_attr = &arg_READDIR4->attr_request;
 	tracker.data = data;
 
-	xdrmem_create(&tracker.xdr, tracker.entries, tracker.mem_avail,
+	xdrmem_create(&tracker.xdr, (char *)tracker.entries, tracker.mem_avail,
 		      XDR_ENCODE);
 
 	/* Assume we need at least the NFS v3 attr.
 	 * Any attr is sufficient for permission checking.
 	 */
-	attrmask = ATTRS_NFS3;
+	if (attrmask == 0)
+		attrmask = ATTRS_NFS3;
 
-	/* If ACL is requested, we need to add that for permission checking. */
-	if (attribute_is_set(tracker.req_attr, FATTR4_ACL))
-		attrmask |= ATTR_ACL;
-
-	/* If seclabel requested, then fetch it too */
+	/* If seclabel is requested but we do not have
+	 * EXPORT_OPTION_SECLABEL_SET, turn off that bit.
+	 */
 	if (attribute_is_set(tracker.req_attr, FATTR4_SEC_LABEL) &&
-	    op_ctx_export_has_option(EXPORT_OPTION_SECLABEL_SET))
-		attrmask |= ATTR4_SEC_LABEL;
+	    !op_ctx_export_has_option(EXPORT_OPTION_SECLABEL_SET)) {
+		attrmask &= ~ATTR4_SEC_LABEL;
+	}
 
 	/* Perform the readdir operation */
 	fsal_status = fsal_readdir(dir_obj,
