@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -53,9 +54,9 @@
 /**
  * @brief NFSv4 SECINFO_NO_NAME operation
  *
- * This function impelments the NFSv4 SECINFO_NO_NAME operation.
+ * This function implements the NFSv4 SECINFO_NO_NAME operation.
  *
- * @param[in]     op   Operation reqest
+ * @param[in]     op   Operation request
  * @param[in,out] data Compound data
  * @param[out]    resp Response
  *
@@ -99,25 +100,22 @@ enum nfs_req_result nfs4_op_secinfo_no_name(struct nfs_argop4 *op,
 
 	/* Get the number of entries */
 #ifdef _HAVE_GSSAPI
-	if (op_ctx->export_perms->options &
-	    EXPORT_OPTION_RPCSEC_GSS_NONE)
+	if (op_ctx->export_perms.options & EXPORT_OPTION_RPCSEC_GSS_NONE)
 		num_entry++;
 
-	if (op_ctx->export_perms->options &
-	    EXPORT_OPTION_RPCSEC_GSS_INTG)
+	if (op_ctx->export_perms.options & EXPORT_OPTION_RPCSEC_GSS_INTG)
 		num_entry++;
 
-	if (op_ctx->export_perms->options &
-	    EXPORT_OPTION_RPCSEC_GSS_PRIV)
+	if (op_ctx->export_perms.options & EXPORT_OPTION_RPCSEC_GSS_PRIV)
 		num_entry++;
 
 	resp_size += (RNDUP(krb5oid.length) + GSS_RESP_SIZE) * num_entry;
 #endif
 
-	if (op_ctx->export_perms->options & EXPORT_OPTION_AUTH_NONE)
+	if (op_ctx->export_perms.options & EXPORT_OPTION_AUTH_NONE)
 		num_entry++;
 
-	if (op_ctx->export_perms->options & EXPORT_OPTION_AUTH_UNIX)
+	if (op_ctx->export_perms.options & EXPORT_OPTION_AUTH_UNIX)
 		num_entry++;
 
 	/* Check for space in response. */
@@ -135,14 +133,13 @@ enum nfs_req_result nfs4_op_secinfo_no_name(struct nfs_argop4 *op,
 
 	/**
 	 * @todo We give here the order in which the client should try
-	 * different authentifications. Might want to give it in the
+	 * different authentications. Might want to give it in the
 	 * order given in the config.
 	 */
 	int idx = 0;
 
 #ifdef _HAVE_GSSAPI
-	if (op_ctx->export_perms->options &
-	    EXPORT_OPTION_RPCSEC_GSS_PRIV) {
+	if (op_ctx->export_perms.options & EXPORT_OPTION_RPCSEC_GSS_PRIV) {
 		resok_val[idx].flavor = RPCSEC_GSS;
 		resok_val[idx].secinfo4_u.flavor_info.service =
 			RPCSEC_GSS_SVC_PRIVACY;
@@ -150,8 +147,7 @@ enum nfs_req_result nfs4_op_secinfo_no_name(struct nfs_argop4 *op,
 		resok_val[idx++].secinfo4_u.flavor_info.oid = v5oid;
 	}
 
-	if (op_ctx->export_perms->options &
-	    EXPORT_OPTION_RPCSEC_GSS_INTG) {
+	if (op_ctx->export_perms.options & EXPORT_OPTION_RPCSEC_GSS_INTG) {
 		resok_val[idx].flavor = RPCSEC_GSS;
 		resok_val[idx].secinfo4_u.flavor_info.service =
 			RPCSEC_GSS_SVC_INTEGRITY;
@@ -159,8 +155,7 @@ enum nfs_req_result nfs4_op_secinfo_no_name(struct nfs_argop4 *op,
 		resok_val[idx++].secinfo4_u.flavor_info.oid = v5oid;
 	}
 
-	if (op_ctx->export_perms->options &
-	    EXPORT_OPTION_RPCSEC_GSS_NONE) {
+	if (op_ctx->export_perms.options & EXPORT_OPTION_RPCSEC_GSS_NONE) {
 		resok_val[idx].flavor = RPCSEC_GSS;
 		resok_val[idx].secinfo4_u.flavor_info.service =
 			RPCSEC_GSS_SVC_NONE;
@@ -169,10 +164,10 @@ enum nfs_req_result nfs4_op_secinfo_no_name(struct nfs_argop4 *op,
 	}
 #endif /* _HAVE_GSSAPI */
 
-	if (op_ctx->export_perms->options & EXPORT_OPTION_AUTH_UNIX)
+	if (op_ctx->export_perms.options & EXPORT_OPTION_AUTH_UNIX)
 		resok_val[idx++].flavor = AUTH_UNIX;
 
-	if (op_ctx->export_perms->options & EXPORT_OPTION_AUTH_NONE)
+	if (op_ctx->export_perms.options & EXPORT_OPTION_AUTH_NONE)
 		resok_val[idx++].flavor = AUTH_NONE;
 
 	res_SECINFO_NO_NAME4->SECINFO4res_u.resok4.SECINFO4resok_len = idx;
@@ -183,11 +178,8 @@ enum nfs_req_result nfs4_op_secinfo_no_name(struct nfs_argop4 *op,
 	data->currentFH.nfs_fh4_len = 0;
 
 	/* Release CurrentFH reference to export. */
-	if (op_ctx->ctx_export) {
-		put_gsh_export(op_ctx->ctx_export);
-		op_ctx->ctx_export = NULL;
-		op_ctx->fsal_export = NULL;
-	}
+	clear_op_context_export();
+
 
 	res_SECINFO_NO_NAME4->status = NFS4_OK;
 
