@@ -166,11 +166,7 @@ enum nfs_req_result nfs4_op_sequence(struct nfs_argop4 *op,
 	LogDebug(COMPONENT_SESSIONS, "SEQUENCE session=%p", session);
 
 	/* Check if lease is expired and reserve it */
-	PTHREAD_MUTEX_lock(&session->clientid_record->cid_mutex);
-
-	if (!reserve_lease(session->clientid_record)) {
-		PTHREAD_MUTEX_unlock(&session->clientid_record->cid_mutex);
-
+	if (!reserve_lease_or_expire(session->clientid_record, false)) {
 		dec_session_ref(session);
 		res_SEQUENCE4->sr_status = NFS4ERR_EXPIRED;
 		LogDebugAlt(COMPONENT_SESSIONS, COMPONENT_CLIENTID,
@@ -181,8 +177,6 @@ enum nfs_req_result nfs4_op_sequence(struct nfs_argop4 *op,
 
 	data->preserved_clientid = session->clientid_record;
 
-	PTHREAD_MUTEX_unlock(&session->clientid_record->cid_mutex);
-
 	slotid = arg_SEQUENCE4->sa_slotid;
 
 	/* Check is slot is compliant with ca_maxrequests */
@@ -198,7 +192,7 @@ enum nfs_req_result nfs4_op_sequence(struct nfs_argop4 *op,
 	slot = &session->fc_slots[slotid];
 
 	/* Serialize use of this slot. */
-	PTHREAD_MUTEX_lock(&slot->lock);
+	PTHREAD_MUTEX_lock(&slot->slot_lock);
 
 	if (slot->sequence + 1 != arg_SEQUENCE4->sa_sequenceid) {
 		/* This sequence is NOT the next sequence */
@@ -240,13 +234,13 @@ enum nfs_req_result nfs4_op_sequence(struct nfs_argop4 *op,
 						slot->cached_result,
 						refcnt);
 
-				PTHREAD_MUTEX_unlock(&slot->lock);
+				PTHREAD_MUTEX_unlock(&slot->slot_lock);
 
 				dec_session_ref(session);
 				return NFS_REQ_REPLAY;
 			} else {
 				/* Illegal replay */
-				PTHREAD_MUTEX_unlock(&slot->lock);
+				PTHREAD_MUTEX_unlock(&slot->slot_lock);
 
 				dec_session_ref(session);
 				res_SEQUENCE4->sr_status =
@@ -263,7 +257,7 @@ enum nfs_req_result nfs4_op_sequence(struct nfs_argop4 *op,
 			}
 		}
 
-		PTHREAD_MUTEX_unlock(&slot->lock);
+		PTHREAD_MUTEX_unlock(&slot->slot_lock);
 
 		dec_session_ref(session);
 		res_SEQUENCE4->sr_status = NFS4ERR_SEQ_MISORDERED;
@@ -328,7 +322,7 @@ enum nfs_req_result nfs4_op_sequence(struct nfs_argop4 *op,
 		/* Indicate the failed response size. */
 		data->op_resp_size = sizeof(nfsstat4);
 
-		PTHREAD_MUTEX_unlock(&slot->lock);
+		PTHREAD_MUTEX_unlock(&slot->slot_lock);
 
 		dec_session_ref(session);
 		data->session = NULL;
