@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -42,7 +43,7 @@
 
 /**
  *
- * @brief Implemtation of NFS4_OP_VERIFY
+ * @brief Implementation of NFS4_OP_VERIFY
  *
  * This function implemtats the NFS4_OP_VERIFY operation.
  *
@@ -61,7 +62,7 @@ enum nfs_req_result nfs4_op_verify(struct nfs_argop4 *op,
 	VERIFY4res * const res_VERIFY4 = &resp->nfs_resop4_u.opverify;
 	fattr4 file_attr4;
 	int rc = 0;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 
 	resp->resop = NFS4_OP_VERIFY;
 	res_VERIFY4->status = NFS4_OK;
