@@ -112,6 +112,7 @@ void cleanup_pseudofs_node(char *pseudo_path,
 	char *pos = pseudo_path + strlen(pseudo_path) - 1;
 	char *name;
 	fsal_status_t fsal_status;
+	op_ctx->flags.pseudo_fsal_internal_lookup = true;
 
 	/* Strip trailing / from pseudo_path */
 	while (*pos == '/')
@@ -165,18 +166,18 @@ void cleanup_pseudofs_node(char *pseudo_path,
 	 * exp_root_obj so we can check if we have reached the root of
 	 * the mounted on export.
 	 */
-	PTHREAD_RWLOCK_rdlock(&op_ctx->ctx_export->lock);
+	PTHREAD_RWLOCK_rdlock(&op_ctx->ctx_export->exp_lock);
 
 	if (parent_obj == op_ctx->ctx_export->exp_root_obj) {
 		LogDebug(COMPONENT_EXPORT,
 			 "Reached root of PseudoFS %s",
 			 CTX_PSEUDOPATH(op_ctx));
 
-		PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->lock);
+		PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->exp_lock);
 		goto out;
 	}
 
-	PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->lock);
+	PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->exp_lock);
 
 	/* Truncate the pseudo_path to be the path to the parent */
 	*pos = '\0';
@@ -197,8 +198,8 @@ bool make_pseudofs_node(char *name, struct pseudofs_state *state)
 	char const *fsal_name;
 
 retry:
-
 	/* First, try to lookup the entry */
+	op_ctx->flags.pseudo_fsal_internal_lookup = true;
 	fsal_status = fsal_lookup(state->obj, name, &new_node, NULL);
 
 	if (!FSAL_IS_ERROR(fsal_status)) {
@@ -216,9 +217,10 @@ retry:
 		}
 
 		LogDebug(COMPONENT_EXPORT,
-			 "BUILDING PSEUDOFS: Parent %p entry %p %s FSAL %s already exists",
-			 state->obj, new_node, name,
-			 new_node->fsal->name);
+			 "BUILDING PSEUDOFS: Parent %p fileid %"PRIx64
+			 "entry %p fileid %"PRIx64" %s FSAL %s already exists",
+			 state->obj, state->obj->fileid, new_node,
+			 new_node->fileid, name, new_node->fsal->name);
 
 		state->obj->obj_ops->put_ref(state->obj);
 		/* Make new node the current node */
@@ -460,8 +462,47 @@ bool pseudo_mount_export(struct gsh_export *export)
 		}
 	}
 
+	/* Fill in the mounted on information for the export. */
+	PTHREAD_RWLOCK_wrlock(&export->exp_lock);
+
+	LogDebug(COMPONENT_EXPORT,
+		 "BUILDING PSEUDOFS: Export_Id %d Path %s Pseudo Path %s mounted_on_fileid %"
+		 PRIx64,
+		 export->export_id,
+		 state.st_fullpath,
+		 state.st_pseudopath,
+		 state.obj->fileid);
+
+	export->exp_mounted_on_file_id = state.obj->fileid;
+
+	/* Get an active reference on the object */
+	state.obj->obj_ops->get_ref(state.obj);
+
+	/* Pass object ref off to export */
+	export->exp_junction_obj = state.obj;
+	export_root_object_get(export->exp_junction_obj);
+
+	/* Take an export ref for the parent export */
+	export->exp_parent_exp = op_ctx->ctx_export;
+	get_gsh_export_ref(export->exp_parent_exp);
+
+	/* Add ourselves to the list of exports mounted on parent */
+	PTHREAD_RWLOCK_wrlock(&export->exp_parent_exp->exp_lock);
+	glist_add_tail(&export->exp_parent_exp->mounted_exports_list,
+		       &export->mounted_exports_node);
+	PTHREAD_RWLOCK_unlock(&export->exp_parent_exp->exp_lock);
+
+	PTHREAD_RWLOCK_unlock(&export->exp_lock);
+
+	/* Set this outside the export->exp_lock, protected by EXPORT_ADMIN_LOCK
+	 */
+	export->is_mounted = true;
+
 	/* Now that all entries are added to pseudofs tree, and we are pointing
 	 * to the final node, make it a proper junction.
+	 * Note that this is the last thing that should be updated on this
+	 * handle, since setting the junction_export is what signals to the
+	 * PSEUDO FSAL that this entry is done.
 	 */
 	PTHREAD_RWLOCK_wrlock(&state.obj->state_hdl->jct_lock);
 	state.obj->state_hdl->dir.junction_export = export;
@@ -475,29 +516,6 @@ bool pseudo_mount_export(struct gsh_export *export)
 
 	PTHREAD_RWLOCK_unlock(&state.obj->state_hdl->jct_lock);
 
-	/* And fill in the mounted on information for the export. */
-	PTHREAD_RWLOCK_wrlock(&export->lock);
-
-	export->exp_mounted_on_file_id = state.obj->fileid;
-	/* Pass object ref off to export */
-	export->exp_junction_obj = state.obj;
-	export_root_object_get(export->exp_junction_obj);
-
-	/* Take an export ref for the parent export */
-	export->exp_parent_exp = op_ctx->ctx_export;
-	get_gsh_export_ref(export->exp_parent_exp);
-
-	/* Add ourselves to the list of exports mounted on parent */
-	PTHREAD_RWLOCK_wrlock(&export->exp_parent_exp->lock);
-	glist_add_tail(&export->exp_parent_exp->mounted_exports_list,
-		       &export->mounted_exports_node);
-	PTHREAD_RWLOCK_unlock(&export->exp_parent_exp->lock);
-
-	PTHREAD_RWLOCK_unlock(&export->lock);
-
-	/* Set this outside the export->lock, protected by EXPORT_ADMIN_LOCK */
-	export->is_mounted = true;
-
 	LogDebug(COMPONENT_EXPORT,
 		 "BUILDING PSEUDOFS: Export_Id %d Path %s Pseudo Path %s junction %p",
 		 export->export_id,
@@ -536,6 +554,7 @@ void create_pseudofs(void)
 	/* Initialize a root context */
 	init_op_context(&op_context, NULL, NULL, NULL,
 			NFS_V4, 0, NFS_REQUEST);
+	op_ctx->flags.pseudo_fsal_internal_lookup = true;
 
 	while (true) {
 		export = export_take_mount_work();
@@ -565,7 +584,7 @@ void pseudo_unmount_export(struct gsh_export *export)
 	 * we jump straight to cleaning up our presence in parent
 	 * export.
 	 */
-	PTHREAD_RWLOCK_wrlock(&export->lock);
+	PTHREAD_RWLOCK_wrlock(&export->exp_lock);
 
 	junction_inode = export->exp_junction_obj;
 	mounted_on_export = export->exp_parent_exp;
@@ -580,7 +599,7 @@ void pseudo_unmount_export(struct gsh_export *export)
 			 "Unmount of export %d unnecessary",
 			 export->export_id);
 
-		PTHREAD_RWLOCK_unlock(&export->lock);
+		PTHREAD_RWLOCK_unlock(&export->exp_lock);
 		return;
 	}
 
@@ -623,48 +642,41 @@ void pseudo_unmount_export(struct gsh_export *export)
 	export->exp_parent_exp = NULL;
 
 	/* Remove ourselves from the list of exports mounted on parent */
-	PTHREAD_RWLOCK_wrlock(&mounted_on_export->lock);
+	PTHREAD_RWLOCK_wrlock(&mounted_on_export->exp_lock);
 	glist_del(&export->mounted_exports_node);
-	PTHREAD_RWLOCK_unlock(&mounted_on_export->lock);
+	PTHREAD_RWLOCK_unlock(&mounted_on_export->exp_lock);
 
 	/* Release the export lock */
-	PTHREAD_RWLOCK_unlock(&export->lock);
+	PTHREAD_RWLOCK_unlock(&export->exp_lock);
 
-	/* Clear this outside the export->lock, protected by EXPORT_ADMIN_LOCK
+	/* Clear this outside the export->exp_lock, protected by
+	 * EXPORT_ADMIN_LOCK
 	 */
 	export->is_mounted = false;
 
+	/* Get a ref to the mounted_on_export and initialize op_context
+	 */
+	get_gsh_export_ref(mounted_on_export);
+
+	init_op_context(&op_context, mounted_on_export,
+			mounted_on_export->fsal_export, NULL,
+			NFS_V4, 0, NFS_REQUEST);
+	op_ctx->flags.pseudo_fsal_internal_lookup = true;
+
 	if (is_export_pseudo(mounted_on_export) && junction_inode != NULL) {
 		char *pseudo_path = gsh_strdup(ref_pseudopath->gr_val);
 
-		/* Get a ref to the mounted_on_export and initialize op_context
-		 */
-		get_gsh_export_ref(mounted_on_export);
-		init_op_context(&op_context, mounted_on_export,
-				mounted_on_export->fsal_export, NULL,
-				NFS_V4, 0, NFS_REQUEST);
-
 		/* Remove the unused PseudoFS nodes */
 		cleanup_pseudofs_node(pseudo_path, junction_inode);
 
 		gsh_free(pseudo_path);
-		release_op_context();
 	} else {
-		/* Get a ref to the mounted_on_export and initialize op_context
-		 */
-		get_gsh_export_ref(mounted_on_export);
-		init_op_context(&op_context, mounted_on_export,
-				mounted_on_export->fsal_export, NULL,
-				NFS_V4, 0, NFS_REQUEST);
-
 		/* Properly unmount at the FSAL layer - primarily this will
 		 * allow mdcache to unmap the junction inode from the parent
 		 * export.
 		 */
 		mounted_on_export->fsal_export->exp_ops.unmount(
 			mounted_on_export->fsal_export, junction_inode);
-
-		release_op_context();
 	}
 
 	/* Release our reference to the export we are mounted on. */
@@ -673,6 +685,11 @@ void pseudo_unmount_export(struct gsh_export *export)
 	/* Release the LRU reference */
 	junction_inode->obj_ops->put_ref(junction_inode);
 
+	/* Release the active reference */
+	junction_inode->obj_ops->put_ref(junction_inode);
+
+	release_op_context();
+
 	LogFullDebug(COMPONENT_EXPORT,
 		     "Finish unexport %s", ref_pseudopath->gr_val);
 
@@ -690,7 +707,7 @@ void pseudo_unmount_export_tree(struct gsh_export *export)
 	while (true) {
 		struct gsh_export *sub_mounted_export;
 
-		PTHREAD_RWLOCK_rdlock(&export->lock);
+		PTHREAD_RWLOCK_rdlock(&export->exp_lock);
 		/* Find a sub_mounted export */
 		sub_mounted_export =
 			glist_first_entry(&export->mounted_exports_list,
@@ -698,7 +715,7 @@ void pseudo_unmount_export_tree(struct gsh_export *export)
 					  mounted_exports_node);
 		if (sub_mounted_export == NULL) {
 			/* If none, break out of the loop */
-			PTHREAD_RWLOCK_unlock(&export->lock);
+			PTHREAD_RWLOCK_unlock(&export->exp_lock);
 			break;
 		}
 
@@ -709,7 +726,7 @@ void pseudo_unmount_export_tree(struct gsh_export *export)
 		get_gsh_export_ref(sub_mounted_export);
 
 		/* Drop the lock */
-		PTHREAD_RWLOCK_unlock(&export->lock);
+		PTHREAD_RWLOCK_unlock(&export->exp_lock);
 
 		/* And unmount it */
 		pseudo_unmount_export_tree(sub_mounted_export);
@@ -788,7 +805,7 @@ void prune_pseudofs_subtree(struct gsh_export *export,
 	 * unmount child_export, it is impossible for any other child exports to
 	 * be unmounted during this time, so glistn continues to be valid.
 	 */
-	PTHREAD_RWLOCK_rdlock(&export->lock);
+	PTHREAD_RWLOCK_rdlock(&export->exp_lock);
 	glist_for_each_safe(glist, glistn, &export->mounted_exports_list) {
 		/* Find a sub_mounted export */
 		child_export = glist_entry(glist,
@@ -802,7 +819,7 @@ void prune_pseudofs_subtree(struct gsh_export *export,
 		get_gsh_export_ref(child_export);
 
 		/* Drop the lock */
-		PTHREAD_RWLOCK_unlock(&export->lock);
+		PTHREAD_RWLOCK_unlock(&export->exp_lock);
 		/* And prune this child */
 		prune_pseudofs_subtree(child_export, generation, defunct);
 
@@ -810,11 +827,11 @@ void prune_pseudofs_subtree(struct gsh_export *export,
 		put_gsh_export(child_export);
 
 		/* And take the lock again. */
-		PTHREAD_RWLOCK_rdlock(&export->lock);
+		PTHREAD_RWLOCK_rdlock(&export->exp_lock);
 	}
 
 	/* Drop the lock */
-	PTHREAD_RWLOCK_unlock(&export->lock);
+	PTHREAD_RWLOCK_unlock(&export->exp_lock);
 
 	if (defunct) {
 		LogDebug(COMPONENT_EXPORT,
