@@ -31,7 +31,7 @@
 #include "nsm.h"
 #include "sal_data.h"
 
-pthread_mutex_t nsm_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t nsm_mutex;
 CLIENT *nsm_clnt;
 AUTH *nsm_auth;
 unsigned long nsm_count;
@@ -234,7 +234,6 @@ static bool nsm_unmonitor_noretry(state_nsm_client_t *host)
 		ret = CLNT_CALL_WAIT(cc);
 	}
 
-	nsm_count--;
 	if (ret != RPC_SUCCESS) {
 		t = rpc_sperror(&cc->cc_error, "failed");
 		LogEventLimited(COMPONENT_NLM,
@@ -251,6 +250,7 @@ static bool nsm_unmonitor_noretry(state_nsm_client_t *host)
 	clnt_req_release(cc);
 
 	atomic_store_int32_t(&host->ssc_monitored, false);
+	nsm_count--;
 
 	LogDebug(COMPONENT_NLM, "Unmonitored %s for nodename %s",
 		 nsm_mon_id.mon_name, nodename);
