@@ -49,9 +49,9 @@
 #include "city.h"
 #include "abstract_mem.h"
 #include "gsh_intrinsic.h"
-#include "gsh_wait_queue.h"
 
 #define DUPREQ_NOCACHE     ((void *)0x02)
+#define DUPREQ_NOCACHE_NORES ((void *)0x03)
 #define DUPREQ_MAX_RETRIES 5
 
 #define NFS_pcp nfs_param.core_param
@@ -131,7 +131,7 @@ const char *dupreq_state_table[] = {
  * freed or in nfs_dupreq_finish() that decides to take out few dupreqs!
  */
 struct drc_st {
-	pthread_mutex_t mtx;
+	pthread_mutex_t drc_st_mtx;
 	drc_t udp_drc;		/* shared DRC */
 	struct rbtree_x tcp_drc_recycle_t;
 	 TAILQ_HEAD(drc_st_tailq, drc) tcp_drc_recycle_q;	/* fifo */
@@ -286,7 +286,7 @@ static inline void init_shared_drc(void)
 	drc->npart = nfs_param.core_param.drc.udp.npart;
 	drc->hiwat = nfs_param.core_param.drc.udp.hiwat;
 
-	gsh_mutex_init(&drc->mtx, NULL);
+	PTHREAD_MUTEX_init(&drc->drc_mtx, NULL);
 
 	/* init dict */
 	code =
@@ -307,6 +307,16 @@ static inline void init_shared_drc(void)
 	}
 }
 
+/* Cleanup on shutdown */
+void dupreq2_cleanup(void)
+{
+	PTHREAD_MUTEX_destroy(&drc_st->drc_st_mtx);
+}
+
+struct cleanup_list_element dupreq2_cleanup_element = {
+	.clean = dupreq2_cleanup,
+};
+
 /**
  * @brief Initialize the DRC package.
  */
@@ -324,7 +334,7 @@ void dupreq2_pkginit(void)
 	drc_st = gsh_calloc(1, sizeof(struct drc_st));
 
 	/* init shared statics */
-	gsh_mutex_init(&drc_st->mtx, NULL);
+	PTHREAD_MUTEX_init(&drc_st->drc_st_mtx, NULL);
 
 	/* recycle_t */
 	code =
@@ -341,6 +351,8 @@ void dupreq2_pkginit(void)
 
 	/* UDP DRC is global, shared */
 	init_shared_drc();
+
+	RegisterCleanup(&dupreq2_cleanup_element);
 }
 
 /**
@@ -407,7 +419,7 @@ static inline drc_t *alloc_tcp_drc(enum drc_type dtype)
 	drc->npart = nfs_param.core_param.drc.tcp.npart;
 	drc->hiwat = nfs_param.core_param.drc.tcp.hiwat;
 
-	PTHREAD_MUTEX_init(&drc->mtx, NULL);
+	PTHREAD_MUTEX_init(&drc->drc_mtx, NULL);
 
 	/* init dict */
 	code =
@@ -448,7 +460,8 @@ static inline void free_tcp_drc(drc_t *drc)
 		if (drc->xt.tree[ix].cache)
 			gsh_free(drc->xt.tree[ix].cache);
 	}
-	PTHREAD_MUTEX_destroy(&drc->mtx);
+	rbtx_cleanup(&drc->xt);
+	PTHREAD_MUTEX_destroy(&drc->drc_mtx);
 	LogFullDebug(COMPONENT_DUPREQ, "free TCP drc %p", drc);
 	pool_free(tcp_drc_pool, drc);
 }
@@ -478,10 +491,15 @@ static inline uint32_t nfs_dupreq_unref_drc(drc_t *drc)
 }
 
 #define DRC_ST_LOCK()				\
-	PTHREAD_MUTEX_lock(&drc_st->mtx)
+	PTHREAD_MUTEX_lock(&drc_st->drc_st_mtx)
 
 #define DRC_ST_UNLOCK()				\
-	PTHREAD_MUTEX_unlock(&drc_st->mtx)
+	PTHREAD_MUTEX_unlock(&drc_st->drc_st_mtx)
+
+#define DRC_QLEN_EXCEED_HIWAT()			\
+	(drc_st->tcp_drc_recycle_qlen >		\
+	nfs_param.core_param.drc.recycle_hiwat)
+
 
 /**
  * @brief Check for expired TCP DRCs.
@@ -498,15 +516,17 @@ static inline void drc_free_expired(void)
 
 	DRC_ST_LOCK();
 
-	if ((drc_st->tcp_drc_recycle_qlen < 1) ||
-	    (now - drc_st->last_expire_check) < 600) /* 10m */
+	if (((drc_st->tcp_drc_recycle_qlen < 1) ||
+	    (now - drc_st->last_expire_check) < 600) && /* 10m */
+			!DRC_QLEN_EXCEED_HIWAT())
 		goto unlock;
 
 	do {
 		drc = TAILQ_FIRST(&drc_st->tcp_drc_recycle_q);
-		if (drc && (drc->d_u.tcp.recycle_time > 0)
+		if ((drc && (drc->d_u.tcp.recycle_time > 0)
 		    && ((now - drc->d_u.tcp.recycle_time) >
-			drc_st->expire_delta)) {
+			drc_st->expire_delta)) ||
+				DRC_QLEN_EXCEED_HIWAT()) {
 
 			assert(drc->refcnt == 0);
 
@@ -590,7 +610,7 @@ retry:
 			/* found, no danger of removal */
 			LogFullDebug(COMPONENT_DUPREQ, "ref DRC=%p for xprt=%p",
 				     drc, req->rq_xprt);
-			PTHREAD_MUTEX_lock(&drc->mtx);	/* LOCKED */
+			PTHREAD_MUTEX_lock(&drc->drc_mtx);	/* LOCKED */
 		} else {
 			drc_t drc_k;
 			struct rbtree_x_part *t = NULL;
@@ -637,7 +657,7 @@ retry:
 				/* reuse old DRC */
 				tdrc = opr_containerof(ndrc, drc_t,
 						       d_u.tcp.recycle_k);
-				PTHREAD_MUTEX_lock(&tdrc->mtx);	/* LOCKED */
+				PTHREAD_MUTEX_lock(&tdrc->drc_mtx);	/* LOCKED */
 
 				/* If the refcnt is zero and it is not
 				 * in the recycle queue, wait for the
@@ -646,7 +666,7 @@ retry:
 				if (tdrc->refcnt == 0) {
 					if (!(tdrc->flags & DRC_FLAG_RECYCLE)) {
 						PTHREAD_MUTEX_unlock(
-								&tdrc->mtx);
+								&tdrc->drc_mtx);
 						DRC_ST_UNLOCK();
 						goto retry;
 					}
@@ -671,7 +691,7 @@ retry:
 				       sizeof(sockaddr_t));
 				/* assign already-computed hash */
 				drc->d_u.tcp.hk = drc_k.d_u.tcp.hk;
-				PTHREAD_MUTEX_lock(&drc->mtx);	/* LOCKED */
+				PTHREAD_MUTEX_lock(&drc->drc_mtx);	/* LOCKED */
 				/* insert dict */
 				opr_rbtree_insert(&t->t,
 						  &drc->d_u.tcp.recycle_k);
@@ -701,7 +721,7 @@ retry:
 
 	/* call path ref */
 	(void)nfs_dupreq_ref_drc(drc);
-	PTHREAD_MUTEX_unlock(&drc->mtx);
+	PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 
 	if (drc_check_expired)
 		drc_free_expired();
@@ -721,7 +741,7 @@ out:
  */
 void nfs_dupreq_put_drc(drc_t *drc)
 {
-	PTHREAD_MUTEX_lock(&drc->mtx);
+	PTHREAD_MUTEX_lock(&drc->drc_mtx);
 
 	/* refcnt is not used on shared UDP DRC, so nothing to do */
 	if (drc->type == DRC_UDP_V234)
@@ -742,13 +762,13 @@ void nfs_dupreq_put_drc(drc_t *drc)
 	if (drc->refcnt != 0) /* quick path */
 		goto unlock;
 
-	/* note t's lock order wrt drc->mtx is the opposite of
+	/* note t's lock order wrt drc->drc_mtx is the opposite of
 	 * drc->xt[*].lock. Drop and reacquire locks in correct
 	 * order.
 	 */
-	PTHREAD_MUTEX_unlock(&drc->mtx);
+	PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 	DRC_ST_LOCK();
-	PTHREAD_MUTEX_lock(&drc->mtx);
+	PTHREAD_MUTEX_lock(&drc->drc_mtx);
 
 	/* Since we dropped and reacquired the drc lock for the
 	 * correct lock order, we need to recheck the drc fields
@@ -766,7 +786,7 @@ void nfs_dupreq_put_drc(drc_t *drc)
 	DRC_ST_UNLOCK();
 
 unlock:
-	PTHREAD_MUTEX_unlock(&drc->mtx);
+	PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 }
 
 /**
@@ -864,7 +884,7 @@ static inline dupreq_entry_t *alloc_dupreq(void)
 	dupreq_entry_t *dv;
 
 	dv = pool_alloc(dupreq_pool);
-	gsh_mutex_init(&dv->mtx, NULL);
+	PTHREAD_MUTEX_init(&dv->dre_mtx, NULL);
 	TAILQ_INIT_ENTRY(dv, fifo_q);
 	TAILQ_INIT(&dv->dupes);
 
@@ -894,7 +914,7 @@ static inline void nfs_dupreq_free_dupreq(dupreq_entry_t *dv)
 		func->free_function(dv->res);
 		free_nfs_res(dv->res);
 	}
-	PTHREAD_MUTEX_destroy(&dv->mtx);
+	PTHREAD_MUTEX_destroy(&dv->dre_mtx);
 	pool_free(dupreq_pool, dv);
 }
 
@@ -918,6 +938,8 @@ static inline void dupreq_entry_put(dupreq_entry_t *dv)
 
 	refcnt = atomic_dec_uint32_t(&dv->refcnt);
 
+	assert(refcnt >= 0);
+
 	/* If ref count is zero, no one should be accessing it other
 	 * than us.  so no lock is needed.
 	 */
@@ -1082,7 +1104,7 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs)
 			/* cached request */
 			nfs_dupreq_free_dupreq(dk);
 			dv = opr_containerof(nv, dupreq_entry_t, rbt_k);
-			PTHREAD_MUTEX_lock(&dv->mtx);
+			PTHREAD_MUTEX_lock(&dv->dre_mtx);
 
 			/* Count the number of duplicate requests received. */
 			dv->dupe_cnt++;
@@ -1102,7 +1124,7 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs)
 				/* Signal to nfs_dupreq_rele this should be
 				 * ignored.
 				 */
-				reqnfs->svc.rq_u1 = DUPREQ_NOCACHE;
+				reqnfs->svc.rq_u1 = DUPREQ_NOCACHE_NORES;
 				LogDebug(COMPONENT_DUPREQ,
 					 "dupreq hit dv=%p, dv xid=%" PRIu32
 					 " cksum %" PRIu64
@@ -1110,7 +1132,7 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs)
 					 dv, dv->hin.tcp.rq_xid, dv->hk,
 					 dupreq_state_table[dv->complete],
 					 dv->dupe_cnt);
-				PTHREAD_MUTEX_unlock(&dv->mtx);
+				PTHREAD_MUTEX_unlock(&dv->dre_mtx);
 				PTHREAD_MUTEX_unlock(&t->mtx);
 				return DUPREQ_DROP;
 			} else {
@@ -1129,12 +1151,12 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs)
 				 dupreq_state_table[dv->complete],
 				 dv->dupe_cnt);
 
-			PTHREAD_MUTEX_unlock(&dv->mtx);
+			PTHREAD_MUTEX_unlock(&dv->dre_mtx);
 
-			PTHREAD_MUTEX_lock(&drc->mtx);
+			PTHREAD_MUTEX_lock(&drc->drc_mtx);
 			/* Extend window */
 			drc_inc_retwnd(drc);
-			PTHREAD_MUTEX_unlock(&drc->mtx);
+			PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 		} else {
 			/* new request */
 			reqnfs->svc.rq_u1 = dk;
@@ -1151,10 +1173,10 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs)
 			dk->refcnt = 2;
 
 			/* add to q tail */
-			PTHREAD_MUTEX_lock(&drc->mtx);
+			PTHREAD_MUTEX_lock(&drc->drc_mtx);
 			TAILQ_INSERT_TAIL(&drc->dupreq_q, dk, fifo_q);
 			++(drc->size);
-			PTHREAD_MUTEX_unlock(&drc->mtx);
+			PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 
 			LogFullDebug(COMPONENT_DUPREQ,
 				     "starting dk=%p xid=%" PRIu32
@@ -1207,10 +1229,10 @@ void nfs_dupreq_finish(nfs_request_t *reqnfs, enum nfs_req_result rc)
 	int16_t cnt = 0;
 
 	/* do nothing if req is marked no-cache */
-	if (dv == DUPREQ_NOCACHE)
+	if (dv == DUPREQ_NOCACHE || dv == DUPREQ_NOCACHE_NORES)
 		return;
 
-	PTHREAD_MUTEX_lock(&dv->mtx);
+	PTHREAD_MUTEX_lock(&dv->dre_mtx);
 
 	assert(dv->res == reqnfs->res_nfs);
 
@@ -1223,17 +1245,17 @@ void nfs_dupreq_finish(nfs_request_t *reqnfs, enum nfs_req_result rc)
 		 * Dequeueing the next duplicate will be done by
 		 * nfs_dupreq_rele().
 		 */
-		PTHREAD_MUTEX_unlock(&dv->mtx);
+		PTHREAD_MUTEX_unlock(&dv->dre_mtx);
 		return;
 	}
 
 	dv->complete = true;
 	dv->rc = rc;
 
-	PTHREAD_MUTEX_unlock(&dv->mtx);
+	PTHREAD_MUTEX_unlock(&dv->dre_mtx);
 
 	drc = reqnfs->svc.rq_xprt->xp_u2; /* req holds a ref on drc */
-	PTHREAD_MUTEX_lock(&drc->mtx);
+	PTHREAD_MUTEX_lock(&drc->drc_mtx);
 
 	LogFullDebug(COMPONENT_DUPREQ,
 		     "completing dv=%p xid=%" PRIu32
@@ -1258,9 +1280,9 @@ dq_again:
 			 * order is partition lock followed by drc lock.
 			 * Drop drc lock and reacquire it!
 			 */
-			PTHREAD_MUTEX_unlock(&drc->mtx);
+			PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 			PTHREAD_MUTEX_lock(&t->mtx);	/* partition lock */
-			PTHREAD_MUTEX_lock(&drc->mtx);
+			PTHREAD_MUTEX_lock(&drc->drc_mtx);
 
 			/* Since we dropped drc lock and reacquired it,
 			 * the drc dupreq list may have changed. Get the
@@ -1280,7 +1302,7 @@ dq_again:
 			TAILQ_REMOVE(&drc->dupreq_q, ov, fifo_q);
 			TAILQ_INIT_ENTRY(ov, fifo_q);
 			--(drc->size);
-			PTHREAD_MUTEX_unlock(&drc->mtx);
+			PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 
 			rbtree_x_cached_remove(&drc->xt, t, &ov->rbt_k, ov->hk);
 
@@ -1298,7 +1320,7 @@ dq_again:
 
 			/* conditionally retire another */
 			if (cnt++ < DUPREQ_MAX_RETRIES) {
-				PTHREAD_MUTEX_lock(&drc->mtx);
+				PTHREAD_MUTEX_lock(&drc->drc_mtx);
 				goto dq_again; /* calls drc_should_retire() */
 			}
 			return;
@@ -1306,7 +1328,7 @@ dq_again:
 	}
 
  unlock:
-	PTHREAD_MUTEX_unlock(&drc->mtx);
+	PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 }
 
 /**
@@ -1331,14 +1353,14 @@ void nfs_dupreq_delete(nfs_request_t *reqnfs, enum nfs_req_result rc)
 	drc_t *drc;
 
 	/* do nothing if req is marked no-cache */
-	if (dv == DUPREQ_NOCACHE)
+	if (dv == DUPREQ_NOCACHE || dv == DUPREQ_NOCACHE_NORES)
 		return;
 
 	/* Check if this entry has any duplicate requests queued against it. If
 	 * so, we will want to resubmit them and NOT delete this drc. It will
 	 * attach as primary to the next request in line.
 	 */
-	PTHREAD_MUTEX_lock(&dv->mtx);
+	PTHREAD_MUTEX_lock(&dv->dre_mtx);
 
 	if (!TAILQ_EMPTY(&dv->dupes)) {
 		/* Return without deleting, we are going to retry this request.
@@ -1351,11 +1373,11 @@ void nfs_dupreq_delete(nfs_request_t *reqnfs, enum nfs_req_result rc)
 		 */
 		dv->rc = rc;
 
-		PTHREAD_MUTEX_unlock(&dv->mtx);
+		PTHREAD_MUTEX_unlock(&dv->dre_mtx);
 		return;
 	}
 
-	PTHREAD_MUTEX_unlock(&dv->mtx);
+	PTHREAD_MUTEX_unlock(&dv->dre_mtx);
 
 	drc = reqnfs->svc.rq_xprt->xp_u2;
 
@@ -1377,15 +1399,15 @@ void nfs_dupreq_delete(nfs_request_t *reqnfs, enum nfs_req_result rc)
 	 * req holds a ref on drc, so it should be valid here.
 	 * assert(drc == (drc_t *)reqnfs->svc.rq_xprt->xp_u2);
 	 */
-	PTHREAD_MUTEX_lock(&drc->mtx);
+	PTHREAD_MUTEX_lock(&drc->drc_mtx);
 	if (!TAILQ_IS_ENQUEUED(dv, fifo_q)) {
-		PTHREAD_MUTEX_unlock(&drc->mtx);
+		PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 		return; /* no more in the hash table/list, nothing todo */
 	}
 	TAILQ_REMOVE(&drc->dupreq_q, dv, fifo_q);
 	TAILQ_INIT_ENTRY(dv, fifo_q);
 	--(drc->size);
-	PTHREAD_MUTEX_unlock(&drc->mtx);
+	PTHREAD_MUTEX_unlock(&drc->drc_mtx);
 
 	t = rbtx_partition_of_scalar(&drc->xt, dv->hk);
 
@@ -1411,7 +1433,11 @@ void nfs_dupreq_rele(nfs_request_t *reqnfs)
 	drc_t *drc;
 
 	/* no-cache cleanup */
-	if (dv == DUPREQ_NOCACHE) {
+	if (dv == DUPREQ_NOCACHE_NORES) {
+		/* Response was never allocated, no need to clean it up. */
+		LogFullDebug(COMPONENT_DUPREQ, "no cache, no res to free");
+		goto out;
+	} else if (dv == DUPREQ_NOCACHE) {
 		LogFullDebug(COMPONENT_DUPREQ, "releasing no-cache res %p",
 			     reqnfs->svc.rq_u2);
 		reqnfs->funcdesc->free_function(reqnfs->svc.rq_u2);
@@ -1429,7 +1455,7 @@ void nfs_dupreq_rele(nfs_request_t *reqnfs)
 	/* Now check if there are any duplicate requests waiting for this
 	 * one to resolve.
 	 */
-	PTHREAD_MUTEX_lock(&dv->mtx);
+	PTHREAD_MUTEX_lock(&dv->dre_mtx);
 
 	if (!TAILQ_EMPTY(&dv->dupes)) {
 		/* Pull the next request of the queue and resume it. If this
@@ -1452,7 +1478,7 @@ void nfs_dupreq_rele(nfs_request_t *reqnfs)
 		svc_resume(&req2->svc);
 	}
 
-	PTHREAD_MUTEX_unlock(&dv->mtx);
+	PTHREAD_MUTEX_unlock(&dv->dre_mtx);
 
 	/* release req's hold on dupreq and drc */
 	dupreq_entry_put(dv);
@@ -1463,11 +1489,3 @@ void nfs_dupreq_rele(nfs_request_t *reqnfs)
 	if (reqnfs->svc.rq_auth)
 		SVCAUTH_RELEASE(&reqnfs->svc);
 }
-
-/**
- * @brief Shutdown the dupreq2 package.
- */
-void dupreq2_pkgshutdown(void)
-{
-	/* XXX do nothing */
-}
