@@ -97,7 +97,7 @@ bool valid_lease(nfs_client_id_t *clientid)
 }
 
 /**
- * @brief Check if lease is valid and reserve it and retain cid_mutex.
+ * @brief Check if lease is valid and reserve it.
  *
  * Lease reservation prevents any other thread from expiring the lease. Caller
  * must call update lease to release the reservation.
@@ -129,6 +129,84 @@ int reserve_lease(nfs_client_id_t *clientid)
 	return valid != 0;
 }
 
+/**
+ * @brief Check if lease is valid and reserve it or expire it.
+ *
+ * Also, if valid, and update is true, update the lease.
+ *
+ * Lease reservation prevents any other thread from expiring the lease. Caller
+ * must call update lease to release the reservation.
+ *
+ * @param[in] clientid   Client record to check lease for
+ * @param[in] update     Indicate that lease should also be updated if valid
+ *
+ * @return true if lease is valid, false if not.
+ *
+ */
+bool reserve_lease_or_expire(nfs_client_id_t *clientid, bool update)
+{
+	unsigned int valid;
+
+	PTHREAD_MUTEX_lock(&clientid->cid_mutex);
+
+	valid = _valid_lease(clientid);
+
+	if (valid != 0)
+		clientid->cid_lease_reservations++;
+
+	if (isFullDebug(COMPONENT_CLIENTID)) {
+		char str[LOG_BUFF_LEN] = "\0";
+		struct display_buffer dspbuf = {sizeof(str), str, str};
+
+		display_client_id_rec(&dspbuf, clientid);
+		LogFullDebug(COMPONENT_CLIENTID,
+			     "Reserve Lease %s (Valid=%s %u seconds left)", str,
+			     valid ? "YES" : "NO", valid);
+	}
+
+	if (valid == 0) {
+		/* Expire the lease */
+		/* Get the client record. */
+		nfs_client_record_t *client_rec =  clientid->cid_client_record;
+
+		/* get a ref to client_id as we might drop the
+		* last reference with expiring.
+		*/
+		inc_client_id_ref(clientid);
+
+		/* if record is STALE, the linkage to client_record is
+		* removed already. Acquire a ref on client record
+		* before we drop the mutex on clientid
+		*/
+		if (client_rec != NULL)
+			inc_client_record_ref(client_rec);
+
+		PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
+
+		if (client_rec != NULL)
+			PTHREAD_MUTEX_lock(&client_rec->cr_mutex);
+
+		nfs_client_id_expire(clientid, false);
+
+		if (client_rec != NULL) {
+			PTHREAD_MUTEX_unlock(&client_rec->cr_mutex);
+			dec_client_record_ref(client_rec);
+		}
+
+		/* drop our reference to the client_id */
+		dec_client_id_ref(clientid);
+
+		return false;
+	}
+
+	if (update)
+		update_lease(clientid);
+
+	PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
+
+	return true;
+}
+
 /**
  * @brief Release a lease reservation and update lease.
  *
