@@ -59,7 +59,7 @@
 
 #ifdef DEBUG_SAL
 struct glist_head state_v4_all = GLIST_HEAD_INIT(state_v4_all);
-pthread_mutex_t all_state_v4_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t all_state_v4_mutex;
 #endif
 
 /**
@@ -99,6 +99,8 @@ state_status_t _state_add_impl(struct fsal_obj_handle *obj,
 	state_status_t status = 0;
 	bool mutex_init = false;
 	struct state_t *openstate = NULL;
+	nfs_client_id_t *clientid =
+		owner_input->so_owner.so_nfs4_owner.so_clientrec;
 
 	if (isFullDebug(COMPONENT_STATE) && pnew_state != NULL) {
 		display_stateid(&dspbuf, pnew_state);
@@ -199,17 +201,17 @@ state_status_t _state_add_impl(struct fsal_obj_handle *obj,
 	 */
 
 	/* Attach this to an export */
-	PTHREAD_RWLOCK_wrlock(&op_ctx->ctx_export->lock);
+	PTHREAD_RWLOCK_wrlock(&op_ctx->ctx_export->exp_lock);
 	PTHREAD_MUTEX_lock(&pnew_state->state_mutex);
 	glist_add_tail(&op_ctx->ctx_export->exp_state_list,
 		&pnew_state->state_export_list);
 	PTHREAD_MUTEX_unlock(&pnew_state->state_mutex);
-	PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->lock);
+	PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->exp_lock);
 
 	/* Add state to list for file */
 	PTHREAD_MUTEX_lock(&pnew_state->state_mutex);
 	glist_add_tail(&ostate->file.list_of_states, &pnew_state->state_list);
-	/* Get ref for this state entry */
+	/* Get active ref for this state entry */
 	obj->obj_ops->get_ref(obj);
 	PTHREAD_MUTEX_unlock(&pnew_state->state_mutex);
 
@@ -240,14 +242,12 @@ state_status_t _state_add_impl(struct fsal_obj_handle *obj,
 
 	if (pnew_state->state_type == STATE_TYPE_DELEG &&
 	    pnew_state->state_data.deleg.sd_type == OPEN_DELEGATE_WRITE) {
-		nfs_client_id_t *client =
-			 owner_input->so_owner.so_nfs4_owner.so_clientrec;
 		ostate->file.write_delegated = true;
 		/* take a ref and save client ptr holding delegation for
 		 * conflict resolution
 		 */
-		inc_client_id_ref(client);
-		ostate->file.write_deleg_client = client;
+		inc_client_id_ref(clientid);
+		ostate->file.write_deleg_client = clientid;
 	}
 
 	/* Copy the result */
@@ -272,8 +272,7 @@ errout:
 		 */
 		(void) obj->obj_ops->close2(obj, pnew_state);
 
-		op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export,
-							pnew_state);
+		free_state(pnew_state);
 	}
 
 	if (got_export_ref)
@@ -306,7 +305,8 @@ state_status_t _state_add(struct fsal_obj_handle *obj,
 	state_status_t status = 0;
 
 	/* Ensure that states are associated only with the appropriate
-	   owners */
+	 * owners
+	 */
 
 	if (((state_type == STATE_TYPE_SHARE)
 	     && (owner_input->so_type != STATE_OPEN_OWNER_NFSV4))
@@ -462,11 +462,22 @@ void _state_del_locked(state_t *state, const char *func, int line)
 	if (state->state_type == STATE_TYPE_LOCK)
 		glist_del(&state->state_data.lock.state_sharelist);
 
-	if (state->state_type == STATE_TYPE_SHARE)
+	if (state->state_type == STATE_TYPE_SHARE) {
 		assert(glist_empty(&state->state_data.share.share_lockstates));
+		/* Write open fd count should be decremented,
+		 * if the client doesn't close the file.
+		 * Else this could result in a file never being delegated
+		 */
+		if ((state->state_data.share.share_access &
+			OPEN4_SHARE_ACCESS_WRITE) &&
+			obj->type == REGULAR_FILE) {
+			obj->state_hdl->file.fdeleg_stats.fds_num_write_opens--;
+		}
+	}
 
 	/* Reset write delegated and release client ref if this is a
-	 * write delegation */
+	 * write delegation
+	 */
 	if (state->state_type == STATE_TYPE_DELEG &&
 	    state->state_data.deleg.sd_type == OPEN_DELEGATE_WRITE &&
 	    obj->state_hdl->file.write_deleg_client) {
@@ -475,18 +486,31 @@ void _state_del_locked(state_t *state, const char *func, int line)
 		obj->state_hdl->file.write_deleg_client = NULL;
 	}
 
+	/* Clean up delegation related flags if file have no active states */
+	if (state->state_type == STATE_TYPE_DELEG &&
+	    state->state_data.deleg.sd_type == OPEN_DELEGATE_READ &&
+	    glist_empty(&obj->state_hdl->file.list_of_states)) {
+		LogEvent(COMPONENT_STATE,
+			"Resetting Deleg Stats(%d/%d) as file have no active states",
+			obj->state_hdl->file.fdeleg_stats.fds_curr_delegations,
+			obj->state_hdl->file.fdeleg_stats.fds_deleg_type);
+		obj->state_hdl->file.fdeleg_stats.fds_curr_delegations = 0;
+		obj->state_hdl->file.fdeleg_stats.fds_deleg_type =
+			OPEN_DELEGATE_NONE;
+	}
+
 	/* Remove from list of states for a particular export.
 	 * In this case, it is safe to look at state_export without yet
 	 * holding the state_mutex because this is the only place where it
 	 * is removed, and we have guaranteed we are the only thread
 	 * proceeding with state deletion.
 	 */
-	PTHREAD_RWLOCK_wrlock(&export->lock);
+	PTHREAD_RWLOCK_wrlock(&export->exp_lock);
 	PTHREAD_MUTEX_lock(&state->state_mutex);
 	glist_del(&state->state_export_list);
 	state->state_export = NULL;
 	PTHREAD_MUTEX_unlock(&state->state_mutex);
-	PTHREAD_RWLOCK_unlock(&export->lock);
+	PTHREAD_RWLOCK_unlock(&export->exp_lock);
 	put_gsh_export(export);
 
 #ifdef DEBUG_SAL
@@ -500,7 +524,7 @@ void _state_del_locked(state_t *state, const char *func, int line)
 	/* Remove from the list of states for a particular file */
 	PTHREAD_MUTEX_lock(&state->state_mutex);
 	glist_del(&state->state_list);
-	/* Put ref for this state entry */
+	/* Put active ref for this state entry */
 	obj->obj_ops->put_ref(obj);
 	state->state_obj = NULL;
 	PTHREAD_MUTEX_unlock(&state->state_mutex);
@@ -552,7 +576,7 @@ void state_del(state_t *state)
  * @param[in,out] owner Place to return the owner (NULL if not desired)
  *
  * @retval true if all desired references were taken
- * @retval flase otherwise (in which case no references are taken)
+ * @retval false otherwise (in which case no references are taken)
  *
  * For convenience, returns false if state is NULL which helps simplify
  * code for some callers.
@@ -674,6 +698,7 @@ void state_nfs4_state_wipe(struct state_hdl *ostate)
 enum nfsstat4 release_lock_owner(state_owner_t *owner)
 {
 	struct saved_export_context saved;
+	bool ok;
 
 	PTHREAD_MUTEX_lock(&owner->so_mutex);
 
@@ -700,6 +725,8 @@ enum nfsstat4 release_lock_owner(state_owner_t *owner)
 
 	while (true) {
 		state_t *state;
+		struct fsal_obj_handle *obj = NULL;
+		struct gsh_export *exp = NULL;
 
 		state = glist_first_entry(&owner->so_owner.so_nfs4_owner
 								.so_state_list,
@@ -713,26 +740,47 @@ enum nfsstat4 release_lock_owner(state_owner_t *owner)
 			return NFS4_OK;
 		}
 
+		/* Get references to the file and export */
+		ok = get_state_obj_export_owner_refs(state, &obj, &exp, NULL);
+
+		if (!ok) {
+			/* The file, export, or state must be about to
+			 * die, skip for now.
+			 */
+
+			PTHREAD_MUTEX_unlock(&owner->so_mutex);
+			continue;
+		}
+
 		/* Make sure the state doesn't go away on us... */
 		inc_state_t_ref(state);
 
 		PTHREAD_MUTEX_unlock(&owner->so_mutex);
 
-		/* Set the op_context properly, since this can be called from
-		 * ops that don't do a putfh
+		STATELOCK_lock(obj);
+
+		/* op_ctx may be used by state_del_locked and others set export
+		 * from the state and release any old ctx_export reference.
+		 * Reference was taken above and will be release by
+		 * clear_op_context_export below.
 		 */
-		get_gsh_export_ref(state->state_export);
-		set_op_context_export(state->state_export);
+		set_op_context_export(exp);
+
+		/* If FSAL supports extended operations, file will be closed by
+		 * state_del_locked.
+		 */
+
+		state_del_locked(state);
 
-		state_del(state);
 		dec_state_t_ref(state);
 
+		STATELOCK_unlock(obj);
+
 		PTHREAD_MUTEX_lock(&owner->so_mutex);
 
-		/* Note the op context set above will either be cleaned up by
-		 * the set_op_context_export_fsal for the next state or by
-		 * restore_op_context_export just before exiting the loop.
-		 */
+		/* Release refs we held during state_del */
+		obj->obj_ops->put_ref(obj);
+		clear_op_context_export();
 	}
 }
 
@@ -973,7 +1021,7 @@ static void release_export_nfs4_state(enum state_type type)
 
  again:
 	first = NULL;
-	PTHREAD_RWLOCK_wrlock(&op_ctx->ctx_export->lock);
+	PTHREAD_RWLOCK_wrlock(&op_ctx->ctx_export->exp_lock);
 	hold_export_lock = true;
 
 	glist_for_each_safe(glist, glistn,
@@ -1018,7 +1066,7 @@ static void release_export_nfs4_state(enum state_type type)
 
 		inc_state_t_ref(state);
 
-		PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->lock);
+		PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->exp_lock);
 		hold_export_lock = false;
 
 		if (type == STATE_TYPE_LAYOUT) {
@@ -1064,7 +1112,7 @@ static void release_export_nfs4_state(enum state_type type)
 	}
 
 	if (hold_export_lock)
-		PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->lock);
+		PTHREAD_RWLOCK_unlock(&op_ctx->ctx_export->exp_lock);
 
 	if (errcnt == STATE_ERR_MAX) {
 		LogFatal(COMPONENT_STATE,
