@@ -500,6 +500,8 @@ void dec_nfs4_state_ref(struct state_t *state)
 
 	refcount = atomic_dec_int32_t(&state->state_refcount);
 
+	assert(refcount >= 0);
+
 	if (refcount > 0) {
 		if (str_valid)
 			LogFullDebug(COMPONENT_STATE,
@@ -511,7 +513,7 @@ void dec_nfs4_state_ref(struct state_t *state)
 
 	PTHREAD_MUTEX_destroy(&state->state_mutex);
 
-	op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export, state);
+	free_state(state);
 
 	if (str_valid)
 		LogFullDebug(COMPONENT_STATE, "Deleted %s", str);
@@ -629,7 +631,18 @@ struct state_t *nfs4_State_Get_Pointer(char *other)
 	buffkey.addr = other;
 	buffkey.len = OTHERSIZE;
 
-	rc = hashtable_getlatch(ht_state_id, &buffkey, &buffval, true, &latch);
+	/* Acquire hashtable latch with may_write == false, as we are not
+	 * updating hashtable data structures but rather we are atomically
+	 * incrementing state_t refcount below. This allows concurrent reads
+	 * from this data structure without contending on hashtable writer
+	 * lock and since the state_t refcount itself is atomic, we are
+	 * thread safe. Furthermore, all writes to the ht_state_id hashtable
+	 * are private to this implementation file and are done under may_write
+	 * lock, so that calls to decrement the state reference count would
+	 * be done under exclusive writer lock.
+	 */
+	rc = hashtable_getlatch(ht_state_id, &buffkey, &buffval,
+				false /* may_write */, &latch);
 
 	if (rc != HASHTABLE_SUCCESS) {
 		if (rc == HASHTABLE_ERROR_NO_SUCH_KEY)
@@ -640,10 +653,13 @@ struct state_t *nfs4_State_Get_Pointer(char *other)
 
 	state = buffval.addr;
 
-	/* Take a reference under latch */
+	/* Take an atomic reference under latch. Hashtable is currently read
+	 * locked, so this state can't be invalidated while this atomic
+	 * increment is occurring.
+	 */
 	inc_state_t_ref(state);
 
-	/* Release latch */
+	/* Release latch, which releases hashtable read lock */
 	hashtable_releaselatched(ht_state_id, &latch);
 
 	return state;
@@ -899,7 +915,10 @@ nfsstat4 nfs4_Check_Stateid(stateid4 *stateid, struct fsal_obj_handle *fsal_obj,
 			LogDebug(COMPONENT_STATE,
 				 "Check %s stateid found stale stateid %s",
 				 tag, str);
-		status = NFS4ERR_STALE_STATEID;
+		if (data->minorversion == 0)
+			status = NFS4ERR_STALE_STATEID;
+		else
+			status = NFS4ERR_BAD_STATEID;
 		goto failure;
 	}
 
@@ -955,29 +974,32 @@ nfsstat4 nfs4_Check_Stateid(stateid4 *stateid, struct fsal_obj_handle *fsal_obj,
 				data->preserved_clientid = NULL;
 			}
 
-			/* Check if lease is expired and reserve it */
-			PTHREAD_MUTEX_lock(&pclientid->cid_mutex);
+			/* Check if lease is expired and reserve it. If it's
+			 * valid and this is STATEID_SPECIAL_CLOSE_40, update
+			 * the lease.
+			 */
+			if (!reserve_lease_or_expire(
+				    pclientid,
+				    (flags & STATEID_SPECIAL_CLOSE_40) != 0)) {
 
-			if (!reserve_lease(pclientid)) {
 				LogDebug(COMPONENT_STATE,
 					 "Returning NFS4ERR_EXPIRED");
-				PTHREAD_MUTEX_unlock(&pclientid->cid_mutex);
+
+				/* Release the clientid reference we just
+				 * acquired.
+				 */
+				dec_client_id_ref(pclientid);
 				status = NFS4ERR_EXPIRED;
 				goto failure;
 			}
 
-			if ((flags & STATEID_SPECIAL_CLOSE_40) != 0) {
-				/* Just update the lease and leave the reserved
-				 * clientid NULL.
-				 */
-				update_lease(pclientid);
-			} else {
-				/* Remember the reserved clientid for the rest
-				 * of the compound.
+			if ((flags & STATEID_SPECIAL_CLOSE_40) == 0) {
+				/* Not STATEID_SPECIAL_CLOSE_40, remember the
+				 * reserved clientid for the rest of the
+				 * compound.
 				 */
 				data->preserved_clientid = pclientid;
 			}
-			PTHREAD_MUTEX_unlock(&pclientid->cid_mutex);
 
 			/* Replayed close, it's ok, but stateid doesn't exist */
 			LogDebug(COMPONENT_STATE,
@@ -994,13 +1016,13 @@ nfsstat4 nfs4_Check_Stateid(stateid4 *stateid, struct fsal_obj_handle *fsal_obj,
 			 * can distinguish between the state_t being in the
 			 * midst of tear down due to expired lease or if
 			 * in fact the entry is actually stale.
+			 *
+			 * If it's valid, Just update the lease and leave the
+			 * reserved clientid NULL.
 			 */
-			PTHREAD_MUTEX_lock(&pclientid->cid_mutex);
-
-			if (!reserve_lease(pclientid)) {
+			if (!reserve_lease_or_expire(pclientid, true)) {
 				LogDebug(COMPONENT_STATE,
 					 "Returning NFS4ERR_EXPIRED");
-				PTHREAD_MUTEX_unlock(&pclientid->cid_mutex);
 
 				/* Release the clientid reference we just
 				 * acquired.
@@ -1010,12 +1032,6 @@ nfsstat4 nfs4_Check_Stateid(stateid4 *stateid, struct fsal_obj_handle *fsal_obj,
 				goto failure;
 			}
 
-			/* Just update the lease and leave the reserved
-			 * clientid NULL.
-			 */
-			update_lease(pclientid);
-			PTHREAD_MUTEX_unlock(&pclientid->cid_mutex);
-
 			/* The lease was valid, so this must be a stale
 			 * entry.
 			 */
@@ -1046,25 +1062,17 @@ nfsstat4 nfs4_Check_Stateid(stateid4 *stateid, struct fsal_obj_handle *fsal_obj,
 		}
 
 		/* Check if lease is expired and reserve it */
-		PTHREAD_MUTEX_lock(
-		    &owner2->so_owner.so_nfs4_owner.so_clientrec->cid_mutex);
-
-		if (!reserve_lease(
-				owner2->so_owner.so_nfs4_owner.so_clientrec)) {
+		if (!reserve_lease_or_expire(
+				owner2->so_owner.so_nfs4_owner.so_clientrec,
+				false)) {
 			LogDebug(COMPONENT_STATE, "Returning NFS4ERR_EXPIRED");
 
-			PTHREAD_MUTEX_unlock(&owner2->so_owner.so_nfs4_owner
-					     .so_clientrec->cid_mutex);
-
 			status = NFS4ERR_EXPIRED;
 			goto failure;
 		}
 
 		data->preserved_clientid =
 		    owner2->so_owner.so_nfs4_owner.so_clientrec;
-
-		PTHREAD_MUTEX_unlock(
-		    &owner2->so_owner.so_nfs4_owner.so_clientrec->cid_mutex);
 	}
 
 	/* Sanity check : Is this the right file ? */
