@@ -782,6 +782,8 @@ void _dec_nsm_client_ref(state_nsm_client_t *client,
 
 	refcount = atomic_dec_int32_t(&client->ssc_refcount);
 
+	assert(refcount >= 0);
+
 	if (refcount > 0) {
 		if (str_valid) {
 			DisplayLogComponentLevel(
@@ -934,12 +936,11 @@ state_nsm_client_t *get_nsm_client(care_t care,  char *caller_name)
 	switch (rc) {
 	case HASHTABLE_SUCCESS:
 		pclient = buffval.addr;
-		if (atomic_inc_int32_t(&pclient->ssc_refcount) == 1) {
+		if (atomic_inc_unless_0_int32_t(&pclient->ssc_refcount) == 0) {
 			/* This nsm client is in the process of getting
 			 * deleted. Delete it from the hash table and
 			 * pretend as though we didn't find it.
 			 */
-			(void)atomic_dec_int32_t(&pclient->ssc_refcount);
 			hashtable_deletelatched(ht_nsm_client, &buffkey,
 						&latch, NULL, NULL);
 			break;
@@ -1102,6 +1103,8 @@ void dec_nlm_client_ref(state_nlm_client_t *client)
 
 	refcount = atomic_dec_int32_t(&client->slc_refcount);
 
+	assert(refcount >= 0);
+
 	if (refcount > 0) {
 		if (str_valid)
 			LogFullDebug(COMPONENT_STATE,
@@ -1224,8 +1227,8 @@ state_nlm_client_t *get_nlm_client(care_t care, SVCXPRT *xprt,
 			LogFullDebug(COMPONENT_STATE, "Found {%s}", str);
 		}
 
-		refcount = atomic_inc_int32_t(&pclient->slc_refcount);
-		if (refcount == 1) {
+		refcount = atomic_inc_unless_0_int32_t(&pclient->slc_refcount);
+		if (refcount == 0) {
 			/* This nlm client is in the process of getting
 			 * deleted. Let us delete it from the hash table
 			 * and pretend as though it isn't found in the
@@ -1233,7 +1236,6 @@ state_nlm_client_t *get_nlm_client(care_t care, SVCXPRT *xprt,
 			 * delete this entry will not find it in the
 			 * hash table but will free its nlm client.
 			 */
-			(void)atomic_dec_int32_t(&pclient->slc_refcount);
 			hashtable_deletelatched(ht_nlm_client, &buffkey,
 						&latch, NULL, NULL);
 			goto not_found;
@@ -1346,6 +1348,7 @@ void free_nlm_owner(state_owner_t *owner)
  */
 static void init_nlm_owner(state_owner_t *owner)
 {
+	owner->so_refcount = 1;
 	inc_nlm_client_ref(owner->so_owner.so_nlm_owner.so_client);
 
 	glist_init(&owner->so_owner.so_nlm_owner.so_nlm_shares);
