@@ -255,6 +255,8 @@ void dec_nlm_state_ref(state_t *state)
 
 	refcount = atomic_dec_int32_t(&state->state_refcount);
 
+	assert(refcount >= 0);
+
 	if (refcount > 0) {
 		if (str_valid)
 			LogFullDebug(COMPONENT_STATE,
@@ -311,18 +313,22 @@ void dec_nlm_state_ref(state_t *state)
 	if (obj == NULL) {
 		LogDebug(COMPONENT_STATE,
 			 "Entry for state is stale");
-		return;
+	} else {
+		/* We need to close the state before freeing the state. */
+		(void) obj->obj_ops->close2(obj, state);
+
+		/* Release the reference just taken */
+		obj->obj_ops->put_ref(obj);
 	}
 
-	/* We need to close the state before freeing the state. */
-	(void) obj->obj_ops->close2(obj, state);
+	PTHREAD_MUTEX_destroy(&state->state_mutex);
 
-	op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export, state);
+	free_state(state);
 
-	/* Release 2 refs: our sentinel one, plus the one from
-	 * get_state_obj_ref() */
-	obj->obj_ops->put_ref(obj);
-	obj->obj_ops->put_ref(obj);
+	if (obj != NULL) {
+		/* Release the active reference */
+		obj->obj_ops->put_ref(obj);
+	}
 }
 
 /**
@@ -391,12 +397,11 @@ int get_nlm_state(enum state_type state_type,
 			break;
 		}
 
-		if (atomic_inc_int32_t(&state->state_refcount) == 1) {
+		if (atomic_inc_unless_0_int32_t(&state->state_refcount) == 0) {
 			/* The state is in the process of getting
 			 * deleted. Delete from the hashtable and
 			 * pretend as though we didn't find it.
 			 */
-			(void)atomic_dec_int32_t(&state->state_refcount);
 			hashtable_deletelatched(ht_nlm_states, &buffkey,
 						&latch, NULL, NULL);
 			break;
@@ -462,6 +467,7 @@ int get_nlm_state(enum state_type state_type,
 	buffval.addr = state;
 	buffval.len = sizeof(*state);
 
+	/* Get active ref for this state entry */
 	state_obj->obj_ops->get_ref(state_obj);
 
 	rc = hashtable_setlatched(ht_nlm_states, &buffval, &buffval, &latch,
@@ -479,8 +485,7 @@ int get_nlm_state(enum state_type state_type,
 		/* Free the ref taken above and the state.
 		 * No need to close here, the state was never opened.
 		 */
-		op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export,
-							state);
+		free_state(state);
 		state_obj->obj_ops->put_ref(state_obj);
 
 		*pstate = NULL;
