@@ -196,6 +196,8 @@ static int fs_ng_create_recov_dir(void)
 		return -error;
 	}
 
+	v4_recov_dir_len = err;
+
 	newdir = mkdtemp(v4_recov_dir);
 	if (newdir != v4_recov_dir) {
 		LogEvent(COMPONENT_CLIENTID,
@@ -369,18 +371,27 @@ static void fs_ng_read_recov_clids(nfs_grace_start_t *gsp,
 				  add_clid_entry_hook add_clid_entry,
 				  add_rfh_entry_hook add_rfh_entry)
 {
-	int rc;
-	char path[PATH_MAX];
 
 	if (!gsp) {
 		fs_ng_read_recov_clids_recover(add_clid_entry, add_rfh_entry);
 		return;
 	}
 
-	/*
-	 * FIXME: make the rest of this work
-	 */
+#ifndef FIXME
 	return;
+#else
+	/**
+	 * More investigation/design is required to determine the best
+	 * implementation for the remainder of this method. In the interim,
+	 * Coverity complains that this is dead code because of the early
+	 * return above. Hence the remainder of this method is ifdef'ed
+	 * out to avoid the Coverity complaint.
+	 *
+	 *  @todo: FIXME: make the rest of this work
+	 */
+
+	int rc;
+	char path[PATH_MAX];
 
 	switch (gsp->event) {
 	case EVENT_TAKE_NODEID:
@@ -418,6 +429,7 @@ static void fs_ng_read_recov_clids(nfs_grace_start_t *gsp,
 			 "Failed to read v4 recovery dir (%s)", path);
 		return;
 	}
+#endif
 }
 
 static void fs_ng_swap_recov_dir(void)
