@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -72,7 +73,7 @@ void rados_kv_add_revoke_fh(nfs_client_id_t *delr_clid, nfs_fh4 *delr_handle);
 /**
  * Convert a clientid into a rados key.
  *
- * @param(in)     clientid  The clientid to conevrt into a key
+ * @param(in)     clientid  The clientid to convert into a key
  * @param(in/out) key       The string buffer to put the key in
  * @param(in)     size      Buffer size - expected to be RADOS_KEY_MAX_LEN
  */
@@ -94,4 +95,5 @@ void rados_kv_add_revoke_fh(nfs_client_id_t *delr_clid, nfs_fh4 *delr_handle);
 
 void rados_ng_pop_clid_entry(char *key, char *val, size_t val_len,
 			     struct pop_args *pop_args);
+int rados_kv_get_nodeid(char **pnodeid);
 #endif	/* _RECOVERY_RADOS_H */
