@@ -53,13 +53,13 @@
 
 struct glist_head cached_open_owners = GLIST_HEAD_INIT(cached_open_owners);
 
-pthread_mutex_t cached_open_owners_lock = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t cached_open_owners_lock;
 
 pool_t *state_owner_pool;	/*< Pool for NFSv4 files's open owner */
 
 #ifdef DEBUG_SAL
 struct glist_head state_owners_all = GLIST_HEAD_INIT(state_owners_all);
-pthread_mutex_t all_state_owners_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t all_state_owners_mutex;
 #endif
 
 /* Error conversion routines */
@@ -149,8 +149,6 @@ const char *state_err_str(state_status_t err)
 		return "STATE_FILE_BIG";
 	case STATE_GRACE_PERIOD:
 		return "STATE_GRACE_PERIOD";
-	case STATE_CACHE_INODE_ERR:
-		return "STATE_CACHE_INODE_ERR";
 	case STATE_SIGNAL_ERROR:
 		return "STATE_SIGNAL_ERROR";
 	case STATE_FILE_OPEN:
@@ -480,7 +478,6 @@ nfsstat4 nfs4_Errno_state(state_status_t error)
 		break;
 
 	case STATE_INVALID_ARGUMENT:
-	case STATE_CACHE_INODE_ERR:
 	case STATE_INCONSISTENT_ENTRY:
 	case STATE_HASH_TABLE_ERROR:
 	case STATE_ASYNC_POST_ERROR:
@@ -635,7 +632,6 @@ nfsstat3 nfs3_Errno_state(state_status_t error)
 		nfserror = NFS3ERR_BADHANDLE;
 		break;
 
-	case STATE_CACHE_INODE_ERR:
 	case STATE_INCONSISTENT_ENTRY:
 	case STATE_HASH_TABLE_ERROR:
 	case STATE_ASYNC_POST_ERROR:
@@ -1088,13 +1084,12 @@ state_owner_t *get_state_owner(care_t care, state_owner_t *key,
 	switch (rc) {
 	case HASHTABLE_SUCCESS:
 		owner = buffval.addr;
-		refcount = atomic_inc_int32_t(&owner->so_refcount);
-		if (refcount == 1) {
+		refcount = atomic_inc_unless_0_int32_t(&owner->so_refcount);
+		if (refcount == 0) {
 			/* This owner is in the process of getting freed.
 			 * Delete from the hash table and pretend as
 			 * though we didn't find it!
 			 */
-			(void)atomic_dec_int32_t(&owner->so_refcount);
 			hashtable_deletelatched(ht_owner, &buffkey, &latch,
 						NULL, NULL);
 			goto not_found;
@@ -1177,8 +1172,6 @@ not_found:
 
 	glist_init(&owner->so_lock_list);
 
-	owner->so_refcount = 1;
-
 	if (isFullDebug(COMPONENT_STATE)) {
 		display_reset_buffer(&dspbuf);
 		display_owner(&dspbuf, owner);
@@ -1224,63 +1217,39 @@ not_found:
  * inc_state_owner_ref() can be called to hold a reference provided
  * someone already has a valid refcount. In other words, one can't call
  * inc_state_owner_ref on a owner whose refcount has possibly gone to
- * zero.
+ * zero so we use atomic_inc_unless_0_int32_t which will not increment
+ * the refcount if it is 0. Since that returns either 0, or the new refcount,
+ * we can easily detect the case where a reference was NOT taken.
  *
- * If we don't know for sure if the owner refcount is positive, this
- * function must be called to hold a reference.  This function places a
- * reference if the owner has a positive refcount already.
- *
- * @return True if we are able to hold a reference, False otherwise.
+ * @return true if we are able to hold a reference, false otherwise.
  */
-bool hold_state_owner(state_owner_t *owner)
+bool hold_state_owner_ref(state_owner_t *owner)
 {
 	char str[LOG_BUFF_LEN] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	bool str_valid = false;
-	hash_table_t *ht_owner;
-	struct gsh_buffdesc buffkey;
-	struct hash_latch latch;
-	hash_error_t rc;
 	int32_t refcount;
 
-	/* We need to increment and possibly decrement the refcount
-	 * atomically. This "increment" to check for the owner refcount
-	 * condition is also done in get_state_owner. Those are done
-	 * under the hash lock, so we do the same here.
-	 */
-	ht_owner = get_state_owner_hash_table(owner);
-	if (ht_owner == NULL) {
-		if (!str_valid)
-			display_owner(&dspbuf, owner);
-		LogCrit(COMPONENT_STATE,
-			"ht=%p Unexpected key {%s}", ht_owner, str);
-		return false;
+	if (isFullDebug(COMPONENT_STATE)) {
+		display_owner(&dspbuf, owner);
+		str_valid = true;
 	}
 
-	buffkey.addr = owner;
-	buffkey.len = sizeof(*owner);
+	refcount = atomic_inc_unless_0_int32_t(&owner->so_refcount);
 
-	/* We don't care if this owner is in the hashtable or not. We
-	 * just need the partition lock in exclusive mode!
-	 */
-	rc = hashtable_acquire_latch(ht_owner, &buffkey, &latch);
-	switch (rc) {
-	case HASHTABLE_SUCCESS:
-		refcount = atomic_inc_int32_t(&owner->so_refcount);
-		if (refcount == 1) {
-			/* This owner is in the process of getting freed */
-			(void)atomic_dec_int32_t(&owner->so_refcount);
-			hashtable_releaselatched(ht_owner, &latch);
-			return false;
+	if (str_valid) {
+		if (refcount == 0) {
+			LogFullDebug(COMPONENT_STATE,
+				     "Did not increment refcount from 0 {%s}",
+				     str);
+		} else {
+			LogFullDebug(COMPONENT_STATE,
+				     "Increment refcount now=%" PRId32 " {%s}",
+				     refcount, str);
 		}
-
-		hashtable_releaselatched(ht_owner, &latch);
-		return true;
-
-	default:
-		assert(0);
-		return false;
 	}
+
+	return refcount != 0;
 }
 
 /**
