@@ -242,19 +242,20 @@ void config_errs_to_log(char *err, void *dest,
 				  log_level, "%s", err);
 }
 
-void report_config_errors(struct config_error_type *err_type, void *dest,
-			  void (*logger)(char *msg, void *dest,
-					 struct config_error_type *err_type))
+int report_config_errors(struct config_error_type *err_type, void *dest,
+			 void (*logger)(char *msg, void *dest,
+					struct config_error_type *err_type))
 {
 	char *msgp, *cp;
+	int count = 0;
 
 	if (err_type->fp == NULL)
-		return;
+		return 0;
 	fclose(err_type->fp);
 	err_type->fp = NULL;
 	msgp = err_type->diag_buf;
 	if (msgp == NULL)
-		return;
+		return 0;
 	while (*msgp != '\0') {
 		cp = index(msgp, '\f');
 		if (cp != NULL) {
@@ -265,9 +266,11 @@ void report_config_errors(struct config_error_type *err_type, void *dest,
 			logger(msgp, dest, err_type);
 			break;
 		}
+		count++;
 	}
 	gsh_free(err_type->diag_buf);
 	err_type->diag_buf = NULL;
+	return count;
 }
 
 static bool convert_bool(struct config_node *node,
@@ -618,10 +621,10 @@ static void convert_inet_addr(struct config_node *node,
 
 	/* Try IPv6 (with mapping) first.  If this fails, fall back on IPv4, if
 	 * a v4 address was given. */
+	memset(&hints, 0, sizeof(struct addrinfo));
 	hints.ai_family = AF_INET6;
 	hints.ai_flags = AI_ADDRCONFIG | AI_V4MAPPED;
-	hints.ai_socktype = 0;
-	hints.ai_protocol = 0;
+
 	rc = getaddrinfo(node->u.term.varvalue, NULL,
 			 &hints, &res);
 
@@ -1363,6 +1366,11 @@ static bool proc_block(struct config_node *node,
 		err_type->errors++;
 		return false;
 	}
+
+	LogFullDebug(COMPONENT_CONFIG,
+		     "------ At (%s:%d): process block %s link_mem = %p",
+		     node->filename, node->linenumber, item->name, link_mem);
+
 	param_struct = item->u.blk.init(link_mem, NULL);
 	if (param_struct == NULL) {
 		config_proc_error(node, err_type,
@@ -1444,6 +1452,31 @@ err_out:
 	return false;
 }
 
+void find_unused_blocks(config_file_t config,
+			struct config_error_type *err_type)
+{
+	struct config_root *tree = (struct config_root *)config;
+	struct glist_head *ns;
+
+	/* We've been marking config nodes as being "seen" during the
+	 * scans.  Report the bogus and typo inflicted bits.
+	 */
+	glist_for_each(ns, &tree->root.u.nterm.sub_nodes) {
+		struct config_node *node;
+
+		node = glist_entry(ns, struct config_node, node);
+
+		if (node->found)
+			node->found = false;
+		else {
+			config_proc_error(node, err_type,
+					  "Unknown block (%s)",
+					  node->u.nterm.name);
+			err_type->bogus = true;
+		}
+	}
+}
+
 /**
  * @brief Find the root of the parse tree given a node.
  *
@@ -1458,11 +1491,11 @@ config_file_t get_parse_root(void *node)
 	struct config_root *root;
 
 	parent = (struct config_node *)node;
-	assert(parent->type == TYPE_BLOCK);
-	while (parent->u.nterm.parent != NULL) {
-		parent = parent->u.nterm.parent;
+	while (parent->parent != NULL) {
+		parent = parent->parent;
 		assert(parent->type == TYPE_ROOT ||
-		       parent->type == TYPE_BLOCK);
+		       parent->type == TYPE_BLOCK ||
+		       parent->type == TYPE_STMT);
 	}
 	assert(parent->type == TYPE_ROOT);
 	root = container_of(parent, struct config_root, root);
@@ -2005,6 +2038,7 @@ int load_config_from_parse(config_file_t config,
 						  "Only one %s block allowed",
 						  blkname);
 			} else {
+				node->found = true;
 				/* Reset cur_exp_create_err which may be used
 				 * if an EXPORT block is processed. */
 				err_type->cur_exp_create_err = false;
@@ -2029,7 +2063,8 @@ int load_config_from_parse(config_file_t config,
 			}
 		}
 	}
-	if (found == 0) {
+
+	if (found == 0 && (conf_blk->blk_desc.flags & CONFIG_NO_DEFAULT) == 0) {
 		/* Found nothing but we have to do the allocate and init
 		 * at least. Use a fake, not NULL link_mem */
 		blk_mem = param != NULL ?
@@ -2045,9 +2080,12 @@ int load_config_from_parse(config_file_t config,
 			err_type->init = true;
 		}
 	}
+
 	if (err_type->errors > prev_errs) {
 		char *errstr = err_type_str(err_type);
 
+		assert(!config_error_no_error(err_type));
+
 		config_proc_error(node, err_type,
 			 "%d %s errors found block %s",
 			 err_type->errors - prev_errs,
