@@ -91,8 +91,7 @@ namespace {
 
     virtual void TearDown() {
       for (int i = 0; i < LOOP_COUNT; ++i) {
-	op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export,
-						file_state[i]);
+	free_state(file_state[i]);
       }
 
       Close2EmptyLatencyTest::TearDown();
@@ -118,7 +117,7 @@ TEST_F(Close2EmptyLatencyTest, SIMPLE)
 
   // create and open a file for test
   status = test_root->obj_ops->open2(test_root, file_state, FSAL_O_RDWR, FSAL_UNCHECKED,
-               TEST_FILE, NULL, NULL, &obj, NULL, &caller_perm_check);
+               TEST_FILE, NULL, NULL, &obj, NULL, &caller_perm_check, nullptr, nullptr);
   ASSERT_EQ(status.major, 0);
 
   status = obj->obj_ops->close2(obj, file_state);
@@ -128,7 +127,7 @@ TEST_F(Close2EmptyLatencyTest, SIMPLE)
   status = fsal_remove(test_root, TEST_FILE);
   ASSERT_EQ(status.major, 0);
   obj->obj_ops->put_ref(obj);
-  op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export, file_state);
+  free_state(file_state);
 }
 
 TEST_F(Close2EmptyLatencyTest, SIMPLE_BYPASS)
@@ -146,7 +145,7 @@ TEST_F(Close2EmptyLatencyTest, SIMPLE_BYPASS)
 
   // create and open a file for test
   status = test_root->obj_ops->open2(test_root, file_state, FSAL_O_RDWR, FSAL_UNCHECKED,
-               TEST_FILE, NULL, NULL, &obj, NULL, &caller_perm_check);
+               TEST_FILE, NULL, NULL, &obj, NULL, &caller_perm_check, nullptr, nullptr);
   ASSERT_EQ(status.major, 0);
 
   sub_hdl = mdcdb_get_sub_handle(obj);
@@ -159,7 +158,7 @@ TEST_F(Close2EmptyLatencyTest, SIMPLE_BYPASS)
   status = fsal_remove(test_root, TEST_FILE);
   ASSERT_EQ(status.major, 0);
   obj->obj_ops->put_ref(obj);
-  op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export, file_state);
+  free_state(file_state);
 }
 
 TEST_F(Close2LoopLatencyTest, LOOP)
@@ -175,7 +174,7 @@ TEST_F(Close2LoopLatencyTest, LOOP)
 
     status = test_root->obj_ops->open2(test_root, file_state[i], FSAL_O_RDWR,
                FSAL_UNCHECKED, fname, NULL, NULL, &obj[i], NULL,
-               &caller_perm_check);
+               &caller_perm_check, nullptr, nullptr);
     ASSERT_EQ(status.major, 0);
   }
 
@@ -216,7 +215,7 @@ TEST_F(Close2LoopLatencyTest, LOOP_BYPASS)
     sprintf(fname, "f-%08x", i);
     status = test_root->obj_ops->open2(test_root, file_state[i], FSAL_O_RDWR,
                FSAL_UNCHECKED, fname, NULL, NULL, &obj[i], NULL,
-               &caller_perm_check);
+               &caller_perm_check, nullptr, nullptr);
     ASSERT_EQ(status.major, 0);
 
     sub_hdl[i] = mdcdb_get_sub_handle(obj[i]);
