@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 // -*- mode:C; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 /*
@@ -67,14 +68,14 @@ namespace {
 
     virtual void SetUp() {
       fsal_status_t status;
-      struct attrlist attrs_out;
+      struct fsal_attrlist attrs_out;
 
       gtest::GaneshaFSALBaseTest::SetUp();
 
       fsal_prepare_attrs(&attrs_out, 0);
 
       status = fsal_create(test_root, TEST_FILE, REGULAR_FILE, &attrs, NULL,
-			   &test_file, &attrs_out);
+			   &test_file, &attrs_out, nullptr, nullptr);
       ASSERT_EQ(status.major, 0);
       ASSERT_NE(test_file, nullptr);
 
