@@ -91,7 +91,7 @@ namespace {
         sprintf(fname, "f-%08x", i);
 
         status = fsal_create(test_root, fname, REGULAR_FILE, &attrs, NULL, &obj,
-                             &attrs_out);
+                             &attrs_out, nullptr, nullptr);
         ASSERT_EQ(status.major, 0);
         ASSERT_NE(obj, nullptr);
 
@@ -124,7 +124,7 @@ TEST_F(MkdirEmptyLatencyTest, SIMPLE)
   struct fsal_obj_handle *mkdir;
   struct fsal_obj_handle *lookup;
 
-  status = test_root->obj_ops->mkdir(test_root, TEST_ROOT, &attrs, &mkdir, NULL);
+  status = test_root->obj_ops->mkdir(test_root, TEST_ROOT, &attrs, &mkdir, NULL, nullptr, nullptr);
   EXPECT_EQ(status.major, 0);
   test_root->obj_ops->lookup(test_root, TEST_ROOT, &lookup, NULL);
   EXPECT_EQ(lookup, mkdir);
@@ -148,7 +148,7 @@ TEST_F(MkdirEmptyLatencyTest, SIMPLE_BYPASS)
   ASSERT_NE(sub_hdl, nullptr);
 
   gtws_subcall(
-    status = sub_hdl->obj_ops->mkdir(sub_hdl, TEST_ROOT, &attrs, &mkdir, NULL)
+    status = sub_hdl->obj_ops->mkdir(sub_hdl, TEST_ROOT, &attrs, &mkdir, NULL, nullptr, nullptr)
     );
 
   EXPECT_EQ(status.major, 0);
@@ -158,7 +158,7 @@ TEST_F(MkdirEmptyLatencyTest, SIMPLE_BYPASS)
   lookup->obj_ops->put_ref(lookup);
 
   /* Remove directory created while running test */
-  status = sub_hdl->obj_ops->unlink(sub_hdl, mkdir, TEST_ROOT);
+  status = sub_hdl->obj_ops->unlink(sub_hdl, mkdir, TEST_ROOT, nullptr, nullptr);
   ASSERT_EQ(status.major, 0);
 
   mkdir->obj_ops->put_ref(mkdir);
@@ -176,7 +176,7 @@ TEST_F(MkdirEmptyLatencyTest, LOOP)
   for (int i = 0; i < LOOP_COUNT; ++i) {
     sprintf(fname, "d-%08x", i);
 
-    status = test_root->obj_ops->mkdir(test_root, fname, &attrs, &obj, NULL);
+    status = test_root->obj_ops->mkdir(test_root, fname, &attrs, &obj, NULL, nullptr, nullptr);
     EXPECT_EQ(status.major, 0);
     obj->obj_ops->put_ref(obj);
   }
@@ -207,7 +207,7 @@ TEST_F(MkdirEmptyLatencyTest, FSALCREATE)
   for (int i = 0; i < LOOP_COUNT; ++i) {
     sprintf(fname, "d-%08x", i);
 
-    status = fsal_create(test_root, fname, DIRECTORY, &attrs, NULL, &obj, NULL);
+    status = fsal_create(test_root, fname, DIRECTORY, &attrs, NULL, &obj, NULL, nullptr, nullptr);
     EXPECT_EQ(status.major, 0);
     obj->obj_ops->put_ref(obj);
   }
@@ -238,7 +238,7 @@ TEST_F(MkdirFullLatencyTest, BIG)
   for (int i = 0; i < LOOP_COUNT; ++i) {
     sprintf(fname, "d-%08x", i);
 
-    status = test_root->obj_ops->mkdir(test_root, fname, &attrs, &obj, NULL);
+    status = test_root->obj_ops->mkdir(test_root, fname, &attrs, &obj, NULL, nullptr, nullptr);
     ASSERT_EQ(status.major, 0) << " failed to mkdir " << fname;
     obj->obj_ops->put_ref(obj);
   }
@@ -274,7 +274,7 @@ TEST_F(MkdirFullLatencyTest, BIG_BYPASS)
     sprintf(fname, "d-%08x", i);
 
     gtws_subcall(
-      status = sub_hdl->obj_ops->mkdir(sub_hdl, fname, &attrs, &obj, NULL)
+      status = sub_hdl->obj_ops->mkdir(sub_hdl, fname, &attrs, &obj, NULL, nullptr, nullptr)
       );
     ASSERT_EQ(status.major, 0) << " failed to mkdir " << fname;
     obj->obj_ops->put_ref(obj);
@@ -290,7 +290,7 @@ TEST_F(MkdirFullLatencyTest, BIG_BYPASS)
     sprintf(fname, "d-%08x", i);
 
     sub_hdl->obj_ops->lookup(sub_hdl, fname, &obj, NULL);
-    status = sub_hdl->obj_ops->unlink(sub_hdl, obj, fname);
+    status = sub_hdl->obj_ops->unlink(sub_hdl, obj, fname, nullptr, nullptr);
     ASSERT_EQ(status.major, 0);
 
     obj->obj_ops->put_ref(obj);
