@@ -83,7 +83,7 @@ namespace {
 
       status = test_root->obj_ops->open2(test_root, test_file_state,
                       FSAL_O_RDWR, FSAL_UNCHECKED, TEST_FILE, &attrs, NULL,
-                      &test_file, NULL, &caller_perm_check);
+                      &test_file, NULL, &caller_perm_check, nullptr, nullptr);
       ASSERT_EQ(status.major, 0);
     }
 
@@ -93,8 +93,7 @@ namespace {
       status = test_file->obj_ops->close2(test_file, test_file_state);
       EXPECT_EQ(0, status.major);
 
-      op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export,
-					      test_file_state);
+      free_state(test_file_state);
 
       status = fsal_remove(test_root, TEST_FILE);
       EXPECT_EQ(status.major, 0);
@@ -137,8 +136,8 @@ TEST_F(Read2EmptyLatencyTest, SIMPLE)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(test_file, true, write_arg, &io_data);
 
@@ -158,8 +157,8 @@ TEST_F(Read2EmptyLatencyTest, SIMPLE)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_read(test_file, true, read_arg, &io_data);
 
@@ -202,8 +201,8 @@ TEST_F(Read2EmptyLatencyTest, SIMPLE_BYPASS)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(sub_hdl, true, write_arg, &io_data);
 
@@ -224,8 +223,8 @@ TEST_F(Read2EmptyLatencyTest, SIMPLE_BYPASS)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_read(sub_hdl, true, read_arg, &io_data);
 
@@ -262,8 +261,8 @@ TEST_F(Read2EmptyLatencyTest, LARGE_DATA_READ)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(test_file, true, write_arg, &io_data);
 
@@ -283,8 +282,8 @@ TEST_F(Read2EmptyLatencyTest, LARGE_DATA_READ)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_read(test_file, true, read_arg, &io_data);
 
@@ -324,8 +323,8 @@ TEST_F(Read2EmptyLatencyTest, LOOP)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(test_file, true, write_arg, &io_data);
 
@@ -349,8 +348,8 @@ TEST_F(Read2EmptyLatencyTest, LOOP)
     io_data.ret.major = ERR_FSAL_NO_ERROR;
     io_data.ret.minor = 0;
     io_data.done = false;
-    io_data.cond = &cond;
-    io_data.mutex = &mutex;
+    io_data.fsa_cond = &cond;
+    io_data.fsa_mutex = &mutex;
 
     fsal_read(test_file, true, read_arg, &io_data);
 
@@ -397,8 +396,8 @@ TEST_F(Read2EmptyLatencyTest, LOOP_BYPASS)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(sub_hdl, true, write_arg, &io_data);
 
@@ -422,8 +421,8 @@ TEST_F(Read2EmptyLatencyTest, LOOP_BYPASS)
     io_data.ret.major = ERR_FSAL_NO_ERROR;
     io_data.ret.minor = 0;
     io_data.done = false;
-    io_data.cond = &cond;
-    io_data.mutex = &mutex;
+    io_data.fsa_cond = &cond;
+    io_data.fsa_mutex = &mutex;
 
     fsal_read(sub_hdl, true, read_arg, &io_data);
 
