@@ -81,7 +81,7 @@ namespace {
 
       status = test_root->obj_ops->open2(test_root, test_file_state,
                       FSAL_O_RDWR, FSAL_UNCHECKED, TEST_FILE, NULL, NULL,
-                      &test_file, NULL, &caller_perm_check);
+                      &test_file, NULL, &caller_perm_check, nullptr, nullptr);
       ASSERT_EQ(status.major, 0);
     }
 
@@ -91,8 +91,7 @@ namespace {
       status = test_file->obj_ops->close2(test_file, test_file_state);
       EXPECT_EQ(0, status.major);
 
-      op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export,
-					      test_file_state);
+      free_state(test_file_state);
 
       status = fsal_remove(test_root, TEST_FILE);
       EXPECT_EQ(status.major, 0);
