@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 // -*- mode:C; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 /*
@@ -34,6 +35,8 @@
 #include <boost/filesystem/exception.hpp>
 #include <boost/program_options.hpp>
 
+#include "gtest.hh"
+
 extern "C" {
 /* Manually forward this, as 9P is not C++ safe */
 void admin_halt(void);
@@ -47,8 +50,6 @@ void admin_halt(void);
 #include "../FSAL/Stackable_FSALs/FSAL_MDCACHE/mdcache_debug.h"
 }
 
-#include "gtest.hh"
-
 #define TEST_ROOT "write2_latency"
 #define TEST_FILE "test_file"
 #define LOOP_COUNT 1000000
@@ -82,7 +83,7 @@ namespace {
 
       status = test_root->obj_ops->open2(test_root, test_file_state,
                       FSAL_O_RDWR, FSAL_UNCHECKED, TEST_FILE, NULL, NULL,
-                      &test_file, NULL, &caller_perm_check);
+                      &test_file, NULL, &caller_perm_check, nullptr, nullptr);
       ASSERT_EQ(status.major, 0);
     }
 
@@ -92,8 +93,7 @@ namespace {
       status = test_file->obj_ops->close2(test_file, test_file_state);
       EXPECT_EQ(0, status.major);
 
-      op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export,
-					      test_file_state);
+      free_state(test_file_state);
 
       status = fsal_remove(test_root, TEST_FILE);
       EXPECT_EQ(status.major, 0);
@@ -133,8 +133,8 @@ TEST_F(Write2EmptyLatencyTest, SIMPLE)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(test_file, true, write_arg, &io_data);
 
@@ -171,8 +171,8 @@ TEST_F(Write2EmptyLatencyTest, SIMPLE_BYPASS)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(sub_hdl, true, write_arg, &io_data);
 
@@ -205,8 +205,8 @@ TEST_F(Write2EmptyLatencyTest, SMALL_STABLE_WRITE)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(test_file, true, write_arg, &io_data);
 
@@ -239,8 +239,8 @@ TEST_F(Write2EmptyLatencyTest, LARGE_UNSTABLE_WRITE)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(test_file, true, write_arg, &io_data);
 
@@ -273,8 +273,8 @@ TEST_F(Write2EmptyLatencyTest, LARGE_STABLE_WRITE)
   io_data.ret.major = ERR_FSAL_NO_ERROR;
   io_data.ret.minor = 0;
   io_data.done = false;
-  io_data.cond = &cond;
-  io_data.mutex = &mutex;
+  io_data.fsa_cond = &cond;
+  io_data.fsa_mutex = &mutex;
 
   fsal_write(test_file, true, write_arg, &io_data);
 
@@ -311,8 +311,8 @@ TEST_F(Write2EmptyLatencyTest, LOOP)
     io_data.ret.major = ERR_FSAL_NO_ERROR;
     io_data.ret.minor = 0;
     io_data.done = false;
-    io_data.cond = &cond;
-    io_data.mutex = &mutex;
+    io_data.fsa_cond = &cond;
+    io_data.fsa_mutex = &mutex;
 
     fsal_write(test_file, true, write_arg, &io_data);
 
@@ -359,8 +359,8 @@ TEST_F(Write2EmptyLatencyTest, LOOP_BYPASS)
     io_data.ret.major = ERR_FSAL_NO_ERROR;
     io_data.ret.minor = 0;
     io_data.done = false;
-    io_data.cond = &cond;
-    io_data.mutex = &mutex;
+    io_data.fsa_cond = &cond;
+    io_data.fsa_mutex = &mutex;
 
     fsal_write(sub_hdl, true, write_arg, &io_data);
 
