@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 // -*- mode:C; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 /*
@@ -55,8 +56,7 @@ namespace {
   uint16_t export_id = 77;
 
   struct req_op_context req_ctx;
-  struct user_cred user_credentials;
-  struct attrlist object_attributes;
+  struct fsal_attrlist object_attributes;
 
   struct gsh_export* a_export = nullptr;
   struct fsal_obj_handle *root_entry = nullptr;
@@ -91,22 +91,14 @@ TEST(CI_HASH_DIST1, INIT)
   ASSERT_NE(root_entry, nullptr);
 
   /* Ganesha call paths need real or forged context info */
-  memset(&user_credentials, 0, sizeof(struct user_cred));
-  memset(&req_ctx, 0, sizeof(struct req_op_context));
+  init_op_context_simple(&req_ctx, a_export, a_export->fsal_export);
   memset(&object_attributes, 0, sizeof(object_attributes));
-
-  req_ctx.ctx_export = a_export;
-  req_ctx.fsal_export = a_export->fsal_export;
-  req_ctx.creds = &user_credentials;
-
-  /* stashed in tls */
-  op_ctx = &req_ctx;
 }
 
 TEST(CI_HASH_DIST1, CREATE_ROOT)
 {
   fsal_status_t status;
-  struct attrlist *attrs_out = nullptr;
+  struct fsal_attrlist *attrs_out = nullptr;
 
   // create root directory for test
   FSAL_SET_MASK(object_attributes.request_mask,
@@ -117,7 +109,7 @@ TEST(CI_HASH_DIST1, CREATE_ROOT)
 
   status = root_entry->obj_ops->mkdir(root_entry, "ci_hash_dist1",
 				    &object_attributes, &test_root,
-				    attrs_out);
+				    attrs_out, nullptr, nullptr);
   ASSERT_NE(test_root, nullptr);
 }
 
