@@ -38,10 +38,11 @@
 #define _9P_REQ_QUEUE_H
 
 #include "gsh_list.h"
+#include "common_utils.h"
 #include "gsh_wait_queue.h"
 
 struct req_q {
-	pthread_spinlock_t sp;
+	pthread_spinlock_t _9p_rq_spinlock;
 	struct glist_head q;	/* LIFO */
 	uint32_t size;
 	uint32_t max;
@@ -50,10 +51,15 @@ struct req_q {
 
 struct req_q_pair {
 	const char *s;
+
 	GSH_CACHE_PAD(0);
+
 	struct req_q producer;	/* from decoder */
+
 	GSH_CACHE_PAD(1);
+
 	struct req_q consumer;	/* to executor */
+
 	GSH_CACHE_PAD(2);
 };
 
@@ -71,35 +77,41 @@ struct _9p_req_st {
 		uint32_t ctr;
 		struct req_q_set _9p_request_q;
 		uint64_t size;
-		pthread_spinlock_t sp;
+		pthread_spinlock_t _9p_rq_st_spinlock;
 		struct glist_head wait_list;
 		uint32_t waiters;
 	} reqs;
+
 	GSH_CACHE_PAD(1);
 };
 
 static inline void _9p_rpc_q_init(struct req_q *q)
 {
 	glist_init(&q->q);
-	pthread_spin_init(&q->sp, PTHREAD_PROCESS_PRIVATE);
+	PTHREAD_SPIN_init(&q->_9p_rq_spinlock, PTHREAD_PROCESS_PRIVATE);
 	q->size = 0;
 	q->waiters = 0;
 }
 
+static inline void _9p_rpc_q_destroy(struct req_q *q)
+{
+	PTHREAD_SPIN_destroy(&q->_9p_rq_spinlock);
+}
+
 static inline void _9p_queue_awaken(void *arg)
 {
 	struct _9p_req_st *st = arg;
 	struct glist_head *g = NULL;
 	struct glist_head *n = NULL;
 
-	pthread_spin_lock(&st->reqs.sp);
+	PTHREAD_SPIN_lock(&st->reqs._9p_rq_st_spinlock);
 	glist_for_each_safe(g, n, &st->reqs.wait_list) {
 		wait_q_entry_t *wqe = glist_entry(g, wait_q_entry_t, waitq);
 
-		pthread_cond_signal(&wqe->lwe.cv);
-		pthread_cond_signal(&wqe->rwe.cv);
+		pthread_cond_signal(&wqe->lwe.wq_cv);
+		pthread_cond_signal(&wqe->rwe.wq_cv);
 	}
-	pthread_spin_unlock(&st->reqs.sp);
+	PTHREAD_SPIN_unlock(&st->reqs._9p_rq_st_spinlock);
 }
 
 #endif				/* _9P_REQ_QUEUE_H */
