@@ -39,7 +39,7 @@
  * int64_t
  * uint64_t
  * int32_t
- * uint21_t
+ * uint32_t
  * int16_t
  * uint16_t
  * int8_t
@@ -59,6 +59,14 @@
  * int64_t atomic_fetch_int64_t(int64_t *var)
  * void atomic_store_int64_t(int64_t *var, int64_t val)
  *
+ * The following is provided for int64_t, uint64_t, int32_t, and uint32_t
+ *
+ * bool atomic_add_unless_int64_t(int64_t *var, int64_t addend, int64_t unless)
+ *
+ * The following is provided for int32_t
+ *
+ * int32_t atomic_inc_unless_0_int32_t(int32_t *var)
+ *
  * The following bit mask operations are provided for
  * uint64_t, uint32_t, uint_16t, and uint8_t:
  *
@@ -74,6 +82,7 @@
 #include <stddef.h>
 #include <stdint.h>
 #include <time.h>
+#include <stdbool.h>
 
 #undef GCC_SYNC_FUNCTIONS
 #undef GCC_ATOMIC_FUNCTIONS
@@ -2133,6 +2142,57 @@ static inline int64_t atomic_fetch_int64_t(int64_t *var)
 }
 #endif
 
+/**
+ * @brief Atomically add to an int64_t unless it has a flag value
+ *
+ * This function atomically adds to the supplied value unless the
+ * variable is equal to the flag value.
+ *
+ * @param[in,out] var    Number to be added to
+ * @param[in]     addend Number to add
+ * @param[in]     unless The flag value
+ *
+ * @return true if the addition occurred
+ */
+
+#ifdef GCC_ATOMIC_FUNCTIONS
+static inline bool atomic_add_unless_int64_t(int64_t *var,
+					     int64_t addend,
+					     int64_t unless)
+{
+	int64_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_int64_t(var);
+	do {
+		if (cur == unless)
+			return false;
+		newv = cur + addend;
+		changed = __atomic_compare_exchange_n(var, &cur, newv, false,
+						      __ATOMIC_SEQ_CST,
+						      __ATOMIC_SEQ_CST);
+	} while (!changed);
+	return true;
+}
+#elif defined(GCC_SYNC_FUNCTIONS)
+static inline bool atomic_add_unless_int64_t(int64_t *var,
+					     int64_t addend,
+					     int64_t unless)
+{
+	int64_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_int64_t(var);
+	do {
+		if (cur == unless)
+			return false;
+		newv = cur + addend;
+		changed = __sync_val_compare_and_swap(var, cur, newv) != cur;
+	} while (!changed);
+	return true;
+}
+#endif
+
 /**
  * @brief Atomically store an int64_t
  *
@@ -2178,6 +2238,57 @@ static inline uint64_t atomic_fetch_uint64_t(uint64_t *var)
 }
 #endif
 
+/**
+ * @brief Atomically add to an uint64_t unless it has a flag value
+ *
+ * This function atomically adds to the supplied value unless the
+ * variable is equal to the flag value.
+ *
+ * @param[in,out] var    Number to be added to
+ * @param[in]     addend Number to add
+ * @param[in]     unless The flag value
+ *
+ * @return true if the addition occurred
+ */
+
+#ifdef GCC_ATOMIC_FUNCTIONS
+static inline bool atomic_add_unless_uint64_t(uint64_t *var,
+					      uint64_t addend,
+					      uint64_t unless)
+{
+	uint64_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_uint64_t(var);
+	do {
+		if (cur == unless)
+			return false;
+		newv = cur + addend;
+		changed = __atomic_compare_exchange_n(var, &cur, newv, false,
+						      __ATOMIC_SEQ_CST,
+						      __ATOMIC_SEQ_CST);
+	} while (!changed);
+	return true;
+}
+#elif defined(GCC_SYNC_FUNCTIONS)
+static inline bool atomic_add_unless_uint64_t(uint64_t *var,
+					      uint64_t addend,
+					      uint64_t unless)
+{
+	uint64_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_uint64_t(var);
+	do {
+		if (cur == unless)
+			return false;
+		newv = cur + addend;
+		changed = __sync_val_compare_and_swap(var, cur, newv) != cur;
+	} while (!changed);
+	return true;
+}
+#endif
+
 /**
  * @brief Atomically store a uint64_t
  *
@@ -2223,6 +2334,102 @@ static inline int32_t atomic_fetch_int32_t(int32_t *var)
 }
 #endif
 
+/**
+ * @brief Atomically add to an int32_t unless it has a flag value
+ *
+ * This function atomically adds to the supplied value unless the
+ * variable is equal to the flag value.
+ *
+ * @param[in,out] var    Number to be added to
+ * @param[in]     addend Number to add
+ * @param[in]     unless The flag value
+ *
+ * @return true if the addition occurred
+ */
+
+#ifdef GCC_ATOMIC_FUNCTIONS
+static inline bool atomic_add_unless_int32_t(int32_t *var,
+					     int32_t addend,
+					     int32_t unless)
+{
+	int32_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_int32_t(var);
+	do {
+		if (cur == unless)
+			return false;
+		newv = cur + addend;
+		changed = __atomic_compare_exchange_n(var, &cur, newv, false,
+						      __ATOMIC_SEQ_CST,
+						      __ATOMIC_SEQ_CST);
+	} while (!changed);
+	return true;
+}
+#elif defined(GCC_SYNC_FUNCTIONS)
+static inline bool atomic_add_unless_int32_t(int32_t *var,
+					     int32_t addend,
+					     int32_t unless)
+{
+	int32_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_int32_t(var);
+	do {
+		if (cur == unless)
+			return false;
+		newv = cur + addend;
+		changed = __sync_val_compare_and_swap(var, cur, newv) != cur;
+	} while (!changed);
+	return true;
+}
+#endif
+
+/**
+ * @brief Atomically increment an int32_t unless it is 0
+ *
+ * This function atomically adds to the supplied value unless the
+ * variable is equal to the flag value.
+ *
+ * @param[in,out] var    Number to be incremented
+ *
+ * @return the new value, if 0, then it failed
+ */
+
+#ifdef GCC_ATOMIC_FUNCTIONS
+static inline int32_t atomic_inc_unless_0_int32_t(int32_t *var)
+{
+	int32_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_int32_t(var);
+	do {
+		if (cur == 0)
+			return 0;
+		newv = cur + 1;
+		changed = __atomic_compare_exchange_n(var, &cur, newv, false,
+						      __ATOMIC_SEQ_CST,
+						      __ATOMIC_SEQ_CST);
+	} while (!changed);
+	return newv;
+}
+#elif defined(GCC_SYNC_FUNCTIONS)
+static inline int32_t atomic_inc_unless_0_int32_t(int32_t *var)
+{
+	int32_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_int32_t(var);
+	do {
+		if (cur == 0)
+			return 0;
+		newv = cur + 1;
+		changed = __sync_val_compare_and_swap(var, cur, newv) != cur;
+	} while (!changed);
+	return newv;
+}
+#endif
+
 /**
  * @brief Atomically store an int32_t
  *
@@ -2268,6 +2475,57 @@ static inline uint32_t atomic_fetch_uint32_t(uint32_t *var)
 }
 #endif
 
+/**
+ * @brief Atomically add to an uint32_t unless it has a flag value
+ *
+ * This function atomically adds to the supplied value unless the
+ * variable is equal to the flag value.
+ *
+ * @param[in,out] var    Number to be added to
+ * @param[in]     addend Number to add
+ * @param[in]     unless The flag value
+ *
+ * @return true if the addition occurred
+ */
+
+#ifdef GCC_ATOMIC_FUNCTIONS
+static inline bool atomic_add_unless_uint32_t(uint32_t *var,
+					      uint32_t addend,
+					      uint32_t unless)
+{
+	uint32_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_uint32_t(var);
+	do {
+		if (cur == unless)
+			return false;
+		newv = cur + addend;
+		changed = __atomic_compare_exchange_n(var, &cur, newv, false,
+						      __ATOMIC_SEQ_CST,
+						      __ATOMIC_SEQ_CST);
+	} while (!changed);
+	return true;
+}
+#elif defined(GCC_SYNC_FUNCTIONS)
+static inline bool atomic_add_unless_uint32_t(uint32_t *var,
+					      uint32_t addend,
+					      uint32_t unless)
+{
+	uint32_t cur, newv;
+	bool changed;
+
+	cur = atomic_fetch_uint32_t(var);
+	do {
+		if (cur == unless)
+			return false;
+		newv = cur + addend;
+		changed = __sync_val_compare_and_swap(var, cur, newv) != cur;
+	} while (!changed);
+	return true;
+}
+#endif
+
 /**
  * @brief Atomically store a uint32_t
  *
