@@ -30,6 +30,10 @@
 #include <stdio.h>
 #include <sys/types.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct config_root *config_file_t;
 
 typedef enum { CONFIG_ITEM_BLOCK = 1, CONFIG_ITEM_VAR } config_item_type;
@@ -88,6 +92,8 @@ enum config_type {
 #define CONFIG_RELAX		0x008  /*< this block has extra params
 					*  so don't complain about them */
 #define CONFIG_MARK_SET		0x010  /*< Mark this param as set */
+#define CONFIG_NO_DEFAULT	0x020  /*< This config block doesn't need a
+					*  default init if not present. */
 
 /**
  * @brief Config file processing error type
@@ -469,28 +475,33 @@ struct config_item {
 	  .type = CONFIG_NULL,			\
 	}
 
-#define CONF_ITEM_FSID(_name_, _def_maj_, _def_min_, _struct_, _mem_) \
-	{ .name = _name_,			    \
-	  .type = CONFIG_FSID,		   	    \
-	  .u.fsid.def_maj = _def_maj_,		    \
-	  .u.fsid.def_min = _def_min_,		    \
-	  .off = offsetof(struct _struct_, _mem_)   \
-	}
 #define CONF_ITEM_FSID_SET(_name_, _def_maj_, _def_min_, _struct_, \
 			   _mem_, _bit_, _set_)     \
 	{ .name = _name_,			    \
 	  .type = CONFIG_FSID,		   	    \
-	  .flags = CONFIG_MARK_SET,			\
+	  .flags = CONFIG_MARK_SET|CONFIG_UNIQUE,   \
 	  .u.fsid.def_maj = _def_maj_,		    \
 	  .u.fsid.def_min = _def_min_,		    \
 	  .u.fsid.bit = _bit_,		   	    \
 	  .u.fsid.set_off = offsetof(struct _struct_, _set_),   \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
+
+#define CONF_ITEM_BLOCK_MULT(_name_, _params_, _init_, _commit_, \
+			     _struct_, _mem_) \
+	{ .name = _name_,			    \
+	  .type = CONFIG_BLOCK,			    \
+	  .u.blk.init = _init_,			    \
+	  .u.blk.params = _params_,		    \
+	  .u.blk.commit = _commit_,		    \
+	  .off = offsetof(struct _struct_, _mem_)   \
+	}
+
 #define CONF_ITEM_BLOCK(_name_, _params_, _init_, _commit_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_BLOCK,		   	    \
-	  .u.blk.init = _init_,		    \
+	  .flags = CONFIG_UNIQUE,		    \
+	  .u.blk.init = _init_,			    \
 	  .u.blk.params = _params_,		    \
 	  .u.blk.commit = _commit_,		    \
 	  .off = offsetof(struct _struct_, _mem_)   \
@@ -499,14 +510,14 @@ struct config_item {
 #define CONF_RELAX_BLOCK(_name_, _params_, _init_, _commit_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_BLOCK,		   	    \
-	  .flags = CONFIG_RELAX,		    \
-	  .u.blk.init = _init_,		    \
+	  .flags = CONFIG_RELAX|CONFIG_UNIQUE,	    \
+	  .u.blk.init = _init_,			    \
 	  .u.blk.params = _params_,		    \
 	  .u.blk.commit = _commit_,		    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
-#define CONF_ITEM_PROC(_name_, _init_, _handler_, _struct_, _mem_)	\
+#define CONF_ITEM_PROC_MULT(_name_, _init_, _handler_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_PROC,			    \
 	  .u.proc.init = _init_,		    \
@@ -517,6 +528,7 @@ struct config_item {
 #define CONF_ITEM_LIST(_name_, _def_, _tokens_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_LIST,			    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.lst.def = _def_,			    \
 	  .u.lst.mask = UINT32_MAX,		    \
 	  .u.lst.set_off = UINT32_MAX,		    \
@@ -524,21 +536,11 @@ struct config_item {
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
-#define CONF_ITEM_LIST_BITS(_name_, _def_, _mask_, _tokens_, _struct_, _mem_) \
-	{ .name = _name_,			    \
-	  .type = CONFIG_LIST,		  	    \
-	  .u.lst.def = _def_,			    \
-	  .u.lst.mask = _mask_,			    \
-	  .u.lst.set_off = UINT32_MAX,		    \
-	  .u.lst.tokens = _tokens_,		    \
-	  .off = offsetof(struct _struct_, _mem_)   \
-	}
-
 #define CONF_ITEM_LIST_BITS_SET(_name_, _def_, _mask_, _tokens_, _struct_, \
 				_mem_, _set_)	    \
 	{ .name = _name_,			    \
 	  .type = CONFIG_LIST,		  	    \
-	  .flags = CONFIG_MARK_SET,			\
+	  .flags = CONFIG_MARK_SET|CONFIG_UNIQUE,   \
 	  .u.lst.def = _def_,			    \
 	  .u.lst.mask = _mask_,			    \
 	  .u.lst.set_off = offsetof(struct _struct_, _set_),   \
@@ -546,33 +548,16 @@ struct config_item {
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
-#define CONF_ITEM_BOOLBIT(_name_, _def_, _bit_, _struct_, _mem_) \
-	{ .name = _name_,			    \
-	  .type = CONFIG_BOOLBIT,		    \
-	  .u.bit.def = _def_,			    \
-	  .u.bit.bit = _bit_,			    \
-	  .off = offsetof(struct _struct_, _mem_)   \
-	}
-
 #define CONF_ITEM_BOOLBIT_SET(_name_, _def_, _bit_, _struct_, _mem_, _set_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_BOOLBIT,		    \
-	  .flags = CONFIG_MARK_SET,			\
+	  .flags = CONFIG_MARK_SET|CONFIG_UNIQUE,   \
 	  .u.bit.def = _def_,			    \
 	  .u.bit.bit = _bit_,			    \
 	  .u.bit.set_off = offsetof(struct _struct_, _set_),   \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
-#define CONF_MAND_LIST(_name_, _def_, _tokens_, _struct_, _mem_) \
-	{ .name = _name_,			    \
-	  .type = CONFIG_LIST,		   	    \
-	  .flags = CONFIG_MANDATORY,		    \
-	  .u.lst.def = _def_,			    \
-	  .u.lst.tokens = _tokens_,		    \
-	  .off = offsetof(struct _struct_, _mem_)   \
-	}
-
 /* Use CONF_ITEM_TOKEN for a variable that is set to a single enum
  * value. The CONF_ITEM_ENUM_* macros are for setting one or more
  * bits within a field (I know, a bit confusing...).
@@ -581,6 +566,7 @@ struct config_item {
 #define CONF_ITEM_ENUM_BITS(_name_, _def_, _mask_, _tokens_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_ENUM,			    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.lst.def = _def_,			    \
 	  .u.lst.mask = _mask_,			    \
 	  .u.lst.tokens = _tokens_,		    \
@@ -591,7 +577,7 @@ struct config_item {
 				_mem_, _set_)	    \
 	{ .name = _name_,			    \
 	  .type = CONFIG_ENUM,			    \
-	  .flags = CONFIG_MARK_SET,			\
+	  .flags = CONFIG_MARK_SET|CONFIG_UNIQUE,   \
 	  .u.lst.def = _def_,			    \
 	  .u.lst.mask = _mask_,			    \
 	  .u.lst.set_off = offsetof(struct _struct_, _set_),   \
@@ -601,7 +587,8 @@ struct config_item {
 
 #define CONF_ITEM_TOKEN(_name_, _def_, _tokens_, _struct_, _mem_) \
 	{ .name = _name_,			    \
-	  .type = CONFIG_TOKEN,		    \
+	  .type = CONFIG_TOKEN,			    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.lst.def = _def_,			    \
 	  .u.lst.tokens = _tokens_,		    \
 	  .off = offsetof(struct _struct_, _mem_)   \
@@ -609,7 +596,8 @@ struct config_item {
 
 #define CONF_INDEX_TOKEN(_name_, _def_, _tokens_, _idx_, _sizeof_) \
 	{ .name = _name_,			    \
-	  .type = CONFIG_TOKEN,		    \
+	  .type = CONFIG_TOKEN,			    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.lst.def = _def_,			    \
 	  .u.lst.tokens = _tokens_,		    \
 	  .off = (sizeof(_sizeof_) * _idx_)  \
@@ -617,7 +605,8 @@ struct config_item {
 
 #define CONF_ITEM_BOOL(_name_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
-	  .type = CONFIG_BOOL,		    \
+	  .type = CONFIG_BOOL,			    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.b.def = _def_,			    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
@@ -625,6 +614,7 @@ struct config_item {
 #define CONF_ITEM_STR(_name_, _minsize_, _maxsize_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_STRING,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.str.minsize = _minsize_,		    \
 	  .u.str.maxsize = _maxsize_,		    \
 	  .u.str.def = _def_,			    \
@@ -640,14 +630,17 @@ struct config_item {
 	  .u.str.def = _def_,			    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
+
 #define CONF_ITEM_PATH(_name_, _minsize_, _maxsize_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
-	  .type = CONFIG_PATH,		    \
+	  .type = CONFIG_PATH,			    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.str.minsize = _minsize_,		    \
 	  .u.str.maxsize = _maxsize_,		    \
 	  .u.str.def = _def_,			    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
+
 #define CONF_MAND_PATH(_name_, _minsize_, _maxsize_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_PATH,		    \
@@ -656,22 +649,16 @@ struct config_item {
 	  .u.str.maxsize = _maxsize_,		    \
 	  .u.str.def = _def_,			    \
 	  .off = offsetof(struct _struct_, _mem_)   \
-	}
-#define CONF_UNIQ_PATH(_name_, _minsize_, _maxsize_, _def_, _struct_, _mem_) \
-	{ .name = _name_,			    \
-	  .type = CONFIG_PATH,		    	    \
-	  .flags = CONFIG_UNIQUE,		    \
-	  .u.str.minsize = _minsize_,		    \
-	  .u.str.maxsize = _maxsize_,		    \
-	  .u.str.def = _def_,			    \
-	  .off = offsetof(struct _struct_, _mem_)   \
-	}
+}
+
 #define CONF_ITEM_IP_ADDR(_name_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_IP_ADDR,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.ip.def = _def_,			    \
 	  .off = offsetof(struct _struct_, _mem_)   \
-	}
+}
+
 #define CONF_MAND_IP_ADDR(_name_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_IP_ADDR,		    \
@@ -683,6 +670,7 @@ struct config_item {
 #define CONF_ITEM_I16(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_INT16,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.i16.minval = _min_,		    \
 	  .u.i16.maxval = _max_,		    \
 	  .u.i16.def = _def_,			    \
@@ -693,6 +681,7 @@ struct config_item {
 #define CONF_ITEM_UI16(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_UINT16,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.ui16.minval = _min_,		    \
 	  .u.ui16.maxval = _max_,		    \
 	  .u.ui16.def = _def_,			    \
@@ -714,6 +703,7 @@ struct config_item {
 #define CONF_ITEM_I32(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_INT32,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.i32.minval = _min_,		    \
 	  .u.i32.maxval = _max_,		    \
 	  .u.i32.def = _def_,			    \
@@ -724,8 +714,8 @@ struct config_item {
 #define CONF_ITEM_I32_SET(_name_, _min_, _max_, _def_, _struct_, _mem_, \
 			  _bit_, _set_) \
 	{ .name = _name_,			    \
-	  .type = CONFIG_INT32,		    \
-	  .flags = CONFIG_MARK_SET,   \
+	  .type = CONFIG_INT32,			    \
+	  .flags = CONFIG_MARK_SET|CONFIG_UNIQUE,   \
 	  .u.i32.minval = _min_,		    \
 	  .u.i32.maxval = _max_,		    \
 	  .u.i32.def = _def_,			    \
@@ -739,7 +729,7 @@ struct config_item {
 			  _bit_, _set_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_ANON_ID,		    \
-	  .flags = CONFIG_MARK_SET,		    \
+	  .flags = CONFIG_MARK_SET|CONFIG_UNIQUE,   \
 	  .u.i64.minval = INT32_MIN,		    \
 	  .u.i64.maxval = UINT32_MAX,		    \
 	  .u.i64.def = _def_,			    \
@@ -752,6 +742,7 @@ struct config_item {
 #define CONF_ITEM_UI32(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_UINT32,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.ui32.minval = _min_,		    \
 	  .u.ui32.maxval = _max_,		    \
 	  .u.ui32.def = _def_,			    \
@@ -762,6 +753,7 @@ struct config_item {
 #define CONF_ITEM_UI32_ZERO(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_UINT32,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.ui32.minval = _min_,		    \
 	  .u.ui32.maxval = _max_,		    \
 	  .u.ui32.def = _def_,			    \
@@ -783,7 +775,7 @@ struct config_item {
 #define CONF_ITEM_MODE(_name_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_UINT32,		    \
-	  .flags = CONFIG_MODE,			    \
+	  .flags = CONFIG_MODE|CONFIG_UNIQUE,	    \
 	  .u.ui32.minval = 0,			    \
 	  .u.ui32.maxval = 0777,		    \
 	  .u.ui32.def = _def_,			    \
@@ -794,6 +786,7 @@ struct config_item {
 #define CONF_ITEM_I64(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_INT64,			    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.i64.minval = _min_,		    \
 	  .u.i64.maxval = _max_,		    \
 	  .u.i64.def = _def_,			    \
@@ -807,6 +800,7 @@ struct config_item {
 #define CONF_ITEM_I64_ZERO(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_INT64,			    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.i64.minval = _min_,		    \
 	  .u.i64.maxval = _max_,		    \
 	  .u.i64.def = _def_,			    \
@@ -817,6 +811,7 @@ struct config_item {
 #define CONF_ITEM_UI64(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_UINT64,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.ui64.minval = _min_,		    \
 	  .u.ui64.maxval = _max_,		    \
 	  .u.ui64.def = _def_,			    \
@@ -827,6 +822,7 @@ struct config_item {
 #define CONF_ITEM_UI64_ZERO(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_UINT64,		    \
+	  .flags = CONFIG_UNIQUE,		    \
 	  .u.ui64.minval = _min_,		    \
 	  .u.ui64.maxval = _max_,		    \
 	  .u.ui64.def = _def_,			    \
@@ -838,7 +834,7 @@ struct config_item {
 			  _mem_, _bit_, _set_)       \
 	{ .name = _name_,			    \
 	  .type = CONFIG_UINT64,		    \
-	  .flags = CONFIG_MARK_SET,		    \
+	  .flags = CONFIG_MARK_SET|CONFIG_UNIQUE,   \
 	  .u.ui64.minval = _min_,		    \
 	  .u.ui64.maxval = _max_,		    \
 	  .u.ui64.def = _def_,			    \
@@ -961,6 +957,9 @@ int load_config_from_parse(config_file_t config,
 			   bool unique,
 			   struct config_error_type *err_type);
 
+void find_unused_blocks(config_file_t config,
+			struct config_error_type *err_type);
+
 /* translate err_type values to log/dbus error string*/
 const char *config_term_name(enum term_type type);
 const char *config_term_desc(enum term_type type);
@@ -972,9 +971,9 @@ void config_errs_to_log(char *err, void *,
 void config_proc_error(void *cnode,
 		       struct config_error_type *err_type,
 		       char *format, ...);
-void report_config_errors(struct config_error_type *err_type, void *dest,
-			  void (*logger)(char *msg, void *dest,
-					 struct config_error_type *err_type));
+int report_config_errors(struct config_error_type *err_type, void *dest,
+			 void (*logger)(char *msg, void *dest,
+					struct config_error_type *err_type));
 
 
 /**
@@ -986,4 +985,8 @@ void *noop_conf_init(void *link_mem, void *self_struct);
 int noop_conf_commit(void *node, void *link_mem, void *self_struct,
 		     struct config_error_type *err_type);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
