@@ -94,8 +94,6 @@ extern uint32_t root_op_export_set;
  */
 extern int g_nodeid;
 
-extern size_t open_fd_count;
-
 void init_ctx_refstr(void);
 void destroy_ctx_refstr(void);
 
@@ -117,8 +115,7 @@ static inline void init_op_context_simple(struct req_op_context *ctx,
 void release_op_context(void);
 void suspend_op_context(void);
 void resume_op_context(struct req_op_context *ctx);
-void set_op_context_export_fsal(struct gsh_export *exp,
-				struct fsal_export *fsal_exp);
+void set_op_context_export(struct gsh_export *exp);
 void clear_op_context_export(void);
 void save_op_context_export_and_clear(struct saved_export_context *saved);
 void save_op_context_export_and_set_export(struct saved_export_context *saved,
@@ -127,12 +124,6 @@ void restore_op_context_export(struct saved_export_context *saved);
 void discard_op_context_export(struct saved_export_context *saved);
 void set_op_context_pnfs_ds(struct fsal_pnfs_ds *pds);
 
-#define set_op_context_export(exp_expr) \
-	do { \
-		struct gsh_export *ex = exp_expr; \
-		set_op_context_export_fsal(ex, ex ? ex->fsal_export : NULL); \
-	} while (0)
-
 /******************************************************
  *                Structure used to define a fsal
  ******************************************************/
@@ -215,7 +206,8 @@ int subfsal_commit(void *node, void *link_mem, void *self_struct,
 
 void destroy_fsals(void);
 void emergency_cleanup_fsals(void);
-void start_fsals(void);
+int start_fsals(config_file_t in_config,
+		struct config_error_type *err_type);
 
 void display_fsinfo(struct fsal_module *fsal);
 
@@ -374,15 +366,7 @@ static inline fsal_status_t fsal_close(struct fsal_obj_handle *obj_hdl)
 	/* Return the result of close method. */
 	fsal_status_t status = obj_hdl->obj_ops->close(obj_hdl);
 
-	if (status.major != ERR_FSAL_NOT_OPENED) {
-		ssize_t count;
-
-		count = atomic_dec_size_t(&open_fd_count);
-		if (count < 0) {
-			LogCrit(COMPONENT_FSAL,
-				"open_fd_count is negative: %zd", count);
-		}
-	} else {
+	if (status.major == ERR_FSAL_NOT_OPENED) {
 		/* Wasn't open.  Not an error, but shouldn't decrement */
 		status = fsalstat(ERR_FSAL_NO_ERROR, 0);
 	}
@@ -497,9 +481,15 @@ static inline void fsal_copy_attrs(struct fsal_attrlist *dest,
 		/* Make sure acl is NULL and don't pass a ref back (so
 		 * caller when calling fsal_release_attrs will not have to
 		 * release the ACL reference).
+		 * It might be that the src did include ATTR_ACL in the valid
+		 * mask but doesn't have a pointer to ACL as an indication
+		 * that ACL was not stored in the BE. In this case we don't
+		 * want to remove the ATTR_ACL from the valid_mask so a valid
+		 * response will be encoded in the reply.
 		 */
+		if (dest->acl != NULL)
+			dest->valid_mask &= ~ATTR_ACL;
 		dest->acl = NULL;
-		dest->valid_mask &= ~ATTR_ACL;
 	}
 
 	if (pass_refs && ((save_request_mask & ATTR4_FS_LOCATIONS) != 0)) {
@@ -647,9 +637,9 @@ struct async_process_data {
 	/** Indicator callback is done. */
 	bool done;
 	/** Mutex to protect done and condition variable. */
-	pthread_mutex_t *mutex;
+	pthread_mutex_t *fsa_mutex;
 	/** Condition variable to signal callback is done. */
-	pthread_cond_t *cond;
+	pthread_cond_t *fsa_cond;
 };
 
 extern void fsal_read(struct fsal_obj_handle *obj_hdl,
@@ -669,5 +659,63 @@ fsal_status_t fsal_listxattr_helper(const char *buf,
 				    bool_t *lxr_eof,
 				    xattrlist4 *lxr_names);
 
+/**
+ * @brief Structure to hold FD LURU paramaters
+ */
+
+struct fd_lru_parameter {
+	/** Base interval in seconds between runs of the LRU cleaner
+	    thread. Defaults to 60, settable with LRU_Run_Interval. */
+	uint32_t lru_run_interval;
+	/**  If Cache_FDs is false then FDs will remained cached till the LRU
+	 *   reaper thread invokes and tries to close the FDs.
+	 *
+	 *   If Cache_FDs is true (default) then FDs get cached and LRU reaper
+	 *   thread on invocation will try to close the FDs only when the
+	 *   currentopen >= fds_lowat (FD low watermark).
+	 */
+	bool Cache_FDs;
+	/** The percentage of the system-imposed maximum of file
+	    descriptors at which Ganesha will deny requests.
+	    Defaults to 99, settable with FD_Limit_Percent. */
+	uint32_t fd_limit_percent;
+	/** The percentage of the system-imposed maximum of file
+	    descriptors above which Ganesha will make greater efforts
+	    at reaping. Defaults to 90, settable with
+	    FD_HWMark_Percent. */
+	uint32_t fd_hwmark_percent;
+	/** The percentage of the system-imposed maximum of file
+	    descriptors below which Ganesha will not reap file
+	    descriptors.  Defaults to 50, settable with FD_LWMark_Percent. */
+	uint32_t fd_lwmark_percent;
+	/** Roughly, the amount of work to do on each pass through the
+	    thread under normal conditions.  (Ideally, a multiple of
+	    the number of lanes.)  Defaults to 1000, settable with
+	    Reaper_Work. */
+	uint32_t reaper_work;
+	/** The amount of work for the reaper thread to do per-lane
+	    under normal conditions. Settable with Repaper_Work_Per_Thread */
+	uint32_t reaper_work_per_lane;
+	/** The largest window (as a percentage of the system-imposed
+	    limit on FDs) of work that we will do in extremis.
+	    Defaults to 40, settable with Biggest_Window */
+	uint32_t biggest_window;
+	/** Percentage of progress toward the high water mark required
+	    in a pass through the thread when in extremis.
+	    Defaults to 5, settable with Required_Progress. */
+	uint32_t required_progress;
+	/** Number of failures to approach the high watermark before
+	    we disable caching, when in extremis.  Defaults to 8,
+	    settable with Futility_Count */
+	uint32_t futility_count;
+	uint32_t fd_fallback_limit;
+};
+
+extern int32_t fsal_fd_global_counter;
+
+void fsal_init_fds_limit(struct fd_lru_parameter *params);
+fsal_status_t fd_lru_pkginit(struct fd_lru_parameter *params);
+fsal_status_t fd_lru_pkgshutdown(void);
+
 #endif				/* !FSAL_H */
 /** @} */
