@@ -162,7 +162,6 @@ typedef uint32_t fsal_aceflag_t;
 
 /** ACE internal flags */
 
-#define FSAL_ACE_IFLAG_MODE_GEN       0x10000000
 #define FSAL_ACE_IFLAG_EXCLUDE_FILES  0x40000000
 #define FSAL_ACE_IFLAG_EXCLUDE_DIRS   0x20000000
 #define FSAL_ACE_IFLAG_SPECIAL_ID     0x80000000
@@ -217,7 +216,7 @@ typedef struct fsal_ace__ {
 typedef struct fsal_acl__ {
 	uint32_t naces;
 	fsal_ace_t *aces;
-	pthread_rwlock_t lock;
+	pthread_rwlock_t acl_lock;
 	uint32_t ref;
 } fsal_acl_t;
 
@@ -290,16 +289,9 @@ typedef struct fsal_acl_data__ {
 	  IS_FSAL_ACE_USER(ACE, FSAL_ACE_SPECIAL_EVERYONE)
 #define IS_FSAL_ACE_SPECIAL_MASK(ACE) \
 	IS_FSAL_ACE_USER(ACE, FSAL_ACE_SPECIAL_MASK)
-#define IS_FSAL_ACE_SPECIAL(ACE) \
-	(IS_FSAL_ACE_SPECIAL_OWNER(ACE) || \
-	 IS_FSAL_ACE_SPECIAL_GROUP(ACE) || \
-	 IS_FSAL_ACE_SPECIAL_EVERYONE(ACE))
 
 /* Macros for internal NFS4 ACE flags. */
 
-#define IS_FSAL_ACE_MODE_GEN(ACE) \
-	IS_FSAL_ACE_BIT(GET_FSAL_ACE_IFLAG(ACE), \
-			FSAL_ACE_IFLAG_MODE_GEN)
 #define IS_FSAL_ACE_SPECIAL_ID(ACE) \
 	IS_FSAL_ACE_BIT(GET_FSAL_ACE_IFLAG(ACE), \
 			FSAL_ACE_IFLAG_SPECIAL_ID)
@@ -357,7 +349,7 @@ typedef struct fsal_acl_data__ {
 typedef struct fsal_fs_locations {
 	uint32_t ref;
 	uint32_t nservers;	/* size of server array */
-	pthread_rwlock_t lock;
+	pthread_rwlock_t fsloc_lock;
 	char *fs_root;
 	char *rootpath;
 	utf8string *server;
@@ -925,6 +917,41 @@ typedef enum {
 
 typedef char fsal_verifier_t[NFS4_VERIFIER_SIZE];
 
+enum fd_states {
+	FD_LOW,
+	FD_MIDDLE,
+	FD_HIGH,
+	FD_LIMIT,
+};
+
+struct fd_lru_state {
+	uint32_t fds_system_imposed;
+	uint32_t fds_hard_limit;
+	uint32_t fds_hiwat;
+	uint32_t fds_lowat;
+	/** This is the actual counter of 'futile' attempts at reaping
+	    made  in a given time period.  When it reaches the futility
+	    count, we turn off caching of file descriptors. */
+	uint32_t futility;
+	uint32_t biggest_window; /* defaults to 40% of fd_limit */
+	uint64_t prev_fd_count;	/* previous # of open fds */
+	time_t prev_time;	/* previous time the gc thread was run. */
+	uint32_t fd_state;
+	uint32_t fd_fallback_limit;
+};
+
+enum fsal_fd_type {
+	/** @todo FSF - to be removed when old style is deprecated */
+	FSAL_FD_OLD_STYLE,
+	FSAL_FD_GLOBAL,
+	FSAL_FD_STATE,
+	FSAL_FD_TEMP,
+};
+
+extern struct fd_lru_state fd_lru_state;
+
+struct fsal_export;
+
 /**
  * @brief Generic file handle.
  */
@@ -933,8 +960,74 @@ struct fsal_fd {
 	 *  file descriptor structure.
 	 */
 	fsal_openflags_t openflags;
+	int32_t fd_work;
+	int32_t io_work;
+	int32_t want_read;
+	int32_t want_write;
+	struct fsal_export *fsal_export;
+	/** LRU for open global fd **/
+	struct glist_head fd_lru;
+	/** work_mutex protects fd work */
+	pthread_mutex_t work_mutex;
+	/** condition to signal when io work may commence */
+	pthread_cond_t io_work_cond;
+	/** condition to signal when fd work may commence */
+	pthread_cond_t fd_work_cond;
+	/** Indicate if should be closed on complete. */
+	bool close_on_complete;
+	/** Indicate if LRU reclaim wants to operate on this fd */
+	uint32_t lru_reclaim;
+	/** Type of fd */
+	enum fsal_fd_type fd_type;
 };
 
+/* FSAL_FD_TEMP initializer, the work_mutex, io_work_cond, and fd_work_cond are
+ * not initialized as these are never used with a FSAL_FD_TEMP.
+ */
+#define FSAL_FD_INIT {	.openflags = FSAL_O_CLOSED, \
+			.fd_work = 0, \
+			.io_work = 0, \
+			.want_read = 0, \
+			.want_write = 0, \
+			.fsal_export = op_ctx->fsal_export, \
+			.fd_lru = {NULL, NULL}, \
+			/* work_mutex unused for FSAL_FD_TEMP */ \
+			/* io_work_cond unused for FSAL_FD_TEMP */ \
+			/* fd_work_cond unused for FSAL_FD_TEMP */ \
+			.close_on_complete = false, \
+			.lru_reclaim = 0, \
+			.fd_type = FSAL_FD_TEMP }
+
+static inline void init_fsal_fd(struct fsal_fd *fsal_fd,
+				enum fsal_fd_type fd_type,
+				struct fsal_export *fsal_export)
+{
+	memset(fsal_fd, 0, sizeof(*fsal_fd));
+	if (fd_type != FSAL_FD_TEMP) {
+		/* Only need to initialize work_mutex and work_cond if not
+		 * a FSAL_FD_TEMP. They are not used for FSAL_FD_TEMP.
+		 */
+		PTHREAD_MUTEX_init(&fsal_fd->work_mutex, NULL);
+		PTHREAD_COND_init(&fsal_fd->fd_work_cond, NULL);
+		PTHREAD_COND_init(&fsal_fd->io_work_cond, NULL);
+	}
+	fsal_fd->fd_type = fd_type;
+	fsal_fd->fsal_export = fsal_export;
+}
+
+static inline void destroy_fsal_fd(struct fsal_fd *fsal_fd)
+{
+	if (fsal_fd->fd_type != FSAL_FD_TEMP) {
+		/* Only need to destroy work_mutex, io_work_cond, and
+		 * fd_work_cond if not a FSAL_FD_TEMP. They are not used for
+		 * FSAL_FD_TEMP.
+		 */
+		PTHREAD_MUTEX_destroy(&fsal_fd->work_mutex);
+		PTHREAD_COND_destroy(&fsal_fd->fd_work_cond);
+		PTHREAD_COND_destroy(&fsal_fd->io_work_cond);
+	}
+}
+
 /**
  * @brief The ref counted share reservation state.
  *
