@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -177,7 +178,7 @@ struct fsal_up_vector {
 	 */
 	fsal_status_t (*update)(const struct fsal_up_vector *vec,
 				struct gsh_buffdesc *obj,
-				struct attrlist *attr,
+				struct fsal_attrlist *attr,
 				uint32_t flags);
 
 	/** Grant a lock to a client
@@ -298,7 +299,8 @@ fsal_status_t up_async_invalidate(struct fridgethr *fr,
 				  void *cb_arg);
 fsal_status_t up_async_update(struct fridgethr *fr,
 			      const struct fsal_up_vector *vec,
-			      struct gsh_buffdesc *obj, struct attrlist *attr,
+			      struct gsh_buffdesc *obj,
+			      struct fsal_attrlist *attr,
 			      uint32_t flags,
 			      void (*cb)(void *, fsal_status_t),
 			      void *cb_arg);
@@ -344,6 +346,7 @@ int async_cbgetattr(struct fridgethr *fr, struct fsal_obj_handle *obj,
 		    nfs_client_id_t *client);
 
 void up_ready_init(struct fsal_up_vector *up_ops);
+void up_ready_destroy(struct fsal_up_vector *up_ops);
 void up_ready_set(struct fsal_up_vector *up_ops);
 void up_ready_wait(struct fsal_up_vector *up_ops);
 void up_ready_cancel(struct fsal_up_vector *up_ops);
