@@ -48,6 +48,7 @@
 #include <stdbool.h>
 
 #include "nfs4.h"
+#include "gsh_recovery.h"
 #include "gsh_rpc.h"
 
 /**
@@ -98,6 +99,11 @@ typedef enum protos {
  */
 #define NB_WORKER_THREAD_DEFAULT 256
 
+/**
+ * @brief Default value for core_param.drc.recycle_hiwat
+ */
+#define DRC_RECYCLE_HIWAT 1024
+
 /**
  * @brief Default value for core_param.drc.tcp.npart
  */
@@ -168,6 +174,11 @@ typedef enum protos {
  */
 #define NFS_DEFAULT_RECV_BUFFER_SIZE 1048576
 
+/**
+ * @brief Default Monitoring Port.
+ */
+#define MONITORING_PORT 9587
+
 /**
  * @brief Turn off all protocols
  */
@@ -215,12 +226,18 @@ typedef enum protos {
 #define UDP_LISTENER_MASK (UDP_LISTENER_ALL | UDP_LISTENER_MOUNT)
 
 typedef struct nfs_core_param {
+	/** The list of hosts allowed to use the HAProxy protocol. These are
+	 *  the hosts running HAProxy, acting as load balancing/proxy. Actual
+	 *  end clients are handled in EXPORT CLIENT lists.
+	 */
+	struct glist_head haproxy_hosts;
+	/** The IPv4 or IPv6 address to which to bind for our
+	    listening port.  Set by the Bind_Addr option.
+	    Must be 8-byte aligned (see sockaddr_t). */
+	sockaddr_t bind_addr;
 	/** An array of port numbers, one for each protocol.  Set by
 	    the NFS_Port, MNT_Port, NLM_Port, and Rquota_Port options. */
 	uint16_t port[P_COUNT];
-	/** The IPv4 or IPv6 address to which to bind for our
-	    listening port.  Set by the Bind_Addr option. */
-	sockaddr_t bind_addr;
 	/** An array of RPC program numbers.  The correct values, by
 	    default, they may be set to incorrect values with the
 	    NFS_Program, MNT_Program, NLM_Program, and
@@ -249,6 +266,8 @@ typedef struct nfs_core_param {
 		/** Whether to disable the DRC entirely.  Defaults to
 		    false, settable by DRC_Disabled. */
 		bool disabled;
+		/** High water mark for len of recycle queue for DRCs */
+		uint32_t recycle_hiwat;
 		/* Parameters controlling TCP specific DRC behavior. */
 		struct {
 			/** Number of partitions in the tree for the
@@ -364,6 +383,8 @@ typedef struct nfs_core_param {
 	/** Whether to support the Network Lock Manager protocol.
 	    Defaults to true and is settable with Enable_NLM. */
 	bool enable_NLM;
+	/** Whether to disable NLM_SHARE and NLM_UNSHARE */
+	bool disable_NLM_SHARE;
 	/** Whether to use the supplied name rather than the IP
 	    address in NSM operations.  Settable with
 	    NSM_Use_Caller_Name. */
@@ -418,6 +439,12 @@ typedef struct nfs_core_param {
 	uint32_t heartbeat_freq;
 	/** Whether to use device major/minor for fsid. Defaults to false. */
 	bool fsid_device;
+	/** How many times to attempt retry of stat while resolving POSIX
+	 *  filesystems */
+	uint32_t resolve_fs_retries;
+	/** Delay (in mili-seconds) between stat calls when trying to resolve
+	 *  POSIX filesystems */
+	uint32_t resolve_fs_delay;
 	/** Whether to use Pseudo (true) or Path (false) for NFS v3 and 9P
 	    mounts. */
 	bool mount_path_pseudo;
@@ -449,6 +476,21 @@ typedef struct nfs_core_param {
 		nfs3_complete_read.
 		Defaults to true and settable by Getattrs_In_Complete_Read. */
 	bool getattrs_in_complete_read;
+	/** Enable malloc trim */
+	bool malloc_trim;
+	/** Minimum threshold value to call malloc_trim. The malloc_trim
+	* will be called once memory allocation exceeds minimum value.
+	* Size in MB's. Note, this setting has no effect when
+	* Enable_malloc_trim is set to false.
+	*/
+	uint32_t malloc_trim_minthreshold;
+#ifdef USE_MONITORING
+	/** Monitoring port number. */
+	uint16_t monitoring_port;
+#endif
+	/** if  Manage_Gids=True and group resolution fails,
+	 *  then use gid data from rpc request */
+	bool enable_rpc_cred_fallback;
 } nfs_core_parameter_t;
 
 /** @} */
@@ -484,19 +526,6 @@ typedef struct nfs_core_param {
  */
 #define DELEG_RECALL_RETRY_DELAY_DEFAULT 1
 
-enum recovery_backend {
-	RECOVERY_BACKEND_FS,
-	RECOVERY_BACKEND_FS_NG,
-	RECOVERY_BACKEND_RADOS_KV,
-	RECOVERY_BACKEND_RADOS_NG,
-	RECOVERY_BACKEND_RADOS_CLUSTER,
-};
-
-/**
- * @brief Default value of recovery_backend.
- */
-#define RECOVERY_BACKEND_DEFAULT RECOVERY_BACKEND_FS
-
 /**
  * @brief NFSv4 minor versions
  */
