@@ -30,10 +30,11 @@
 /**
  * @brief Trace an increase in refcount of an entry
  *
- * @param[in] function	Name of function taking ref
- * @param[in] line	Line number of call
+ * @param[in] function		Name of function taking ref
+ * @param[in] line		Line number of call
  * @param[in] obj_handle	Address of obj_handle
- * @param[in] refcnt	Refcount after increase
+ * @param[in] refcn		Refcount after increase
+ * @param[in] active_refcnt	Active Refcnt after increase
  */
 TRACEPOINT_EVENT(
 	mdcache,
@@ -42,13 +43,16 @@ TRACEPOINT_EVENT(
 		int, line,
 		void *, obj_handle,
 		void *, sub_handle,
-		int32_t, refcnt),
+		int32_t, refcnt,
+		int32_t, active_refcnt),
 	TP_FIELDS(
 		ctf_string(function, function)
 		ctf_integer(int, line, line)
 		ctf_integer_hex(void *, obj_handle, obj_handle)
 		ctf_integer_hex(void *, sub_handle, sub_handle)
+		ctf_string(active_refcnt, active_refcnt)
 		ctf_integer(int32_t, refcnt, refcnt)
+		ctf_integer(int32_t, active_refcnt, active_refcnt)
 	)
 )
 
@@ -60,10 +64,11 @@ TRACEPOINT_LOGLEVEL(
 /**
  * @brief Trace a decrease in refcount of an entry
  *
- * @param[in] function	Name of function releasing ref
- * @param[in] line	Line number of call
+ * @param[in] function		Name of function releasing ref
+ * @param[in] line		Line number of call
  * @param[in] obj_handle	Address of obj_handle
- * @param[in] refcnt	Refcount after decrease
+ * @param[in] refcnt		Refcount after decrease
+ * @param[in] active_refcnt	Active Refcnt after decrease
  */
 TRACEPOINT_EVENT(
 	mdcache,
@@ -72,13 +77,15 @@ TRACEPOINT_EVENT(
 		int, line,
 		void *, obj_handle,
 		void *, sub_handle,
-		int32_t, refcnt),
+		int32_t, refcnt,
+		int32_t, active_refcnt),
 	TP_FIELDS(
 		ctf_string(function, function)
 		ctf_integer(int, line, line)
 		ctf_integer_hex(void *, obj_handle, obj_handle)
 		ctf_integer_hex(void *, sub_handle, sub_handle)
 		ctf_integer(int32_t, refcnt, refcnt)
+		ctf_integer(int32_t, active_refcnt, active_refcnt)
 	)
 )
 
