@@ -44,6 +44,8 @@ struct gssd_k5_kt_princ;
 
 extern char *ccachesearch[];
 
+void gssd_init_cred_cache(void);
+void gssd_shutdown_cred_cache(void);
 int gssd_check_mechs(void);
 int gssd_refresh_krb5_machine_credential(char *,
 					 struct gssd_k5_kt_princ *,
