@@ -41,7 +41,6 @@
 
 #include "sal_data.h"
 #include "gsh_config.h"
-#include "gsh_wait_queue.h"
 
 #ifdef _USE_9P
 #include "9p.h"
@@ -58,6 +57,9 @@
 
 extern char *nfs_host_name;
 
+extern char cid_server_owner[MAXNAMLEN+1]; /* max hostname length */
+extern char *cid_server_scope;
+
 /*
  * Bind protocol family, pending a richer interface model.
  */
