@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  *
  * contributeur : Sachin Bhamare   sbhamare@panasas.com
@@ -23,7 +24,7 @@
 /**
  * @file subr.h
  * @author Sachin Bhamare <sbhamare@panasas.com>
- * @brief platform dependant subroutine type definitions
+ * @brief platform dependent subroutine type definitions
  */
 
 #ifndef SUBR_OS_H
@@ -31,6 +32,7 @@
 
 #include <stdbool.h>
 #include <extended_types.h>
+#include <unistd.h>
 
 #ifndef UTIME_NOW
 #define UTIME_NOW	-1
@@ -53,8 +55,10 @@ int vfs_readents(int fd, char *buf, unsigned int bcount, off_t *basepp);
 bool to_vfs_dirent(char *buf, int bpos, struct vfs_dirent *vd, off_t base);
 void setuser(uid_t uid);
 void setgroup(gid_t gid);
-uid_t getuser(void);
-gid_t getgroup(void);
 int set_threadgroups(size_t size, const gid_t *list);
 
+/* Define these aliases for all OSs for consistency with setuser/setgroup */
+#define getuser geteuid
+#define getgroup getegid
+
 #endif/* SUBR_OS_H */
