@@ -131,16 +131,36 @@ extern hash_table_t *ht_session_id;
  */
 #define NFS41_NB_SLOTS_DEF 64
 
+
+/**
+ * @brief Default string length of all operations in one compound
+ */
+#define NFS4_COMPOUND_OPERATIONS_STR_LEN 256
+
+/**
+ * @brief the simple info of last request to save in slot
+ */
+
+struct nfs41_cached_req {
+	nfs_opnum4 opcodes[NFS4_MAX_OPERATIONS];	/*< opcodes
+		of compound */
+	uint32_t opcode_num;	/*< number of opcode in one compound */
+	uint32_t xid;	/*< Xid of request */
+	sequenceid4 seq_id;	/*< Sequence id of request */
+	uint64_t finish_time_ms;	/*< Finish time of request in ms */
+};
+
 /**
  * @brief Members in the slot table
  */
 
 typedef struct nfs41_session_slot__ {
 	sequenceid4 sequence;	/*< Sequence number of this operation */
-	pthread_mutex_t lock;	/*< Lock on the slot */
+	pthread_mutex_t slot_lock;	/*< Lock on the slot */
 	struct COMPOUND4res_extended *cached_result;	/*< NFv41: pointer to
 							   cached RPC result in
 							   a session's slot */
+	struct nfs41_cached_req last_req; /*< Last request of the slot */
 } nfs41_session_slot_t;
 
 /**
@@ -283,6 +303,16 @@ struct state_nlm_share {
 	unsigned int share_deny_counts[fsm_DRW + 1];
 };
 
+/**
+ * @brief The number of bytes in the stateid.other
+ *
+ * The value 12 is fixed by RFC 3530/RFC 5661.
+ */
+#define OTHERSIZE 12
+
+extern char all_zero[OTHERSIZE];
+extern char all_ones[OTHERSIZE];
+
 /**
  * @brief Data for a set of locks
  */
@@ -293,7 +323,7 @@ struct state_lock {
 						   This field MUST be first */
 	struct glist_head state_sharelist;	/*< List of states related
 						   to a share */
-	state_t *openstate;	/*< The related open-state */
+	char openstate_key[OTHERSIZE]; /*< The hash key of related open-state */
 };
 
 /**
@@ -368,16 +398,6 @@ union state_data {
 	uint32_t io_advise;
 };
 
-/**
- * @brief The number of bytes in the stateid.other
- *
- * The value 12 is fixed by RFC 3530/RFC 5661.
- */
-#define OTHERSIZE 12
-
-extern char all_zero[OTHERSIZE];
-extern char all_ones[OTHERSIZE];
-
 /**
  * @brief Structure representing a single NFSv4 state
  *
@@ -386,6 +406,7 @@ extern char all_ones[OTHERSIZE];
  *
  * The lists are protected by the st_lock
  */
+typedef void (state_free_t) (struct state_t *state);
 
 struct state_t {
 	struct glist_head state_list;	/**< List of states on a file */
@@ -396,11 +417,12 @@ struct state_t {
 	struct glist_head state_list_all;    /**< Global list of all stateids */
 #endif
 	pthread_mutex_t state_mutex; /**< Mutex protecting following pointers */
+	state_free_t *state_free; /**< function to free if default gsh_free
+				      doesn't work. */
 	struct gsh_export *state_export; /**< Export this entry belongs to */
 	/* Don't re-order or move these next two.  They are used for hashing */
 	state_owner_t *state_owner;	/**< State Owner related to state */
 	struct fsal_obj_handle *state_obj; /**< owning object */
-	struct fsal_export *state_exp;  /**< FSAL export */
 	union state_data state_data;
 	enum state_type state_type;
 	u_int32_t state_seqid;		/**< The NFSv4 Sequence id */
@@ -412,6 +434,14 @@ struct state_t {
 					   state. */
 };
 
+static inline void free_state(struct state_t *state)
+{
+	if (state->state_free != NULL)
+		state->state_free(state);
+	else
+		gsh_free(state);
+}
+
 /* Macros to compare and copy state_t to a struct stateid4 */
 #define SAME_STATEID(id4, state) \
 	((id4)->seqid == (state)->state_seqid && \
@@ -487,8 +517,22 @@ typedef enum care_t {
 	CARE_MONITOR		/*< Will actively monitor client status */
 } care_t;
 
+#ifdef _USE_NLM
 extern hash_table_t *ht_nsm_client;
 extern hash_table_t *ht_nlm_client;
+extern pthread_mutex_t granted_mutex;
+extern pthread_mutex_t nlm_async_resp_mutex;
+extern pthread_cond_t nlm_async_resp_cond;
+extern pthread_mutex_t nsm_mutex;
+#endif
+
+extern pthread_mutex_t cached_open_owners_lock;
+
+#ifdef DEBUG_SAL
+extern pthread_mutex_t all_state_owners_mutex;
+extern pthread_mutex_t all_state_v4_mutex;
+#endif
+
 
 /**
  * @brief NSM (rpc.statd) state for a given client.
@@ -887,6 +931,10 @@ struct file_deleg_stats {
 	uint32_t fds_num_opens;         /* total num of opens so far. */
 	time_t fds_first_open;          /* time that we started recording
 					   num_opens */
+	uint32_t fds_num_write_opens; /* total num of opens in write mode
+					   incremented irrespective of
+					   whether or not delegation
+					   was offered for the file */
 };
 
 enum cbgetattr_state {
@@ -971,18 +1019,18 @@ struct state_dir {
  * It is a rwlock since most of the time junctions are being looked at not
  * modified.
  *
- * Both of these locks are often used in conjunction with the export->lock, but
- * the rules of lock order are different.
+ * Both of these locks are often used in conjunction with the export->exp_lock,
+ * but the rules of lock order are different.
  *
- * For st_lock, the export->lock MUST NOT be held, the export->lock is often
- * taken while already holding st_lock, so the order is:
+ * For st_lock, the export->exp_lock MUST NOT be held, the export->exp_lock is
+ * often taken while already holding st_lock, so the order is:
  *
- * st_lock THEN export->lock
+ * st_lock THEN export->exp_lock
  *
- * For jct_lock, there is one place where the export->lock is already held,
+ * For jct_lock, there is one place where the export->exp_lock is already held,
  * so it makes sense for the order to be:
  *
- * export->lock THEN jct_lock.
+ * export->exp_lock THEN jct_lock.
  *
  */
 struct state_hdl {
