@@ -42,6 +42,7 @@
 #include <stdint.h>
 #include "sal_data.h"
 #include "fsal.h"
+#include "gsh_recovery.h"
 
 /**
  * @brief Divisions in state and clientid tables.
@@ -65,7 +66,7 @@ const char *state_owner_type_to_str(state_owner_type_t type);
 bool different_owners(state_owner_t *owner1, state_owner_t *owner2);
 int display_owner(struct display_buffer *dspbuf, state_owner_t *owner);
 void inc_state_owner_ref(state_owner_t *owner);
-bool hold_state_owner(state_owner_t *owner);
+bool hold_state_owner_ref(state_owner_t *owner);
 void dec_state_owner_ref(state_owner_t *owner);
 void free_state_owner(state_owner_t *owner);
 
@@ -449,6 +450,7 @@ struct state_t *nfs4_State_Get_Obj(struct fsal_obj_handle *obj,
  ******************************************************************************/
 
 int reserve_lease(nfs_client_id_t *clientid);
+bool reserve_lease_or_expire(nfs_client_id_t *clientid, bool update);
 void update_lease(nfs_client_id_t *clientid);
 bool valid_lease(nfs_client_id_t *clientid);
 
@@ -928,7 +930,6 @@ bool nfs4_check_deleg_reclaim(nfs_client_id_t *, nfs_fh4 *);
 void nfs4_record_revoke(nfs_client_id_t *, nfs_fh4 *);
 
 /* Recovery backend management */
-enum recovery_backend;
 const char *recovery_backend_str(enum recovery_backend recovery_backend);
 int nfs4_recovery_init(void);
 void nfs4_recovery_shutdown(void);
