@@ -45,6 +45,11 @@
 #include <execinfo.h>
 #include <assert.h>
 
+#ifdef USE_UNWIND
+#define UNW_LOCAL_ONLY
+#include <libunwind.h>
+#endif
+
 #include "log.h"
 #include "gsh_list.h"
 #include "gsh_rpc.h"
@@ -96,7 +101,7 @@
 			assert(0);					\
 	} while (0)
 
-pthread_rwlock_t log_rwlock = PTHREAD_RWLOCK_INITIALIZER;
+pthread_rwlock_t log_rwlock;
 
 /* Variables to control log fields */
 
@@ -123,6 +128,7 @@ enum log_flag_index_t {
 	LF_FUNCTION_NAME,	/*< Function name message occurred in. */
 	LF_COMPONENT,		/*< Log component. */
 	LF_LEVEL,		/*< Log level. */
+	LF_OP_ID,		/*< Op id. */
 };
 
 /**
@@ -159,6 +165,7 @@ struct logfields {
 	bool disp_funct;
 	bool disp_comp;
 	bool disp_level;
+	bool disp_op_id;
 	enum timedate_formats_t datefmt;
 	enum timedate_formats_t timefmt;
 	char *user_date_fmt;
@@ -268,12 +275,23 @@ static char program_name[1024];
 static char hostname[256];
 static int syslog_opened;
 
+/* Number of bytes actually usable in the log buffer */
+#define LOG_BUF_USE (LOG_BUFF_LEN + 1)
+
+/* Extra bytes to reserve in log buffer, currently 1 for the '\n' that
+ * log_to_file and log_to_stream adds.
+ */
+#define LOG_BUF_EXTRA 1
+
+/* Final total size of log buffer */
+#define LOG_BUF_ALLOC (LOG_BUF_USE + LOG_BUF_EXTRA)
+
 /*
  * Variables specifiques aux threads.
  */
 
 __thread char thread_name[32];
-__thread char log_buffer[LOG_BUFF_LEN + 1];
+__thread char log_buffer[LOG_BUF_ALLOC];
 __thread char *clientip = NULL;
 
 /* threads keys */
@@ -286,37 +304,13 @@ __thread char *clientip = NULL;
 				 "LOG: " format, \
 				 ## args)
 
-struct cleanup_list_element *cleanup_list;
-
-void RegisterCleanup(struct cleanup_list_element *clean)
-{
-	clean->next = cleanup_list;
-	cleanup_list = clean;
-}
-
-void Cleanup(void)
-{
-	struct cleanup_list_element *c = cleanup_list;
-
-	while (c != NULL) {
-		c->clean();
-		c = c->next;
-	}
-}
-
-void Fatal(void)
-{
-	Cleanup();
-	_exit(2);
-}
-
 #ifdef _DONT_HAVE_LOCALTIME_R
 
 /* Localtime is not reentrant...
  * So we are obliged to have a mutex for calling it.
  * pffff....
  */
-static pthread_mutex_t mutex_localtime = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t mutex_localtime;
 
 /* thread-safe and PORTABLE version of localtime */
 
@@ -344,6 +338,34 @@ static struct tm *Localtime_r(const time_t *p_time, struct tm *p_tm)
 #define Localtime_r localtime_r
 #endif
 
+struct cleanup_list_element *cleanup_list;
+
+void RegisterCleanup(struct cleanup_list_element *clean)
+{
+	clean->next = cleanup_list;
+	cleanup_list = clean;
+}
+
+void Cleanup(void)
+{
+	struct cleanup_list_element *c = cleanup_list;
+
+	while (c != NULL) {
+		c->clean();
+		c = c->next;
+	}
+
+	PTHREAD_RWLOCK_destroy(&log_rwlock);
+#ifdef _DONT_HAVE_LOCALTIME_R
+	PTHREAD_MUTEX_destroy(&mutex_localtime);
+#endif
+}
+
+void Fatal(void)
+{
+	_exit(2);
+}
+
 /*
  * Convert a numeral log level in ascii to
  * the numeral value.
@@ -1027,6 +1049,10 @@ void init_logging(const char *log_path, const int debug_level)
 	int rc;
 
 	/* Finish initialization of and register log facilities. */
+	PTHREAD_RWLOCK_init(&log_rwlock, NULL);
+#ifdef _DONT_HAVE_LOCALTIME_R
+	PTHREAD_MUTEX_init(&mutex_localtime, NULL);
+#endif
 	glist_init(&facility_list);
 	glist_init(&active_facility_list);
 
@@ -1131,7 +1157,7 @@ static int log_to_file(log_header_t headers, void *private,
 
 	len = display_buffer_len(buffer);
 
-	/* Add newline to end of buffer */
+	/* Add newline to end of buffer, this is why LOG_BUF_EXTRA is 1 */
 	buffer->b_start[len] = '\n';
 	buffer->b_start[len + 1] = '\0';
 
@@ -1185,7 +1211,7 @@ static int log_to_stream(log_header_t headers, void *private,
 
 	len = display_buffer_len(buffer);
 
-	/* Add newline to end of buffer */
+	/* Add newline to end of buffer, this is why LOG_BUF_EXTRA is 1 */
 	buffer->b_start[len] = '\n';
 	buffer->b_start[len + 1] = '\0';
 
@@ -1378,6 +1404,16 @@ static int display_log_component(struct display_buffer *dsp_log,
 		    display_printf(dsp_log, "%s :",
 				   tabLogLevel[level].short_str);
 
+	if (b_left > 0 && logfields->disp_op_id) {
+		if (op_ctx)
+			b_left =
+				display_printf(dsp_log, "op_id=%u :",
+					op_ctx->op_id);
+		else
+			b_left =
+				display_printf(dsp_log, "op_id=none :");
+	}
+
 	/* If we overflowed the buffer with the header, just skip it. */
 	if (b_left == 0) {
 		display_reset_buffer(dsp_log);
@@ -1397,7 +1433,7 @@ void display_log_component_level(log_components_t component, const char *file,
 	int b_left;
 	struct glist_head *glist;
 	struct log_facility *facility;
-	struct display_buffer dsp_log = {sizeof(log_buffer),
+	struct display_buffer dsp_log = {LOG_BUF_USE,
 					 log_buffer, log_buffer};
 
 	/* Build up the message and capture the various positions in it. */
@@ -1463,8 +1499,8 @@ static log_levels_t default_log_levels[] = {
 	[COMPONENT_EXPORT] = NIV_EVENT,
 	[COMPONENT_FILEHANDLE] = NIV_EVENT,
 	[COMPONENT_DISPATCH] = NIV_EVENT,
-	[COMPONENT_CACHE_INODE] = NIV_EVENT,
-	[COMPONENT_CACHE_INODE_LRU] = NIV_EVENT,
+	[COMPONENT_MDCACHE] = NIV_EVENT,
+	[COMPONENT_MDCACHE_LRU] = NIV_EVENT,
 	[COMPONENT_HASHTABLE] = NIV_EVENT,
 	[COMPONENT_HASHTABLE_CACHE] = NIV_EVENT,
 	[COMPONENT_DUPREQ] = NIV_EVENT,
@@ -1489,7 +1525,7 @@ static log_levels_t default_log_levels[] = {
 	[COMPONENT_9P_DISPATCH] = NIV_EVENT,
 	[COMPONENT_FSAL_UP] = NIV_EVENT,
 	[COMPONENT_DBUS] = NIV_EVENT,
-	[COMPONENT_NFS_MSK] = NIV_EVENT
+	[COMPONENT_NFS_MSK] = NIV_EVENT,
 };
 
 /* Active set of log levels */
@@ -1535,12 +1571,12 @@ struct log_component_info LogComponents[COMPONENT_COUNT] = {
 	[COMPONENT_DISPATCH] = {
 		.comp_name = "COMPONENT_DISPATCH",
 		.comp_str = "DISP",},
-	[COMPONENT_CACHE_INODE] = {
-		.comp_name = "COMPONENT_CACHE_INODE",
-		.comp_str = "INODE",},
-	[COMPONENT_CACHE_INODE_LRU] = {
-		.comp_name = "COMPONENT_CACHE_INODE_LRU",
-		.comp_str = "INODE LRU",},
+	[COMPONENT_MDCACHE] = {
+		.comp_name = "COMPONENT_MDCACHE",
+		.comp_str = "MDCACHE",},
+	[COMPONENT_MDCACHE_LRU] = {
+		.comp_name = "COMPONENT_MDCACHE_LRU",
+		.comp_str = "MDCACHE LRU",},
 	[COMPONENT_HASHTABLE] = {
 		.comp_name = "COMPONENT_HASHTABLE",
 		.comp_str = "HT",},
@@ -1615,7 +1651,7 @@ struct log_component_info LogComponents[COMPONENT_COUNT] = {
 		.comp_str = "DBUS",},
 	[COMPONENT_NFS_MSK] = {
 		.comp_name = "COMPONENT_NFS_MSK",
-		.comp_str = "NFS_MSK",}
+		.comp_str = "NFS_MSK",},
 };
 
 void DisplayLogComponentLevel(log_components_t component, const char *file,
@@ -1750,8 +1786,8 @@ HANDLE_PROP(NFS_V4);
 HANDLE_PROP(EXPORT);
 HANDLE_PROP(FILEHANDLE);
 HANDLE_PROP(DISPATCH);
-HANDLE_PROP(CACHE_INODE);
-HANDLE_PROP(CACHE_INODE_LRU);
+HANDLE_PROP(MDCACHE);
+HANDLE_PROP(MDCACHE_LRU);
 HANDLE_PROP(HASHTABLE);
 HANDLE_PROP(HASHTABLE_CACHE);
 HANDLE_PROP(DUPREQ);
@@ -1789,8 +1825,8 @@ static struct gsh_dbus_prop *log_props[] = {
 	LOG_PROPERTY_ITEM(EXPORT),
 	LOG_PROPERTY_ITEM(FILEHANDLE),
 	LOG_PROPERTY_ITEM(DISPATCH),
-	LOG_PROPERTY_ITEM(CACHE_INODE),
-	LOG_PROPERTY_ITEM(CACHE_INODE_LRU),
+	LOG_PROPERTY_ITEM(MDCACHE),
+	LOG_PROPERTY_ITEM(MDCACHE_LRU),
 	LOG_PROPERTY_ITEM(HASHTABLE),
 	LOG_PROPERTY_ITEM(HASHTABLE_CACHE),
 	LOG_PROPERTY_ITEM(DUPREQ),
@@ -1914,6 +1950,8 @@ static struct config_item format_options[] = {
 		       logfields, disp_comp),
 	CONF_ITEM_BOOL("LEVEL", true,
 		       logfields, disp_level),
+	CONF_ITEM_BOOL("OP_ID", false,
+		       logfields, disp_op_id),
 	CONFIG_EOL
 };
 
@@ -2045,13 +2083,17 @@ static struct config_item component_levels[] = {
 	CONF_INDEX_TOKEN("DISP", NB_LOG_LEVEL, log_levels,
 			 COMPONENT_DISPATCH, int),
 	CONF_INDEX_TOKEN("CACHE_INODE", NB_LOG_LEVEL, log_levels,
-			 COMPONENT_CACHE_INODE, int),
+			 COMPONENT_MDCACHE, int),
 	CONF_INDEX_TOKEN("INODE", NB_LOG_LEVEL, log_levels,
-			 COMPONENT_CACHE_INODE, int),
+			 COMPONENT_MDCACHE, int),
+	CONF_INDEX_TOKEN("MDCACHE", NB_LOG_LEVEL, log_levels,
+			 COMPONENT_MDCACHE, int),
 	CONF_INDEX_TOKEN("CACHE_INODE_LRU", NB_LOG_LEVEL, log_levels,
-			 COMPONENT_CACHE_INODE_LRU, int),
+			 COMPONENT_MDCACHE_LRU, int),
 	CONF_INDEX_TOKEN("INODE_LRU", NB_LOG_LEVEL, log_levels,
-			 COMPONENT_CACHE_INODE_LRU, int),
+			 COMPONENT_MDCACHE_LRU, int),
+	CONF_INDEX_TOKEN("MDCACHE_LRU", NB_LOG_LEVEL, log_levels,
+			 COMPONENT_MDCACHE_LRU, int),
 	CONF_INDEX_TOKEN("HASHTABLE", NB_LOG_LEVEL, log_levels,
 			 COMPONENT_HASHTABLE, int),
 	CONF_INDEX_TOKEN("HT", NB_LOG_LEVEL, log_levels,
@@ -2543,9 +2585,9 @@ static struct config_item logging_params[] = {
 	CONF_ITEM_UI32("RPC_Debug_Flags", 0, UINT32_MAX,
 		       TIRPC_DEBUG_FLAG_DEFAULT,
 		       logger_config, rpc_debug_flags),
-	CONF_ITEM_BLOCK("Facility", facility_params,
-			facility_init, facility_commit,
-			logger_config, facility_list),
+	CONF_ITEM_BLOCK_MULT("Facility", facility_params,
+			     facility_init, facility_commit,
+			     logger_config, facility_list),
 	CONF_ITEM_BLOCK("Format", format_options,
 			format_init, format_commit,
 			logger_config, logfields),
@@ -2594,6 +2636,94 @@ int read_log_config(config_file_t in_config,
 		return -1;
 }				/* read_log_config */
 
+#ifdef USE_UNWIND
+void gsh_libunwind(void)
+{
+	unw_cursor_t cursor;
+	unw_context_t uc;
+	unsigned int i = 0;
+	char procname[256];
+	unw_word_t ip, sp, off;
+
+	struct log_facility *facility;
+	struct glist_head *glist;
+	int fd = -1;
+	char buffer[256];
+	int n;
+
+	procname[sizeof(procname) - 1] = '\0';
+
+	if (unw_getcontext(&uc) != 0)
+		goto libunwind_failed;
+
+	if (unw_init_local(&cursor, &uc) != 0)
+		goto libunwind_failed;
+
+	/* Find an active log facility */
+	PTHREAD_RWLOCK_rdlock(&log_rwlock);
+	glist_for_each(glist, &active_facility_list) {
+		facility = glist_entry(glist, struct log_facility, lf_active);
+		if (facility->lf_func == log_to_file) {
+			fd = open((char *)facility->lf_private,
+				O_WRONLY | O_APPEND | O_CREAT, log_mask);
+			break;
+		}
+	}
+
+	LogMajor(COMPONENT_INIT, "BACKTRACE:");
+
+	do {
+		ip = sp = 0;
+		unw_get_reg(&cursor, UNW_REG_IP, &ip);
+		unw_get_reg(&cursor, UNW_REG_SP, &sp);
+		switch (unw_get_proc_name(&cursor,
+			procname, sizeof(procname) - 1, &off)) {
+			case 0:
+				/* Name found. */
+			case -UNW_ENOMEM:
+				/* Name truncated. */
+				if (fd != -1) {
+					n = snprintf(buffer, sizeof(buffer),
+						" #%u %s + %#llx [ip=%#llx] [sp=%#llx]\n",
+						i, procname, (long long)off,
+						(long long)ip, (long long) sp);
+						write(fd, buffer, n);
+				} else {
+					LogMajor(COMPONENT_INIT,
+					" #%u %s + %#llx [ip=%#llx] [sp=%#llx]",
+					i, procname, (long long)off,
+					(long long)ip, (long long) sp);
+				}
+				break;
+			default:
+			/* case -UNW_ENOINFO: */
+			/* case -UNW_EUNSPEC: */
+				if (fd != -1) {
+					snprintf(buffer, sizeof(buffer),
+						" #%u %s [ip=%#llx] [sp=%#llx]\n",
+						i, "<unknown symbol>",
+						(long long)ip, (long long) sp);
+						write(fd, buffer, n);
+				} else {
+					LogMajor(COMPONENT_INIT,
+						" #%u %s [ip=%#llx] [sp=%#llx]",
+						i, "<unknown symbol>",
+						(long long)ip, (long long) sp);
+				}
+		}
+		++i;
+	} while (unw_step(&cursor) > 0);
+
+PTHREAD_RWLOCK_unlock(&log_rwlock);
+return;
+
+libunwind_failed:
+	LogCrit(COMPONENT_INIT,
+		"unable to produce a stack trace with libunwind");
+}
+
+#endif
+
 void gsh_backtrace(void)
 {
 #define MAX_STACK_DEPTH		32	/* enough ? */
