@@ -340,6 +340,54 @@ class AdminInterface():
         msg = reply[1]
         return status, msg
 
+    def trim_enable(self):
+        method = self.dbusobj.get_dbus_method("trim_enable",
+                                              self.dbus_interface)
+        try:
+           reply = method()
+        except dbus.exceptions.DBusException as e:
+           return False, e
+
+        status = reply[0]
+        msg = reply[1]
+        return status, msg
+
+    def trim_disable(self):
+        method = self.dbusobj.get_dbus_method("trim_disable",
+                                              self.dbus_interface)
+        try:
+           reply = method()
+        except dbus.exceptions.DBusException as e:
+           return False, e
+
+        status = reply[0]
+        msg = reply[1]
+        return status, msg
+
+    def trim_call(self):
+        method = self.dbusobj.get_dbus_method("trim_call",
+                                              self.dbus_interface)
+        try:
+           reply = method()
+        except dbus.exceptions.DBusException as e:
+           return False, e
+
+        status = reply[0]
+        msg = reply[1]
+        return status, msg
+
+    def trim_status(self):
+        method = self.dbusobj.get_dbus_method("trim_status",
+                                              self.dbus_interface)
+        try:
+           reply = method()
+        except dbus.exceptions.DBusException as e:
+           return False, e
+
+        status = reply[0]
+        msg = reply[1]
+        return status, msg
+
     def GetAll(self):
         method = self.dbusobj.get_dbus_method("GetAll",
                                               "org.freedesktop.DBus.Properties")
