@@ -65,9 +65,10 @@ To enable/disable stat counters use:
     print(message.format(progname=sys.argv[0]))
     sys.exit(return_code)
 
+DEFAULT_COMMAND = 'global'
 output_json = False
 if (len(sys.argv) < 2):
-    command = 'global'
+    command = DEFAULT_COMMAND
 else:
     command = sys.argv[1]
     opts = sys.argv[2:]
@@ -80,7 +81,7 @@ if command == 'json':
     else:
         command = DEFAULT_COMMAND
 
-json_not_available = ('fsal', 'reset', 'enable')
+json_not_available = ('fsal', 'reset', 'enable', 'disable')
 if output_json and command in json_not_available:
     sys.exit("{0} command is not available in json.".format(command))
 
