@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: MIT
 /* city.c - cityhash-c
  * CityHash on C
  * Copyright (c) 2011-2012, Alexander Nusov
@@ -35,6 +36,7 @@
 
 #include <string.h>
 #include "city.h"
+#include <misc/portable.h>
 
 static uint64 UNALIGNED_LOAD64(const char *p)
 {
@@ -59,21 +61,6 @@ static uint32 UNALIGNED_LOAD32(const char *p)
 
 #else
 
-#ifdef _MSC_VER
-#include <stdlib.h>
-#define bswap_32(x) _byteswap_ulong(x)
-#define bswap_64(x) _byteswap_uint64(x)
-
-#elif defined(__APPLE__)
-/* Mac OS X / Darwin features */
-#include <libkern/OSByteOrder.h>
-#define bswap_32(x) OSSwapInt32(x)
-#define bswap_64(x) OSSwapInt64(x)
-
-#else
-#include <byteswap.h>
-#endif
-
 #define uint32_in_expected_order(x) (bswap_32(x))
 #define uint64_in_expected_order(x) (bswap_64(x))
 
