@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -50,6 +51,7 @@
 #include <pthread.h>
 #include <assert.h>
 #include <arpa/inet.h>
+#include <fnmatch.h>
 #include "gsh_list.h"
 #include "fsal.h"
 #include "nfs_core.h"
@@ -66,13 +68,15 @@
 #include "gsh_intrinsic.h"
 #include "server_stats.h"
 #include "sal_functions.h"
+#include "nfs_ip_stats.h"
+#include "netgroup_cache.h"
 
 /* Clients are stored in an AVL tree
  */
 
 struct client_by_ip {
 	struct avltree t;
-	pthread_rwlock_t lock;
+	pthread_rwlock_t cip_lock;
 	struct avltree_node **cache;
 	uint32_t cache_sz;
 };
@@ -83,7 +87,7 @@ static struct client_by_ip client_by_ip;
  * @brief Compute cache slot for an entry
  *
  * This function computes a hash slot, taking an address modulo the
- * number of cache slotes (which should be prime).
+ * number of cache slots (which should be prime).
  *
  * @param wt [in] The table
  * @param ptr [in] Entry address
@@ -137,7 +141,7 @@ struct gsh_client *get_gsh_client(sockaddr_t *client_ipaddr, bool lookup_only)
 	uint64_t hash = hash_sockaddr(client_ipaddr, true);
 
 	memcpy(&v.cl_addrbuf, client_ipaddr, sizeof(v.cl_addrbuf));
-	PTHREAD_RWLOCK_rdlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_rdlock(&client_by_ip.cip_lock);
 
 	/* check cache */
 	cache_slot = (void **)
@@ -163,10 +167,10 @@ struct gsh_client *get_gsh_client(sockaddr_t *client_ipaddr, bool lookup_only)
 		atomic_store_voidptr(cache_slot, node);
 		goto out;
 	} else if (lookup_only) {
-		PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
+		PTHREAD_RWLOCK_unlock(&client_by_ip.cip_lock);
 		return NULL;
 	}
-	PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_unlock(&client_by_ip.cip_lock);
 
 	server_st = gsh_calloc(1, sizeof(*server_st));
 
@@ -180,13 +184,13 @@ struct gsh_client *get_gsh_client(sockaddr_t *client_ipaddr, bool lookup_only)
 			       sizeof(cl->hostaddr_str));
 	}
 
-	PTHREAD_RWLOCK_wrlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_wrlock(&client_by_ip.cip_lock);
 	node = avltree_insert(&cl->node_k, &client_by_ip.t);
 	if (node) {
 		gsh_free(server_st);	/* somebody beat us to it */
 		cl = avltree_container_of(node, struct gsh_client, node_k);
 	} else {
-		PTHREAD_RWLOCK_init(&cl->lock, NULL);
+		PTHREAD_RWLOCK_init(&cl->client_lock, NULL);
 		/* update cache */
 		atomic_store_voidptr(cache_slot, &cl->node_k);
 	}
@@ -194,7 +198,7 @@ struct gsh_client *get_gsh_client(sockaddr_t *client_ipaddr, bool lookup_only)
  out:
 	/* we will hold a ref starting out... */
 	inc_gsh_client_refcount(cl);
-	PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_unlock(&client_by_ip.cip_lock);
 	return cl;
 }
 
@@ -233,7 +237,10 @@ int remove_gsh_client(sockaddr_t *client_ipaddr)
 	void **cache_slot;
 	uint64_t hash = hash_sockaddr(client_ipaddr, true);
 
-	PTHREAD_RWLOCK_wrlock(&client_by_ip.lock);
+	/* Copy the search address into the key */
+	memcpy(&v.cl_addrbuf, client_ipaddr, sizeof(v.cl_addrbuf));
+
+	PTHREAD_RWLOCK_wrlock(&client_by_ip.cip_lock);
 	node = avltree_lookup(&v.node_k, &client_by_ip.t);
 	if (node) {
 		cl = avltree_container_of(node, struct gsh_client, node_k);
@@ -252,11 +259,12 @@ int remove_gsh_client(sockaddr_t *client_ipaddr)
 		removed = ENOENT;
 	}
  out:
-	PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_unlock(&client_by_ip.cip_lock);
 	if (removed == 0) {
 		server_st = container_of(cl, struct server_stats, client);
 		server_stats_free(&server_st->st);
 		server_stats_allops_free(&server_st->c_all);
+		PTHREAD_RWLOCK_destroy(&cl->client_lock);
 		gsh_free(server_st);
 	}
 	return removed;
@@ -276,7 +284,7 @@ int foreach_gsh_client(bool(*cb) (struct gsh_client *cl, void *state),
 	struct gsh_client *cl;
 	int cnt = 0;
 
-	PTHREAD_RWLOCK_rdlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_rdlock(&client_by_ip.cip_lock);
 	for (client_node = avltree_first(&client_by_ip.t); client_node != NULL;
 	     client_node = avltree_next(client_node)) {
 		cl = avltree_container_of(client_node, struct gsh_client,
@@ -285,7 +293,7 @@ int foreach_gsh_client(bool(*cb) (struct gsh_client *cl, void *state),
 			break;
 		cnt++;
 	}
-	PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_unlock(&client_by_ip.cip_lock);
 	return cnt;
 }
 
@@ -428,19 +436,17 @@ static bool client_to_dbus(struct gsh_client *cl_node, void *state)
 	struct server_stats *cl;
 	char *ipaddr = alloca(SOCK_NAME_MAX);
 	DBusMessageIter struct_iter;
-	struct timespec last_as_ts = nfs_ServerBootTime;
 
 	cl = container_of(cl_node, struct server_stats, client);
 
 	if (!sprint_sockip(&cl_node->cl_addrbuf, ipaddr, SOCK_NAME_MAX))
 		(void) strlcpy(ipaddr, "<unknown>", SOCK_NAME_MAX);
 
-	timespec_add_nsecs(cl_node->last_update, &last_as_ts);
 	dbus_message_iter_open_container(&iter_state->client_iter,
 					 DBUS_TYPE_STRUCT, NULL, &struct_iter);
 	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING, &ipaddr);
 	server_stats_summary(&struct_iter, &cl->st);
-	gsh_dbus_append_timestamp(&struct_iter, &last_as_ts);
+	gsh_dbus_append_timestamp(&struct_iter, &cl_node->last_update);
 	dbus_message_iter_close_container(&iter_state->client_iter,
 					  &struct_iter);
 	return true;
@@ -459,7 +465,7 @@ static bool gsh_client_showclients(DBusMessageIter *args,
 	dbus_message_iter_init_append(reply, &iter);
 	gsh_dbus_append_timestamp(&iter, &timestamp);
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
-					 "(sbbbbbbbb(tt))",
+					 CLIENT_CONTAINER,
 					 &iter_state.client_iter);
 
 	(void)foreach_gsh_client(client_to_dbus, (void *)&iter_state);
@@ -472,10 +478,7 @@ static struct gsh_dbus_method cltmgr_show_clients = {
 	.name = "ShowClients",
 	.method = gsh_client_showclients,
 	.args = {TIMESTAMP_REPLY,
-		 {
-		  .name = "clients",
-		  .type = "a(sbbbbbbbb(tt))",
-		  .direction = "out"},
+		 CLIENTS_REPLY,
 		 END_ARG_LIST}
 };
 
@@ -487,7 +490,7 @@ void reset_client_stats(void)
 	struct gsh_client *cl;
 	struct server_stats *clnt;
 
-	PTHREAD_RWLOCK_rdlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_rdlock(&client_by_ip.cip_lock);
 	for (client_node = avltree_first(&client_by_ip.t); client_node != NULL;
 	     client_node = avltree_next(client_node)) {
 		cl = avltree_container_of(client_node, struct gsh_client,
@@ -497,7 +500,7 @@ void reset_client_stats(void)
 		/* reset stats counter for allops structs */
 		reset_gsh_allops_stats(&clnt->c_all);
 	}
-	PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_unlock(&client_by_ip.cip_lock);
 }
 
 /* Reset Client specific stats counters for allops
@@ -508,7 +511,7 @@ void reset_clnt_allops_stats(void)
 	struct gsh_client *cl;
 	struct server_stats *clnt;
 
-	PTHREAD_RWLOCK_rdlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_rdlock(&client_by_ip.cip_lock);
 	for (client_node = avltree_first(&client_by_ip.t); client_node != NULL;
 	     client_node = avltree_next(client_node)) {
 		cl = avltree_container_of(client_node, struct gsh_client,
@@ -516,7 +519,7 @@ void reset_clnt_allops_stats(void)
 		clnt = container_of(cl, struct server_stats, client);
 		reset_gsh_allops_stats(&clnt->c_all);
 	}
-	PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
+	PTHREAD_RWLOCK_unlock(&client_by_ip.cip_lock);
 }
 
 static struct gsh_dbus_method *cltmgr_client_methods[] = {
@@ -630,13 +633,42 @@ static bool gsh_client_all_ops(DBusMessageIter *args,
 static struct gsh_dbus_method cltmgr_client_all_ops = {
 	.name = "GetClientAllops",
 	.method = gsh_client_all_ops,
-	.args = {IPADDR_ARG,
-		 STATUS_REPLY,
-		 TIMESTAMP_REPLY,
-		 CLNT_ALL_OPS_REPLY,
-		 END_ARG_LIST}
+	.args = {
+			IPADDR_ARG,
+			STATUS_REPLY,
+			TIMESTAMP_REPLY,
+#ifdef _USE_NFS3
+			{
+				.name = "clnt_v3",
+				.type = "b",
+				.direction = "out"
+			},
+			CLNT_V3NLM_OPS_REPLY,
+#endif
+#ifdef _USE_NLM
+			{
+				.name = "clnt_nlm",
+				.type = "b",
+				.direction = "out"
+			},
+			CLNT_V3NLM_OPS_REPLY,
+#endif
+			{
+				.name = "clnt_v4",
+				.type = "b",
+				.direction = "out"
+			},
+			CLNT_V4_OPS_REPLY,
+			{
+				.name = "clnt_cmp",
+				.type = "b",
+				.direction = "out"
+			},
+			CLNT_CMP_OPS_REPLY,
+			END_ARG_LIST}
 };
 
+#ifdef _USE_NFS3
 /**
  * DBUS method to report NFSv3 I/O statistics
  *
@@ -685,6 +717,7 @@ static struct gsh_dbus_method cltmgr_show_v3_io = {
 		 IOSTATS_REPLY,
 		 END_ARG_LIST}
 };
+#endif
 
 /**
  * DBUS method to report NFSv40 I/O statistics
@@ -834,6 +867,105 @@ static struct gsh_dbus_method cltmgr_show_v41_layouts = {
 		 END_ARG_LIST}
 };
 
+/**
+ * DBUS method to report NFSv42 I/O statistics
+ *
+ */
+
+static bool get_nfsv42_stats_io(DBusMessageIter *args,
+				DBusMessage *reply,
+				DBusError *error)
+{
+	struct gsh_client *client = NULL;
+	struct server_stats *server_st = NULL;
+	bool success = true;
+	char *errormsg = "OK";
+	DBusMessageIter iter;
+
+	dbus_message_iter_init_append(reply, &iter);
+	if (!nfs_param.core_param.enable_NFSSTATS)
+		errormsg = "NFS stat counting disabled";
+	client = lookup_client(args, &errormsg);
+	if (client == NULL) {
+		success = false;
+		if (errormsg == NULL)
+			errormsg = "Client IP address not found";
+	} else {
+		server_st = container_of(client, struct server_stats, client);
+		if (server_st->st.nfsv42 == NULL) {
+			success = false;
+			errormsg = "Client does not have any NFSv4.2 activity";
+		}
+	}
+	gsh_dbus_status_reply(&iter, success, errormsg);
+	if (success)
+		server_dbus_v42_iostats(server_st->st.nfsv42, &iter);
+
+	if (client != NULL)
+		put_gsh_client(client);
+	return true;
+}
+
+static struct gsh_dbus_method cltmgr_show_v42_io = {
+	.name = "GetNFSv42IO",
+	.method = get_nfsv42_stats_io,
+	.args = {IPADDR_ARG,
+		 STATUS_REPLY,
+		 TIMESTAMP_REPLY,
+		 IOSTATS_REPLY,
+		 END_ARG_LIST}
+};
+
+
+/**
+ * DBUS method to report NFSv42 layout statistics
+ *
+ */
+
+static bool get_nfsv42_stats_layouts(DBusMessageIter *args,
+				     DBusMessage *reply,
+				     DBusError *error)
+{
+	struct gsh_client *client = NULL;
+	struct server_stats *server_st = NULL;
+	bool success = true;
+	char *errormsg = "OK";
+	DBusMessageIter iter;
+
+	dbus_message_iter_init_append(reply, &iter);
+	if (!nfs_param.core_param.enable_NFSSTATS)
+		errormsg = "NFS stat counting disabled";
+	client = lookup_client(args, &errormsg);
+	if (client == NULL) {
+		success = false;
+		if (errormsg == NULL)
+			errormsg = "Client IP address not found";
+	} else {
+		server_st = container_of(client, struct server_stats, client);
+		if (server_st->st.nfsv42 == NULL) {
+			success = false;
+			errormsg = "Client does not have any NFSv4.2 activity";
+		}
+	}
+	gsh_dbus_status_reply(&iter, success, errormsg);
+	if (success)
+		server_dbus_v42_layouts(server_st->st.nfsv42, &iter);
+
+	if (client != NULL)
+		put_gsh_client(client);
+	return true;
+}
+
+static struct gsh_dbus_method cltmgr_show_v42_layouts = {
+	.name = "GetNFSv42Layouts",
+	.method = get_nfsv42_stats_layouts,
+	.args = {IPADDR_ARG,
+		 STATUS_REPLY,
+		 TIMESTAMP_REPLY,
+		 LAYOUTS_REPLY,
+		 END_ARG_LIST}
+};
+
 /**
  * DBUS method to report NFSv4 delegation statistics
  */
@@ -1029,10 +1161,14 @@ static struct gsh_dbus_method cltmgr_show_9p_op_stats = {
 
 
 static struct gsh_dbus_method *cltmgr_stats_methods[] = {
+#ifdef _USE_NFS3
 	&cltmgr_show_v3_io,
+#endif
 	&cltmgr_show_v40_io,
 	&cltmgr_show_v41_io,
 	&cltmgr_show_v41_layouts,
+	&cltmgr_show_v42_io,
+	&cltmgr_show_v42_layouts,
 	&cltmgr_show_delegations,
 	&cltmgr_client_io_ops,
 	&cltmgr_client_all_ops,
@@ -1067,26 +1203,539 @@ void dbus_client_init(void)
 
 #endif				/* USE_DBUS */
 
+/* Cleanup on shutdown */
+void client_mgr_cleanup(void)
+{
+	PTHREAD_RWLOCK_destroy(&client_by_ip.cip_lock);
+}
+
+struct cleanup_list_element client_mgr_cleanup_element = {
+	.clean = client_mgr_cleanup,
+};
+
 /**
  * @brief Initialize client manager
  */
 
 void client_pkginit(void)
 {
-	pthread_rwlockattr_t rwlock_attr;
-
-	pthread_rwlockattr_init(&rwlock_attr);
-#ifdef GLIBC
-	pthread_rwlockattr_setkind_np(
-		&rwlock_attr,
-		PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP);
-#endif
-	PTHREAD_RWLOCK_init(&client_by_ip.lock, &rwlock_attr);
+	PTHREAD_RWLOCK_init(&client_by_ip.cip_lock, NULL);
 	avltree_init(&client_by_ip.t, client_ip_cmpf, 0);
 	client_by_ip.cache_sz = 32767;
 	client_by_ip.cache =
 	    gsh_calloc(client_by_ip.cache_sz, sizeof(struct avltree_node *));
-	pthread_rwlockattr_destroy(&rwlock_attr);
+	RegisterCleanup(&client_mgr_cleanup_element);
+}
+
+static char *client_types[] = {
+	[PROTO_CLIENT] = "PROTO_CLIENT",
+	[NETWORK_CLIENT] = "NETWORK_CLIENT",
+	[NETGROUP_CLIENT] = "NETGROUP_CLIENT",
+	[WILDCARDHOST_CLIENT] = "WILDCARDHOST_CLIENT",
+	[GSSPRINCIPAL_CLIENT] = "GSSPRINCIPAL_CLIENT",
+	[MATCH_ANY_CLIENT] = "MATCH_ANY_CLIENT",
+	[BAD_CLIENT] = "BAD_CLIENT"
+	 };
+
+int StrClient(struct display_buffer *dspbuf, struct base_client_entry *client)
+{
+	char *paddr = NULL;
+	char *free_paddr = NULL;
+	int b_left = display_start(dspbuf);
+
+	switch (client->type) {
+	case NETWORK_CLIENT:
+		free_paddr = cidr_to_str(client->client.network.cidr,
+					 CIDR_NOFLAGS);
+		paddr = free_paddr;
+		break;
+
+	case NETGROUP_CLIENT:
+		paddr = client->client.netgroup.netgroupname;
+		break;
+
+	case WILDCARDHOST_CLIENT:
+		paddr = client->client.wildcard.wildcard;
+		break;
+
+	case GSSPRINCIPAL_CLIENT:
+		paddr = client->client.gssprinc.princname;
+		break;
+
+	case MATCH_ANY_CLIENT:
+		paddr = "*";
+		break;
+
+	case PROTO_CLIENT:
+	case BAD_CLIENT:
+		paddr = "<unknown>";
+		break;
+
+	default:
+		break;
+	}
+
+	if (client->type > BAD_CLIENT) {
+		b_left = display_printf(dspbuf, "UNKNOWN_CLIENT_TYPE: 0x%08x",
+					client->type);
+	} else {
+		b_left = display_printf(dspbuf, "%s: %s",
+					client_types[client->type], paddr);
+	}
+
+	gsh_free(free_paddr);
+
+	return b_left;
+}
+
+void LogClientListEntry(enum log_components component,
+			log_levels_t level,
+			int line,
+			const char *func,
+			const char *tag,
+			struct base_client_entry *entry)
+{
+	char buf[1024] = "\0";
+	struct display_buffer dspbuf = {sizeof(buf), buf, buf};
+	int b_left = display_start(&dspbuf);
+
+	if (!isLevel(component, level))
+		return;
+
+	if (b_left > 0 && tag != NULL)
+		b_left = display_cat(&dspbuf, tag);
+
+	if (b_left > 0 && level >= NIV_DEBUG)
+		b_left = display_printf(&dspbuf, "%p ", entry);
+
+	if (b_left > 0)
+		b_left = StrClient(&dspbuf, entry);
+
+	DisplayLogComponentLevel(component,
+				 (char *) __FILE__, line, func, level,
+				 "%s", buf);
+
+}
+
+void FreeClientList(struct glist_head *clients, client_free_func free_func)
+{
+	struct glist_head *glist;
+	struct glist_head *glistn;
+
+	glist_for_each_safe(glist, glistn, clients) {
+		struct base_client_entry *client;
+
+		client = glist_entry(glist,
+				     struct base_client_entry,
+				     cle_list);
+
+		glist_del(&client->cle_list);
+		switch (client->type) {
+		case NETWORK_CLIENT:
+			if (client->client.network.cidr != NULL)
+				cidr_free(client->client.network.cidr);
+			break;
+		case NETGROUP_CLIENT:
+			gsh_free(client->client.netgroup.netgroupname);
+			break;
+		case WILDCARDHOST_CLIENT:
+			gsh_free(client->client.wildcard.wildcard);
+			break;
+		case GSSPRINCIPAL_CLIENT:
+			gsh_free(client->client.gssprinc.princname);
+			break;
+		case PROTO_CLIENT:
+		case MATCH_ANY_CLIENT:
+		case BAD_CLIENT:
+			/* Do nothing for these client types */
+			break;
+		}
+		free_func(client);
+	}
+}
+
+void *base_client_allocator(void)
+{
+	return gsh_calloc(1, sizeof(struct base_client_entry));
+}
+
+/**
+ * @brief Expand the client name token into one or more client entries
+ *
+ * @param component     [IN]  component for logging
+ * @param client_list   [IN]  the client list this gets linked to (in tail order)
+ * @param client_tok    [IN]  the name string.  We modify it.
+ * @param type_hint     [IN]  type hint from parser for client_tok
+ * @param cnode         [IN]  opaque pointer needed for config_proc_error()
+ * @param err_type      [OUT] error handling ref
+ * @param cle_allocator [IN]  function to allocate a list entry
+ * @param cle_filler    [IN]  function to fill in a list entry
+ * @param private_data  [IN]  data to be passed to cle_filler function
+ *
+ * @returns 0 on success, error count on failure
+ */
+
+int add_client(enum log_components component,
+	       struct glist_head *client_list,
+	       const char *client_tok,
+	       enum term_type type_hint,
+	       void *cnode,
+	       struct config_error_type *err_type,
+	       client_list_entry_allocator_t cle_allocator,
+	       client_list_entry_filler_t cle_filler,
+	       void *private_data)
+{
+	int errcnt = 0;
+	struct addrinfo *info;
+	CIDR *cidr;
+	int rc;
+	struct base_client_entry *cli;
+
+	if (cle_allocator == NULL)
+		cle_allocator = base_client_allocator;
+
+	cli = cle_allocator();
+
+	cli->client.network.cidr = NULL;
+	glist_init(&cli->cle_list);
+	switch (type_hint) {
+	case TERM_V4_ANY:
+		cli->type = MATCH_ANY_CLIENT;
+		break;
+	case TERM_NETGROUP:
+		if (strlen(client_tok) > MAXHOSTNAMELEN) {
+			config_proc_error(cnode, err_type,
+					  "netgroup (%s) name too long",
+					  client_tok);
+			err_type->invalid = true;
+			errcnt++;
+			goto out;
+		}
+		cli->client.netgroup.netgroupname = gsh_strdup(client_tok + 1);
+		cli->type = NETGROUP_CLIENT;
+		break;
+	case TERM_V4CIDR:
+	case TERM_V6CIDR:
+	case TERM_V4ADDR:
+	case TERM_V6ADDR:
+		cidr = cidr_from_str(client_tok);
+		if (cidr == NULL) {
+			switch (type_hint) {
+			case TERM_V4CIDR:
+				config_proc_error(cnode, err_type,
+						  "Expected a IPv4 CIDR address, got (%s)",
+						  client_tok);
+				break;
+			case TERM_V6CIDR:
+				config_proc_error(cnode, err_type,
+						  "Expected a IPv6 CIDR address, got (%s)",
+						  client_tok);
+				break;
+			case TERM_V4ADDR:
+				config_proc_error(cnode, err_type,
+						  "IPv4 addr (%s) not in presentation format",
+						  client_tok);
+				break;
+			case TERM_V6ADDR:
+				config_proc_error(cnode, err_type,
+						  "IPv6 addr (%s) not in presentation format",
+						  client_tok);
+				break;
+			default:
+				break;
+			}
+			err_type->invalid = true;
+			errcnt++;
+			goto out;
+		}
+		cli->client.network.cidr = cidr;
+		cli->type = NETWORK_CLIENT;
+		break;
+	case TERM_REGEX:
+		if (strlen(client_tok) > MAXHOSTNAMELEN) {
+			config_proc_error(cnode, err_type,
+					  "Wildcard client (%s) name too long",
+					  client_tok);
+			err_type->invalid = true;
+			errcnt++;
+			goto out;
+		}
+		cli->client.wildcard.wildcard = gsh_strdup(client_tok);
+		cli->type = WILDCARDHOST_CLIENT;
+		break;
+	case TERM_TOKEN: /* only dns names now. */
+		rc = gsh_getaddrinfo(client_tok, NULL, NULL, &info,
+				nfs_param.core_param.enable_AUTHSTATS);
+		if (rc == 0) {
+			struct addrinfo *ap, *ap_last = NULL;
+			struct in_addr in_addr_last;
+			struct in6_addr in6_addr_last;
+
+			for (ap = info; ap != NULL; ap = ap->ai_next) {
+				LogFullDebug(COMPONENT_EXPORT,
+					     "flags=%d family=%d socktype=%d protocol=%d addrlen=%d name=%s",
+					     ap->ai_flags,
+					     ap->ai_family,
+					     ap->ai_socktype,
+					     ap->ai_protocol,
+					     (int) ap->ai_addrlen,
+					     ap->ai_canonname);
+
+				if (cli == NULL) {
+					cli = cle_allocator();
+					glist_init(&cli->cle_list);
+				}
+
+				if (ap->ai_family == AF_INET &&
+				    (ap->ai_socktype == SOCK_STREAM ||
+				     ap->ai_socktype == SOCK_DGRAM)) {
+					struct in_addr infoaddr =
+						((struct sockaddr_in *)
+						 ap->ai_addr)->sin_addr;
+					if (ap_last != NULL &&
+					    ap_last->ai_family
+					    == ap->ai_family &&
+					    memcmp(&infoaddr,
+						   &in_addr_last,
+						   sizeof(struct in_addr)) == 0)
+						continue;
+					cli->client.network.cidr =
+						cidr_from_inaddr(&infoaddr);
+					cli->type = NETWORK_CLIENT;
+					ap_last = ap;
+					in_addr_last = infoaddr;
+
+				} else if (ap->ai_family == AF_INET6 &&
+					   (ap->ai_socktype == SOCK_STREAM ||
+					    ap->ai_socktype == SOCK_DGRAM)) {
+					struct in6_addr infoaddr =
+						((struct sockaddr_in6 *)
+						 ap->ai_addr)->sin6_addr;
+
+					if (ap_last != NULL &&
+					    ap_last->ai_family == ap->ai_family
+					    &&  !memcmp(&infoaddr,
+						       &in6_addr_last,
+						       sizeof(struct in6_addr)))
+						continue;
+					/* IPv6 address */
+					cli->client.network.cidr =
+						cidr_from_in6addr(&infoaddr);
+					cli->type = NETWORK_CLIENT;
+					ap_last = ap;
+					in6_addr_last = infoaddr;
+				} else {
+					continue;
+				}
+
+				if (cle_filler != NULL)
+					cle_filler(cli, private_data);
+				else
+					LogMidDebug_ClientListEntry(component,
+								    "", cli);
+
+				glist_add_tail(client_list, &cli->cle_list);
+				cli = NULL; /* let go of it */
+			}
+			freeaddrinfo(info);
+			goto out;
+		} else {
+			config_proc_error(cnode, err_type,
+					  "Client (%s) not found because %s",
+					  client_tok, gai_strerror(rc));
+			err_type->bogus = true;
+			errcnt++;
+		}
+		break;
+	default:
+		config_proc_error(cnode, err_type,
+				  "Expected a client, got a %s for (%s)",
+				  config_term_desc(type_hint),
+				  client_tok);
+		err_type->bogus = true;
+		errcnt++;
+		goto out;
+	}
+
+	if (cle_filler != NULL)
+		cle_filler(cli, private_data);
+	else
+		LogMidDebug_ClientListEntry(component, "", cli);
+
+	glist_add_tail(client_list, &cli->cle_list);
+	cli = NULL;
+out:
+	gsh_free(cli);
+	return errcnt;
+}
+
+/**
+ * @brief Match a specific client in a client list
+ *
+ * @param[in]  hostaddr      Host to search for
+ * @param[in]  clients       Client list to search
+ *
+ * @return the client entry or NULL if failure.
+ */
+struct base_client_entry *client_match(enum log_components component,
+				       const char *str,
+				       sockaddr_t *clientaddr,
+				       struct glist_head *clients)
+{
+	struct glist_head *glist;
+	int rc;
+	int ipvalid = -1;	/* -1 need to print, 0 - invalid, 1 - ok */
+	char hostname[NI_MAXHOST];
+	char ipstring[SOCK_NAME_MAX];
+	CIDR *host_prefix = NULL;
+	struct base_client_entry *client;
+	sockaddr_t alt_hostaddr;
+	sockaddr_t *hostaddr = NULL;
+
+	hostaddr = convert_ipv6_to_ipv4(clientaddr, &alt_hostaddr);
+
+	if (isMidDebug(component)) {
+		char ipstring[SOCK_NAME_MAX];
+		struct display_buffer dspbuf = {
+					sizeof(ipstring), ipstring, ipstring};
+
+		display_sockip(&dspbuf, hostaddr);
+
+		LogMidDebug(component,
+			    "Check for address %s%s",
+			    ipstring, str ? str : "");
+	}
+
+	glist_for_each(glist, clients) {
+		client = glist_entry(glist, struct base_client_entry, cle_list);
+		LogMidDebug_ClientListEntry(component, "Match V4: ", client);
+
+		switch (client->type) {
+		case NETWORK_CLIENT:
+			if (host_prefix == NULL) {
+				if (hostaddr->ss_family == AF_INET6) {
+					host_prefix = cidr_from_in6addr(
+						&((struct sockaddr_in6 *)
+							hostaddr)->sin6_addr);
+				} else {
+					host_prefix = cidr_from_inaddr(
+						&((struct sockaddr_in *)
+							hostaddr)->sin_addr);
+				}
+			}
+
+			if (cidr_contains(client->client.network.cidr,
+					  host_prefix) == 0) {
+				goto out;
+			}
+			break;
+
+		case NETGROUP_CLIENT:
+			/* Try to get the entry from th IP/name cache */
+			rc = nfs_ip_name_get(hostaddr, hostname,
+					     sizeof(hostname));
+
+			if (rc == IP_NAME_NOT_FOUND) {
+				/* IPaddr was not cached, add it to the cache */
+				rc = nfs_ip_name_add(hostaddr,
+						     hostname,
+						     sizeof(hostname));
+			}
+
+			if (rc != IP_NAME_SUCCESS)
+				break; /* Fatal failure */
+
+			/* At this point 'hostname' should contain the
+			 * name that was found
+			 */
+			if (ng_innetgr(client->client.netgroup.netgroupname,
+				    hostname)) {
+				goto out;
+			}
+			break;
+
+		case WILDCARDHOST_CLIENT:
+			/* Now checking for IP wildcards */
+			if (ipvalid < 0)
+				ipvalid = sprint_sockip(hostaddr,
+							ipstring,
+							sizeof(ipstring));
+
+			if (ipvalid &&
+			    (fnmatch(client->client.wildcard.wildcard,
+				     ipstring,
+				     FNM_PATHNAME) == 0)) {
+				goto out;
+			}
+
+			/* Try to get the entry from th IP/name cache */
+			rc = nfs_ip_name_get(hostaddr, hostname,
+					     sizeof(hostname));
+
+			if (rc == IP_NAME_NOT_FOUND) {
+				/* IPaddr was not cached, add it to the cache */
+
+				/** @todo this change from 1.5 is not IPv6
+				 * useful.  come back to this and use the
+				 * string from client mgr inside op_context...
+				 */
+				rc = nfs_ip_name_add(hostaddr,
+						     hostname,
+						     sizeof(hostname));
+			}
+
+			if (rc != IP_NAME_SUCCESS)
+				break;
+
+			/* At this point 'hostname' should contain the
+			 * name that was found
+			 */
+			if (fnmatch
+			    (client->client.wildcard.wildcard, hostname,
+			     FNM_PATHNAME) == 0) {
+				goto out;
+			}
+			break;
+
+		case GSSPRINCIPAL_CLIENT:
+	  /** @todo BUGAZOMEU a completer lors de l'integration de RPCSEC_GSS */
+			LogCrit(COMPONENT_EXPORT,
+				"Unsupported type GSS_PRINCIPAL_CLIENT");
+			break;
+
+		case MATCH_ANY_CLIENT:
+			goto out;
+
+		case BAD_CLIENT:
+		default:
+			continue;
+		}
+	}
+
+	client = NULL;
+
+out:
+
+	if (host_prefix != NULL)
+		cidr_free(host_prefix);
+
+	return client;
+
+}
+
+bool haproxy_match(SVCXPRT *xprt)
+{
+	struct base_client_entry *host = NULL;
+
+	if (glist_empty(&nfs_param.core_param.haproxy_hosts))
+		return false;
+
+	/* Does the host match anyone on the host list? */
+	host = client_match(COMPONENT_DISPATCH, " for HAProxy",
+			    &xprt->xp_proxy.ss,
+			    &nfs_param.core_param.haproxy_hosts);
+
+	return host != NULL;
 }
 
 /** @} */
