@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright (c) 2013, The Linux Box Corporation
  *
@@ -42,6 +43,8 @@
 #elif FREEBSD
 #include <signal.h>
 #endif
+#include <urcu-bp.h>
+
 #include "abstract_mem.h"
 #include "delayed_exec.h"
 #include "log.h"
@@ -104,14 +107,14 @@ struct delayed_thread {
 /** list of all threads */
 static struct delayed_threadlist thread_list;
 /** Mutex for delayed execution */
-static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t dle_mtx;
 /** Condition variable for delayed execution */
-static pthread_cond_t cv = PTHREAD_COND_INITIALIZER;
+static pthread_cond_t dle_cv;
 /** The timer tree */
 static struct avltree tree;
 
 /**
- * @brief Posssible states for the delayed executor
+ * @brief Possible states for the delayed executor
  */
 enum delayed_state {
 	delayed_running,	/*< Executor is running */
@@ -207,8 +210,9 @@ void *delayed_thread(void *arg)
 	sigset_t old_sigmask;
 
 	SetNameFunction("Async");
+	rcu_register_thread();
 
-	/* Excplicitly and definitely enable cancellation */
+	/* Explicitly and definitely enable cancellation */
 	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &old_state);
 
 	/**
@@ -218,7 +222,7 @@ void *delayed_thread(void *arg)
 
 	pthread_sigmask(SIG_SETMASK, NULL, &old_sigmask);
 
-	PTHREAD_MUTEX_lock(&mtx);
+	PTHREAD_MUTEX_lock(&dle_mtx);
 	while (delayed_state == delayed_running) {
 		struct timespec then;
 		void (*func)(void *);
@@ -226,27 +230,28 @@ void *delayed_thread(void *arg)
 
 		switch (delayed_get_work(&then, &func, &farg)) {
 		case delayed_unemployed:
-			pthread_cond_wait(&cv, &mtx);
+			pthread_cond_wait(&dle_cv, &dle_mtx);
 			break;
 
 		case delayed_on_break:
-			pthread_cond_timedwait(&cv, &mtx, &then);
+			pthread_cond_timedwait(&dle_cv, &dle_mtx, &then);
 			break;
 
 		case delayed_employed:
-			PTHREAD_MUTEX_unlock(&mtx);
+			PTHREAD_MUTEX_unlock(&dle_mtx);
 			func(farg);
-			PTHREAD_MUTEX_lock(&mtx);
+			PTHREAD_MUTEX_lock(&dle_mtx);
 			break;
 		}
 	}
 	LIST_REMOVE(thr, link);
 	if (LIST_EMPTY(&thread_list))
-		pthread_cond_broadcast(&cv);
+		pthread_cond_broadcast(&dle_cv);
 
-	PTHREAD_MUTEX_unlock(&mtx);
+	PTHREAD_MUTEX_unlock(&dle_mtx);
 	gsh_free(thr);
 
+	rcu_unregister_thread();
 	return NULL;
 }
 
@@ -263,6 +268,8 @@ void delayed_start(void)
 	/* Thread index */
 	int i;
 
+	PTHREAD_MUTEX_init(&dle_mtx, NULL);
+	PTHREAD_COND_init(&dle_cv, NULL);
 	LIST_INIT(&thread_list);
 	avltree_init(&tree, comparator, 0);
 
@@ -271,13 +278,10 @@ void delayed_start(void)
 			 "You can't execute tasks with zero threads.");
 	}
 
-	if (pthread_attr_init(&attr) != 0)
-		LogFatal(COMPONENT_THREAD, "can't init pthread's attributes");
+	PTHREAD_ATTR_init(&attr);
+	PTHREAD_ATTR_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
 
-	if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED) != 0)
-		LogFatal(COMPONENT_THREAD, "can't set pthread's join state");
-
-	PTHREAD_MUTEX_lock(&mtx);
+	PTHREAD_MUTEX_lock(&dle_mtx);
 	delayed_state = delayed_running;
 
 	for (i = 0; i < threads_to_start; ++i) {
@@ -285,15 +289,15 @@ void delayed_start(void)
 		    gsh_malloc(sizeof(struct delayed_thread));
 		int rc = 0;
 
-		rc = pthread_create(&thread->id, &attr, delayed_thread, thread);
+		rc = PTHREAD_create(&thread->id, &attr, delayed_thread, thread);
 		if (rc != 0) {
 			LogFatal(COMPONENT_THREAD,
 				 "Unable to start delayed executor: %d", rc);
 		}
 		LIST_INSERT_HEAD(&thread_list, thread, link);
 	}
-	PTHREAD_MUTEX_unlock(&mtx);
-	pthread_attr_destroy(&attr);
+	PTHREAD_MUTEX_unlock(&dle_mtx);
+	PTHREAD_ATTR_destroy(&attr);
 }
 
 /**
@@ -308,11 +312,11 @@ void delayed_shutdown(void)
 	now(&then);
 	then.tv_sec += 120;
 
-	PTHREAD_MUTEX_lock(&mtx);
+	PTHREAD_MUTEX_lock(&dle_mtx);
 	delayed_state = delayed_stopping;
-	pthread_cond_broadcast(&cv);
+	pthread_cond_broadcast(&dle_cv);
 	while ((rc != ETIMEDOUT) && !LIST_EMPTY(&thread_list))
-		rc = pthread_cond_timedwait(&cv, &mtx, &then);
+		rc = pthread_cond_timedwait(&dle_cv, &dle_mtx, &then);
 
 	if (!LIST_EMPTY(&thread_list)) {
 		struct delayed_thread *thr;
@@ -325,7 +329,9 @@ void delayed_shutdown(void)
 			gsh_free(thr);
 		}
 	}
-	PTHREAD_MUTEX_unlock(&mtx);
+	PTHREAD_MUTEX_unlock(&dle_mtx);
+	PTHREAD_MUTEX_destroy(&dle_mtx);
+	PTHREAD_COND_destroy(&dle_cv);
 }
 
 /**
@@ -333,7 +339,7 @@ void delayed_shutdown(void)
  *
  * @param[in] func  The function to run
  * @param[in] arg   The argument to run it with
- * @param[in] delay The dleay in nanoseconds
+ * @param[in] delay The delay in nanoseconds
  *
  * @retval 0 on success.
  * @retval ENOMEM on inability to allocate memory causing other than success.
@@ -352,7 +358,7 @@ int delayed_submit(void (*func) (void *), void *arg, nsecs_elapsed_t delay)
 	now(&mul->realtime);
 	timespec_add_nsecs(delay, &mul->realtime);
 
-	PTHREAD_MUTEX_lock(&mtx);
+	PTHREAD_MUTEX_lock(&dle_mtx);
 	first = avltree_first(&tree);
 	collision = avltree_insert(&mul->node, &tree);
 	if (unlikely(collision)) {
@@ -368,9 +374,9 @@ int delayed_submit(void (*func) (void *), void *arg, nsecs_elapsed_t delay)
 	task->arg = arg;
 	LIST_INSERT_HEAD(&mul->list, task, link);
 	if (!first || comparator(&mul->node, first) < 0)
-		pthread_cond_broadcast(&cv);
+		pthread_cond_broadcast(&dle_cv);
 
-	PTHREAD_MUTEX_unlock(&mtx);
+	PTHREAD_MUTEX_unlock(&dle_mtx);
 
 	return 0;
 }
