@@ -108,7 +108,52 @@ static struct config_item_list protocols[] = {
 #define DEFAULT_PROTOCOLS  (DEFAULT_INCLUDES_NFSV3 | \
 			    DEFAULT_INCLUDES_NFSV4)
 
+/**
+ * @brief Process a list of hosts for haproxy_hosts
+ *
+ * CONFIG_PROC handler that gets called for each token in the term list.
+ * Create a exportlist_client_entry for each token and link it into
+ * the proto host's cle_list list head.  We will pass that head to the
+ * export in commit.
+ *
+ * NOTES: this is the place to expand a node list with perhaps moving the
+ * call to add_client into the expander rather than build a list there
+ * to be then walked here...
+ *
+ * @param token [IN] pointer to token string from parse tree
+ * @param type_hint [IN] a type hint from what the parser recognized
+ * @param item [IN] pointer to the config item table entry
+ * @param param_addr [IN] pointer to prototype host entry
+ * @param err_type [OUT] error handling
+ * @return error count
+ */
+
+static int haproxy_host_adder(const char *token,
+			      enum term_type type_hint,
+			      struct config_item *item,
+			      void *param_addr,
+			      void *cnode,
+			      struct config_error_type *err_type)
+{
+	struct base_client_entry *host;
+	int rc;
+
+	host = container_of(param_addr,
+			      struct base_client_entry,
+			      cle_list);
+
+	LogMidDebug(COMPONENT_CONFIG, "Adding host %s", token);
+
+	rc = add_client(COMPONENT_CONFIG,
+			&host->cle_list,
+			token, type_hint, cnode, err_type,
+			NULL, NULL, NULL);
+	return rc;
+}
+
 static struct config_item core_params[] = {
+	CONF_ITEM_PROC_MULT("HAProxy_Hosts", noop_conf_init,
+			    haproxy_host_adder, base_client_entry, cle_list),
 	CONF_ITEM_UI16("NFS_Port", 0, UINT16_MAX, NFS_PORT,
 		       nfs_core_param, port[P_NFS]),
 #ifdef _USE_NFS3
@@ -154,6 +199,8 @@ static struct config_item core_params[] = {
 		       nfs_core_param, drop_delay_errors),
 	CONF_ITEM_BOOL("DRC_Disabled", false,
 		       nfs_core_param, drc.disabled),
+	CONF_ITEM_UI32("DRC_Recycle_Hiwat", 1, 1000000, DRC_RECYCLE_HIWAT,
+			nfs_core_param, drc.recycle_hiwat),
 	CONF_ITEM_UI32("DRC_TCP_Npart", 1, 20, DRC_TCP_NPART,
 		       nfs_core_param, drc.tcp.npart),
 	CONF_ITEM_UI32("DRC_TCP_Size", 1, 32767, DRC_TCP_SIZE,
@@ -178,7 +225,7 @@ static struct config_item core_params[] = {
 		       nfs_core_param, drc.udp.hiwat),
 	CONF_ITEM_BOOL("DRC_UDP_Checksum", DRC_UDP_CHECKSUM,
 		       nfs_core_param, drc.udp.checksum),
-	CONF_ITEM_UI32("RPC_Max_Connections", 1, 10000, 1024,
+	CONF_ITEM_UI32("RPC_Max_Connections", 1, 1000000, 1024,
 		       nfs_core_param, rpc.max_connections),
 	CONF_ITEM_UI32("RPC_Idle_Timeout_S", 0, 60*60, 300,
 		       nfs_core_param, rpc.idle_timeout_s),
@@ -209,6 +256,8 @@ static struct config_item core_params[] = {
 #ifdef _USE_NLM
 	CONF_ITEM_BOOL("Enable_NLM", true,
 		       nfs_core_param, enable_NLM),
+	CONF_ITEM_BOOL("Disable_NLM_SHARE", false,
+		       nfs_core_param, disable_NLM_SHARE),
 	CONF_ITEM_BOOL("NSM_Use_Caller_Name", false,
 		       nfs_core_param, nsm_use_caller_name),
 #endif
@@ -254,6 +303,10 @@ static struct config_item core_params[] = {
 		       nfs_core_param, heartbeat_freq),
 	CONF_ITEM_BOOL("fsid_device", false,
 		       nfs_core_param, fsid_device),
+	CONF_ITEM_UI32("resolve_fs_retries", 1, 1000, 10,
+		       nfs_core_param, resolve_fs_retries),
+	CONF_ITEM_UI32("resolve_fs_delay", 1, 1000, 100,
+		       nfs_core_param, resolve_fs_delay),
 	CONF_ITEM_BOOL("mount_path_pseudo", false,
 		       nfs_core_param, mount_path_pseudo),
 	CONF_ITEM_ENUM_BITS("Enable_UDP", UDP_LISTENER_ALL, UDP_LISTENER_MASK,
@@ -264,12 +317,22 @@ static struct config_item core_params[] = {
 		       nfs_core_param, max_uid_to_grp_reqs),
 	CONF_ITEM_BOOL("Enable_V3fh_Validation_For_V4", false,
 		       nfs_core_param, enable_v3_fh_for_v4),
-	CONF_ITEM_UI32("Readdir_Res_Size", 4096, FSAL_MAXIOSIZE, FSAL_MAXIOSIZE,
+	CONF_ITEM_UI32("Readdir_Res_Size", 4096, FSAL_MAXIOSIZE, 32*1024,
 		       nfs_core_param, readdir_res_size),
 	CONF_ITEM_UI32("Readdir_Max_Count", 32, 1024*1024, 1024*1024,
 		       nfs_core_param, readdir_max_count),
 	CONF_ITEM_BOOL("Getattrs_In_Complete_Read", true,
-			   nfs_core_param, getattrs_in_complete_read),
+		       nfs_core_param, getattrs_in_complete_read),
+	CONF_ITEM_BOOL("Enable_malloc_trim", false,
+		       nfs_core_param, malloc_trim),
+	CONF_ITEM_UI32("Malloc_trim_MinThreshold", 1, INT32_MAX, 15*1024,
+		       nfs_core_param, malloc_trim_minthreshold),
+#ifdef USE_MONITORING
+	CONF_ITEM_UI16("Monitoring_Port", 0, UINT16_MAX, MONITORING_PORT,
+		       nfs_core_param, monitoring_port),
+#endif
+	CONF_ITEM_BOOL("enable_rpc_cred_fallback", false,
+		       nfs_core_param, enable_rpc_cred_fallback),
 	CONFIG_EOL
 };
 
@@ -277,6 +340,7 @@ struct config_block nfs_core = {
 	.dbus_interface_name = "org.ganesha.nfsd.config.core",
 	.blk_desc.name = "NFS_Core_Param",
 	.blk_desc.type = CONFIG_BLOCK,
+	.blk_desc.flags = CONFIG_UNIQUE,  /* too risky to have more */
 	.blk_desc.u.blk.init = noop_conf_init,
 	.blk_desc.u.blk.params = core_params,
 	.blk_desc.u.blk.commit = noop_conf_commit
@@ -305,6 +369,7 @@ struct config_block krb5_param = {
 	.dbus_interface_name = "org.ganesha.nfsd.config.krb5",
 	.blk_desc.name = "NFS_KRB5",
 	.blk_desc.type = CONFIG_BLOCK,
+	.blk_desc.flags = CONFIG_UNIQUE,  /* too risky to have more */
 	.blk_desc.u.blk.init = noop_conf_init,
 	.blk_desc.u.blk.params = krb5_params,
 	.blk_desc.u.blk.commit = noop_conf_commit
@@ -340,7 +405,7 @@ static struct config_item_list recovery_backend_types[] = {
 static struct config_item version4_params[] = {
 	CONF_ITEM_BOOL("Graceless", false,
 		       nfs_version4_parameter, graceless),
-	CONF_ITEM_UI32("Lease_Lifetime", 0, 180, LEASE_LIFETIME_DEFAULT,
+	CONF_ITEM_UI32("Lease_Lifetime", 1, 180, LEASE_LIFETIME_DEFAULT,
 		       nfs_version4_parameter, lease_lifetime),
 	CONF_ITEM_UI32("Grace_Period", 0, 270, GRACE_PERIOD_DEFAULT,
 		       nfs_version4_parameter, grace_period),
@@ -363,9 +428,9 @@ static struct config_item version4_params[] = {
 	CONF_ITEM_UI32("Deleg_Recall_Retry_Delay", 0, 10,
 			DELEG_RECALL_RETRY_DELAY_DEFAULT,
 			nfs_version4_parameter, deleg_recall_retry_delay),
-	CONF_ITEM_BOOL("PNFS_MDS", true,
+	CONF_ITEM_BOOL("PNFS_MDS", false,
 		       nfs_version4_parameter, pnfs_mds),
-	CONF_ITEM_BOOL("PNFS_DS", true,
+	CONF_ITEM_BOOL("PNFS_DS", false,
 		       nfs_version4_parameter, pnfs_ds),
 	CONF_ITEM_TOKEN("RecoveryBackend", RECOVERY_BACKEND_DEFAULT,
 			recovery_backend_types, nfs_version4_parameter,
@@ -391,6 +456,7 @@ struct config_block version4_param = {
 	.dbus_interface_name = "org.ganesha.nfsd.config.nfsv4",
 	.blk_desc.name = "NFSv4",
 	.blk_desc.type = CONFIG_BLOCK,
+	.blk_desc.flags = CONFIG_UNIQUE,  /* too risky to have more */
 	.blk_desc.u.blk.init = noop_conf_init,
 	.blk_desc.u.blk.params = version4_params,
 	.blk_desc.u.blk.commit = noop_conf_commit
