@@ -63,16 +63,9 @@
 #include "nfs_proto_functions.h"
 #include "nfs_convert.h"
 
-#define NFS_V3_NB_COMMAND (NFSPROC3_COMMIT + 1)
-#define NFS_V4_NB_COMMAND 2
-#define MNT_V1_NB_COMMAND (MOUNTPROC3_EXPORT + 1)
-#define MNT_V3_NB_COMMAND (MOUNTPROC3_EXPORT + 1)
-#define NLM_V4_NB_OPERATION (NLMPROC4_FREE_ALL + 1)
-#define RQUOTA_NB_COMMAND (RQUOTAPROC_SETACTIVEQUOTA + 1)
-#define NFS_V40_NB_OPERATION (NFS4_OP_RELEASE_LOCKOWNER + 1)
-#define NFS_V41_NB_OPERATION (NFS4_OP_RECLAIM_COMPLETE + 1)
-#define NFS_V42_NB_OPERATION (NFS4_OP_WRITE_SAME + 1)
-#define _9P_NB_COMMAND 33
+#ifdef USE_MONITORING
+#include "monitoring.h"
+#endif
 
 #define NFS_pcp nfs_param.core_param
 #define NFS_program NFS_pcp.program
@@ -187,7 +180,7 @@ struct op_latency {
 /* v3 ops
  */
 struct nfsv3_ops {
-	uint64_t op[NFSPROC3_COMMIT+1];
+	uint64_t op[NFS_V3_NB_COMMAND];
 };
 #endif
 
@@ -195,7 +188,7 @@ struct nfsv3_ops {
 /* quota ops
  */
 struct qta_ops {
-	uint64_t op[RQUOTAPROC_SETACTIVEQUOTA+1];
+	uint64_t op[RQUOTA_NB_COMMAND];
 };
 #endif
 
@@ -203,7 +196,7 @@ struct qta_ops {
 /* nlm ops
  */
 struct nlm_ops {
-	uint64_t op[NLMPROC4_FREE_ALL+1];
+	uint64_t op[NLM_V4_NB_OPERATION];
 };
 #endif
 
@@ -211,7 +204,7 @@ struct nlm_ops {
 /* mount ops
  */
 struct mnt_ops {
-	uint64_t op[MOUNTPROC3_EXPORT+1];
+	uint64_t op[MNT_V3_NB_COMMAND];
 };
 #endif
 
@@ -265,7 +258,7 @@ struct nfsv3_stats {
 };
 
 struct clnt_allops_v3_stats {
-	struct op_count cmds[NFSPROC3_COMMIT + 1];	/* all NFSv3 ops */
+	struct op_count cmds[NFS_V3_NB_COMMAND];	/* all NFSv3 ops */
 };
 
 /* Mount statistics counters
@@ -285,7 +278,7 @@ struct nlmv4_stats {
 };
 
 struct clnt_allops_nlm_stats {
-	struct op_count cmds[NLMPROC4_FREE_ALL + 1];	/* all NLMv4 ops */
+	struct op_count cmds[NLM_V4_NB_OPERATION];	/* all NLMv4 ops */
 };
 #endif
 
@@ -390,11 +383,11 @@ static struct global_stats global_st;
 
 #ifdef _USE_NFS3
 /* NFSv3 Detailed stats holder */
-struct proto_op v3_full_stats[NFSPROC3_COMMIT+1];
+struct proto_op v3_full_stats[NFS_V3_NB_COMMAND];
 #endif
 
 /* NFSv4 Detailed stats holder */
-struct proto_op v4_full_stats[NFS_V42_NB_OPERATION+1];
+struct proto_op v4_full_stats[NFS4_OP_LAST_ONE];
 
 /**
  * @brief Get stats struct helpers
@@ -1297,7 +1290,7 @@ void server_stats_transport_done(struct gsh_client *client,
 {
 	struct server_stats *server_st =
 		container_of(client, struct server_stats, client);
-	struct _9p_stats *sp = get_9p(&server_st->st, &client->lock);
+	struct _9p_stats *sp = get_9p(&server_st->st, &client->client_lock);
 
 	if (sp != NULL)
 		record_transport_stats(&sp->trans, rx_bytes, rx_pkt, rx_err,
@@ -1320,7 +1313,7 @@ void server_stats_9p_done(u8 opc, struct _9p_request_data *req9p)
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		sp = get_9p(&server_st->st, &client->lock);
+		sp = get_9p(&server_st->st, &client->client_lock);
 		if (sp->opcodes[opc] == NULL)
 			sp->opcodes[opc] =
 				gsh_calloc(1, sizeof(struct proto_op));
@@ -1332,7 +1325,7 @@ void server_stats_9p_done(u8 opc, struct _9p_request_data *req9p)
 
 		export = op_ctx->ctx_export;
 		exp_st = container_of(export, struct export_stats, export);
-		sp = get_9p(&exp_st->st, &export->lock);
+		sp = get_9p(&exp_st->st, &export->exp_lock);
 		if (sp->opcodes[opc] == NULL)
 			sp->opcodes[opc] =
 				gsh_calloc(1, sizeof(struct proto_op));
@@ -1344,12 +1337,12 @@ void server_stats_9p_done(u8 opc, struct _9p_request_data *req9p)
 #ifdef _USE_NFS3
 static void record_v3_full_stats(struct svc_req *req,
 			       nsecs_elapsed_t request_time,
-			       bool success, bool dup);
+			       int status, bool dup);
 #endif
 
 static void record_v4_full_stats(uint32_t proc,
 			       nsecs_elapsed_t request_time,
-			       bool success);
+			       nfsstat4 status);
 
 /**
  * @brief record NFS op finished
@@ -1394,16 +1387,17 @@ void server_stats_nfs_done(nfs_request_t *reqdata, int rc, bool dup)
 #ifdef _USE_NFS3
 	if (nfs_param.core_param.enable_FULLV3STATS)
 		record_v3_full_stats(req, time_diff,
-			    rc == NFS_REQ_OK, dup);
+			    rc, dup);
 #endif
 	if (client != NULL) {
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		record_clnt_stats(&server_st->st, &client->lock, reqdata,
+		record_clnt_stats(&server_st->st, &client->client_lock, reqdata,
 			     rc == NFS_REQ_OK, dup);
 		if (nfs_param.core_param.enable_CLNTALLSTATS)
-			record_clnt_all_stats(&server_st->c_all, &client->lock,
+			record_clnt_all_stats(&server_st->c_all,
+					&client->client_lock,
 					program_op, proto_op, NFS_V3,
 					rc == NFS_REQ_OK, dup);
 		timespec_update(&client->last_update, &current_time);
@@ -1414,8 +1408,8 @@ void server_stats_nfs_done(nfs_request_t *reqdata, int rc, bool dup)
 		exp_st =
 		    container_of(op_ctx->ctx_export, struct export_stats,
 			    export);
-		record_stats(&exp_st->st, &op_ctx->ctx_export->lock, reqdata,
-			     time_diff,
+		record_stats(&exp_st->st, &op_ctx->ctx_export->exp_lock,
+			     reqdata, time_diff,
 			     rc == NFS_REQ_OK, dup, true);
 		timespec_update(&op_ctx->ctx_export->last_update,
 				&current_time);
@@ -1447,18 +1441,18 @@ void server_stats_nfsv4_op_done(int proto_op,
 	time_diff = timespec_diff(start_time, &current_time);
 
 	if (nfs_param.core_param.enable_FULLV4STATS)
-		record_v4_full_stats(proto_op, time_diff,
-			    status == NFS4_OK);
+		record_v4_full_stats(proto_op, time_diff, status);
 
 	if (client != NULL) {
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		record_nfsv4_op(&server_st->st, &client->lock, proto_op,
+		record_nfsv4_op(&server_st->st, &client->client_lock, proto_op,
 				op_ctx->nfs_minorvers, time_diff,
 				status, false);
 		if (nfs_param.core_param.enable_CLNTALLSTATS)
-			record_clnt_all_stats(&server_st->c_all, &client->lock,
+			record_clnt_all_stats(&server_st->c_all,
+				&client->client_lock,
 				NFS_program[P_NFS], proto_op, NFS_V4,
 				status == NFS_REQ_OK, false);
 		timespec_update(&client->last_update, &current_time);
@@ -1480,7 +1474,7 @@ void server_stats_nfsv4_op_done(int proto_op,
 		exp_st =
 		    container_of(op_ctx->ctx_export, struct export_stats,
 			    export);
-		record_nfsv4_op(&exp_st->st, &op_ctx->ctx_export->lock,
+		record_nfsv4_op(&exp_st->st, &op_ctx->ctx_export->exp_lock,
 				proto_op, op_ctx->nfs_minorvers,
 				time_diff, status, true);
 		timespec_update(&op_ctx->ctx_export->last_update,
@@ -1509,7 +1503,7 @@ void server_stats_compound_done(int num_ops, int status)
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		record_compound(&server_st->st, &client->lock,
+		record_compound(&server_st->st, &client->client_lock,
 				op_ctx->nfs_minorvers,
 				num_ops, time_diff,
 				status == NFS4_OK);
@@ -1521,7 +1515,7 @@ void server_stats_compound_done(int num_ops, int status)
 		exp_st =
 		    container_of(op_ctx->ctx_export, struct export_stats,
 			    export);
-		record_compound(&exp_st->st, &op_ctx->ctx_export->lock,
+		record_compound(&exp_st->st, &op_ctx->ctx_export->exp_lock,
 				op_ctx->nfs_minorvers, num_ops,
 				time_diff,
 				status == NFS4_OK);
@@ -1547,7 +1541,7 @@ void server_stats_io_done(size_t requested,
 
 		server_st = container_of(op_ctx->client, struct server_stats,
 					 client);
-		record_io_stats(&server_st->st, &op_ctx->client->lock,
+		record_io_stats(&server_st->st, &op_ctx->client->client_lock,
 				requested, transferred, success,
 				is_write);
 	}
@@ -1557,9 +1551,23 @@ void server_stats_io_done(size_t requested,
 		exp_st =
 		    container_of(op_ctx->ctx_export, struct export_stats,
 			    export);
-		record_io_stats(&exp_st->st, &op_ctx->ctx_export->lock,
+		record_io_stats(&exp_st->st, &op_ctx->ctx_export->exp_lock,
 				requested, transferred, success, is_write);
 	}
+#ifdef USE_MONITORING
+	if (op_ctx->req_type == NFS_REQUEST) {
+		uint16_t export_id = 0;
+		struct fsal_export *export = op_ctx->fsal_export;
+		struct gsh_client *client = op_ctx->client;
+		const char *client_ip =
+			client == NULL ? "" : client->hostaddr_str;
+
+		if (export != NULL)
+			export_id = export->export_id;
+		monitoring_nfs_io(requested, transferred, success, is_write,
+				  export_id, client_ip);
+	}
+#endif
 }
 
 /**
@@ -1583,7 +1591,7 @@ void inc_grants(struct gsh_client *client)
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		check_deleg_struct(&server_st->st, &client->lock);
+		check_deleg_struct(&server_st->st, &client->client_lock);
 		server_st->st.deleg->curr_deleg_grants++;
 	}
 }
@@ -1593,7 +1601,7 @@ void dec_grants(struct gsh_client *client)
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		check_deleg_struct(&server_st->st, &client->lock);
+		check_deleg_struct(&server_st->st, &client->client_lock);
 		server_st->st.deleg->curr_deleg_grants++;
 	}
 }
@@ -1603,7 +1611,7 @@ void inc_revokes(struct gsh_client *client)
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		check_deleg_struct(&server_st->st, &client->lock);
+		check_deleg_struct(&server_st->st, &client->client_lock);
 		server_st->st.deleg->num_revokes++;
 	}
 }
@@ -1613,7 +1621,7 @@ void inc_recalls(struct gsh_client *client)
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		check_deleg_struct(&server_st->st, &client->lock);
+		check_deleg_struct(&server_st->st, &client->client_lock);
 		server_st->st.deleg->tot_recalls++;
 	}
 }
@@ -1623,7 +1631,7 @@ void inc_failed_recalls(struct gsh_client *client)
 		struct server_stats *server_st;
 
 		server_st = container_of(client, struct server_stats, client);
-		check_deleg_struct(&server_st->st, &client->lock);
+		check_deleg_struct(&server_st->st, &client->client_lock);
 		server_st->st.deleg->failed_recalls++;
 	}
 }
@@ -2003,7 +2011,7 @@ void server_dbus_client_all_ops(DBusMessageIter *iter,
 	if (c_all->nfsv3) {
 		dbus_message_iter_open_container(iter, DBUS_TYPE_STRUCT,
 						 NULL, &array_iter);
-		for (i = 0; i < NFSPROC3_COMMIT + 1; i++) {
+		for (i = 0; i < NFS_V3_NB_COMMAND; i++) {
 			if (c_all->nfsv3->cmds[i].total) {
 				op_name = (char *) nfsproc3_to_str(i);
 				dbus_message_iter_append_basic(&array_iter,
@@ -2031,7 +2039,7 @@ void server_dbus_client_all_ops(DBusMessageIter *iter,
 	if (c_all->nlm4) {
 		dbus_message_iter_open_container(iter, DBUS_TYPE_STRUCT,
 						 NULL, &array_iter);
-		for (i = 0; i < NLMPROC4_FREE_ALL + 1; i++) {
+		for (i = 0; i < NLM_V4_NB_OPERATION; i++) {
 			if (c_all->nlm4->cmds[i].total) {
 				dbus_message_iter_append_basic(&array_iter,
 					DBUS_TYPE_STRING, &optnlm[i].name);
@@ -2280,7 +2288,7 @@ void global_dbus_fast(DBusMessageIter *iter)
 	version = "NFSv3:";
 	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING,
 				       &version);
-	for (i = 0; i <= NFSPROC3_COMMIT; i++) {
+	for (i = 0; i < NFS_V3_NB_COMMAND; i++) {
 		if (global_st.v3.op[i] > 0) {
 			op = (char *) nfsproc3_to_str(i);
 			dbus_message_iter_append_basic(&struct_iter,
@@ -2306,7 +2314,7 @@ void global_dbus_fast(DBusMessageIter *iter)
 	version = "\nNLM:";
 	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING,
 				       &version);
-	for (i = 0; i <= NLMPROC4_FREE_ALL; i++) {
+	for (i = 0; i < NLM_V4_NB_OPERATION; i++) {
 		if (global_st.lm.op[i] > 0) {
 			op = optnlm[i].name;
 			dbus_message_iter_append_basic(&struct_iter,
@@ -2320,7 +2328,7 @@ void global_dbus_fast(DBusMessageIter *iter)
 	version = "\nMNT:";
 	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING,
 				       &version);
-	for (i = 0; i <= MOUNTPROC3_EXPORT; i++) {
+	for (i = 0; i < MNT_V3_NB_COMMAND; i++) {
 		if (global_st.mn.op[i] > 0) {
 			op = optmnt[i].name;
 			dbus_message_iter_append_basic(&struct_iter,
@@ -2334,7 +2342,7 @@ void global_dbus_fast(DBusMessageIter *iter)
 	version = "\nQUOTA:";
 	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING,
 				       &version);
-	for (i = 0; i <= RQUOTAPROC_SETACTIVEQUOTA; i++) {
+	for (i = 0; i < RQUOTA_NB_COMMAND; i++) {
 		if (global_st.qt.op[i] > 0) {
 			op = optqta[i].name;
 			dbus_message_iter_append_basic(&struct_iter,
@@ -2609,21 +2617,18 @@ void reset_gsh_allops_stats(struct gsh_clnt_allops_stats *st)
 
 #ifdef _USE_NFS3
 	if (st->nfsv3) {
-		for (i = 0; i < NFSPROC3_COMMIT + 1 ; i++) {
+		for (i = 0; i < NFS_V3_NB_COMMAND ; i++)
 			reset_op_count(&(st->nfsv3->cmds[i]));
-		}
 	}
 #endif
 	if (st->nfsv4) {
-		for (i = 0; i < NFS4_OP_LAST_ONE ; i++) {
+		for (i = 0; i < NFS4_OP_LAST_ONE ; i++)
 			reset_op_count(&(st->nfsv4->cmds[i]));
-		}
 	}
 #ifdef _USE_NLM
 	if (st->nlm4) {
-		for (i = 0; i < NLMPROC4_FREE_ALL + 1 ; i++) {
+		for (i = 0; i < NLM_V4_NB_OPERATION ; i++)
 			reset_op_count(&(st->nlm4->cmds[i]));
-		}
 	}
 #endif
 }
@@ -2633,31 +2638,26 @@ void reset_global_stats(void)
 	int i;
 #ifdef _USE_NFS3
 	/* Reset all ops counters of nfsv3 */
-	for (i = 0; i < NFSPROC3_COMMIT; i++) {
+	for (i = 0; i < NFS_V3_NB_COMMAND; i++)
 		(void)atomic_store_uint64_t(&global_st.v3.op[i], 0);
-	}
 #endif
 	/* Reset all ops counters of nfsv4 */
-	for (i = 0; i < NFS4_OP_LAST_ONE; i++) {
+	for (i = 0; i < NFS4_OP_LAST_ONE; i++)
 		(void)atomic_store_uint64_t(&global_st.v4.op[i], 0);
-	}
 #ifdef _USE_NLM
 	/* Reset all ops counters of lock manager */
-	for (i = 0; i < NLM4_FAILED; i++) {
+	for (i = 0; i < NLM_V4_NB_OPERATION; i++)
 		(void)atomic_store_uint64_t(&global_st.lm.op[i], 0);
-	}
 #endif
 #ifdef _USE_NFS3
 	/* Reset all ops counters of mountd */
-	for (i = 0; i < MOUNTPROC3_EXPORT; i++) {
+	for (i = 0; i < MNT_V3_NB_COMMAND; i++)
 		(void)atomic_store_uint64_t(&global_st.mn.op[i], 0);
-	}
 #endif
 #ifdef _USE_RQUOTA
 	/* Reset all ops counters of rquotad */
-	for (i = 0; i < RQUOTAPROC_SETACTIVEQUOTA; i++) {
+	for (i = 0; i < RQUOTA_NB_COMMAND; i++)
 		(void)atomic_store_uint64_t(&global_st.qt.op[i], 0);
-	}
 #endif
 #ifdef _USE_NFS3
 	reset_nfsv3_stats(&global_st.nfsv3);
@@ -2826,7 +2826,7 @@ void server_dbus_v3_full_stats(DBusMessageIter *iter)
 	gsh_dbus_append_timestamp(iter, &v3_full_stats_time);
 	dbus_message_iter_open_container(iter, DBUS_TYPE_ARRAY,
 					 "(stttddd)", &array_iter);
-	for (op = 1; op < NFSPROC3_COMMIT+1; op++) {
+	for (op = 1; op < NFS_V3_NB_COMMAND; op++) {
 		if (v3_full_stats[op].total) {
 			op_name = (char *) nfsproc3_to_str(op);
 			dbus_message_iter_open_container(&array_iter,
@@ -2901,7 +2901,7 @@ void server_dbus_v4_full_stats(DBusMessageIter *iter)
 	gsh_dbus_append_timestamp(iter, &v4_full_stats_time);
 	dbus_message_iter_open_container(iter, DBUS_TYPE_ARRAY,
 					 "(sttddd)", &array_iter);
-	for (op = 1; op < NFS_V42_NB_OPERATION+1; op++) {
+	for (op = 1; op < NFS_V42_NB_OPERATION; op++) {
 		if (v4_full_stats[op].total) {
 			op_name = (char *) nfsop4_to_str(op);
 			dbus_message_iter_open_container(&array_iter,
@@ -3047,20 +3047,35 @@ void server_stats_allops_free(struct gsh_clnt_allops_stats *statsp)
 #ifdef _USE_NFS3
 static void record_v3_full_stats(struct svc_req *req,
 			       nsecs_elapsed_t request_time,
-			       bool success, bool dup)
+			       int status, bool dup)
 {
 	uint32_t prog = req->rq_msg.cb_prog;
 	uint32_t vers = req->rq_msg.cb_vers;
 	uint32_t proc = req->rq_msg.cb_proc;
 
+#ifdef USE_MONITORING
+	if (prog == NFS_PROGRAM) {
+		uint16_t export_id = 0;
+		struct fsal_export *export = op_ctx->fsal_export;
+		struct gsh_client *client = op_ctx->client;
+		const char *client_ip =
+			client == NULL ? "" : client->hostaddr_str;
+		if (export != NULL)
+			export_id = export->export_id;
+		monitoring_nfs3_request(proc, request_time, status,
+					export_id, client_ip);
+	}
+#endif
+
 	if (prog == NFS_program[P_NFS] && vers == NFS_V3) {
-		if (proc > NFSPROC3_COMMIT) {
+		if (proc >= NFS_V3_NB_COMMAND) {
 			LogCrit(COMPONENT_DBUS,
-				"req->rq_proc is more than COMMIT: %d\n",
+				"req->rq_proc is more than NFS_V3_NB_COMMAND: %d\n",
 				proc);
 			return;
 		}
-		record_op(&v3_full_stats[proc], request_time, success, dup);
+		record_op(&v3_full_stats[proc], request_time,
+			  status == NFS_REQ_OK, dup);
 	}
 }
 
@@ -3068,7 +3083,7 @@ void reset_v3_full_stats(void)
 {
 	int op;
 
-	for (op = 1; op < NFSPROC3_COMMIT+1; op++) {
+	for (op = 1; op < NFS_V3_NB_COMMAND; op++) {
 		v3_full_stats[op].total = 0;
 		v3_full_stats[op].errors = 0;
 		v3_full_stats[op].dups = 0;
@@ -3081,22 +3096,33 @@ void reset_v3_full_stats(void)
 
 static void record_v4_full_stats(uint32_t proc,
 			       nsecs_elapsed_t request_time,
-			       bool success)
+			       nfsstat4 status)
 {
-	if (proc > NFS_V42_NB_OPERATION) {
+#ifdef USE_MONITORING
+	uint16_t export_id = 0;
+	struct fsal_export *export = op_ctx->fsal_export;
+	struct gsh_client *client = op_ctx->client;
+	const char *client_ip = client == NULL ? "" : client->hostaddr_str;
+
+	if (export != NULL)
+		export_id = export->export_id;
+	monitoring_nfs4_request(proc, request_time, status, export_id,
+				client_ip);
+#endif
+	if (proc >= NFS4_OP_LAST_ONE) {
 		LogCrit(COMPONENT_DBUS,
-			"proc is more than NFS4_OP_WRITE_SAME: %d\n",
+			"proc is more than NFS4_OP_LAST_ONE: %d\n",
 			proc);
 		return;
 	}
-	record_op(&v4_full_stats[proc], request_time, success, false);
+	record_op(&v4_full_stats[proc], request_time, status == NFS4_OK, false);
 }
 
 void reset_v4_full_stats(void)
 {
 	int op;
 
-	for (op = 1; op < NFS_V42_NB_OPERATION+1; op++) {
+	for (op = 1; op < NFS4_OP_LAST_ONE; op++) {
 		v4_full_stats[op].total = 0;
 		v4_full_stats[op].errors = 0;
 		v4_full_stats[op].dups = 0;
